package anl.aida.algo;

import anl.aida.util.ObjScore;

/**
 * Interface for a Cluster as produced from a Clustering algorithm.
 * The cluster should contain items of type T.
 *
 */
public interface Cluster<T> {


  /**
   * Gets the label for this cluster. This can be a list of keywords
   * or a descriptive label, etc.
   *
   * @return the label for this cluster
   */
  String getCentroidLabel();

  /**
   * The number of items in this cluster.
   *
   * @return the number of items in this cluster
   */
  int getItemCount();

  /**
   * Gets the vector for the cluster centroid.
   *
   * @return
   */
  float[] getCentroid();

  /**
   * Gets the similarity score of the specified item with the centroid of this
   * cluster.
   *
   * @param item the item whose score we want
   * @return the similarity score of the specified item with the centroid of this
   *         cluster.
   */
  float getSimilarity(T item);

  /**
   * Gets an iterable over all the items and score in this cluster.
   *
   * @return an iterable over all the items and score in this cluster.
   */
  Iterable<ObjScore<T>> items();
}
