package anl.aida.algo;

/**
 *         Date: Feb 16, 2009 11:56:53 AM
 */
public interface Clusters<T> {
  /**
   * Gets the similarity score of the specified item
   * with the specified cluster centriod.
   *
   * @param item         the item whose score we want
   * @param clusterIndex the index of the cluster containing the relevant centroid
   * @return the similarity score of the specified item
   *         with the specified cluster centriod.
   */
  float getScore(T item, int clusterIndex);

  /**
   * Gets the number of clusters in this collection.
   *
   * @return the number of clusters in this collection.
   */
  int size();

  /**
   * Gets the total number of clustered items.
   *
   * @return the total number of clustered items.
   */
  int itemCount();

  /**
   * Gets an iterable over all the clustered items in this collection.
   * Each item is contained by one or more clusters.
   *
   * @return an iterable over all the clustered items in this collection.
   */
  Iterable<T> items();

  /**
   * Gets an iterable over all the Clusters contained by this collection.
   *
   * @return an iterable over all the Clusters contained by this collection.
   */
  Iterable<? extends Cluster<T>> clusters();

  /**
   * Gets the cluster at the specified index.
   *
   * @param index the index of the cluster
   * @return the cluster at the specified index.
   */
  public Cluster<T> getCluster(int index);
}
