package anl.aida.algo;

import anl.aida.core.DocumentID;
import anl.aida.core.KeywordGenerator;
import anl.aida.core.MIFSDocument;
import anl.aida.sv.SVKeywordGenerator;
import anl.aida.sv.SemanticVectorStore;
import anl.aida.util.HighestTermScores;
import anl.aida.util.AIDAException;
import anl.aida.util.TermScore;
import anl.aida.util.VectorUtilities;

import java.io.IOException;
import java.util.List;

/**
 */
public class ConceptClusterer {

  private Iterable<MIFSDocument> docIterator;
  private SemanticVectorStore vecStore;

  public ConceptClusterer(Iterable<MIFSDocument> docIterator, SemanticVectorStore vecStore) {
    this.docIterator = docIterator;
    this.vecStore = vecStore;
  }

  public Clusters<DocumentID> generateClusters(List<String> concepts) throws IOException {
    ClustersBuilder<DocumentID> builder = new ClustersBuilder<DocumentID>();
    float[][] centroids = new float[concepts.size()][];
    int index = 0;
    for (String concept : concepts) {
      float[] vector = vecStore.getTermVector(concept);
      builder.addCluster(vector, concept);
      centroids[index++] = vector;
    }

    for (MIFSDocument doc : docIterator) {
      String path = doc.getPath();
      float[] vector = vecStore.getDocumentVector(path);
      for (int j = 0; j < centroids.length; j++) {
        float[] centroid = centroids[j];
        float score = VectorUtilities.scalarProduct(centroid, vector);
        builder.addItemToCluster(new DocumentID(doc.getSTCIndex(), path), j, score);
      }
    }

    return builder.buildClusters();
  }
}
