/**
 * 
 */
package anl.aida.algo;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import anl.aida.client.Keyword;
import anl.aida.core.MIFSTerm;
import anl.aida.core.ScoredTermsCollection;
import anl.aida.util.AIDAException;

/**
 * Generates scored concepts from a predetermined list of concepts.
 * 
 */
public class DefaultConceptGenerator implements ConceptGenerator {
  
  private List<Keyword> concepts = new ArrayList<Keyword>();
  private ScoredTermsCollection stc;
  

  /**
   * Creates a DefaultConceptGenerator. The scored
   * results will be retrieved from the stc.
   * 
   * @param concepts
   * @param stc
   */
  public DefaultConceptGenerator(Collection<Keyword> concepts, ScoredTermsCollection stc) {
    this.concepts.addAll(concepts);
    this.stc = stc;
  }


  /* (non-Javadoc)
   * @see anl.mifs.algo.ConceptGenerator#generateConcepts(int)
   */
  @Override
  public Iterable<ScoredKeyword> generateConcepts(int count) throws AIDAException, IOException {
    List<ScoredKeyword> concepts = new ArrayList<ScoredKeyword>();
    for (Keyword keyword : this.concepts) {
      // must lower case as that's how the index is created
      int index = stc.getTermIndex(keyword.getTerm());
      if (index == -1) {
	// concept is not found in this stc
	concepts.add(new DefaultScoredKeyword(0, 0, keyword));
      } else {
	MIFSTerm term = stc.getTerm(index);
	concepts.add(new ScoredKeywordImpl(keyword, term));
      }
    }
    
    return concepts;
  }
  
  private static class ScoredKeywordImpl implements ScoredKeyword {
    
    private MIFSTerm term;
    private String dispString;
  
    public ScoredKeywordImpl(Keyword word, MIFSTerm term) {
      this.term = term;
      this.dispString = word.getLabel();
    }

    @Override
    public String getLabel() {
      return dispString;
    }

    @Override
    public int getDocumentCount() {
      return term.getDocumentCount();
    }

    @Override
    public float getScore() {
      return (term.getMaxScore() + term.getMinScore()) / 2;
    }

    @Override
    public String getTerm() {
      return term.getText();
    }

    @Override
    public int compareTo(Keyword o) {
      return dispString.compareTo(o.getLabel());
    }
    
    
  }
  
  
}
