/**
 * 
 */
package anl.aida.algo;

import java.util.Collection;

import anl.aida.client.CacheElement;
import anl.aida.client.Keyword;
import anl.aida.client.Keywords;

/**
 * ConceptGenerator producer that produces a ConceptGenerator that creates concepts
 * from a list of keywords.
 * 
 */
public class KeywordsConceptGeneratorProducer implements ConceptGeneratorProducer {
  
  private static String ALL_KEYWORDS = "__ALL__";
  
  private Keywords keywords;
  private String keywordSet;

  /**
   * Creates a KeywordsConceptGeneratorProducer that will produce concepts
   * from the specified keyword set.
   * 
   * @param keywords the keywords collection to use
   * @param keywordSet the name of the keyword set to use
   */
  public KeywordsConceptGeneratorProducer(Keywords keywords, String keywordSet) {
    this.keywords = keywords;
    this.keywordSet = keywordSet;
  }
  
  /**
   * Creates a KeywordsConceptGeneratorProducer that will produce concepts
   * from all the keywords in the keywords collection.
   * 
   * @param keywords the keywords collection to use
   */
  public KeywordsConceptGeneratorProducer(Keywords keywords) {
    this(keywords, ALL_KEYWORDS);
  }

  /* (non-Javadoc)
   * @see anl.mifs.client.gui.ConceptGeneratorProducer#createGenerator(anl.mifs.client.CacheElement)
   */
  @Override
  public ConceptGenerator createGenerator(CacheElement element) {
    Collection<Keyword> words;
    if (keywordSet.equals(ALL_KEYWORDS)) {
      words = keywords.getAllKeywords();
    } else {
      words = keywords.getKeywords(keywordSet);
    }
    DefaultConceptGenerator generator = new DefaultConceptGenerator(words, element.getSTC());
    return generator;
  }

  @Override
  public String getName() {
    return keywordSet.equals(ALL_KEYWORDS) ? "All Keywords" : keywordSet;
  }
}
