/**
 * 
 */
package anl.aida.algo;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import anl.aida.core.KeywordGenerator;
import anl.aida.core.MIFSDocument;
import anl.aida.core.ScoredTermsCollection;
import anl.aida.sv.SVKeywordGenerator;
import anl.aida.sv.SemanticVectorStore;
import anl.aida.util.HighestTermScores;
import anl.aida.util.AIDAException;
import anl.aida.util.TermScore;

/**
 * Generates concepts based on Semantic Vector scores.
 * 
 */
public class SVConceptGenerator implements ConceptGenerator {
  
  private SemanticVectorStore vecStore;
  private ScoredTermsCollection stc;

  public SVConceptGenerator(SemanticVectorStore vecStore, ScoredTermsCollection stc) {
    this.stc = stc;
    this.vecStore = vecStore;
  }

  /* (non-Javadoc)
   * @see algo.ConceptGenerator#generateConcepts(int)
   */
  @Override
  public Iterable<ScoredKeyword> generateConcepts(int count) throws AIDAException, IOException {
    KeywordGenerator gen = new SVKeywordGenerator(vecStore);
    // for now this just gets count number of terms
    // with the highest scores
    HighestTermScores total = new HighestTermScores(count);
    for (MIFSDocument doc : stc.documents()) {
      HighestTermScores scores = gen.generate(doc.getPath(), 10);
      for (TermScore score : scores.termScores()) {
        total.add(score.getTerm(), score.getScore());
      }
    }
    
    List<ScoredKeyword> concepts = new ArrayList<ScoredKeyword>();
    for (TermScore ts : total.termScores()) {
      String text = ts.getTerm();
      int docCount = stc.getTerm(stc.getTermIndex(text)).getDocumentCount();
      concepts.add(new DefaultScoredKeyword(docCount, ts.getScore(),text));
    }
    
    return concepts;
  }
}
