package anl.aida.bubble;

import java.awt.Rectangle;

import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.nodes.PPath;
import edu.umd.cs.piccolo.nodes.PText;
import edu.umd.cs.piccolo.util.PBounds;

/**
 * Abstract implementation of a Domain Axis.
 * 
 */
@SuppressWarnings("serial")
public abstract class AbstractDomainAxis extends AbstractAxis {
  
  public AbstractDomainAxis() {
    super(new NumericRange(0, 100), "X");
  }

  public AbstractDomainAxis(NumericRange range, String label) {
    super(range, label);
  }
  
  public void highlight(double val) {
    float pX = getPixelLocation(val);
    highlight.update(highlightFormatter.format(val));
    PBounds bounds = highlight.computeFullBounds(null);
    highlight.setOffset(pX - bounds.width / 2, 5);
    addChild(highlight);
  }
  
  protected abstract void layoutTicks(PPath ticks);
  
  protected double addLabel(PPath ticks, String val, double pTick, double labelOffset) {
    if (val.equals("-0")) val = "0";
    PText label = new PText(val);
    PBounds labelBounds = label.getBoundsReference();

    double xOffset = pTick - (labelBounds.width / 2);
    if (xOffset > labelOffset) {
      label.setOffset(xOffset, labelBounds.height);
      ticks.addChild(label);
      // 8 for some gap betweeen
      labelOffset = 8 + pTick + labelBounds.width / 2;
    }
    return labelOffset;
  }
  
  protected void layout() {
    removeAllChildren();
    tickLocations.clear();
    Rectangle bounds = canvas.getBounds();

    float xOffset = (float) getOffset().getX();
    pixelExtent = bounds.width - insets - xOffset;
    PNode axis = PPath.createLine(0, 0, pixelExtent, 0);
    addChild(axis);
    
    PPath ticks = new PPath();
    layoutTicks(ticks);
    addChild(ticks);

    axisLabel.setText(label);
    PBounds myBounds = getUnionOfChildrenBounds(null);
    axisLabel.setOffset(myBounds.width / 2 - axisLabel.getBounds().width / 2, myBounds.height + 4);
    addChild(axisLabel);
  }
}
