/**
 * 
 */
package anl.aida.bubble;

import java.awt.Color;
import java.awt.Font;
import java.awt.Stroke;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.nodes.PPath;
import edu.umd.cs.piccolo.nodes.PText;
import edu.umd.cs.piccolo.util.PAffineTransform;
import edu.umd.cs.piccolo.util.PBounds;
import edu.umd.cs.piccolox.nodes.PComposite;
import edu.umd.cs.piccolox.util.PBoundsLocator;
import edu.umd.cs.piccolox.util.PFixedWidthStroke;

/**
 * PNode that represents the bubble in the bubble graph.
 * 
 */
@SuppressWarnings("serial")
public class Bubble extends PComposite {
  
  public enum State {SELECTED, UNSELECTED, NEUTRAL}

  private static Stroke stroke = new PFixedWidthStroke(1f);
  private PPath shape;
  private PText pText = new PText();
  private PPath highlight;
  private boolean selected = false;
  private float alpha = .7f, unselectedAlpha = .2f;
  private State state = State.NEUTRAL;

  public Bubble(Color color, float size) {
    setPickable(true);
    setChildrenPickable(true);
    shape = PPath.createEllipse(0, 0, size, size);
    shape.setStroke(stroke);
    shape.setPaint(color);
    shape.setTransparency(alpha);
    addChild(shape);

    Font font = pText.getFont();
    pText.setFont(font.deriveFont(Font.BOLD));
    pText.setPickable(false);
  }

  public boolean isHighlighted() {
    return highlight != null;
  }

  /**
   * @return the selected
   */
  public boolean isSelected() {
    return selected;
  }
  
  public void setState(State state){
    this.state = state;
    setSelected(state == State.SELECTED);
    if (state == State.NEUTRAL || state == State.SELECTED) shape.setTransparency(alpha);
    else if (state == State.UNSELECTED) shape.setTransparency(unselectedAlpha);
  }
  
  public State getState() {
    return this.state;
  }

  /**
   * @param selected
   *          the selected to set
   */
  public void setSelected(boolean selected) {
    this.selected = selected;
    if (selected) moveToFront();
    if (selected && !isHighlighted())
      setHighlight(true);
    else if (!selected)
      setHighlight(false);
  }

  public void setHighlight(boolean enabled) {
    if (enabled && highlight == null) {
      BubbleData data = (BubbleData) getAttribute(BubbleConstants.DATA);
      pText.setText(data.category);
      PBounds bounds = pText.computeFullBounds(null);
      highlight = PPath.createRoundRectangle(0, 0, (float) (bounds.width + 8),
	  (float) (bounds.height + 8), 6, 6);
      highlight.setPickable(false);
      PBounds backBounds = highlight.computeFullBounds(null);
      pText.setOffset(backBounds.getCenterX() - bounds.width / 2, backBounds.getCenterY()
	  - bounds.height / 2);
      // need to get the inverse of the shape scale so that the
      // text is always the same size and not effected by the
      // scale of its parent (i.e. this).
      highlight.setScale(1 / getScale());
      highlight.addChild(pText);
      resetHighlightOffset();
      addChild(highlight);
      moveToFront();
    } else {
      if (!selected && highlight != null) {
	removeChild(highlight);
	highlight.removeAllChildren();
	highlight = null;
      }
    }
  }
  
  private void resetHighlightOffset() {
    PBounds backBounds = highlight.computeFullBounds(null);
    PBoundsLocator locator = PBoundsLocator.createNorthWestLocator(shape);
    // make sure that the label doesn't overlap the left most edge.
    // transform coords into parent and see if label would be off the
    // left edge of parent
    Point2D pt = new Point2D.Double();
    locator.locatePoint(pt);
    pt = localToParent(pt);
    Rectangle2D rect = localToParent(new PBounds(backBounds));
    if (pt.getX() - rect.getWidth() < 0) {
	PBounds shapeBounds = shape.computeFullBounds(null);
	highlight.setOffset(shapeBounds.x + shapeBounds.width, shapeBounds.y - backBounds.height);
    } else {
	highlight.setOffset(locator.locateX() - backBounds.width, locator.locateY()
	    - backBounds.height);
    }
  }

  @Override
  public void addAttribute(Object key, Object value) {
    shape.addAttribute(key, value);
  }

  @Override
  public Object getAttribute(Object key, Object def) {
    return shape.getAttribute(key, def);
  }

  @Override
  public Object getAttribute(Object key) {
    return shape.getAttribute(key);
  }

  public PNode getShape() {
    return shape;
  }
  
  public void updateAnimate(float px, float py, double scale) {
    setVisible(true);
    setPickable(true);
    PBounds bounds = shape.getBounds();
    PAffineTransform transform = shape.getTransform();
    transform.setScale(scale);
    transform.transform(bounds, bounds);

    animateToPositionScaleRotation(px - bounds.width / 2, py - bounds.height / 2, scale, 0,
	DefaultBubbleRenderer.ANIM_DURATION);
  }

  public void update(float px, float py, double scale) {
    setVisible(true);
    setPickable(true);
    setScale(scale);
    PBounds bounds = shape.getBounds();
    PAffineTransform transform = getTransform();
    transform.transform(bounds, bounds);
    if (highlight != null) {
      highlight.setScale(1 / getScale());
      resetHighlightOffset();
    }
    setOffset(px - bounds.width / 2, py - bounds.height / 2);
  }
  
}
