/**
 * 
 */
package anl.aida.bubble;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.DefaultListModel;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;
import javax.swing.ListModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import anl.aida.client.util.ClientUtils;

import com.jidesoft.swing.CheckBoxList;

import edu.umd.cs.piccolo.PCanvas;

/**
 */
public class BubbleChart {

  private static class CategoryComparator implements Comparator<Category> {

    private Comparator<String> comparator;

    public CategoryComparator(Comparator<String> comparator) {
      this.comparator = comparator;
    }

    @Override
    public int compare(Category o1, Category o2) {
      return comparator.compare(o1.category, o2.category);
    }
  }

  private static class Category {

    String category;
    int index;

    public Category(String category, int index) {
      this.category = category;
      this.index = index;
    }

    public String toString() {
      return category;
    }
  }

  private BubblePlot plot;
  private PCanvas canvas = new PCanvas();
  private JPanel panel, listPanel;
  private CheckBoxList list;
  private boolean ignoreSelection = false, catShowing = true;
  private Set<Category> selections = new HashSet<Category>();
  private BubbleSlider slider;
  private BubbleDataset data;
  private CategoryComparator catComparator;
  private JButton deselBtn, entityPanelBtn;

  private Icon raiseIcon = ClientUtils.loadIcon("raiselayer.png");
  private Icon lowerIcon = ClientUtils.loadIcon("lowerlayer.png");

  public BubbleChart(BubbleDataset data) {
    this.data = data;
    plot = new BubblePlot(canvas, data);
    panel = new JPanel(new BorderLayout());
    panel.setBorder(BorderFactory.createEmptyBorder(6, 6, 6, 6));

    JPanel canvasPanel = new JPanel(new BorderLayout());
    canvasPanel.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 6));
    canvasPanel.add(canvas, BorderLayout.CENTER);
    panel.add(canvasPanel, BorderLayout.CENTER);

    slider = new BubbleSlider(data, plot);
    JToolBar bar = new JToolBar();
    bar.setBackground(Color.WHITE);
    bar.setFloatable(false);
    bar.add(slider.getPlayButton());
    bar.add(slider.getTimeSlider());
    canvasPanel.add(bar, BorderLayout.SOUTH);

    bar = new JToolBar();
    JButton button = new JButton(ClientUtils.loadIcon("configure.png"));
    button.setToolTipText("Configure Chart");
    bar.add(button);

    button = new JButton(ClientUtils.loadIcon("camera.png"));
    button.setToolTipText("Take Snapshot");
    bar.add(button);

    entityPanelBtn = new JButton(ClientUtils.loadIcon("raiselayer.png"));
    entityPanelBtn.setToolTipText("Hide Entity List");
    entityPanelBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	flipEntityPanel();
      }
    });
    entityPanelBtn.setBorderPainted(false);
    bar.add(Box.createGlue());
    bar.add(entityPanelBtn);

    bar.setFloatable(false);
    panel.add(bar, BorderLayout.NORTH);

    catComparator = new CategoryComparator(new Comparator<String>() {
      @Override
      public int compare(String o1, String o2) {
	return o1.compareTo(o2);
      }
    });

    initCatList();
    initListeners();
  }

  /**
   * Sets the comparator to use when sorting the category list.
   * 
   * @param comparator
   *          the comparator to use
   */
  public void setCategoryComparator(Comparator<String> comparator) {
    catComparator.comparator = comparator;
    initCatList();
  }

  /**
   * Gets the comparator to use when sorting the category list.
   * 
   * @return the comparator to use when sorting the category list.
   */
  public Comparator<String> getCategoryComparator() {
    return catComparator.comparator;
  }

  private void initCatList() {

    List<Category> categories = new ArrayList<Category>();
    for (String cat : data.categories()) {
      categories.add(new Category(cat, data.getCategoryIndex(cat)));
    }
    Collections.sort(categories, catComparator);

    DefaultListModel model = new DefaultListModel();
    for (Category cat : categories) {
      model.addElement(cat);
    }

    if (list == null) {
      list = new CheckBoxList();
      listPanel = new JPanel(new BorderLayout());
      JToolBar bar = new JToolBar();
      bar.setFloatable(false);
      listPanel.add(bar, BorderLayout.NORTH);
      JButton selBtn = new JButton(ClientUtils.loadIcon("select_multiple.png"));

      selBtn.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent evt) {
	  selectAll();
	}
      });

      deselBtn = new JButton(ClientUtils.loadIcon("deselect_multiple.png"));
      deselBtn.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent evt) {
	  unselectAll();
	}
      });

      deselBtn.setEnabled(false);
      bar.add(selBtn);
      bar.add(deselBtn);
      listPanel.add(new JScrollPane(list), BorderLayout.CENTER);
      panel.add(listPanel, BorderLayout.EAST);
    }

    list.setModel(model);
  }

  private void selectAll() {
    int size = list.getModel().getSize();
    list.getCheckBoxListSelectionModel().setSelectionInterval(0, size - 1);
  }

  private void unselectAll() {
    for (int index : list.getCheckBoxListSelectedIndices()) {
      list.removeCheckBoxListSelectedIndex(index);
    }
    deselBtn.setEnabled(false);
  }

  private void initListeners() {

    plot.addSelectionListener(new BubbleSelectionListener() {
      @Override
      public void bubbleSelected(int categoryIndex) {
	if (!ignoreSelection)
	  selectBubbleBox(categoryIndex);
      }

      @Override
      public void bubbleUnselected(int categoryIndex) {
	if (!ignoreSelection)
	  unselectBubbleBox(categoryIndex);
      }
    });

    list.getCheckBoxListSelectionModel().addListSelectionListener(new ListSelectionListener() {
      @Override
      public void valueChanged(ListSelectionEvent e) {
	if (!ignoreSelection) {
	  ignoreSelection = true;
	  if (!e.getValueIsAdjusting()) {

	    List<Category> tmpSel = new ArrayList<Category>();
	    for (Object obj : list.getCheckBoxListSelectedValues()) {
	      Category cat = (Category) obj;
	      tmpSel.add(cat);
	      if (!selections.remove(cat)) {
		selectBubble(cat.index);
	      }
	    }

	    for (Category cat : selections) {
	      plot.selectBubble(cat.index, false);
	    }
	    selections.clear();
	    selections.addAll(tmpSel);

	    deselBtn.setEnabled(true);
	  }
	  ignoreSelection = false;
	}
      }
    });
  }

  private void selectBubble(int catIndex) {
    int timestep = slider.getTimestep();
    double xVal = data.getXValue(catIndex, timestep);
    double yVal = data.getYValue(catIndex, timestep);
    double zVal = data.getZValue(catIndex, timestep);

    while ((Double.isNaN(xVal) || Double.isNaN(yVal) || Double.isNaN(zVal))
	&& timestep < data.getTimeStepCount()) {
      timestep++;
      xVal = data.getXValue(catIndex, timestep);
      yVal = data.getYValue(catIndex, timestep);
      zVal = data.getZValue(catIndex, timestep);
    }

    if (timestep > slider.getTimestep())
      slider.setTimestep(timestep);
    plot.selectBubble(catIndex, true);
  }

  private void selectBubbleBox(int catIndex) {
    ListModel model = list.getModel();
    for (int i = 0; i < model.getSize(); i++) {
      Category cat = (Category) model.getElementAt(i);
      if (cat.index == catIndex) {
	ignoreSelection = true;
	selections.add(cat);
	list.getCheckBoxListSelectionModel().addSelectionInterval(i, i);
	ignoreSelection = false;
	deselBtn.setEnabled(true);
	break;
      }
    }
  }

  private void unselectBubbleBox(int catIndex) {
    ListModel model = list.getModel();
    for (int i = 0; i < model.getSize(); i++) {
      Category cat = (Category) model.getElementAt(i);
      if (cat.index == catIndex) {
	ignoreSelection = true;
	selections.remove(cat);
	list.getCheckBoxListSelectionModel().removeSelectionInterval(i, i);
	deselBtn.setEnabled(list.getCheckBoxListSelectedValues().length != 0);
	ignoreSelection = false;
	break;
      }
    }
  }

  private void flipEntityPanel() {
    if (catShowing) {
      panel.remove(listPanel);
      entityPanelBtn.setIcon(lowerIcon);
      entityPanelBtn.setToolTipText("Show Entity List");
    } else {
      panel.add(listPanel, BorderLayout.EAST);
      entityPanelBtn.setIcon(raiseIcon);
      entityPanelBtn.setToolTipText("Hide Entity List");
    }
    catShowing = !catShowing;
    panel.validate();
    panel.repaint();
  }

  /*
   * private void showConfigurePopup(ActionEvent evt) { JPopupMenu popup = new
   * JPopupMenu(); popup.setBorder(BorderFactory.createEmptyBorder(6, 6, 6, 6));
   * JCheckBoxMenuItem item = new JCheckBoxMenuItem("Entity List", catShowing);
   * item.addActionListener(new ActionListener() { public void
   * actionPerformed(ActionEvent evt) {
   * 
   * } });
   * 
   * popup.add(item); popup.add("Plot Properties..."); JComponent comp =
   * (JComponent) evt.getSource(); popup.show(comp, comp.getX(), comp.getY() +
   * comp.getHeight()); }
   */

  public BubblePlot getPlot() {
    return plot;
  }

  public JPanel getPanel() {
    return panel;
  }
}
