package anl.aida.bubble;

import java.awt.BorderLayout;
import java.awt.Color;
import java.util.ArrayList;
import java.util.concurrent.ExecutionException;

import javax.swing.JPanel;
import javax.swing.SwingWorker;

import simphony.util.messages.MessageCenter;
import anl.aida.client.gui.AIDAGui;
import anl.aida.client.gui.BubbleConfigBean;
import anl.aida.client.gui.FormulaListElement;
import anl.aida.client.gui.ProgressLayer;
import anl.aida.client.gui.BubbleConfigBean.AxisType;
import anl.aida.data.AxisRange;
import anl.aida.data.DataFrame;
import anl.aida.data.DataManager;
import anl.aida.formula.Formula;
import anl.aida.formula.IllegalFormulaException;

public class BubblePlotCreator extends SwingWorker<BubbleDataset, Object> {

  private static MessageCenter center = MessageCenter.getMessageCenter(BubblePlotCreator.class);

  private JPanel panel;
  private AIDAGui gui;
  private BubbleConfigBean bean;
  private DataManager manager;

  public BubblePlotCreator(BubbleConfigBean bean, JPanel panel, AIDAGui gui, DataManager manager) {
    this.panel = panel;
    this.bean = bean;
    this.gui = gui;
  }

  private DataFrame evaluateFormula(FormulaListElement item) throws IllegalFormulaException {
    Formula formula = item.getFormula();
    return formula.evaluate(manager, new ArrayList<AxisRange>());
  }

  @Override
  protected BubbleDataset doInBackground() throws Exception {
    gui.clearStatusMessage();
    try {
      DataFrame xFrame = evaluateFormula(bean.getXItem().getElement());
      DataFrame yFrame = evaluateFormula(bean.getYItem().getElement());
      DataFrame zFrame = null;
      FormulaListElement zElement = bean.getZItem().getElement();
      if (zElement != null) {
	zFrame = evaluateFormula(zElement);
      }

      return new DefaultBubbleDataset(xFrame, yFrame, zFrame, bean.getEntityCount());

    } catch (IllegalFormulaException ex) {
      center.error("Error while evaluating formula", ex);
      gui.addStatusMessage("Error while evaluating formula: " + ex.getMessage(), Color.RED);
      return null;
    }
  }

  @Override
  protected void done() {
    try {
      BubbleDataset data = get();
      if (data != null) {
	BubbleChart chart = new BubbleChart(data);
	if (bean.getXAxisType() == AxisType.LOG) {
	  chart.getPlot().setDomainAxis(new LogDomainAxis(data.getXRange(), bean.getXLabel()));
	} else {
	  chart.getPlot().getDomainAxis().setLabel(bean.getXLabel());
	}

	if (bean.getYAxisType() == AxisType.LOG) {
	  // todo log axis for Y
	  chart.getPlot().getRangeAxis().setLabel(bean.getYLabel());
	} else {
	  chart.getPlot().getRangeAxis().setLabel(bean.getYLabel());
	}

	panel.add(chart.getPanel(), BorderLayout.CENTER);
	panel.validate();
      }

    } catch (InterruptedException e) {
      center.error("Error while creating bubble plot", e);
    } catch (ExecutionException e) {
      center.error("Error while creating bubble plot", e);
    } finally {
      ((ProgressLayer) panel.getClientProperty(ProgressLayer.CLIENT_KEY)).setEnabled(true);
    }
  }
}
