package anl.aida.bubble;

import java.awt.Color;
import java.util.HashMap;
import java.util.Map;

import anl.aida.bubble.Bubble.State;
import edu.umd.cs.piccolo.PLayer;
import edu.umd.cs.piccolo.PNode;

public class DefaultBubbleRenderer {

  public static final long ANIM_DURATION = 250;
  private static final int MAX_BUBBLE_SIZE = 72;
  // maps category index to bubble.
  private Map<Integer, Bubble> bubbles = new HashMap<Integer, Bubble>();
  private PLayer layer = new BubbleLayer();
  private int selectedCount = 0;

  // private PLayer selectedLayer = new BubbleLayer();

  public DefaultBubbleRenderer(BubbleDataset dataset, BubbleTooltip tooltip) {
    // PaletteCreator creator = new PaletteCreator();
    // Palette palette =
    // creator.createPalettes(dataset.getCategoryCount()).get(0);
    Color[] palette = generateColors(dataset.getCategoryCount());
    int i = 0;
    for (String category : dataset.categories()) {
      Color pColor = palette[i];
      Bubble bubble = new Bubble(pColor, MAX_BUBBLE_SIZE);
      int categoryIndex = dataset.getCategoryIndex(category);
      BubbleData data = new BubbleData(category, categoryIndex);
      data.category = category;
      bubble.addAttribute(BubbleConstants.DATA, data);
      layer.addChild(bubble);
      bubbles.put(categoryIndex, bubble);
      i++;
    }
    layer.addInputEventListener(tooltip);
    // selectedLayer.addInputEventListener(tooltip);
  }

  public Color[] generateColors(int n) {
    Color[] cols = new Color[n];
    for (int i = 0; i < n; i++) {
      cols[i] = Color.getHSBColor((float) i / (float) n, 0.85f, 1.0f);
    }
    return cols;
  }

  /**
   * Selects or unselects the bubble with the specified category index.
   * 
   * @param categoryIndex
   *          in the index of the bubble
   * @param selected
   *          whether or not the bubble is selected or unselected.
   */
  public void selectBubble(int categoryIndex, boolean selected) {
    Bubble bubble = bubbles.get(categoryIndex);
    if (selected) {
	selectedCount++;
	//System.out.println("selected count on select: " + selectedCount);
	bubble.setState(State.SELECTED);
	if (selectedCount == 1) {
	  // set everything else to unselected state.
	  for (int i = 0, n = layer.getChildrenCount(); i < n; i++) {
	    Bubble aBubble = (Bubble) layer.getChild(i);
	    if (aBubble.getState() != State.SELECTED)
	      aBubble.setState(State.UNSELECTED);
	  }
	}
    } else {
      bubble.setSelected(selected);
      bubble.setState(State.UNSELECTED);
      selectedCount--;
      //System.out.println("selected Count: " + selectedCount);
      // is everything unselected, if so then set all the bubbles
      // state to neutral.
      if (selectedCount < 1) {
	selectedCount = 0;
	for (Bubble aBubble : bubbles.values()) {
	  aBubble.setState(State.NEUTRAL);
	}
      }
    }
  }

  public void addLayers(PNode parent) {
    parent.addChild(layer);
    // parent.addChild(selectedLayer);
  }

  public PNode update(int category, float px, float py, double scale) {
    scale *= .9;
    scale += .1;
    Bubble bubble = bubbles.get(category);
    bubble.update(px, py, scale);
    return bubble;
  }

  public PNode updateAnimate(int category, float px, float py, double scale) {
    Bubble node = bubbles.get(category);
    // don't animate into existence
    if (!node.getVisible())
      update(category, px, py, scale);
    scale *= .9;
    scale += .1;
    node.updateAnimate(px, py, scale);
    return node;
  }

  public void setNodeVisible(int category, boolean visible) {
    Bubble node = bubbles.get(category);
    node.setVisible(visible);
    node.setPickable(visible);
  }
}
