/**
 * 
 */
package anl.aida.bubble;

import edu.umd.cs.piccolo.nodes.PPath;

/**
 * Domain Axis with logarithmic scale.
 * 
 */
@SuppressWarnings("serial")
public class LogDomainAxis extends AbstractDomainAxis {

  public LogDomainAxis() {
    this(new NumericRange(0, 100), "X");
  }

  public LogDomainAxis(NumericRange range, String label) {
    super(range, label);
    labelFormatter.setMaximumFractionDigits(0);
  }

  private int findStartingExp() {
    int exp = 1;
    double value = Math.pow(10, exp);
    while (value < range.getStart()) {
      exp++;
      value = Math.pow(10, exp);
    }

    return exp;
  }

  private int findFirstTick(int startingVal) {
    while (startingVal < range.getStart()) {
      startingVal += startingVal;
    }
    return startingVal;
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * anl.aida.bubble.AbstractDomainAxis#layoutTicks(edu.umd.cs.piccolo.nodes
   * .PPath)
   */
  @Override
  protected void layoutTicks(PPath ticks) {
    double labelOffset = Integer.MIN_VALUE;
    int exp = findStartingExp();
    // first tick is the first multiple of pow(10, exp - 1)
    // that is greater than range.getStart()
    int tick = findFirstTick((int) Math.pow(10, exp - 1));
    // we want to place the first tick about 20 pixels in
    // so for everytick we subtract that amount
    // float pTickOffset = getPixelLocationLog(tick) - 20;
    if (tick < range.getEnd()) {
      labelOffset = addTickMark(ticks, tick, labelOffset);
      tick += (int) Math.pow(10, exp - 1);
    }
    
    while (Math.pow(10, exp) <= range.getEnd()) {
      int increment = (int) Math.pow(10, exp - 1);
      int limit = (int) Math.pow(10, exp);
      tick += increment;
      while (tick < limit) {
	labelOffset = addTickMark(ticks, tick, labelOffset);
	tick += increment;
      }
      exp++;
      tick = limit;

    }
  }

  private double addTickMark(PPath ticks, int tick, double labelOffset) {
    float pTick = getPixelLocationLog(tick); // - pTickOffset;
    ticks.moveTo(pTick, 0);
    ticks.lineTo(pTick, 5);
    tickLocations.add(pTick);

    labelOffset = addLabel(ticks, labelFormatter.format(tick), pTick, labelOffset);
    return labelOffset;

  }

  /*
   * (non-Javadoc)
   * 
   * @see anl.aida.bubble.AbstractAxis#getPixelLocation(double)
   */
  @Override
  public float getPixelLocation(double val) {
    return super.getPixelLocationLog(val);
  }

  @Override
  public double getValueForPixel(double pixel) {
    return super.getValueForPixelLog(pixel);
  }
}
