package anl.aida.bubble;

import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.nodes.PPath;
import edu.umd.cs.piccolo.nodes.PText;
import edu.umd.cs.piccolo.util.PBounds;

@SuppressWarnings("serial")
public class RangeAxis extends AbstractAxis {

  private float yEnd = 0;

  public RangeAxis() {
    super(new NumericRange(0, 100), "Y Axis");
  }

  public RangeAxis(NumericRange range, String label) {
    super(range, label);
    super.initRange(range.getLength() * .05);
  }

  public void setYEnd(float yEnd) {
    this.yEnd = yEnd;
  }

  public float getYEnd() {
    return yEnd;
  }

  public void highlight(double val) {
    float pY = getPixelLocation(val);
    highlight.update(highlightFormatter.format(val));
    PBounds bounds = highlight.computeFullBounds(null);
    highlight.setOffset(-bounds.width, pY - bounds.height / 2);
    addChild(highlight);
  }

  public float getPixelLocation(double val) {
    return yEnd - super.getPixelLocationLin(val);
  }

  @Override
  public double getValueForPixel(double pixel) {
    pixel = yEnd - pixel;
    return super.getValueForPixelLin(pixel);
  }

  protected void layout() {
    removeAllChildren();
    tickLocations.clear();

    // yEnd will be 0 on the first layout so we need to adjust
    // so the first layout is approx. correct so that the bounds for
    // rAxis aren't widly incorrect.
    if (yEnd == 0) yEnd = canvas.getHeight();
    pixelExtent = yEnd - insets;
    PNode axis = PPath.createLine(0, insets, 0, yEnd);
    addChild(axis);

    PPath ticks = new PPath();
    TickLocator locator = new TickLocator(range, (int) (pixelExtent / 20));
    labelFormatter.setMaximumFractionDigits(locator.getFracDigits());
    while (locator.hasNext()) {
      double val = locator.next();
      float pTick = getPixelLocation(val);
      ticks.moveTo(0, pTick);
      ticks.lineTo(-5, pTick);
      tickLocations.add(pTick);

      String textVal = labelFormatter.format(val);
      if (textVal.equals("-0"))
	textVal = "0";
      PText label = new PText(textVal);
      PBounds labelBounds = label.getBoundsReference();
      label.setOffset(-labelBounds.width - 8, pTick - labelBounds.height / 2);
      ticks.addChild(label);
    }

    addChild(ticks);
    PBounds pBounds = getUnionOfChildrenBounds(null);
    axisLabel.setText(label);
    axisLabel.setRotation(0);
    PBounds labelBounds = axisLabel.computeFullBounds(null);
    axisLabel.rotateAboutPoint(Math.toRadians(270), labelBounds.getCenterX(), labelBounds
	.getCenterY());
    axisLabel.setOffset(-pBounds.width - labelBounds.height - 8, pBounds.height / 2
	+ labelBounds.width / 2);

    addChild(axisLabel);
  }
}
