/**
 * 
 */
package anl.aida.bubble;

/**
 * Produces Tick mark locations given a range and the desired number of ticks.
 * Uses the loose tick method from Graphics Gems.
 * 
 */
public class TickLocator {
  
  private double spacing, min, max;
  private int nfrac;
  private double current;
  
  public TickLocator(NumericRange nRange, int numTicks) {
    double range = niceNum(nRange.getLength(), false);
    if (numTicks < 0) numTicks = 2;
    spacing = niceNum(range / (numTicks - 1), true);
    min = Math.ceil(nRange.getStart() / spacing) * spacing;
    max = Math.floor(nRange.getEnd() / spacing) * spacing;
    nfrac = (int)Math.max(-Math.floor(Math.log10(spacing)), 0);
    current = min;
  }
  
  public double getMin() {
    return min;
  }
  
  public double getMax() {
    return max;
  }
  
  public int getFracDigits() {
    return nfrac;
  }
  
  public boolean hasNext() {
    return current < max + .5 * spacing;
  }
  
  public double next() {
    double tmp = current;
    current += spacing;
    return tmp;
  }
  
  private double niceNum(double val, boolean round) {
    int exp = (int)Math.floor(Math.log10(val));
    double f = val / Math.pow(10, exp);
    double nf;
    if (round) {
      if (f < 1.5) nf = 1;
      else if (f < 3) nf = 2;
      else if (f < 7) nf = 5;
      else nf = 10;

    } else {
      if (f <= 1.5) nf = 1;
      else if (f <= 2) nf = 2;
      else if (f <= 5) nf = 5;
      else nf = 10;
    }
    return nf * Math.pow(10, exp);
  }

}
