package anl.aida.bubble;

import java.awt.Color;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import edu.umd.cs.piccolo.PCamera;
import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.event.PBasicInputEventHandler;
import edu.umd.cs.piccolo.event.PInputEvent;
import edu.umd.cs.piccolo.nodes.PPath;
import edu.umd.cs.piccolo.util.PBounds;

/**
 * Handles zooms of the bubble chart
 * 
 */
public class Zoomer extends PBasicInputEventHandler {
  
  private Point2D start;
  private PNode plot, zoomBox;
  private PCamera camera;
  private PBounds bounds;
  private BubblePlot bubblePlot;
  
  public Zoomer(BubblePlot bubblePlot) {
    this.plot = bubblePlot.getPlot();
    this.camera = bubblePlot.getCanvas().getCamera();
    this.bubblePlot = bubblePlot;
  }

  @Override
  public void mousePressed(PInputEvent e) {
    super.mousePressed(e);
    bounds = plot.getGlobalFullBounds();
    //System.out.println(bounds);
    if (bounds.contains(e.getCanvasPosition())) {
      bubblePlot.setIgnoreMouseOver(true);
      start = e.getPosition();
      createBox(start);
    } else {
      start = null;
    }
  }
  
  private Rectangle2D createBox(Point2D end) {
    if (zoomBox != null) camera.removeChild(zoomBox);
    float width = (float)(end.getX() - start.getX());
    float height = (float)(end.getY() - start.getY());
    zoomBox = PPath.createRectangle((float)start.getX(), (float)start.getY(), width, height);
    zoomBox.setPaint(Color.RED);
    zoomBox.setTransparency(.4f);
    camera.addChild(zoomBox);
    return zoomBox.getBounds();
    
  }
  
  public void mouseDragged(PInputEvent e) {
    super.mouseDragged(e);
    if (start != null) {
      Point2D end = e.getPosition();
      if (end.getY() > bounds.height + bounds.y) {
	end = new Point2D.Double(end.getX(), bounds.height + bounds.y);
      }
      
      if (end.getX() > bounds.width + bounds.x) {
	end = new Point2D.Double(bounds.width + bounds.x, end.getY());
      }
      createBox(end);
    }
  }
  
  public void mouseReleased(PInputEvent e) {
    super.mouseReleased(e);
    if (start != null) {
      Rectangle2D bounds = createBox(e.getPosition());
      camera.removeChild(zoomBox);
      this.bounds = null;
      zoomBox = null;
      if (bounds.getWidth() >= 5 || bounds.getHeight() >= 5) {
	bubblePlot.zoom(bounds);
      } else if (bounds.isEmpty()) {
	bubblePlot.resetBounds();
      }
      
    }
    bubblePlot.setIgnoreMouseOver(false);
    start = null;
  }
}
