package anl.aida.client;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.stream.XMLStreamException;

import simphony.util.messages.MessageCenter;
import anl.aida.client.gui.AIDAGui;
import anl.aida.client.gui.TriggerDialog;
import anl.aida.client.io.AlertFileIO;
import anl.aida.client.io.CacheFileReader;
import anl.aida.query.Trigger;
import anl.aida.util.FileUtilities;

/**
 * Date: Feb 3, 2009 3:22:37 PM
 */
public class AIDAClient {

  private static final MessageCenter center = MessageCenter.getMessageCenter(AIDAClient.class);

  private AIDAGui gui;
  private Cache cache;

  void init(AIDAGui gui) {
    this.gui = gui;
  }

  public void openTriggerDialog() {
    TriggerDialog dialog = new TriggerDialog(gui.getFrame());
    List<Trigger> triggers = new ArrayList<Trigger>();
    AlertFileIO io = new AlertFileIO();
    try {
      triggers = io.read("./cache/alerts.txt");
    } catch (IOException ex) {
      center.error("Error while reading alerts file", ex);
    }
    dialog.init(triggers, cache);
    dialog.setLocationRelativeTo(gui.getFrame());
    dialog.setSize(560, 400);
    dialog.setVisible(true);

    if (!dialog.isCanceled()) {
      triggers = dialog.getAlerts();
      try {
	io.write(triggers, "./cache/alerts.txt");
      } catch (IOException ex) {
	center.error("Error while writing alerts file", ex);
      }
    }
  }

  /**
   * Opens a cache from the specified file.
   * 
   * @param file
   *          the cache description file
   */
  public void openCache(File file) {
    try {

      CacheFileReader reader = new CacheFileReader();
      cache = reader.read(file);
      gui.resetCache(cache);

    } catch (XMLStreamException e) {
      e.printStackTrace();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  /**
   * Called when the client is exiting.
   * 
   * @return true if the client should exit, otherwise false.
   */
  public boolean exit() {
    // delete the all the created semantic vectors
    if (cache != null)
    FileUtilities.delete(new File(cache.getCacheDirectory(), CacheElement.SV_FILES_DIR));
    return true;
  }
}
