package anl.aida.client;

import anl.aida.algo.Clusterer;
import anl.aida.client.doc.DocumentsModel;
import anl.aida.core.DocumentID;
import anl.aida.core.KeywordGenerator;
import anl.aida.core.ScoredTermsCollection;

/**
 *
 */
public interface CacheDocumentSet {

  /**
   * Gets a descriptive label for this CacheDocumentSet.
   *
   * @return a descriptive label for this CacheDocumentSet.
   */

  String getLabel();

  /**
   * Gets the id of this document set.
   *
   * @return the id of this document set.
   */
  DocSetID getId();

  /**
   * Gets the Clusterer that should be used to Cluster this document set.
   *
   * @return the Clusterer that should be used to Cluster this document set.
   */
  Clusterer<DocumentID> getClusterer();

  /**
   * Gets the number of documents in this CacheDocumentSet.
   *
   * @return the number of documents in this CacheDocumentSet.
   */
  int getDocumentCount();

  /**
   * Gets the number of terms in this CacheDocumentSet.
   *
   * @return the number of terms in this CacheDocumentSet.
   */
  int getTermCount();

  /**
   * Gets the timestamp of this CacheDocumentSet.
   *
   * @return the timestamp of this CacheDocumentSet.
   */
  String getTimeStampAsString();

  /**
   * Gets the document ids of all the docs this set contains.
   *
   * @return the document ids of all the docs this set contains.
   */
  Iterable<DocumentID> documents();

  /**
   * Gets a keyword generator that can be used to generate
   * keywords for this doc set.
   *
   * @return a keyword generator that can be used to generate
   *         keywords for this doc set.
   */
  KeywordGenerator getKeyWordGenerator();

  /**
   * Gets the ScoredTermsCollection associated with this CacheDocumentSet.
   *
   * @return the ScoredTermsCollection associated with this CacheDocumentSet.
   */
  ScoredTermsCollection getSTC();

  /**
   * Gets a DocumentsModel for the documents in this CacheDocumentSet.
   *
   * @return a DocumentsModel for the documents in this CacheDocumentSet.
   */
  DocumentsModel getDocumentsModel();

  /**
   * Gets the CacheElement that is the parent of this CacheDocumentSet.
   *
   * @return the CacheElement that is the parent of this CacheDocumentSet.
   */
  CacheElement getParent();
}
