/**
 * 
 */
package anl.aida.client;

import anl.aida.core.MIFSTerm;
import anl.aida.util.Filter;

/**
 * Filters MIFSTerm based on document count or
 * whether or not the term is a keyword.
 * 
 */
public class KeywordDocCountFilter implements Filter<MIFSTerm> {
  
  private int minFreq;
  private Keywords keywords;
  
  /**
   * Creates a filter that filters based on keywords and minimum frequency.
   * A term will pass if it is in the keyword collection or if its document frequency is 
   * >= minFreq.
   * 
   * @param minFreq the minimum frequency
   * @param keywords the keyword collection
   */
  public KeywordDocCountFilter(int minFreq, Keywords keywords) {
    this.minFreq = minFreq;
    this.keywords = keywords;
  }




  /* (non-Javadoc)
   * @see anl.mifs.util.Filter#evaluate(java.lang.Object)
   */
  @Override
  public boolean evaluate(MIFSTerm item) {
    return item.getDocumentCount() >= minFreq || keywords.contains(item.getText());
  }
}
