package anl.aida.client;

import java.util.ArrayList;
import java.util.List;

import anl.aida.algo.Clusterer;
import anl.aida.client.doc.DocumentsModel;
import anl.aida.client.doc.SubsetDocumentsModel;
import anl.aida.core.DocumentID;
import anl.aida.core.KeywordGenerator;
import anl.aida.core.ScoredTermsCollection;

/**
 * A temporary cache document set that represents some subset of
 * documents from a CacheElement. DocSetID.isTemporary() will return
 * true for a TempCachDocSet's id.
 *
 */
public class TempCacheDocSet implements CacheDocumentSet {

  private static int tempID = Integer.MIN_VALUE;

  private List<DocumentID> selectedItems = new ArrayList<DocumentID>();
  private CacheElement element;
  private DocSetID id;
  private DocumentsModel model;

  public TempCacheDocSet(CacheElement element, List<DocumentID> selectedItems) {
    this.element = element;
    this.selectedItems.addAll(selectedItems);
    this.id = new DocSetID(element.getID(), tempID++) {
      @Override
      public boolean isTemporary() {
        return true;
      }
    };
  }

  /**
   * Gets a descriptive label for this CacheDocumentSet.
   *
   * @return a descriptive label for this CacheDocumentSet.
   */
  public String getLabel() {
    return "";
  }

  /**
   * Gets the id of this document set.
   *
   * @return the id of this document set.
   */
  public DocSetID getId() {
    return id;
  }

  /**
   * Gets the Clusterer that should be used to Cluster this document set.
   *
   * @return the Clusterer that should be used to Cluster this document set.
   */
  public Clusterer<DocumentID> getClusterer() {
    //ClustererSV clusterer = new ClustererSV(element.getSVS(), "clusterer");
    //for (DocumentID id : selectedItems) {
     // clusterer.addDocument(id);
    //}

    return null;
  }

  /**
   * Gets the number of documents in this CacheDocumentSet.
   *
   * @return the number of documents in this CacheDocumentSet.
   */
  public int getDocumentCount() {
    return selectedItems.size();
  }

  /**
   * Gets the number of terms in this CacheDocumentSet.
   *
   * @return the number of terms in this CacheDocumentSet.
   */
  public int getTermCount() {
    return 0;
  }

  /**
   * Gets the timestamp of this CacheDocumentSet.
   *
   * @return the timestamp of this CacheDocumentSet.
   */
  public String getTimeStampAsString() {
    return "REPLACE WITH DATE RANGE";
  }

  /**
   * Gets a keyword generator that can be used to generate
   * keywords for this doc set.
   *
   * @return a keyword generator that can be used to generate
   *         keywords for this doc set.
   */
  public KeywordGenerator getKeyWordGenerator() {
    return element.getKeywordGenerator();
  }

  /**
   * Gets the ScoredTermsCollection associated with this CacheDocumentSet.
   *
   * @return the ScoredTermsCollection associated with this CacheDocumentSet.
   */
  public ScoredTermsCollection getSTC() {
    return element.getSTC();
  }

  /**
   * Gets the document ids of all the docs this set contains.
   *
   * @return the document ids of all the docs this set contains.
   */
  public Iterable<DocumentID> documents() {
    return selectedItems;
  }

  /**
   * Gets a DocumentsModel for the documents in this CacheDocumentSet.
   *
   * @return a DocumentsModel for the documents in this CacheDocumentSet.
   */
  public DocumentsModel getDocumentsModel() {
    if (model == null) {
      model = new SubsetDocumentsModel(id, element, selectedItems);
    }
    return model;
  }

  /**
   * Gets the CacheElement that is the parent of this CacheDocumentSet.
   *
   * @return the CacheElement that is the parent of this CacheDocumentSet.
   */
  public CacheElement getParent() {
    return element;
  }
}
