package anl.aida.client.doc;

import anl.aida.client.CacheElement;
import anl.aida.client.DocSetID;
import anl.aida.core.KeywordGenerator;
import anl.aida.core.ScoredTermsCollection;
import anl.aida.util.DateRange;
import anl.aida.util.HighestTermScores;
import anl.aida.util.AIDAException;
import anl.aida.util.TermScore;
import simphony.util.messages.MessageCenter;

import java.util.HashMap;
import java.util.Map;

/**
 * Abstract base class for a DocumentsModel.
 *
 */
public abstract class AbstractDocumentsModel implements DocumentsModel {

  protected static final MessageCenter center = MessageCenter.getMessageCenter(DefaultDocumentsModel.class);

  protected CacheElement element;
  protected KeywordGenerator keyGen;
  private Map<String, String[]> keywordCache = new HashMap<String, String[]>();
  private String description = "";
  protected DocSetID id;
  protected DateRange range;

  public AbstractDocumentsModel(DocSetID id, CacheElement element) {
    range = element.getHeader().getDateRange();
    this.element = element;
    this.id = id;
    keyGen = element.getKeywordGenerator();
    element.release();
  }

  /**
   * Gets the DocumentSetID for this model. This will return
   * NO_ID if this model  s not associated with a DocumentSet.
   *
   * @return the DocumentSetID for this model, or NO_ID if this model
   *         is not associated with a DocumentSet.
   */
  public DocSetID getID() {
    return id;
  }

  /**
   * Sets the description of the documents contained by this model.
   *
   * @param description the description
   */
  public void setDescription(String description) {
    this.description = description;
  }

  /**
   * Gets the description of the documents contained by this model.
   *
   * @return the description of the documents contained by this model.
   */
  public String getDescription() {
    return description;
  }

  /**
   * Gets keywords for the document with the specified path.
   *
   * @param docPath the path of the document to get keywords for
   * @return keywords for the document with the specified path.
   */
  public String[] getKeywords(String docPath) {
    String[] words = keywordCache.get(docPath);
    if (words == null) {
      try {
        HighestTermScores scores = keyGen.generate(docPath, 4);
        words = new String[scores.size()];
        int i = scores.size() - 1;
        //System.out.println("\nKeywords for " + docPath.toExternalForm());
        for (TermScore term : scores.termScores()) {
          //System.out.printf("\t%s: %f%n", term.getTerm(), term.getScore());
          words[i] = term.getTerm();
          i--;
        }
        keywordCache.put(docPath, words);
      } catch (AIDAException ex) {
        center.error("Error generating keywords for '" + docPath + "'", ex);
      }
    }
    return words;
  }
}
