package anl.aida.client.doc;

import java.util.Iterator;

import org.apache.lucene.document.DateTools;
import org.apache.lucene.search.Filter;
import org.apache.lucene.search.RangeFilter;

import anl.aida.ae.lucene.LuceneConstants;
import anl.aida.client.CacheElement;
import anl.aida.client.DocSetID;
import anl.aida.core.MIFSDocument;
import anl.aida.core.ScoredTermsCollection;

/**
 * Model class for the DocumentsView.
 *
 */
public class DefaultDocumentsModel extends AbstractDocumentsModel {

  private int offset;

  public DefaultDocumentsModel(DocSetID id, CacheElement element) {
    super(id, element);
    offset = 0;
  }

  /**
   * Creates a DocSets with each set of the specified size.
   *
   * @param setSize the set size.
   * @return the created DocSets.
   */
  public DocSets createDocumentSets(int setSize) {
    return new DocSetsImpl(offset, setSize);
  }

  /**
   * Returns a date range filter that filters out anything
   * greater than associated CacheElements timestamp.
   *
   * @return a date range filter that filters out anything
   *         greater than associated CacheElements timestamp.
   */

  public Filter getSearchFilter() {
    String end = DateTools.timeToString(range.getEnd().getTime(), DateTools.Resolution.DAY);
    String start = DateTools.timeToString(range.getStart().getTime(), DateTools.Resolution.DAY);
    return new RangeFilter(LuceneConstants.TIMESTAMP_FIELD, start, end, true, true);
  }

  class DocSetsImpl implements DocSets {

    private int start, setSize, numSets;

    DocSetsImpl(int start, int setSize) {
      this.start = start;
      this.setSize = setSize;
      numSets = (int) Math.floor(((float) element.getSTC().getDocumentCount()) / setSize) + 1;
      element.release();
    }


    public int getSetCount() {
      return numSets;
    }

    public Iterable<ClientDocument> getDocuments(int index) {
      if (index < 0 || index >= numSets) throw new ArrayIndexOutOfBoundsException();
      start = setSize * index;
      int end = start + setSize;
      ScoredTermsCollection stc = element.getSTC();
      if (end > stc.getDocumentCount()) end = stc.getDocumentCount();
      element.release();
      return new DocIterator(start, end);
    }
  }

  class DocIterator implements Iterator<ClientDocument>, Iterable<ClientDocument> {

    private int index, end;

    DocIterator(int start, int end) {
      this.index = start;
      this.end = end;
    }

    @Override
    public boolean hasNext() {
      return index < end;
    }

    @Override
    public ClientDocument next() {
      ScoredTermsCollection stc = element.getSTC();
      MIFSDocument doc = stc.getDocument(index);
      String title = stc.getDocument(doc.getSTCIndex()).getTitle();
      ClientDocument cd = new ClientDocument(doc.getId(), title, getKeywords(doc.getPath()));
      element.release();
      index++;
      return cd;
    }

    @Override
    public void remove() {
      throw new UnsupportedOperationException();
    }

    @Override
    public Iterator<ClientDocument> iterator() {
      return this;
    }
  }
}
