package anl.aida.client.doc;

import anl.aida.client.DocSetID;

import org.apache.lucene.search.Filter;

/**
 * Interface for classes that encapsulate some
 * set of documents that can be displayed in document view.
 */
public interface DocumentsModel {

  DocSetID NO_ID = new DocSetID(-1, -1);

  /**
   * Gets the DocumentSetID for this model. This will return
   * NO_ID if this model  s not associated with a DocumentSet.
   *
   * @return the DocumentSetID for this model, or NO_ID if this model
   *         is not associated with a DocumentSet.
   */
  public DocSetID getID();

  /**
   * Sets the description of the documents contained by this model.
   *
   * @param description the description
   */
  void setDescription(String description);

  /**
   * Gets the description of the documents contained by this model.
   *
   * @return the description of the documents contained by this model.
   */
  String getDescription();

  /**
   * Gets keywords for the document with the specified path.
   *
   * @param docPath the path of the document to get keywords for
   * @return keywords for the document with the specified path.
   */
  String[] getKeywords(String docPath);

  /**
   * Creates a DocSets with each set of the specified size.
   *
   * @param setSize the set size.
   * @return the created DocSets.
   */
  DocSets createDocumentSets(int setSize);

  /**
   * Gets a lucene filter appropriate for filtering a lucene search.
   * If no filtering is required then this should return null.
   *
   * @return the appropriate filter, or null if no filter is required.
   */
  Filter getSearchFilter();
}
