package anl.aida.client.doc;

import anl.aida.client.DocSetID;

import org.apache.lucene.search.Filter;

/**
 * Delegates DocumentsModel to a wrapped DocumentsModel but
 * replaced the DocSetID.
 *
 */
public class WrappedDocumentsModel implements DocumentsModel {

  private DocumentsModel model;
  private DocSetID id;

  public WrappedDocumentsModel(DocSetID id, DocumentsModel model) {
    this.id = id;
    this.model = model;
  }

  /**
   * Gets the DocumentSetID for this model. This will return
   * NO_ID if this model  s not associated with a DocumentSet.
   *
   * @return the DocumentSetID for this model, or NO_ID if this model
   *         is not associated with a DocumentSet.
   */
  public DocSetID getID() {
    return id;
  }

  /**
   * Sets the description of the documents contained by this model.
   *
   * @param description the description
   */
  public void setDescription(String description) {
    model.setDescription(description);
  }

  /**
   * Gets the description of the documents contained by this model.
   *
   * @return the description of the documents contained by this model.
   */
  public String getDescription() {
    return model.getDescription();
  }

  /**
   * Gets keywords for the document with the specified path.
   *
   * @param docPath the path of the document to get keywords for
   * @return keywords for the document with the specified path.
   */
  public String[] getKeywords(String docPath) {
    return model.getKeywords(docPath);
  }

  /**
   * Creates a DocSets with each set of the specified size.
   *
   * @param setSize the set size.
   * @return the created DocSets.
   */
  public DocSets createDocumentSets(int setSize) {
    return model.createDocumentSets(setSize);
  }

  /**
   * Gets a lucene filter appropriate for filtering a lucene search.
   * If no filtering is required then this should return null.
   *
   * @return the appropriate filter, or null if no filter is required.
   */
  public Filter getSearchFilter() {
    return model.getSearchFilter();
  }
}
