package anl.aida.client.gui;

import static anl.aida.client.gui.AIDAGuiConstants.CACHE_GROUP;
import static anl.aida.client.gui.AIDAGuiConstants.MAIN_PERSPECTIVE;
import static anl.aida.client.gui.AIDAGuiConstants.VIZ_GROUP;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.measure.unit.NonSI;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;
import javax.swing.SwingWorker;
import javax.xml.stream.XMLStreamException;

import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.search.Filter;

import saf.core.ui.GUIBarManager;
import saf.core.ui.dock.DockableFrame;
import saf.core.ui.dock.DockingManager;
import saf.core.ui.event.DockableFrameAdapter;
import saf.core.ui.event.DockableFrameEvent;
import simphony.util.messages.MessageCenter;
import anl.aida.AIDADatasetProducer;
import anl.aida.algo.Clusters;
import anl.aida.bubble.BubblePlotCreator;
import anl.aida.client.Cache;
import anl.aida.client.CacheDocumentSet;
import anl.aida.client.CacheElement;
import anl.aida.client.DocSetID;
import anl.aida.client.ShowDocViewOp;
import anl.aida.client.TempCacheDocSet;
import anl.aida.client.WrappedDocSet;
import anl.aida.client.doc.DocumentsModel;
import anl.aida.client.io.ImageExportAction;
import anl.aida.client.net.ConceptGraph;
import anl.aida.client.net.NetworkOutputter;
import anl.aida.client.net.TSConceptGraphs;
import anl.aida.client.net.TermNode;
import anl.aida.client.net.ValueGraphCreator;
import anl.aida.client.net.ValueNode;
import anl.aida.client.search.LuceneSearcher;
import anl.aida.client.util.ClientUtils;
import anl.aida.client.widget.NetworkTimeSlider2;
import anl.aida.core.DocumentID;
import anl.aida.core.ScoredTermsCollection;
import anl.aida.data.AxisType;
import anl.aida.data.DataFrame;
import anl.aida.data.DataManager;
import anl.aida.gis.GISDisplay;
import anl.aida.plot.DFTimeSeriesParams;
import anl.aida.plot.Dashboard;
import anl.aida.plot.GraphPlotRenderer;
import anl.aida.plot.GroupScatterPlot;
import anl.aida.plot.Map3DPlotCreator;
import anl.aida.plot.MapPlotCreator;
import anl.aida.plot.TimeSeriesPlotWidget;
import anl.aida.plot.io.Project;
import anl.aida.plot.io.ProjectReader;
import anl.aida.table.TableCreator;
import anl.aida.util.DateRange;
import anl.aida.viz.Canvas2D;
import anl.aida.viz.KeywordTipper;
import anl.aida.viz.NetworkDisplay;
import anl.aida.viz.SimilarityClusterDisplay;
import anl.aida.viz.layout.FRLayout;
import anl.aida.viz3d.Canvas3D;
import anl.aida.viz3d.DocOpsCanvasListener;
import anl.aida.viz3d.GraphRenderer;
import edu.uci.ics.jung.graph.Graph;

/**
 * Mediates interactions between GUI components and non-GUI components with GUI
 * components.
 * 
 */
public class AIDAGui {

  private static final MessageCenter center = MessageCenter.getMessageCenter(AIDAGui.class);

  private static final String DOCS_ICON = "page_white_stack.png";
  private static final String STATUS_BAR_ID = "status.first";

  private DockingManager dockingManager;
  private CachePanel cachePanel = new CachePanel();
  private StructuredDataPanel structPanel = new StructuredDataPanel();
  private DocsTabPanel docTabs = null;

  private CacheElement cElement;
  private Cache cache;

  private int vizCount = 0;
  private JFrame frame;

  private DockableFrame docsFrame, cacheFrame, structFrame;
  private java.util.List<DockableFrame> displayFrames = new ArrayList<DockableFrame>();
  private boolean triggerListener = true;
  private Dashboard dashboard;

  private GUIBarManager barManager;
  private DataManager dataManager = new DataManager();
  private AIDADockableHelper dockableHelper = new AIDADockableHelper();

  private boolean textOutput=true;
  
  public AIDAGui(DockingManager dockingManager) {
    this.dockingManager = dockingManager;
    
    cacheFrame = dockingManager.createDockable("Cache", cachePanel,
	DockingManager.MinimizeLocation.LEFT, DockingManager.FLOAT | DockingManager.MINIMIZE
	    | DockingManager.MAXIMIZE);
    cacheFrame.setTitle("Data Sources");
    dockingManager.addDockableToGroup(MAIN_PERSPECTIVE, CACHE_GROUP, cacheFrame);
    cachePanel.init(this);
    
   
    structFrame = dockingManager.createDockable("Structured Data", structPanel,
	DockingManager.MinimizeLocation.LEFT, DockingManager.FLOAT | DockingManager.MINIMIZE
	    | DockingManager.MAXIMIZE);
    structFrame.setTitle("Structured Data");
    dockingManager.addDockableToGroup(MAIN_PERSPECTIVE, CACHE_GROUP, structFrame);
    dockingManager.dock(structFrame, cacheFrame);
    structPanel.init(this, dataManager);
    structFrame.toFront();
  
    dockingManager.addDockableListener(new DockableFrameAdapter() {
      @Override
      public void dockableClosed(DockableFrameEvent evt) {
	if (evt.getDockable().equals(docsFrame)) {
	  docTabs.removeAllDocs();
	  docTabs = null;
	  docsFrame = null;
	} else if (triggerListener) {
	  displayFrames.remove(evt.getDockable());
	}
      }
    });
    
    dockingManager.addDockableListener(dockableHelper);
   
    cachePanel.requestFocus();
    
  }
  
  /**
   * Notifies the gui that a visualization has been added.
   * 
   * @param id the id of the dockable that was added
   * @param display the gis display that was added.
   */
  public void visualizationAdded(String id, final GISDisplay display) {
    dockableHelper.addVisualization(id, new AIDAVisualizationAdapter() {
      public void closed() {
	display.destroy();
      }
    });
    
  }

  public JFrame getFrame() {
    return frame;
  }

  public void resetCache(Cache cache) {
    this.cache = cache;
    if (docTabs != null)
      docTabs.removeAllDocs();
    triggerListener = false;
    for (DockableFrame dockable : displayFrames) {
      dockable.close();
    }
    triggerListener = true;
    displayFrames.clear();
    cachePanel.resetCache(cache);

   
    try {
      Project project = new ProjectReader().read(new File("./projects/project.xml"));
      for (String classname : project.datasetClasses()) {
	dataManager.createDataset(classname);
      }
      dataManager.addDataset(new AIDADatasetProducer(AxisType.NAMED_LOCATION, "Country", NonSI.YEAR).
	  createDataset("./data/gap_minder_data.bin", "GAP Minder Data"));
      dataManager.addDataset(new AIDADatasetProducer(AxisType.CATEGORY, "Driver", NonSI.YEAR).
	  createDataset("./data/H2CAS.bin", "H2CAS Model Data"));
      
      // loads mock WI data
      structPanel.resetFeed(dataManager.getDatasets());
      dashboard = new Dashboard(this, structPanel);
      
      dashboard.addProject(project);
      JPanel panel = createVizDockable("Plots", "dashboard");
      panel.add(dashboard.getPanel(), BorderLayout.CENTER);
      structPanel.resetProject(project);
    } catch (IOException ex) {
      center.error("Error reading project data", ex);
    } catch (XMLStreamException ex) {
      center.error("Error reading project data", ex);
    }
  }

  /**
   * Loads a CacheDocumentSet into the documents view.
   * 
   * @param title
   *          the title to display in the view
   * @param docSet
   *          the docs set to display
   */
  public void loadDocs(String title, CacheDocumentSet docSet) {
    // the divider in the cachePanel goes wonky when
    // the docsFrame is added, so get its current location
    // and set it to that after docs frame has been added.
    final double val = cachePanel.getSplitDividerLocation();
    if (docsFrame == null) {
      docTabs = new DocsTabPanel();
      docTabs.init(this);
      docsFrame = dockingManager.createDockable("Documents", docTabs,
	  DockingManager.MinimizeLocation.RIGHT);
      docsFrame.setTitle("Documents");
      dockingManager.addDockableToGroup(MAIN_PERSPECTIVE, CACHE_GROUP, docsFrame);
    }

    docTabs.addDocs(title, docSet);
    docsFrame.toFront();

    // need to add the divider location reset at the end
    // of the event queue because some event already in the
    // queue is resetting it after this method completes.
    SwingUtilities.invokeLater(new Runnable() {
      @Override
      public void run() {
	docTabs.invalidate();
	docTabs.validate();
	docTabs.repaint();
	cachePanel.setSplitDividerLocation(val);
      }
    });
  }
 
  public DockableFrame getDockable(String id) {
    return dockingManager.getDockable(id);
  }

  public void showVizDialog(VizPropertyBean bean) {
    VizDialog dialog = new VizDialog(frame);
    dialog.init(bean);
    dialog.pack();
    dialog.setVisible(true);
  }
  
  public String add3DGISPlotDisplay(DataFrame frame) {
    String id = createDockableID();
    String title = "3D Map Plot: " + frame.getVariable().getName();
    JPanel panel = createVizDockable(title, id);
    ProgressLayer pLayer = (ProgressLayer) panel.getClientProperty(ProgressLayer.CLIENT_KEY);
    pLayer.setEnabled(false);
    pLayer.setBusyText("Creating Map");
    Map3DPlotCreator creator = new Map3DPlotCreator(frame, panel, this);
    creator.execute();
    
    return id;
  }

  /**
   * Adds the specified panel as a plot.
   * 
   * @param panel
   */
  public String addGISPlotDisplay(DataFrame frame) {
    String id = createDockableID();
    String title = "Map Plot: " + frame.getVariable().getName();
    JPanel panel = createVizDockable(title, id);
    ProgressLayer pLayer = (ProgressLayer) panel.getClientProperty(ProgressLayer.CLIENT_KEY);
    pLayer.setEnabled(false);
    pLayer.setBusyText("Creating Map");
    MapPlotCreator creator = new MapPlotCreator(id, frame, panel, this);
    creator.execute();
    return id;
  }

  public String addDataTable(DataFrame frame) {
    JTable table = new TableCreator().createTable(frame);
    DockableFrame dockable = dockingManager.createDockable("viz" + vizCount++, new JScrollPane(
	table), DockingManager.MinimizeLocation.BOTTOM);
    dockable.setTitle("Data Table: " + frame.getVariable().getName());
    displayFrames.add(dockable);
    dockingManager.addDockableToGroup(MAIN_PERSPECTIVE, VIZ_GROUP, dockable);
    return dockable.getID();
  }
  
  public String addScatterPlot(DataFrame xFrame, DataFrame yFrame) {

    GroupScatterPlot plot = new GroupScatterPlot(xFrame, yFrame);
    DockableFrame dockable = dockingManager.createDockable("viz" + vizCount++, plot.getPanel(),
	DockingManager.MinimizeLocation.BOTTOM);
    dockable.setTitle(xFrame.getVariable().getName() + " x " + yFrame.getVariable().getName());
    displayFrames.add(dockable);
    dockingManager.addDockableToGroup(MAIN_PERSPECTIVE, VIZ_GROUP, dockable);
    return dockable.getID();
  }
  
  public String addBubblePlot(BubbleConfigBean bean) {
    String id = createDockableID();
    String title = "Bubble Plot: " + bean.getTitle();
    JPanel panel = createVizDockable(title, id);
    ProgressLayer pLayer = (ProgressLayer) panel.getClientProperty(ProgressLayer.CLIENT_KEY);
    pLayer.setEnabled(false);
    pLayer.setBusyText("Creating Bubble Plot");
    BubblePlotCreator creator = new BubblePlotCreator(bean, panel, this, dataManager);
    creator.execute();
    return id;
  }

  /**
   * Adds a 3D plot using the data in the data frame.
   * 
   * @param frame
   *          the frame to create the data from
   */
  public String add3DPlot(DataFrame frame) {
    Graph<ValueNode, Object> graph = new ValueGraphCreator().createGraph(frame, 0);
    GraphPlotRenderer renderer = new GraphPlotRenderer(frame, graph);
    String id = createDockableID();
    JPanel panel = createVizDockable("3D Plot: " + frame.getVariable().getName(), id);
    panel.add(renderer.getPanel());
    return id;
  }

  private String createDockableID() {
    String id = "viz" + vizCount++;
    return id;
  }
  
  /**
   * Adds a time series plot from the data in the frame.
   * 
   * @param frame
   *          the frame holding the data to plot
   */
  public String addTimeSeries(List<DataFrame> frames) {
    return addTimeSeries(frames, new int[0]);
  }

  /**
   * Adds a time series plot from the data in the frame.
   * 
   * @param frame
   *          the frame holding the data to plot
   * @param indices
   *          the indices in the frame whose values we want to plot. If this is
   *          an empty array then all the indices are plotted.
   */
  public String addTimeSeries(List<DataFrame> frames, int ... indices) {
    DFTimeSeriesParams params = new DFTimeSeriesParams(frames, indices);
    TimeSeriesPlotWidget plot = new TimeSeriesPlotWidget(params);
    DockableFrame dockable = dockingManager.createDockable("viz" + vizCount++, plot.getPanel(),
	DockingManager.MinimizeLocation.BOTTOM);
    StringBuilder buf = new StringBuilder();
    boolean first = true;
    for (DataFrame frame : frames) {
      if (!first)
	buf.append(", ");
      buf.append(frame.getVariable().getName());
      first = false;
    }
    String title = buf.toString();
    if (title.length() > 15) {
      title = title.substring(0, 15) + "...";
    }
    dockable.setTitle("Time Series: " + title);
    displayFrames.add(dockable);
    dockingManager.addDockableToGroup(MAIN_PERSPECTIVE, VIZ_GROUP, dockable);
    return dockable.getID();
  }

  /**
   * Adds a cluster display to the gui.
   * 
   * @param id
   *          the id of the documents set we are visualizing
   * @param clusters
   *          the clusters to visualize
   * @param title
   *          a title for the display
   */
  public void addClusterDisplay(final DocSetID id, Clusters<DocumentID> clusters, String title) {
    Canvas2D canvas = new Canvas2D(700, 500);
    JPanel panel = new JPanel(new BorderLayout());
    panel.add(canvas, BorderLayout.CENTER);
    panel.add(canvas.getToolBar(), BorderLayout.NORTH);
    DockableFrame dockable = dockingManager.createDockable("viz" + vizCount++, panel,
	DockingManager.MinimizeLocation.BOTTOM);
    dockable.setTitle(title);
    displayFrames.add(dockable);
    dockingManager.addDockableToGroup(MAIN_PERSPECTIVE, VIZ_GROUP, dockable);

    SimilarityClusterDisplay<DocumentID> display = new SimilarityClusterDisplay<DocumentID>(
	clusters, canvas.getSize());
    display
	.setLensListener(new KeywordTipper(cElement.getDocumentSet(0).getKeyWordGenerator(), 20));

    JButton button = canvas.addDocOp(new ShowDocViewOp(this, cache.getElement(id
	.getCacheElementId())), DOCS_ICON);
    button.setToolTipText("Display documents for selected nodes and links");

    display.initialize();
    canvas.addLayer(display);
  }

  // creates a dockable frame with the specified title
  // and returns the panel that is the contents of that frame.
  private JPanel createVizDockable(String title, String id) {
    JPanel panel = new JPanel(new BorderLayout());
    ProgressLayer layer = new ProgressLayer(panel);
    DockableFrame dockable = dockingManager.createDockable(id, layer.getJXLayer(),
	DockingManager.MinimizeLocation.BOTTOM);
    displayFrames.add(dockable);
    dockable.setTitle(title);
    dockingManager.addDockableToGroup(MAIN_PERSPECTIVE, VIZ_GROUP, dockable);

    return panel;
  }

  /**
   * Adds a time series display using the bean properties.
   * 
   * @param bean
   *          a bean that holds the display properties.
   */
  public void addTimeSeriesDisplay(VizPropertyBean bean) {
    JPanel panel = createVizDockable(bean.getTitle(), "viz" + vizCount++);
    ProgressLayer pLayer = (ProgressLayer) panel.getClientProperty(ProgressLayer.CLIENT_KEY);
    pLayer.setEnabled(false);
    CreateTSWorker worker = new CreateTSWorker(bean.getKeywords(), cache, cache.getRanger()
	.ranges(), panel);
    worker.execute();
  }

  public void produceOutput(Cache cache, VizPropertyBean bean,TSConceptGraphs graphs){
	  final NetworkOutputter outputter = new NetworkOutputter(cache, bean,graphs);
	  outputter.writeOutputOut();
  }
  /**
   * Adds a newtwork display (2D or 3D) using the bean properties.
   * 
   * @param bean
   *          a bean that holds the display properties.
   */
  public void addNetworkDisplay(VizPropertyBean bean) {
    // get graph for current cache element range and add that.
    final TSConceptGraphs graphs = new TSConceptGraphs(cache, bean);
    if(textOutput)
    	produceOutput(cache,bean,graphs);
    final DateRange dateRange = cElement.getHeader().getDateRange();
    final ConceptGraph<TermNode> graph = graphs.getGraph(dateRange);

    String id = createDockableID();
    JPanel panel = createVizDockable(graph.getTitle(), id);
    Dimension size = new Dimension(800, 800);

    if (bean.isThreed()) {
      GraphRenderer graphRenderer = new GraphRenderer(cache, graph, dateRange);
      NetworkTimeSlider2 timeSlider = new NetworkTimeSlider2();
      timeSlider.init(graphRenderer, graphs, dateRange);

      Canvas3D canvas = graphRenderer.getCanvas();
      JToolBar bar = canvas.getToolBar();
      bar.setFloatable(false);
      DocOpsWidget widget = new DocOpsWidget();
      JButton button = widget.addOp(canvas.getNodeSelector().getDocViewOp(this), DOCS_ICON);
      canvas.addCanvasListener(new DocOpsCanvasListener(widget));
      bar.addSeparator();
      bar.add(button);
      timeSlider.getAnimator().addToToolBar(bar);

      // order the bar items so they match the 2D
      // remove the hide nodes checkbox
      Component comp = bar.getComponent(3);
      bar.remove(comp);
      // remove the docs button
      Component comp2 = bar.getComponent(4);
      bar.remove(comp2);
      // add the docs button back
      bar.add(comp2, 3);
      // add the hide nodes box back
      bar.add(comp, 5);

      panel.add(bar, BorderLayout.NORTH);
      panel.add(canvas.getPanel());
      panel.add(timeSlider, BorderLayout.SOUTH);
    } else if (bean.isTwod()) {

      Canvas2D canvas = new Canvas2D(size);
      JButton button = canvas.addDocOp(graphs.getDocSetOp(this), DOCS_ICON);
      button.setToolTipText("Display documents for selected nodes and links");
      // KKLayout layout = new KKLayout(graph, size);
      FRLayout layout = new FRLayout(graph, size);
      NetworkDisplay display = new NetworkDisplay(graph, layout);
      NetworkTimeSlider2 timeSlider = new NetworkTimeSlider2();
      timeSlider.init(display, graphs, cElement.getHeader().getDateRange());
      canvas.addLayer(display);
      timeSlider.getAnimator().addToToolBar(canvas.getToolBar());
      button = new JButton(new ImageExportAction(canvas));
      button.setToolTipText("Export as Image");
      button.setIcon(ClientUtils.loadIcon("camera.png"));
      button.setText("");
      button.setBorderPainted(false);
      canvas.getToolBar().addSeparator();
      canvas.getToolBar().add(button);
      panel.add(canvas, BorderLayout.CENTER);
      panel.add(canvas.getToolBar(), BorderLayout.NORTH);
      panel.add(timeSlider, BorderLayout.SOUTH);
    }
  }

  /**
   * Called when a cache element is selected.
   * 
   * @param element
   *          the selected cache element
   */
  public void cacheElementSelected(CacheElement element) {
    if (cElement != null) {
      cElement.release();
    }
    if (element == null) {
      // todo clear and disable the panels

    } else {
      cElement = element;
      cachePanel.docSetSelected(cElement.getDocumentSet(0));
    }
  }

  /**
   * Displays the documents associated with docSetNode in the documents view.
   * 
   * @param docSetNode
   *          the node whose docs should be displayed
   */
  public void showDocumentList(DocSetNode docSetNode) {
    CacheDocumentSet docSet = docSetNode.getDocumentSet();
    if (docTabs == null) {
      loadDocs(docSet.getLabel(), docSet);
    } else if (!docTabs.selectDocSet(docSet.getId())) {
      loadDocs(docSet.getLabel(), docSet);
    }

  }

  /**
   * Adds a documents set to the cache tree. This will return a new added set or
   * null if the user has canceled the operation.
   * 
   * @param name
   *          the name of the set to add. This becomes the tree node label.
   * @param set
   *          the set to add
   * @return the added CacheDoumentSet or null if the user has canceled the
   *         operation.
   */
  public CacheDocumentSet addDocumentSet(String name, CacheDocumentSet set) {
    AddDocSetDialog dialog = new AddDocSetDialog(frame);
    dialog.setDocSetName(name);
    dialog.pack();
    dialog.setVisible(true);
    CacheElement element = cache.getElement(set.getId().getCacheElementId());

    WrappedDocSet newSet = new WrappedDocSet(new DocSetID(element.getID(), element
	.getDocumentSetCount()), dialog.getDocSetName(), set);
    element.addDocumentSet(newSet);

    if (!dialog.isCanceled()) {
      cachePanel.addCacheDocumentSet(newSet);
      return newSet;
    }
    return null;
  }

  /**
   * Sets the main application frame.
   * 
   * @param frame
   *          the main application frame
   */
  public void setFrame(JFrame frame) {
    this.frame = frame;
    structFrame.toFront();
  }

  public void addStatusMessage(String msg, Color color) {
    barManager.setStatusBarTextColor(STATUS_BAR_ID, color);
    barManager.setStatusBarText(STATUS_BAR_ID, msg);
  }

  public void clearStatusMessage() {
    barManager.setStatusBarText(STATUS_BAR_ID, "");
  }

  /**
   * Searches the cache index using the specified query and places the results,
   * if any in the documents panel.
   * 
   * @param query
   *          the query string
   * @param model
   *          the Documents model to perform the search over
   * @param pLayer
   *          a progress layer used to indicate progress while the search is
   *          proceeding.
   */
  public void performSearch(String query, DocumentsModel model, ProgressLayer pLayer) {
    try {
      pLayer.setEnabled(false);
      SearchWorker worker = new SearchWorker(query, model, pLayer);
      worker.execute();
    } catch (IOException ex) {
      center.error("Error while searching documents", ex);
      pLayer.setEnabled(true);
    }
  }

  /**
   * SwingWorker that does the lucence search in a background thread
   */
  class SearchWorker extends SwingWorker<List<DocumentID>, Object> {

    private LuceneSearcher searcher;
    private String query;
    private Filter filter;
    private ProgressLayer pLayer;
    private CacheElement element;

    SearchWorker(String query, DocumentsModel model, ProgressLayer pLayer) throws IOException {
      element = cache.getElement(model.getID().getCacheElementId());
      ScoredTermsCollection stc = element.getSTC();
      filter = model.getSearchFilter();
      searcher = new LuceneSearcher(cache.getLuceneLocation().getCanonicalPath(), stc);
      this.query = query;
      this.pLayer = pLayer;
    }

    @Override
    protected List<DocumentID> doInBackground() throws Exception {
      return searcher.search(query, 100, filter);
    }

    protected void done() {
      try {

	List<DocumentID> results = get();

	if (results.size() > 0) {
	  TempCacheDocSet set = new TempCacheDocSet(element, results);
	  set.getDocumentsModel().setDescription("Search Result for '" + query + "'");
	  loadDocs("Search Result", set);
	} else {
	  String msg = "Your search '" + query + "' did not match any documents.";
	  JOptionPane.showMessageDialog(frame, msg, "No Results", JOptionPane.INFORMATION_MESSAGE);
	}
      } catch (Exception ex) {
	if (ex instanceof ParseException) {
	  String msg = "Invalid search query: " + ex.getMessage();
	  JOptionPane.showMessageDialog(frame, msg, "Search Error", JOptionPane.ERROR_MESSAGE);
	} else {
	  center.error("Error while searching documents", ex);
	}
      } finally {
	element.release();
	pLayer.setEnabled(true);
      }
    }
  }

  public void addBarManager(GUIBarManager guiBarManager) {
    this.barManager = guiBarManager;
  }
  
}
