/*
 * Created by JFormDesigner on Thu Feb 19 10:38:50 EST 2009
 */

package anl.aida.client.gui;

import anl.aida.client.Cache;
import anl.aida.client.CacheDocumentSet;
import anl.aida.client.CacheElement;

import com.jgoodies.forms.factories.FormFactory;
import com.jgoodies.forms.layout.*;
import com.jidesoft.swing.CheckBoxTree;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.*;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Enumeration;

/**
 * @author User #1
 */
public class CacheTreePanel extends JPanel {

  private static final String CACHE_ICON = "page_white_stack.png";
  private static final String DOC_SET_CON = "page_white_star.png";

  private class Renderer extends DefaultTreeCellRenderer {
    @Override
    public Component getTreeCellRendererComponent(JTree tree, Object value, boolean sel, boolean expanded,
                                                  boolean leaf, int row, boolean hasFocus) {
      ImageIcon icon = null;
      if (value instanceof CacheNode) {
        CacheNode node = (CacheNode) value;
        value = node.getLabel();
        if (node.getAllowsChildren()) icon = cacheIcon;
        else icon = docIcon;
      }

      JLabel label = (JLabel) super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf, row, hasFocus);
      if (icon != null) label.setIcon(icon);
      return label;
    }
  }

  private DefaultTreeModel model;
  private ImageIcon cacheIcon, docIcon;

  public CacheTreePanel() {
    initComponents();
    setPreferredSize(new Dimension(getPreferredSize().width, 250));

    model = new DefaultTreeModel(new DefaultMutableTreeNode("root"));
    cacheTree.setModel(model);
    cacheTree.setRootVisible(false);
    cacheTree.setShowsRootHandles(true);

    cacheTree.setCellRenderer(new Renderer());

    cacheIcon = new ImageIcon(getClass().getClassLoader().getResource(CACHE_ICON));
    docIcon = new ImageIcon(getClass().getClassLoader().getResource(DOC_SET_CON));
  }

  public void init(final AIDAGui gui) {
    cacheTree.addTreeSelectionListener(new TreeSelectionListener() {
      public void valueChanged(TreeSelectionEvent e) {
        TreePath path = e.getPath();
        if (path != null) {
          CacheNode node = (CacheNode) path.getLastPathComponent();
          if (!node.getAllowsChildren()) {
            // its a doc node
            //gui.documentNodeSelected((DocSetNode) node);
          }
        }
      }
    });

    cacheTree.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseClicked(MouseEvent e) {
        if (e.getButton() == 1 && e.getClickCount() == 2) {
          JTree tree = (JTree) e.getSource();
          TreePath path = tree.getPathForLocation(e.getX(), e.getY());
          if (path != null) {
            CacheNode node = (CacheNode) path.getLastPathComponent();
            if (!node.getAllowsChildren()) {
              // its a doc node
              gui.showDocumentList((DocSetNode) node);
            }
          }
        }
      }
    });
  }

  public CacheElement getElement(int id) {
    Object root = model.getRoot();
    // we know cache elements are children of root
    // so we don't have to recurse the search
    for (int i = 0, n = model.getChildCount(root); i < n; i++) {
      CacheElementNode node = (CacheElementNode) model.getChild(root, i);
      if (node.getItem().getID() == id) return node.getItem();
    }

    return null;
  }

  public void addCacheDocumentSet(CacheDocumentSet set) {
    int elementID = set.getId().getCacheElementId();
    Object root = model.getRoot();
    CacheElementNode parent = null;
    for (int i = 0, n = model.getChildCount(root); i < n; i++) {
      CacheElementNode node = (CacheElementNode) model.getChild(root, i);
      if (node.getItem().getID() == elementID) {
        parent = node;
        break;
      }
    }

    if (parent != null) {
      DocSetNode node = new DocSetNode(set);
      parent.add(node);
      model.nodeStructureChanged(parent);
      cacheTree.setSelectionPath(new TreePath(model.getPathToRoot(node)));
    }
  }

  public void resetCache(Cache cache) {
    DefaultMutableTreeNode root = (DefaultMutableTreeNode) model.getRoot();
    root.removeAllChildren();
    for (CacheElement item : cache.elements()) {
      CacheElementNode child = new CacheElementNode(item);
      root.add(child);
      for (CacheDocumentSet set : item.documentSets()) {
        child.add(new DocSetNode(set));
      }
    }

    model.nodeStructureChanged(root);
    expandAll();

    cacheTree.setSelectionRow(1);
  }

  public void expandAll() {
    TreeNode root = (TreeNode) model.getRoot();
    // Traverse tree from root
    expandAll(new TreePath(root), true);
  }

  private void expandAll(TreePath parent, boolean expand) {
    // Traverse children
    TreeNode node = (TreeNode) parent.getLastPathComponent();
    if (node.getChildCount() >= 0) {
      for (Enumeration e = node.children(); e.hasMoreElements();) {
        TreeNode n = (TreeNode) e.nextElement();
        TreePath path = parent.pathByAddingChild(n);
        expandAll(path, expand);
      }
    }

    // Expansion or collapse must be done bottom-up
    if (expand) {
      cacheTree.expandPath(parent);
    } else {
      cacheTree.collapsePath(parent);
    }
  }


  private void initComponents() {
    // JFormDesigner - Component initialization - DO NOT MODIFY  //GEN-BEGIN:initComponents
    // Generated using JFormDesigner non-commercial license
    toolBar = new JToolBar();
    button2 = new JButton();
    button3 = new JButton();
    scrollPane1 = new JScrollPane();
    cacheTree = new CheckBoxTree();
    CellConstraints cc = new CellConstraints();

    //======== this ========
    setBorder(new EmptyBorder(5, 5, 5, 5));
    setLayout(new FormLayout(
            ColumnSpec.decodeSpecs("default:grow"),
            new RowSpec[]{
                    FormFactory.DEFAULT_ROWSPEC,
                    FormFactory.LINE_GAP_ROWSPEC,
                    new RowSpec(RowSpec.CENTER, Sizes.DEFAULT, FormSpec.DEFAULT_GROW)
            }));

    //======== toolBar ========
    {
      toolBar.setRollover(true);
      toolBar.setFloatable(false);

      //---- button2 ----
      button2.setText("One");
      toolBar.add(button2);

      //---- button3 ----
      button3.setText("Two");
      toolBar.add(button3);
    }
    add(toolBar, cc.xy(1, 1));

    //======== scrollPane1 ========
    {
      scrollPane1.setViewportView(cacheTree);
    }
    add(scrollPane1, cc.xy(1, 3));
    // JFormDesigner - End of component initialization  //GEN-END:initComponents
  }

  // JFormDesigner - Variables declaration - DO NOT MODIFY  //GEN-BEGIN:variables
  // Generated using JFormDesigner non-commercial license
  private JToolBar toolBar;
  private JButton button2;
  private JButton button3;
  private JScrollPane scrollPane1;
  private CheckBoxTree cacheTree;
  // JFormDesigner - End of variables declaration  //GEN-END:variables
}
