/*
 * Created by JFormDesigner on Thu Feb 12 16:29:07 EST 2009
 */

package anl.aida.client.gui;

import anl.aida.algo.Clusterer;
import anl.aida.algo.Clusters;
import anl.aida.client.CacheDocumentSet;
import anl.aida.client.DocSetID;
import anl.aida.core.DocumentID;

import com.jgoodies.forms.factories.FormFactory;
import com.jgoodies.forms.layout.*;
import simphony.util.messages.MessageCenter;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import javax.swing.table.DefaultTableModel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.util.HashMap;
import java.util.Map;

/**
 * @author User #1
 */
public class ClusterPanel extends JPanel {

  class ClusterWorker extends SwingWorker<Clusters<DocumentID>, Object> {
    private int count;

    ClusterWorker(int count) {
      this.count = count;
    }

    @Override
    protected Clusters<DocumentID> doInBackground() throws Exception {
      return clusterer.cluster(count);
    }

    protected void done() {
      try {
	Clusters<DocumentID> results = get();
	ClusterTableModel model = (ClusterTableModel) clusterTbl.getModel();
	model.reset(results);
      } catch (Exception ex) {
	center.error("Error while generating clusters", ex);
      } finally {
	ProgressLayer pLayer = (ProgressLayer) getClientProperty(ProgressLayer.CLIENT_KEY);
	pLayer.setEnabled(true);
      }
    }
  }

  private static MessageCenter center = MessageCenter.getMessageCenter(ClusterPanel.class);

  private boolean first = true;
  private Clusterer<DocumentID> clusterer;
  private AIDAGui gui;
  private DocSetID curId;
  private Map<DocSetID, ClusterTableModel> tableMap = new HashMap<DocSetID, ClusterTableModel>();

  public ClusterPanel() {
    initComponents();
    clusterTbl.setModel(new ClusterTableModel());
    clusterTbl.setShowGrid(true);
    clusterTbl.setShowHorizontalLines(true);
    clusterTbl.setShowVerticalLines(true);

    clusterTbl.addComponentListener(new ComponentAdapter() {
      public void componentResized(ComponentEvent e) {
	if (first) {
	  clusterTbl.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
	  int total = 0;

	  for (int i = 0; i < 4; i++) {
	    total += clusterTbl.getColumnModel().getColumn(i).getWidth();
	  }

	  float[] percents = { .1f, .5f, .25f, .15f };
	  for (int i = 0; i < 4; i++) {
	    clusterTbl.getColumnModel().getColumn(i).setPreferredWidth((int) (total * percents[i]));
	  }
	  first = false;
	}
      }
    });

    addListeners();
  }

  private void addListeners() {
    runBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	runClusterer();
      }
    });

    vizBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	Clusters<DocumentID> clusters = ((ClusterTableModel) clusterTbl.getModel()).getClusters();
	gui.addClusterDisplay(curId, clusters, "Cluster Exploration");
      }
    });
  }

  private void runClusterer() {
    ProgressLayer pLayer = (ProgressLayer) getClientProperty(ProgressLayer.CLIENT_KEY);
    pLayer.setEnabled(false);
    int numClusters = ((Number) numSpn.getModel().getValue()).intValue();
    ClusterWorker worker = new ClusterWorker(numClusters);
    worker.execute();
  }

  public void init(AIDAGui gui) {
    this.gui = gui;
  }

  /**
   * Resets to blank default state.
   */
  public void reset() {
    tableMap.clear();
    ClusterTableModel model = new ClusterTableModel();
    resetTable(model);
    curId = null;
    clusterer = null;
  }

  public void reset(CacheDocumentSet set) {
    if (curId != null) {
      tableMap.put(curId, (ClusterTableModel) clusterTbl.getModel());
    }
    curId = set.getId();
    clusterer = set.getClusterer();
    ClusterTableModel model = tableMap.get(curId);
    if (model == null) {
      model = new ClusterTableModel();
    }

    resetTable(model);
  }

  private void resetTable(ClusterTableModel model) {

    // the table resets its columns sizes when the model
    // is reset so we need to keep the column sizes constant
    int count = clusterTbl.getColumnCount();
    int widths[] = new int[count];
    for (int i = 0; i < count; i++) {
      widths[i] = clusterTbl.getColumnModel().getColumn(i).getWidth();
    }
    clusterTbl.setModel(model);
    for (int i = 0; i < count; i++) {
      clusterTbl.getColumnModel().getColumn(i).setPreferredWidth(widths[i]);
      clusterTbl.getColumnModel().getColumn(i).setWidth(widths[i]);
    }
  }

  private void initComponents() {
    // JFormDesigner - Component initialization - DO NOT MODIFY
    // //GEN-BEGIN:initComponents
    // Generated using JFormDesigner non-commercial license
    label1 = new JLabel();
    numSpn = new JSpinner();
    runBtn = new JButton();
    scrollPane1 = new JScrollPane();
    clusterTbl = new JTable();
    vizBtn = new JButton();
    CellConstraints cc = new CellConstraints();

    // ======== this ========
    setBorder(new EmptyBorder(5, 5, 5, 5));
    setLayout(new FormLayout(new ColumnSpec[] { FormFactory.DEFAULT_COLSPEC,
	FormFactory.LABEL_COMPONENT_GAP_COLSPEC, new ColumnSpec("max(default;30dlu)"),
	FormFactory.LABEL_COMPONENT_GAP_COLSPEC,
	new ColumnSpec(ColumnSpec.LEFT, Sizes.DEFAULT, FormSpec.DEFAULT_GROW) }, new RowSpec[] {
	FormFactory.DEFAULT_ROWSPEC, FormFactory.LINE_GAP_ROWSPEC,
	new RowSpec(RowSpec.FILL, Sizes.DEFAULT, FormSpec.DEFAULT_GROW),
	FormFactory.LINE_GAP_ROWSPEC, FormFactory.DEFAULT_ROWSPEC }));

    // ---- label1 ----
    label1.setText("Cluster Count:");
    add(label1, cc.xy(1, 1));

    // ---- numSpn ----
    numSpn.setModel(new SpinnerNumberModel(2, 2, null, 1));
    add(numSpn, cc.xy(3, 1));

    // ---- runBtn ----
    runBtn.setText("Run");
    add(runBtn, cc.xy(5, 1));

    // ======== scrollPane1 ========
    {

      // ---- clusterTbl ----
      clusterTbl.setModel(new DefaultTableModel(new Object[][] { { null, null, null },
	  { null, null, null }, }, new String[] { null, null, null }) {
	boolean[] columnEditable = new boolean[] { false, true, true };

	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
	  return columnEditable[columnIndex];
	}
      });
      scrollPane1.setViewportView(clusterTbl);
    }
    add(scrollPane1, cc.xywh(1, 3, 5, 1));

    // ---- vizBtn ----
    vizBtn.setText("Viz");
    add(vizBtn, cc.xy(1, 5));
    // JFormDesigner - End of component initialization //GEN-END:initComponents
  }

  // JFormDesigner - Variables declaration - DO NOT MODIFY //GEN-BEGIN:variables
  // Generated using JFormDesigner non-commercial license
  private JLabel label1;
  private JSpinner numSpn;
  private JButton runBtn;
  private JScrollPane scrollPane1;
  private JTable clusterTbl;
  private JButton vizBtn;
  // JFormDesigner - End of variables declaration //GEN-END:variables
}
