/**
 * 
 */
package anl.aida.client.gui;

import java.awt.BorderLayout;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.concurrent.ExecutionException;

import javax.swing.JPanel;
import javax.swing.SwingWorker;

import simphony.util.messages.MessageCenter;
import anl.aida.client.Cache;
import anl.aida.client.Keyword;
import anl.aida.plot.CacheTimeSeriesParams;
import anl.aida.plot.ProgressListener;
import anl.aida.plot.TimeSeriesParameters;
import anl.aida.plot.TimeSeriesPlotWidget;
import anl.aida.util.DateRange;

/**
 * Swing worker that will create a Time series plot 
 * and finish by adding to a panel. 
 * 
 */
public class CreateTSWorker extends SwingWorker<TimeSeriesParameters, Object> implements ProgressListener<DateRange> {
  
  private static MessageCenter center = MessageCenter.getMessageCenter(CreateTSWorker.class);
  private static DateFormat format = new SimpleDateFormat("MM/dd/yy");
  
  private Collection<Keyword> keywords;
  private Cache cache;
  private Iterable<DateRange> ranges;
  private JPanel panel;
  private ProgressLayer pLayer;

  /**
   * @param keywords
   * @param cache
   * @param ranges
   */
  public CreateTSWorker(Collection<? extends Keyword> keywords, Cache cache, Iterable<DateRange> ranges, JPanel panel) {
    this.keywords = new ArrayList<Keyword>(keywords);
    this.cache = cache;
    this.ranges = ranges;
    this.panel = panel;
    pLayer = (ProgressLayer)panel.getClientProperty(ProgressLayer.CLIENT_KEY);
  }

  @SuppressWarnings("unchecked")
  @Override
  protected TimeSeriesParameters doInBackground() throws Exception {
    return CacheTimeSeriesParams.createParams(cache, keywords, ranges, this);
  }

  @Override
  protected void done() {
    try {
      TimeSeriesPlotWidget widget = new TimeSeriesPlotWidget(get());
      panel.add(widget.getPanel(), BorderLayout.CENTER);
      panel.validate();
      panel.repaint();
    } catch (InterruptedException e) {
      center.error("Error while creating a time series", e);
    } catch (ExecutionException e) {
      center.error("Error while creating a time series", e);
    } finally {
      pLayer.setEnabled(true);
    }
  }

  @Override
  public void progressUpdated(DateRange range) {
    String update = "Processing " + format.format(range.getStart()) + " - " + format.format(range.getEnd());
    pLayer.setBusyText(update);
  }
}
