/*
 * Created by JFormDesigner on Fri Feb 06 10:31:32 EST 2009
 */

package anl.aida.client.gui;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Desktop;
import java.awt.Font;
import java.awt.Toolkit;
import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;

import org.jdesktop.jxlayer.JXLayer;

import anl.aida.client.doc.ClientDocument;
import anl.aida.client.doc.DocSets;
import anl.aida.client.doc.DocumentsModel;
import anl.aida.client.widget.MTaskPaneGroup;

import com.jgoodies.forms.factories.FormFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.ColumnSpec;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.layout.FormSpec;
import com.jgoodies.forms.layout.RowSpec;
import com.jgoodies.forms.layout.Sizes;
import com.l2fprod.common.swing.JTaskPane;
import com.l2fprod.common.swing.JTaskPaneGroup;

/**
 * @author User #1
 */
public class DocumentsView extends JPanel {

  private static final String KEYWORD_KEY = "keyword.key";
  private static final String PATH_KEY = "path.key";
  private static final String TITLE_KEY = "title.key";

  private static final int SET_SIZE = 100;
  private DocumentsModel model;
  private DocSets sets;
  private int setIndex = 0;

  private Font nonTitleFont, titleFont;
  private AIDAGui gui;
  private ProgressLayer pLayer;

  /**
   * Gets the model for this view.
   *
   * @return the model for this view.
   */
  public DocumentsModel getModel() {
    return model;
  }

  public DocumentsView() {
    initComponents();
    addListeners();

    nonTitleFont = new JLabel().getFont().deriveFont(11f);
    titleFont = new JLabel().getFont().deriveFont(Font.BOLD, 11f);

    pLayer = new ProgressLayer(this);
  }

  private void updateIndexDisplay() {
    nextBtn.setEnabled(setIndex < sets.getSetCount() - 1);
    prevBtn.setEnabled(setIndex > 0);
    pagelbl.setText((setIndex + 1) + " of " + sets.getSetCount());
  }

  private void addListeners() {
    prevBtn.setEnabled(false);
    nextBtn.setEnabled(false);

    prevBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        setIndex--;
        updateIndexDisplay();
        addDocs();
      }
    });

    nextBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        setIndex++;
        updateIndexDisplay();
        addDocs();
      }
    });

    searchBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        search();
      }
    });

    searchFld.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        search();
      }
    });
  }

  public JXLayer getJXLayer() {
    return pLayer.getJXLayer();
  }

  private void search() {
    String query = searchFld.getText().trim();
    if (query.length() > 0) gui.performSearch(query, model, pLayer);
  }

  public void init(AIDAGui gui) {
    this.gui = gui;
  }

  public void reset(DocumentsModel model) {
    this.model = model;
    this.sets = model.createDocumentSets(SET_SIZE);
    setIndex = 0;
    updateIndexDisplay();
    addDocs();
  }

  private void addDocs() {
    docItemsPane.removeAll();
    for (ClientDocument doc : sets.getDocuments(setIndex)) {
      docItemsPane.add(createGroup(doc));
    }
    searchBtn.setEnabled(docItemsPane.getComponentCount() > 1);
  }

  private JTaskPaneGroup createGroup(ClientDocument doc) {
    JTaskPaneGroup grp = new MTaskPaneGroup();
    grp.setExpanded(false);
    String title = doc.getTitle();
    grp.setTitle(title);
    grp.setFont(titleFont);
    grp.putClientProperty(TITLE_KEY, title);

    String keywords = doc.getKeywords();
    JLabel label = new JLabel(keywords);
    label.setFont(nonTitleFont);
    grp.add(label);
    grp.putClientProperty(KEYWORD_KEY, keywords);

    String path = doc.getPath();
    label = new JLabel(path);
    label.setToolTipText(label.getText());
    label.setFont(nonTitleFont);
    grp.add(label);
    grp.putClientProperty(PATH_KEY, path);

    label.addMouseListener(new MouseAdapter() {
      public void mouseClicked(MouseEvent e) {
        JLabel label = (JLabel) e.getSource();
        try {
          //AppStarter.start(label.getText());
          Desktop.getDesktop().browse(new URI(label.getText()));
        } catch (IOException ex) {
          ex.printStackTrace();
        } catch (URISyntaxException ex) {
	  ex.printStackTrace();
	}
      }

      public void mouseEntered(MouseEvent e) {
        JLabel label = (JLabel) e.getSource();
        label.setForeground(Color.BLUE);
        label.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
      }


      public void mouseExited(MouseEvent e) {
        JLabel label = (JLabel) e.getSource();
        label.setForeground(Color.BLACK);
        label.setCursor(null);
      }
    });

    grp.addMouseListener(new MouseAdapter() {
      @Override
      public void mousePressed(MouseEvent e) {
        if (e.isPopupTrigger()) {
          showPopup(e);
        }
      }

      @Override
      public void mouseReleased(MouseEvent e) {
        if (e.isPopupTrigger()) {
          showPopup(e);
        }
      }
    });

    return grp;
  }


  private static class CopyAction extends AbstractAction {

    private String[] keys;
    private JComponent comp;

    private CopyAction(String name, JComponent comp, String... keys) {
      super(name);
      this.keys = keys;
      this.comp = comp;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
      String newline = System.getProperty("line.separator");
      StringBuilder buf = new StringBuilder();
      for (int i = 0; i < keys.length; i++) {
        String str = (String) comp.getClientProperty(keys[i]);
        if (str != null) {
          if (i > 0) buf.append(newline);
          buf.append(str);
        }
      }

      StringSelection ss = new StringSelection(buf.toString());
      Toolkit.getDefaultToolkit().getSystemClipboard().setContents(ss, null);
    }
  }

  private void showPopup(MouseEvent evt) {
    JPopupMenu popup = new JPopupMenu();
    JComponent comp = (JComponent) evt.getSource();
    popup.add(new CopyAction("Copy Title", comp, TITLE_KEY));
    popup.add(new CopyAction("Copy Keywords", comp, KEYWORD_KEY));
    popup.add(new CopyAction("Copy Path", comp, PATH_KEY));
    popup.add(new CopyAction("Copy All", comp, TITLE_KEY, KEYWORD_KEY, PATH_KEY));

    popup.show((JComponent) evt.getSource(), evt.getX(), evt.getY());

  }

  private void initComponents() {
    // JFormDesigner - Component initialization - DO NOT MODIFY  //GEN-BEGIN:initComponents
    // Generated using JFormDesigner non-commercial license
    searchFld = new JTextField();
    searchBtn = new JButton();
    scrollPane1 = new JScrollPane();
    docItemsPane = new JTaskPane();
    panel1 = new JPanel();
    prevBtn = new JButton();
    nextBtn = new JButton();
    pagelbl = new JLabel();
    CellConstraints cc = new CellConstraints();

    //======== this ========
    setBorder(new EmptyBorder(5, 5, 5, 5));
    setLayout(new FormLayout(
            new ColumnSpec[]{
                    new ColumnSpec(ColumnSpec.FILL, Sizes.PREFERRED, FormSpec.DEFAULT_GROW),
                    FormFactory.LABEL_COMPONENT_GAP_COLSPEC,
                    FormFactory.DEFAULT_COLSPEC
            },
            new RowSpec[]{
                    FormFactory.DEFAULT_ROWSPEC,
                    FormFactory.LINE_GAP_ROWSPEC,
                    new RowSpec(RowSpec.FILL, Sizes.DEFAULT, FormSpec.DEFAULT_GROW),
                    FormFactory.LINE_GAP_ROWSPEC,
                    FormFactory.DEFAULT_ROWSPEC
            }));
    add(searchFld, cc.xy(1, 1));

    //---- searchBtn ----
    searchBtn.setText("Search");
    add(searchBtn, cc.xy(3, 1));

    //======== scrollPane1 ========
    {
      scrollPane1.setViewportView(docItemsPane);
    }
    add(scrollPane1, cc.xywh(1, 3, 3, 1));

    //======== panel1 ========
    {
      panel1.setLayout(new FormLayout(
              new ColumnSpec[]{
                      FormFactory.DEFAULT_COLSPEC,
                      FormFactory.LABEL_COMPONENT_GAP_COLSPEC,
                      FormFactory.DEFAULT_COLSPEC,
                      FormFactory.LABEL_COMPONENT_GAP_COLSPEC,
                      new ColumnSpec(ColumnSpec.FILL, Sizes.PREFERRED, FormSpec.DEFAULT_GROW)
              },
              RowSpec.decodeSpecs("default")));

      //---- prevBtn ----
      prevBtn.setText("Prev");
      panel1.add(prevBtn, cc.xy(1, 1));

      //---- nextBtn ----
      nextBtn.setText("Next");
      panel1.add(nextBtn, cc.xy(3, 1));
      panel1.add(pagelbl, cc.xy(5, 1));
    }
    add(panel1, cc.xywh(1, 5, 3, 1));
    // JFormDesigner - End of component initialization  //GEN-END:initComponents
  }

  // JFormDesigner - Variables declaration - DO NOT MODIFY  //GEN-BEGIN:variables
  // Generated using JFormDesigner non-commercial license
  private JTextField searchFld;
  private JButton searchBtn;
  private JScrollPane scrollPane1;
  private JTaskPane docItemsPane;
  private JPanel panel1;
  private JButton prevBtn;
  private JButton nextBtn;
  private JLabel pagelbl;
  // JFormDesigner - End of variables declaration  //GEN-END:variables
}
