package anl.aida.client.gui;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.swing.table.AbstractTableModel;

import anl.aida.algo.ScoredKeyword;

/**
 *         Date: Feb 12, 2009 4:42:49 PM
 */
public class KeywordTableModel extends AbstractTableModel {
  
  public static final int KEYWORD_COL = 1;
  public static final int DOC_COUNT_COL = 2;
  public static final int PERCENT_COL = 3;
  public static final int SELECTED_COL = 0;

  private class Item {

    private ScoredKeyword concept;
    boolean selected = false;

    private Item(ScoredKeyword term) {
      this.concept = term;
    }

    public Object get(int index) {
      if (index == SELECTED_COL) return selected;
      else if (index == KEYWORD_COL) return concept.getLabel();
      //else if (index == 2) return concept.getScore();
      else if (index == DOC_COUNT_COL) return concept.getDocumentCount();
      else if (index == PERCENT_COL) return concept.getDocumentCount() / ((float)totalDocs)* 100f;
      return null;
    }

    public void setSelected(boolean selected) {
      this.selected = selected;
    }
  }

  private static final String[] COL_NAMES = {"", "Keyword", /*"Score",*/ "Document Count", "[%]"};
  private static final Class<?>[] COL_TYPES = {Boolean.class, /*String.class,*/ String.class, String.class, String.class};

  private int totalDocs = 1;
  private java.util.List<Item> items = new ArrayList<Item>();
  private Set<ScoredKeyword> selectedTerms = new HashSet<ScoredKeyword>();

  public void reset(int totalDocs) {
    items.clear();
    this.totalDocs = totalDocs;
    selectedTerms.clear();
    fireTableDataChanged();
  }

  @Override
  public int getRowCount() {
    return items.size();
  }

  @Override
  public int getColumnCount() {
    return COL_NAMES.length;
  }

  public void addItem(ScoredKeyword term) {
    items.add(new Item(term));
  }

  @Override
  public Object getValueAt(int row, int col) {
    Item item = items.get(row);
    Object obj = item.get(col);
    //if (col == 2) return String.format("%.2f", ((Number) obj).floatValue());
    if (col == DOC_COUNT_COL) return obj.toString();
    else if (col == PERCENT_COL) return String.format("%.1f", ((Number) obj).floatValue());
    return obj;
  }

  @Override
  public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
    if (columnIndex == 0) {
      Boolean selected = (Boolean) aValue;
      KeywordTableModel.Item item = items.get(rowIndex);
      item.setSelected(selected);
      if (selected) {
        selectedTerms.add(item.concept);
      } else {
        selectedTerms.remove(item.concept);
      }
      fireTableRowsUpdated(rowIndex, rowIndex);
    }
  }

  @Override
  public boolean isCellEditable(int rowIndex, int columnIndex) {
    return columnIndex == 0;
  }

  @Override
  public Class<?> getColumnClass(int columnIndex) {
    return COL_TYPES[columnIndex];
  }

  @Override
  public String getColumnName(int column) {
    return COL_NAMES[column];
  }

  public int getSelectedItemCount() {
    return selectedTerms.size();
  }

  public List<ScoredKeyword> getSelectedTerms() {
    return new ArrayList<ScoredKeyword>(selectedTerms);
  }

  public void selectAll() {
    for (Item item : items) {
      item.setSelected(true);
      selectedTerms.add(item.concept);
    }
    fireTableDataChanged();
  }
  
  public void setRowsSelected(int[] rows, boolean selected) {
    for (int row : rows) {
      setRowSelected(row, selected);
    }
    fireTableDataChanged();
  }
  
  private void setRowSelected(int row, boolean selected) {
    Item item = items.get(row);
    item.selected = selected;
    if (selected) selectedTerms.add(item.concept);
    else selectedTerms.remove(item.concept);
  }
  
  public void flipSelectionRows(int[] rows) {
    for (int row : rows) {
      Item item = items.get(row);
      item.selected = !item.selected;
      if (item.selected) {
	selectedTerms.add(item.concept);
      } else {
	selectedTerms.remove(item.concept);
      }
    }
    fireTableDataChanged();
  }

  public void unselectAll() {
    selectedTerms.clear();
    for (Item item : items) {
      item.setSelected(false);
    }
    fireTableDataChanged();
  }
}