package anl.aida.client.gui;

import org.jdesktop.jxlayer.JXLayer;
import org.jdesktop.jxlayer.plaf.AbstractLayerUI;
import org.jdesktop.swingx.JXBusyLabel;
import org.jdesktop.swingx.painter.MattePainter;
import org.jdesktop.swingx.painter.Painter;

import javax.swing.*;
import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;

/**
 *         Date: Feb 17, 2009 9:55:54 AM
 */
public class ProgressLayer {

  public static final String CLIENT_KEY = ProgressLayer.class.getName();

  private JXLayer layer;
  private JXBusyLabel busyLabel = new JXBusyLabel();
  private JComponent comp;

  public ProgressLayer(JComponent comp) {
    this.comp = comp;
    layer = new JXLayer<JComponent>(comp, new ProgressLayerUI());
    layer.getGlassPane().setLayout(new BorderLayout());
    busyLabel.setHorizontalAlignment(JLabel.CENTER);
    busyLabel.setHorizontalTextPosition(JLabel.CENTER);
    busyLabel.setVerticalTextPosition(JLabel.BOTTOM);
    comp.putClientProperty(CLIENT_KEY, this);
  }

  private void setEnabled(JComponent comp, boolean enabled) {
    comp.setEnabled(enabled);
    for (Component child : comp.getComponents()) {
      if (child instanceof JComponent) setEnabled((JComponent) child, enabled);
    }
  }
  
  public String getBusyText() {
    return busyLabel.getText();
  }
  
  public void setBusyText(String text) {
    busyLabel.setText(text);
  }

  public void setEnabled(boolean enabled) {
    if (enabled) {
      busyLabel.setBusy(false);
      layer.getGlassPane().remove(busyLabel);
    } else {
      layer.getGlassPane().add(busyLabel, BorderLayout.CENTER);
      busyLabel.setBusy(true);
      layer.getGlassPane().invalidate();
      layer.getGlassPane().revalidate();
      layer.getGlassPane().repaint();
    }
    layer.setEnabled(enabled);
  }

  public boolean isEnabled() {
    return layer.isEnabled();
  }

  public JXLayer getJXLayer() {
    return layer;
  }

  private class ProgressLayerUI extends AbstractLayerUI<JComponent> {

    private Painter<JXLayer> painter = new MattePainter<JXLayer>(new Color(255, 255, 255, 150));

    @Override
    protected void processKeyEvent(KeyEvent keyEvent, JXLayer<JComponent> layer) {
      if (layer.isEnabled()) super.processKeyEvent(keyEvent, layer);
      else {
        keyEvent.consume();
      }
    }

    @Override
    protected void processMouseEvent(MouseEvent mouseEvent, JXLayer<JComponent> layer) {
      if (layer.isEnabled()) {
        super.processMouseEvent(mouseEvent, layer);
      } else {
        mouseEvent.consume();
      }
    }

    @Override
    protected void paintLayer(Graphics2D g2, JXLayer<JComponent> layer) {
      super.paintLayer(g2, layer);
      if (!layer.isEnabled()) painter.paint(g2, layer, layer.getWidth(), layer.getHeight());
    }
  }
}
