package anl.aida.client.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;

import anl.aida.data.Axes;
import anl.aida.data.CompatibilityChecker;
import anl.aida.data.CoordAxis;
import anl.aida.formula.FormulaVariable;

import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.factories.DefaultComponentFactory;
import com.jgoodies.forms.factories.FormFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.ColumnSpec;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.layout.RowSpec;

/**
 * @author User #2
 */
public class ScatterDialog extends JDialog {

  private FormulaListElement xElement, yElement;

  public ScatterDialog(Frame owner) {
    super(owner);
    initComponents();
    setTitle("Plot Configuration");
    errorLbl.setForeground(errorLbl.getBackground());
    errorLbl.setText("DUMMY PLACEHOLDER");
    fldX.setPrototypeDisplayValue("Formula(s) missing Group Dimensions.");
  }

  public ScatterDialog(Dialog owner) {
    super(owner);
    initComponents();
    setTitle("Plot Configuration");
    errorLbl.setForeground(errorLbl.getBackground());
    errorLbl.setText("DUMMY PLACEHOLDER");
    fldX.setPrototypeDisplayValue("Formula(s) missing Group Dimensions.");
  }

  public FormulaListElement getXElement() {
    return xElement;
  }

  public FormulaListElement getYElement() {
    return yElement;
  }

  public void setFormulas(java.util.List<FormulaListElement> elements) {
    DefaultComboBoxModel xModel = new DefaultComboBoxModel();
    DefaultComboBoxModel yModel = new DefaultComboBoxModel();

    for (FormulaListElement item : elements) {
      xModel.addElement(item);
      yModel.addElement(item);
    }

    fldX.setModel(xModel);
    fldY.setModel(yModel);
    fldX.setSelectedIndex(0);
    fldY.setSelectedIndex(0);

    fldX.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	errorLbl.setText("");
      }
    });

    fldY.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	errorLbl.setText("");
      }
    });

    okButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	boolean result = doCheck();
	if (result) {
	  xElement = (FormulaListElement) fldX.getSelectedItem();
	  yElement = (FormulaListElement) fldY.getSelectedItem();
	  dispose();
	}
      }
    });

    cancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	dispose();
      }
    });
  }

  private boolean doCheck() {
    errorLbl.setText("");
    CompatibilityChecker checker = new CompatibilityChecker();

    FormulaListElement item = (FormulaListElement) fldX.getSelectedItem();
    for (FormulaVariable var : item.getVariables()) {
      Axes<CoordAxis> axes = var.getDataset().getAxes(var.getVariable());
      checker.addAxes(axes);
    }
    item = (FormulaListElement) fldY.getSelectedItem();
    for (FormulaVariable var : item.getVariables()) {
      Axes<CoordAxis> axes = var.getDataset().getAxes(var.getVariable());
      checker.addAxes(axes);
    }

    CompatibilityChecker.Result result = checker.check();
    if (!result.isTimeCompatible()) {
      errorLbl.setForeground(Color.RED);
      errorLbl.setText("Incompatible Time Dimensions.");
      return false;
    }

    if (result.isCategoryAllNull()) {
      errorLbl.setForeground(Color.RED);
      errorLbl.setText("Formula(s) missing Group Dimensions.");
      return false;
    }
    
    

    return true;
  }

  private void initComponents() {
    // JFormDesigner - Component initialization - DO NOT MODIFY
    // //GEN-BEGIN:initComponents
    // Generated using JFormDesigner non-commercial license
    DefaultComponentFactory compFactory = DefaultComponentFactory.getInstance();
    dialogPane = new JPanel();
    contentPanel = new JPanel();
    separator2 = compFactory.createSeparator("Select Formulas for Axes");
    label1 = new JLabel();
    fldX = new JComboBox();
    label2 = new JLabel();
    fldY = new JComboBox();
    errorLbl = new JLabel();
    buttonBar = new JPanel();
    okButton = new JButton();
    cancelButton = new JButton();
    CellConstraints cc = new CellConstraints();

    // ======== this ========
    setModal(true);
    Container contentPane = getContentPane();
    contentPane.setLayout(new BorderLayout());

    // ======== dialogPane ========
    {
      dialogPane.setBorder(Borders.DIALOG_BORDER);
      dialogPane.setLayout(new BorderLayout());

      // ======== contentPanel ========
      {
	contentPanel.setLayout(new FormLayout(new ColumnSpec[] { FormFactory.RELATED_GAP_COLSPEC,
	    FormFactory.LABEL_COMPONENT_GAP_COLSPEC, new ColumnSpec("max(min;75dlu):grow") },
	    new RowSpec[] { FormFactory.DEFAULT_ROWSPEC, FormFactory.LINE_GAP_ROWSPEC,
		FormFactory.DEFAULT_ROWSPEC, FormFactory.LINE_GAP_ROWSPEC,
		FormFactory.DEFAULT_ROWSPEC, FormFactory.LINE_GAP_ROWSPEC,
		FormFactory.DEFAULT_ROWSPEC, FormFactory.LINE_GAP_ROWSPEC,
		FormFactory.DEFAULT_ROWSPEC, FormFactory.LINE_GAP_ROWSPEC,
		FormFactory.DEFAULT_ROWSPEC }));
	contentPanel.add(separator2, cc.xywh(1, 1, 3, 1));

	// ---- label1 ----
	label1.setText("X-Axis:");
	contentPanel.add(label1, cc.xywh(2, 3, 2, 1));

	// ---- fldX ----
	fldX.setEditable(false);
	contentPanel.add(fldX, cc.xy(3, 5));

	// ---- label2 ----
	label2.setText("Y-Axis:");
	contentPanel.add(label2, cc.xywh(2, 7, 2, 1));

	// ---- fldY ----
	fldY.setEditable(false);
	contentPanel.add(fldY, cc.xy(3, 9));

	// ---- errorLbl ----
	errorLbl.setForeground(Color.red);
	contentPanel.add(errorLbl, cc.xywh(1, 11, 3, 1));
      }
      dialogPane.add(contentPanel, BorderLayout.CENTER);

      // ======== buttonBar ========
      {
	buttonBar.setBorder(Borders.BUTTON_BAR_GAP_BORDER);
	buttonBar.setLayout(new FormLayout(
	    new ColumnSpec[] { FormFactory.GLUE_COLSPEC, FormFactory.BUTTON_COLSPEC,
		FormFactory.RELATED_GAP_COLSPEC, FormFactory.BUTTON_COLSPEC }, RowSpec
		.decodeSpecs("pref")));

	// ---- okButton ----
	okButton.setText("OK");
	buttonBar.add(okButton, cc.xy(2, 1));

	// ---- cancelButton ----
	cancelButton.setText("Cancel");
	buttonBar.add(cancelButton, cc.xy(4, 1));
      }
      dialogPane.add(buttonBar, BorderLayout.SOUTH);
    }
    contentPane.add(dialogPane, BorderLayout.CENTER);
    pack();
    setLocationRelativeTo(getOwner());
    // JFormDesigner - End of component initialization //GEN-END:initComponents
  }

  // JFormDesigner - Variables declaration - DO NOT MODIFY //GEN-BEGIN:variables
  // Generated using JFormDesigner non-commercial license
  private JPanel dialogPane;
  private JPanel contentPanel;
  private JComponent separator2;
  private JLabel label1;
  private JComboBox fldX;
  private JLabel label2;
  private JComboBox fldY;
  private JLabel errorLbl;
  private JPanel buttonBar;
  private JButton okButton;
  private JButton cancelButton;
  // JFormDesigner - End of variables declaration //GEN-END:variables
}
