/*
 * Created by JFormDesigner on Wed Aug 12 10:15:56 EDT 2009
 */

package anl.aida.client.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.ListModel;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import simphony.util.messages.MessageCenter;
import anl.aida.data.Axes;
import anl.aida.data.AxisRange;
import anl.aida.data.Combiner;
import anl.aida.data.CompatibilityChecker;
import anl.aida.data.CoordAxis;
import anl.aida.data.DataFrame;
import anl.aida.data.DataManager;
import anl.aida.data.Dataset;
import anl.aida.data.Units;
import anl.aida.data.Variable;
import anl.aida.formula.Formula;
import anl.aida.formula.FormulaFactory;
import anl.aida.formula.FormulaVariable;
import anl.aida.formula.IllegalFormulaException;
import anl.aida.formula.ValidationResult;
import anl.aida.plot.PlotType;
import anl.aida.plot.io.PlotData;
import anl.aida.plot.io.Project;

import com.jgoodies.forms.factories.DefaultComponentFactory;
import com.jgoodies.forms.factories.FormFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.ColumnSpec;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.layout.FormSpec;
import com.jgoodies.forms.layout.RowSpec;
import com.jgoodies.forms.layout.Sizes;

/**
 * @author User #1
 */
@SuppressWarnings("serial")
public class StructuredDataPanel extends JPanel {

  private MessageCenter center = MessageCenter.getMessageCenter(StructuredDataPanel.class);

  private static class FeedRenderer extends DefaultListCellRenderer {

    @Override
    public Component getListCellRendererComponent(JList list, Object value, int index,
	boolean isSelected, boolean cellHasFocus) {
      if (value != null) {
	Dataset dataset = (Dataset) value;
	value = dataset.getAlias() + " " + dataset.getName();
      }
      return super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
    }
  }

  private static class VariableRenderer extends DefaultListCellRenderer {

    @Override
    public Component getListCellRendererComponent(JList list, Object value, int index,
	boolean isSelected, boolean cellHasFocus) {
      if (value != null) {
	Variable var = (Variable) value;
	String unitName = Units.getFormattedName(var.getUnit()).trim();
	if (unitName.length() == 0) value = var.getName();
	else value = var.getName() + " (" + unitName + ")";
      }
      return super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
    }
  }

  private AIDAGui gui;
  private DataManager dataManager;

  public StructuredDataPanel() {
    initComponents();
    feedLst.setModel(new DefaultListModel());
    feedLst.setCellRenderer(new FeedRenderer());
    varLst.setModel(new DefaultListModel());
    varLst.setCellRenderer(new VariableRenderer());
    formulaLst.setModel(new DefaultListModel());
    deleteBtn.setEnabled(false);
    mapBtn.setEnabled(false);
    tsBtn.setEnabled(false);
    tableBtn.setEnabled(false);
    scatterPlotBtn.setEnabled(false);
    plot3DBtn.setEnabled(false);

    addListeners();
  }

  private void addListeners() {
    feedLst.addListSelectionListener(new ListSelectionListener() {
      @Override
      public void valueChanged(ListSelectionEvent e) {
	if (!e.getValueIsAdjusting()) {
	  resetVarLst();
	}
      }
    });

    varLst.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseClicked(MouseEvent e) {
	if (e.getClickCount() == 2 && e.getButton() == MouseEvent.BUTTON1
	    && varLst.getSelectedIndex() != -1) {
	  addVariableAsFormula((Variable) varLst.getSelectedValue());
	  formulaLst.setSelectedIndex(formulaLst.getModel().getSize() - 1);
	}
      }

      public void mouseReleased(MouseEvent evt) {
	showVarMenu(evt);
      }

      public void mousePressed(MouseEvent evt) {
	showVarMenu(evt);
      }
    });

    formulaLst.addListSelectionListener(new ListSelectionListener() {
      @Override
      public void valueChanged(ListSelectionEvent e) {
	if (!e.getValueIsAdjusting()) {

	}
      }
    });

    formulaLst.addListSelectionListener(new ListSelectionListener() {
      @Override
      public void valueChanged(ListSelectionEvent e) {
	deleteBtn.setEnabled(formulaLst.getSelectedIndex() != -1);
	if (!e.getValueIsAdjusting()) {
	  updateFormulaButtons();
	}
      }
    });

    deleteBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	DefaultListModel model = (DefaultListModel) formulaLst.getModel();
	model.remove(formulaLst.getSelectedIndex());
	scatterPlotBtn.setEnabled(formulaLst.getModel().getSize() > 0);
      }
    });

    mapBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	createMap();
      }
    });
    
    map3DBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	create3DMap();
      }
    });

    tsBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	createTimeSeries();
      }
    });

    plot3DBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	create3DPlot();
      }
    });

    scatterPlotBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	createScatterPlot();
      }
    });
    
    bubbleBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	createBubblePlot();
      }
    });

    addBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	addTextAsFormula(formulaFld.getText().trim());
      }
    });

    formulaFld.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	addTextAsFormula(formulaFld.getText().trim());
      }
    });

    tableBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	createDataTable();
      }
    });
  }

  private void addTextAsFormula(String formula) {
    if (formula.length() > 0) {
      FormulaListElement item = createFormulaElement(formula);
      if (item != null) {
	DefaultListModel model = (DefaultListModel) formulaLst.getModel();
	model.addElement(item);
	formulaLst.setSelectedValue(item, true);
      }
    }
    scatterPlotBtn.setEnabled(true);
  }

  private FormulaListElement createFormulaElement(Variable var) {
    gui.clearStatusMessage();
    String strFormula = var.getName() + var.getDataset().getAlias();
    return createFormulaElement(strFormula);
  }

  private FormulaListElement createFormulaElement(String strFormula) {
    Formula formula = new FormulaFactory().createFormula(strFormula);
    ValidationResult result = formula.validate(dataManager, new ArrayList<AxisRange>());
    ValidationResult.Status status = result.getStatus();
    if (status == ValidationResult.Status.FAIL) {
      Exception ex = result.getException();
      String msg = "Formula '" + strFormula + "' is invalid";
      if (ex != null) {
	msg += ": " + ex.getMessage();
	center.error(msg, ex);
      }
      gui.addStatusMessage(msg, Color.RED);
      return null;
    }

    return new FormulaListElement(formula, result.getVariables());
  }

  private void addVariableAsFormula(Variable var) {
    FormulaListElement item = createFormulaElement(var);
    if (item != null) {
      DefaultListModel model = (DefaultListModel) formulaLst.getModel();
      model.addElement(item);
    }
    scatterPlotBtn.setEnabled(true);
  }

  private void addVarsAsFormulas() {
    DefaultListModel model = (DefaultListModel) formulaLst.getModel();
    for (Object obj : varLst.getSelectedValues()) {
      Variable var = (Variable) obj;
      addVariableAsFormula(var);
    }
    formulaLst.setSelectedIndex(model.getSize() - 1);
    scatterPlotBtn.setEnabled(true);
  }

  private void addVarsToEditor() {
    StringBuilder buf = new StringBuilder();
    for (Object obj : varLst.getSelectedValues()) {
      Variable var = (Variable) obj;
      buf.append(var.getName() + var.getDataset().getAlias());
      buf.append("  ");
    }
    formulaFld.setText(buf.toString().trim());
    formulaFld.requestFocus();
    formulaFld.setCaretPosition(0);
  }

  private void showVarMenu(MouseEvent evt) {
    if (varLst.getSelectedIndex() != -1 && evt.isPopupTrigger()) {
      JPopupMenu menu = new JPopupMenu();
      menu.add(new AbstractAction("Add As Formula(s)") {
	public void actionPerformed(ActionEvent evt) {
	  addVarsAsFormulas();
	}
      });

      menu.add(new AbstractAction("Add To Formula Editor") {
	public void actionPerformed(ActionEvent evt) {
	  addVarsToEditor();
	}
      });

      menu.show(varLst, evt.getX(), evt.getY());
    }
  }

  private void updateFormulaButtons() {
    gui.clearStatusMessage();
    CompatibilityChecker checker = new CompatibilityChecker();
    Object[] items = formulaLst.getSelectedValues();
    if (items == null || items.length == 0) {
      mapBtn.setEnabled(false);
      tsBtn.setEnabled(false);
      deleteBtn.setEnabled(false);
      plot3DBtn.setEnabled(false);
    } else {
      deleteBtn.setEnabled(true);

      for (Object obj : items) {
	FormulaListElement item = (FormulaListElement) obj;
	for (FormulaVariable var : item.getVariables()) {
	  Axes<CoordAxis> axes = var.getDataset().getAxes(var.getVariable());
	  checker.addAxes(axes);
	}
      }

      CompatibilityChecker.Result result = checker.check();
      mapBtn.setEnabled(result.isMapCompatible());
      tsBtn.setEnabled(result.isCategoryCompatible() && result.isTimeCompatible());
      plot3DBtn.setEnabled(tsBtn.isEnabled());
      tableBtn.setEnabled(tsBtn.isEnabled());

      String msg = result.getMessage();
      if (msg.length() > 0) {
	gui.addStatusMessage(msg, Color.RED);
      }
    }
  }

  private DataFrame dataFrameFromFormula() {
    Object[] objs = formulaLst.getSelectedValues();
    FormulaListElement[] items = new FormulaListElement[objs.length];
    int i = 0;
    for (Object obj : objs) {
      items[i++] = (FormulaListElement) obj;
    }
    return dataFrameFromFormula(items);
  }

  private DataFrame dataFrameFromFormula(FormulaListElement[] items) {
    DataFrame frame = null;
    if (items.length == 1) {
      FormulaListElement item = items[0];
      if (item != null) {
	frame = evaluateFormulaItem(item);
      }
    } else if (items.length > 1) {
      // combine into a single DataFrame
      Combiner combiner = new Combiner();
      boolean error = false;
      for (FormulaListElement item : items) {
	DataFrame aFrame = evaluateFormulaItem(item);
	if (aFrame == null) {
	  error = true;
	  break;
	} else {
	  combiner.addDataFrame(aFrame);
	}
      }
      if (!error) {
	try {
	  gui.clearStatusMessage();
	  frame = combiner.combine();
	} catch (IllegalFormulaException ex) {
	  center.error("Error creating time series", ex);
	  gui.addStatusMessage("Error creating times series: " + ex.getMessage(), Color.RED);
	}
      }
    }

    return frame;
  }

  private void createDataTable() {
    DataFrame frame = dataFrameFromFormula();
    if (frame != null) {
      gui.addDataTable(frame);
    }
  }

  private void createTimeSeries() {
    DataFrame frame = dataFrameFromFormula();
    if (frame != null) {
      createTimeSeries(frame);
    }
  }

  private String createTimeSeries(DataFrame frame) {
    List<DataFrame> frames = new ArrayList<DataFrame>();
    frames.add(frame);
    return gui.addTimeSeries(frames);
  }

  private void create3DPlot() {
    DataFrame frame = dataFrameFromFormula();
    if (frame != null) {
      gui.add3DPlot(frame);
    }
  }
  
  private void createBubblePlot() {
    BubbleDialog dialog = new BubbleDialog(gui.getFrame());
    List<FormulaListElement> list = new ArrayList<FormulaListElement>();
    for (int i = 0; i < formulaLst.getModel().getSize(); i++) {
      list.add((FormulaListElement) formulaLst.getModel().getElementAt(i));
    }
    
    BubbleConfigBean bean = new BubbleConfigBean();
    dialog.init(list, bean);
    dialog.pack();
    dialog.setVisible(true);

    if (!bean.isCanceled()) {
      gui.addBubblePlot(bean);
    }
    
  }

  private void createScatterPlot() {
    ScatterDialog dialog = new ScatterDialog(gui.getFrame());
    List<FormulaListElement> list = new ArrayList<FormulaListElement>();
    for (int i = 0; i < formulaLst.getModel().getSize(); i++) {
      list.add((FormulaListElement) formulaLst.getModel().getElementAt(i));
    }
    dialog.setFormulas(list);
    dialog.pack();
    dialog.setVisible(true);

    if (dialog.getXElement() != null && dialog.getYElement() != null) {
      DataFrame xFrame = evaluateFormulaItem(dialog.getXElement());
      DataFrame yFrame = evaluateFormulaItem(dialog.getYElement());
      gui.addScatterPlot(xFrame, yFrame);
    }
  }

  private DataFrame evaluateFormulaItem(FormulaListElement item) {
    gui.clearStatusMessage();
    DataFrame frame = null;
    try {
      Formula formula = item.getFormula();
      frame = formula.evaluate(dataManager, new ArrayList<AxisRange>());
    } catch (IllegalFormulaException ex) {
      center.error("Error while evaluating formula", ex);
      gui.addStatusMessage("Error while evaluating formula: " + ex.getMessage(), Color.RED);
    }

    return frame;
  }

  private void createMap() {
    FormulaListElement item = (FormulaListElement) formulaLst.getSelectedValue();
    if (item != null) {
      DataFrame frame = evaluateFormulaItem(item);
      if (frame != null) {
	createMap(frame);
      }
    }
  }
  
  private void create3DMap() {
    FormulaListElement item = (FormulaListElement) formulaLst.getSelectedValue();
    if (item != null) {
      DataFrame frame = evaluateFormulaItem(item);
      if (frame != null) {
	create3DMap(frame);
      }
    }
  }
  
  private String create3DMap(DataFrame frame) {
    return gui.add3DGISPlotDisplay(frame);
  }

  private String createMap(DataFrame frame) {
    
    //gui.clearStatusMessage();
    return gui.addGISPlotDisplay(frame);
    /*
    MapPlotCreator plotCreator = new MapPlotCreator();
    try {
      GISDisplay display = plotCreator.createPlot(frame, gui);
      

    } catch (PlotCreationException ex) {
      center.error("Error while creating map plot", ex);
      gui.addStatusMessage("Error while creating map plot: " + ex.getMessage(), Color.RED);
    }
    */
    
  }

  private void resetVarLst() {
    DefaultListModel model = (DefaultListModel) varLst.getModel();
    model.clear();
    if (feedLst.getSelectedValue() != null) {
      Dataset set = (Dataset) feedLst.getSelectedValue();
      List<Variable> vars = set.getVariables();
      Collections.sort(vars, new Comparator<Variable>() {
	@Override
	public int compare(Variable o1, Variable o2) {
	  return o1.getName().compareTo(o2.getName());
	}
      });

      for (Variable var : vars) {
	model.addElement(var);
      }
    }
  }

  public void init(AIDAGui gui, DataManager manager) {
    this.gui = gui;
    this.dataManager = manager;
  }

  public void resetFeed(List<Dataset> feeds) {
    // this.feeds = feeds;
    Collections.sort(feeds, new Comparator<Dataset>() {
      @Override
      public int compare(Dataset o1, Dataset o2) {
	return (o1.getAlias() + o1.getName()).compareTo(o2.getAlias() + o2.getName());
      }
    });
    
    DefaultListModel model = (DefaultListModel) feedLst.getModel();
    model.clear();
    for (Dataset feed : feeds) {
      model.addElement(feed);
    }
    if (model.size() > 0) {
      feedLst.setSelectedIndex(0);
    }
  }

  private FormulaListElement getFormulaElementFor(String formula) {
    ListModel model = formulaLst.getModel();
    for (int i = 0; i < model.getSize(); i++) {
      FormulaListElement item = (FormulaListElement) model.getElementAt(i);
      if (item.getFormula().getFormulaAsString().equals(formula))
	return item;
    }
    return null;
  }

  public void resetProject(Project project) {
    List<String> list = new ArrayList<String>();
    for (String formula : project.formulas()) {
      list.add(formula);
    }

    Collections.sort(list);
    for (String formula : list) {
      addTextAsFormula(formula);
    }
  }
  
  
  /**
   * Creates a plot based on the PlotData and returns the dockable frame id
   * of the created plot.
   * 
   * @param data the data to create the plot from
   * @return the dockable frame id of the created plot.
   */
  public String createPlot(PlotData data) {
    String id = null;
    FormulaListElement[] items = new FormulaListElement[data.getFormulaCount()];
    int i = 0;
    for (String formula : data.formulas()) {
	items[i++] = getFormulaElementFor(formula);
    }

   
    PlotType type = data.getType();
    if (type != PlotType.SCATTER) {
	DataFrame frame = dataFrameFromFormula(items);

	if (frame != null) {
	  if (type == PlotType.MAP)
	    id = createMap(frame);
	  else if (type == PlotType.TIME_SERIES)
	    id = createTimeSeries(frame);
	  else if (type == PlotType.DATA_TABLE)
	    id = gui.addDataTable(frame);
	  else if (type == PlotType.STACK_3D)
	    id = gui.add3DPlot(frame);
	}
    } else {
	DataFrame xFrame = evaluateFormulaItem(items[0]);
	DataFrame yFrame = evaluateFormulaItem(items[1]);
	id = gui.addScatterPlot(xFrame, yFrame);
    }
    
    return id;
  }

  private void initComponents() {
    // JFormDesigner - Component initialization - DO NOT MODIFY
    // //GEN-BEGIN:initComponents
    // Generated using JFormDesigner non-commercial license
    DefaultComponentFactory compFactory = DefaultComponentFactory.getInstance();
    separator3 = compFactory.createSeparator("Feeds");
    scrollPane3 = new JScrollPane();
    feedLst = new JList();
    separator1 = compFactory.createSeparator("Variables");
    label1 = new JLabel();
    scrollPane1 = new JScrollPane();
    varLst = new JList();
    separator2 = compFactory.createSeparator("Formulas");
    toolBar1 = new JToolBar();
    deleteBtn = new JButton();
    mapBtn = new JButton();
    map3DBtn = new JButton();
    tsBtn = new JButton();
    tableBtn = new JButton();
    plot3DBtn = new JButton();
    scatterPlotBtn = new JButton();
    bubbleBtn = new JButton();
    scrollPane2 = new JScrollPane();
    formulaLst = new JList();
    formulaFld = new JTextField();
    addBtn = new JButton();
    CellConstraints cc = new CellConstraints();

    //======== this ========
    setBorder(new EmptyBorder(5, 5, 5, 5));
    setLayout(new FormLayout(
      new ColumnSpec[] {
	new ColumnSpec(ColumnSpec.FILL, Sizes.DEFAULT, FormSpec.DEFAULT_GROW),
	FormFactory.LABEL_COMPONENT_GAP_COLSPEC,
	FormFactory.DEFAULT_COLSPEC
      },
      new RowSpec[] {
	FormFactory.DEFAULT_ROWSPEC,
	FormFactory.LINE_GAP_ROWSPEC,
	new RowSpec(RowSpec.FILL, Sizes.DEFAULT, 0.5),
	FormFactory.UNRELATED_GAP_ROWSPEC,
	FormFactory.DEFAULT_ROWSPEC,
	FormFactory.LINE_GAP_ROWSPEC,
	FormFactory.DEFAULT_ROWSPEC,
	FormFactory.LINE_GAP_ROWSPEC,
	new RowSpec(RowSpec.FILL, Sizes.DEFAULT, FormSpec.DEFAULT_GROW),
	FormFactory.UNRELATED_GAP_ROWSPEC,
	FormFactory.DEFAULT_ROWSPEC,
	FormFactory.LINE_GAP_ROWSPEC,
	FormFactory.DEFAULT_ROWSPEC,
	FormFactory.LINE_GAP_ROWSPEC,
	new RowSpec(RowSpec.FILL, Sizes.DEFAULT, FormSpec.DEFAULT_GROW),
	FormFactory.LINE_GAP_ROWSPEC,
	FormFactory.DEFAULT_ROWSPEC
      }));
    add(separator3, cc.xywh(1, 1, 3, 1));

    //======== scrollPane3 ========
    {

      //---- feedLst ----
      feedLst.setVisibleRowCount(5);
      scrollPane3.setViewportView(feedLst);
    }
    add(scrollPane3, cc.xywh(1, 3, 3, 1));
    add(separator1, cc.xywh(1, 5, 3, 1));

    //---- label1 ----
    label1.setText("Double-Click to add the variable as a formula");
    add(label1, cc.xywh(1, 7, 3, 1));

    //======== scrollPane1 ========
    {

      //---- varLst ----
      varLst.setVisibleRowCount(5);
      scrollPane1.setViewportView(varLst);
    }
    add(scrollPane1, cc.xywh(1, 9, 3, 1));
    add(separator2, cc.xywh(1, 11, 3, 1));

    //======== toolBar1 ========
    {
      toolBar1.setFloatable(false);

      //---- deleteBtn ----
      deleteBtn.setText("D");
      toolBar1.add(deleteBtn);

      //---- mapBtn ----
      mapBtn.setText("M");
      toolBar1.add(mapBtn);

      //---- map3DBtn ----
      map3DBtn.setText("M3D");
      toolBar1.add(map3DBtn);

      //---- tsBtn ----
      tsBtn.setText("TS");
      toolBar1.add(tsBtn);

      //---- tableBtn ----
      tableBtn.setText("DT");
      toolBar1.add(tableBtn);

      //---- plot3DBtn ----
      plot3DBtn.setText("3D");
      toolBar1.add(plot3DBtn);

      //---- scatterPlotBtn ----
      scatterPlotBtn.setText("SP");
      toolBar1.add(scatterPlotBtn);

      //---- bubbleBtn ----
      bubbleBtn.setText("B");
      toolBar1.add(bubbleBtn);
    }
    add(toolBar1, cc.xywh(1, 13, 3, 1));

    //======== scrollPane2 ========
    {

      //---- formulaLst ----
      formulaLst.setVisibleRowCount(5);
      scrollPane2.setViewportView(formulaLst);
    }
    add(scrollPane2, cc.xywh(1, 15, 3, 1));
    add(formulaFld, cc.xy(1, 17));

    //---- addBtn ----
    addBtn.setText("Add");
    add(addBtn, cc.xy(3, 17));
    // JFormDesigner - End of component initialization //GEN-END:initComponents
  }

  // JFormDesigner - Variables declaration - DO NOT MODIFY //GEN-BEGIN:variables
  // Generated using JFormDesigner non-commercial license
  private JComponent separator3;
  private JScrollPane scrollPane3;
  private JList feedLst;
  private JComponent separator1;
  private JLabel label1;
  private JScrollPane scrollPane1;
  private JList varLst;
  private JComponent separator2;
  private JToolBar toolBar1;
  private JButton deleteBtn;
  private JButton mapBtn;
  private JButton map3DBtn;
  private JButton tsBtn;
  private JButton tableBtn;
  private JButton plot3DBtn;
  private JButton scatterPlotBtn;
  private JButton bubbleBtn;
  private JScrollPane scrollPane2;
  private JList formulaLst;
  private JTextField formulaFld;
  private JButton addBtn;
  // JFormDesigner - End of variables declaration //GEN-END:variables
}
