package anl.aida.client.gui;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import javax.swing.JTextArea;
import javax.swing.SwingWorker;

import anl.aida.client.Cache;
import anl.aida.client.util.Emailer;
import anl.aida.query.Trigger;
import anl.aida.query.TriggerResult;
import anl.aida.query.TriggerUpdateListener;
import anl.aida.util.DateRange;

public class TriggerRunnerWorker extends SwingWorker<TriggerResult, DateRange> {

  private static DateFormat format = new SimpleDateFormat("MM/dd/yyyy");

  private Trigger trigger;
  private Cache cache;
  private JTextArea resultsArea;

  /**
   * @param trigger
   * @param cache
   * @param area
   */
  public TriggerRunnerWorker(Trigger trigger, Cache cache, JTextArea area) {
    this.trigger = trigger;
    this.cache = cache;
    resultsArea = area;
  }

  /*
   * (non-Javadoc)
   * 
   * @see javax.swing.SwingWorker#doInBackground()
   */
  @Override
  protected TriggerResult doInBackground() throws Exception {

    return trigger.run(cache, new TriggerUpdateListener() {
      @Override
      public void rangeProcessing(DateRange range) {
	publish(range);
      }
    });
  }

  private String formatDateRange(DateRange range) {
    return format.format(range.getStart()) + " - " + format.format(range.getEnd());
  }

  @Override
  protected void process(List<DateRange> chunks) {
    for (DateRange chunk : chunks) {
      resultsArea.append("\nQuerying time slice: " + formatDateRange(chunk) + " ... ");
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see javax.swing.SwingWorker#done()
   */
  @Override
  protected void done() {
    try {
      TriggerResult result = get();
      if (result.isTriggered()) {
	resultsArea.append("\n\nAlert was TRIGGERED for the following time slices: ");
	StringBuilder buf = new StringBuilder("A CDPH MIFS Alert was triggered.\n");
	buf.append("Alert Name: ");
	buf.append(trigger.getName());
	buf.append("\nQuery: ");
	buf.append(trigger.getQuery());
	buf.append("\nRelevant Time Ranges: ");
	for (DateRange range : result.ranges()) {
	  String rangeStr = "\n  " + formatDateRange(range);
	  resultsArea.append(rangeStr);
	  buf.append(rangeStr);
	}
	buf.append("\n\nSee MIFS Client for additional information.");
	
	if (trigger.getEmail().contains("@")) {
	  resultsArea.append("\n\nSending alert email(s) to: " + trigger.getEmail());
	  Emailer emailer = new Emailer();
	  List<String> to = new ArrayList<String>();
	  StringTokenizer tok = new StringTokenizer(trigger.getEmail(), ",");
	  while (tok.hasMoreTokens()) {
	    to.add(tok.nextToken());
	  }
	  emailer.send(buf.toString(), to.toArray(new String[to.size()]), trigger.getName());
	  resultsArea.append("\nEmail sent.");
	} else {
	  resultsArea.append("\n\nInvalid email address -- email not sent.");
	}
      } else {
	resultsArea.append("\nAlert was NOT triggered");
      }
    } catch (Exception ex) {
      // resultsArea.setForeground(Color.RED);
      resultsArea.append("\n\nError while testing alert: " + ex.getMessage());
      ex.printStackTrace();
    }
  }

}
