package anl.aida.client.net;

import anl.aida.core.DocumentID;

import java.util.Collection;
import java.util.Set;

/**
 * Default implementation of CGEdge.
 */
public class DefaultCGEdge implements CGEdge {

  private String label;
  private Set<DocumentID> docs;

  public DefaultCGEdge(String label, Set<DocumentID> docs) {
    this.label = label;
    this.docs = docs;
  }

  /**
   * Gets an iterable over the docs associated with this
   * edge.
   *
   * @return an iterable over the docs associated with this
   *         edge.
   */
  public Iterable<DocumentID> documents() {
    return docs;
  }

  /**
   * Gets the number of documents associated with this edge.
   *
   * @return the number of documents associated with this edge.
   */
  public int getDocumentCount() {
    return docs.size();
  }

  /**
   * Gets the documents contained by this container and
   * puts them in the specified collection.
   *
   * @param docs the collection to add the documents to
   */
  public void getDocuments(Collection<DocumentID> docs) {
    docs.addAll(this.docs);
  }

  /**
   * Gets a label for this edge.
   *
   * @return a label for this edge.
   */
  public String getLabel() {
    return label;
  }
}
