package anl.aida.client.net;

import anl.aida.core.DocumentID;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.util.EdgeType;

import java.util.HashSet;
import java.util.Set;

/**
 * Creates an edge between two term nodes if the
 * number of documents that occur in both nodes is
 * greater than or equal to some threshold.
 *
 */
public class OccurrenceEdgeCreator implements EdgeCreator {

  private int threshold;

  /**
   * Creates an OccurenceEdgeCreator with the specified threshold.
   *
   * @param threshold the threshold for document co-occurence that triggers edge creation
   */
  public OccurrenceEdgeCreator(int threshold) {
    this.threshold = threshold;
  }

  /**
   * Creates an edge between the two nodes if the
   * number of documents that occur in both nodes is
   * greater than or equal to the threshold.
   *
   * @param node1 the first node
   * @param node2 the second node
   * @param graph the graph where the edge will be created
   * @return boolean if the edge created, otherwise false.
   */
  public boolean createEdge(TermNode node1, TermNode node2, Graph<TermNode, CGEdge> graph) {
    Set<DocumentID> set1 = node1.documents();
    Set<DocumentID> set2 = node2.documents();
    
    if (set1.size() < 1 || set2.size() < 1) return false;

    Set<DocumentID> docs = new HashSet<DocumentID>();

    // get the intersection of the two sets
    for (DocumentID id : set1) {
      if (set2.contains(id)) {
        docs.add(id);
      }
    }

    if (docs.size() >= threshold) {
      graph.addEdge(new DefaultCGEdge("A label", docs), node1, node2, EdgeType.UNDIRECTED);
      return true;
    }

    return false;
  }
}
