package anl.aida.client.net;

import anl.aida.client.CacheElement;
import anl.aida.score.Scorer;
import anl.aida.score.ScorerThresholdFilter;
import anl.aida.score.TFIDFScorer;
import anl.aida.score.TermScoreFilter;

/**
 * Creates a ScorerThresholdFilter that filters based on
 * tf/idf score being >= to some threshold.
 *
 */
public class TFIDFFilterCreator implements TSFilterCreator {

  /**
   * Gets a label for this creator.
   *
   * @return
   */
  public String getLabel() {
    return "TF/IDF Score";
  }

  /**
   * Creates a TermScoreFilter using the element and val.
   *
   * @param element the element to use -- getSTC etc from
   * @param val     a value to use (e.g. threshold).
   * @return the created TermScoreFilter
   */
  public TermScoreFilter createFilter(CacheElement element, float val) {
    Scorer scorer = new TFIDFScorer(element.getSTC());
    return new ScorerThresholdFilter(scorer, val);
  }
}