package anl.aida.client.net;

import anl.aida.client.doc.DocumentContainer;
import anl.aida.core.DocumentID;

import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

/**
 * A node in a concept graph.
 *
 */
public class TermNode  extends ValueNode implements DocumentContainer {

  private String term;
  private Set<DocumentID> docs = new HashSet<DocumentID>();

  public TermNode(String term) {
    super(0, term);
    this.term = term;
  }

  /**
   * Gets the number of documents associated with this node.
   *
   * @return the number of documents associated with this node.
   */
  public int getDocumentCount() {
    return docs.size();
  }

  /**
   * Adds a document to this node.
   *
   * @param doc the doc id of the document to add.
   */
  public void addDocument(DocumentID doc) {
    docs.add(doc);
  }

  @Override
  public double getValue() {
    return getDocumentCount();
  }

  /**
   * Gets the documents associated with this node.
   *
   * @return the documents associated with this node.
   */
  public Set<DocumentID> documents() {
    return Collections.unmodifiableSet(docs);
  }

  /**
   * Gets the documents contained by this container and
   * puts them in the specified collection.
   *
   * @param docs the collection to add the documents to
   */
  public void getDocuments(Collection<DocumentID> docs) {
    docs.addAll(this.docs);
  }

  /**
   * Gets the term associated with this TermNode.
   *
   * @return the term associated with this TermNode.
   */
  public String getLabel() {
    return term;
  }
}
