package anl.aida.client.search;

import anl.aida.ae.lucene.LuceneConstants;
import anl.aida.core.DocumentID;

import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.Term;
import org.apache.lucene.index.TermDocs;
import org.apache.lucene.search.DocIdSet;
import org.apache.lucene.search.Filter;
import org.apache.lucene.util.OpenBitSet;

import java.io.IOException;
import java.util.List;

/**
 * Lucene filter that filters based on document ids passed in on the constructor.
 *
 */
public class PathFilter extends Filter {

  private List<DocumentID> ids;

  public PathFilter(List<DocumentID> ids) {
    this.ids = ids;
  }

  @Override
  public DocIdSet getDocIdSet(IndexReader reader) throws IOException {
    OpenBitSet bitSet = new OpenBitSet(reader.maxDoc());
    for (DocumentID id : ids) {
      Term term = new Term(LuceneConstants.PATH_FIELD, id.getPath());
      TermDocs docs = reader.termDocs(term);
      while (docs.next()) {
        bitSet.set(docs.doc());
      }
    }

    return bitSet;
  }
}
