package anl.aida.client.util;

import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;

import javax.swing.Icon;
import javax.swing.ImageIcon;

/**
 * Utility methods used by the client.
 * 
 */
public class ClientUtils {

  private static NumberFormat nFormat = DecimalFormat.getInstance();
  private static NumberFormat eFormat = new DecimalFormat("0.###E0");
  
  private static SimpleDateFormat shortFormat = new SimpleDateFormat("MM/dd/yyyy");
  private static SimpleDateFormat longFormat = new SimpleDateFormat("MMMMM dd, yyyy HH:mm:ss z");
  private static DateFormat overlayFormat = new SimpleDateFormat("MMMMM dd, yyyy");

  static {
    nFormat.setMaximumFractionDigits(3);
  }

  public static String formatDate(Date date) {
    longFormat.setTimeZone(TimeZone.getTimeZone("UTC"));
    return longFormat.format(date);
  }

  public static String formatShortDate(Date date) {
    shortFormat.setTimeZone(TimeZone.getTimeZone("UTC"));
    return shortFormat.format(date);
  }
  
  public static String formatOverlayDate(Date date) {
    return overlayFormat.format(date);
  }

  public static String formatNumber(double val) {
    if (val < .001 && val != 0)
      return eFormat.format(val);

    return nFormat.format(val);
  }

  /**
   * Attemps to load an icon as resource and if that fails then directly from
   * the "icons" directory.
   * 
   * @param iconFile
   *          the icon file name
   * @return the loaded icon
   */
  public static Icon loadIcon(String iconFile) {
    // a specific classloader that may not be there in testing.
    Icon icon;
    try {
      icon = new ImageIcon(ClientUtils.class.getClassLoader().getResource(iconFile));
    } catch (Exception ex) {
      icon = new ImageIcon("./icons/" + iconFile);
    }

    return icon;
  }
}
