package anl.aida.client.widget;

import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;
import javax.swing.Timer;

import anl.aida.client.gui.AnimationConfigDialog;
import anl.aida.client.util.ClientUtils;
import anl.aida.util.DateRangeProducer;

/**
 * Handles animating a network display.
 * 
 */
public class Animator {
  
  
  
private class ReverseTimerAction implements ActionListener {
    
    private int index = 0, interval, minVal, indexIncrement;
    
    public ReverseTimerAction(AnimatorPropertyBean bean) {
      index = bean.getMaxValue();
      interval = bean.getInterval();
      minVal = bean.getMinValue();
      indexIncrement = bean.getIndexIncrement();
    }

    @Override
    public void actionPerformed(ActionEvent e) {
      if (index - interval < minVal) {
	stop();
      } else {
	slider.setRange(index - interval, index);
      }
      index-= indexIncrement;
    }
  }
  
  private class TimerAction implements ActionListener {
    
    private int index = 0, interval, maxVal, indexIncrement;
    
    public TimerAction(AnimatorPropertyBean bean) {
      index = bean.getMinValue();
      interval = bean.getInterval();
      maxVal = bean.getMaxValue();
      indexIncrement = bean.getIndexIncrement();
    }

    @Override
    public void actionPerformed(ActionEvent e) {
      if (index + interval > maxVal) {
	stop();
      } else {
	slider.setRange(index, index + interval);
      }
      index+= indexIncrement;
    }
  }
  
  private static final String PLAY_ICON = "player_play.png";
  private static final String PAUSE_ICON = "player_pause.png";
  private static final String STOP_ICON = "player_stop.png";
  private static final String CONFIG_ICON = "configure.png";
  
  private NetworkTimeSlider2 slider;
  private boolean playing = false, runInReverse;
  private JButton configureBtn, playBtn, stopBtn;
  private Icon playIcon, pauseIcon;
  private Timer timer;
  private AnimatorPropertyBean bean;

  public Animator(NetworkTimeSlider2 slider) {
    this(slider, false);
  }
  
  public Animator(NetworkTimeSlider2 slider, boolean runInReverse) {
    this.slider = slider;
    configButtons();
  }
  
  public boolean isRunInReverse() {
    return runInReverse;
  }

  public void setRunInReverse(boolean runInReverse) {
    this.runInReverse = runInReverse;
  }

  private void configButtons() {
    
    configureBtn = new JButton();
    configureBtn.setIcon(ClientUtils.loadIcon(CONFIG_ICON));
    configureBtn.setBorderPainted(false);
    configureBtn.setToolTipText("Configure the semantic map animation");
    configureBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	config();
      }
    });
    
    pauseIcon = ClientUtils.loadIcon(PAUSE_ICON);
    playIcon = ClientUtils.loadIcon(PLAY_ICON);
    playBtn = new JButton(playIcon);
    playBtn.setToolTipText("Animate the semantic map over time");
    playBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	if (playing) pause();
	else play();
      }
    });
    playBtn.setBorderPainted(false);
    
    stopBtn = new JButton(ClientUtils.loadIcon(STOP_ICON));
    stopBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	stop();
      }
    });
    stopBtn.setToolTipText("Stop the semantic map animation");
    stopBtn.setBorderPainted(false);
    stopBtn.setEnabled(false);
  }
  
  private void configTimer() {
    int delay = 1000 / bean.getFramesPer();
    if (runInReverse) {
    timer = new Timer(0, new ReverseTimerAction(bean));
    } else {
      timer = new Timer(0, new TimerAction(bean));
    }
    timer.setDelay(delay);
  }
  
  private void config() {
    AnimationConfigDialog dialog;
    Window w = SwingUtilities.getWindowAncestor(configureBtn);
    if (w instanceof Frame) {
      dialog = new AnimationConfigDialog((Frame) w);
    } else {
      dialog = new AnimationConfigDialog((Dialog) w);
    }
    dialog.init(bean);
    dialog.pack();
    Dimension d = dialog.getSize();
    dialog.setSize(d.width + 100, d.height);
    dialog.setVisible(true);
    AnimatorPropertyBean bean = dialog.getProperties();
    if (bean != null) {
      this.bean = bean;
    }
  }
  
  public void resetDefaultRanger(DateRangeProducer ranger) {
    bean = new AnimatorPropertyBean(ranger);
  }
  
  public void addToToolBar(JToolBar bar) {
    bar.addSeparator();
    bar.add(configureBtn);
    bar.add(playBtn);
    bar.add(stopBtn);
  }
  
  private void play() {
    playing = true;
    configureBtn.setEnabled(false);
    playBtn.setToolTipText("Pause the semantic map animation");
    playBtn.setIcon(pauseIcon);
    if (!stopBtn.isEnabled()) {
      configTimer();
    }
    stopBtn.setEnabled(true);
    timer.start();
  }
  
  private void pause() {
    playing = false;
    playBtn.setToolTipText("Start the semantic map animation");
    playBtn.setIcon(playIcon);
    timer.stop();
  }
  
  private void stop() {
    playing = false;
    timer.stop();
    playBtn.setToolTipText("Animate the semantic map over time");
    playBtn.setIcon(playIcon);
    stopBtn.setEnabled(false);
    configureBtn.setEnabled(true);
  }
 
  
  

}
