package anl.aida.client.widget;

import com.l2fprod.common.swing.JTaskPaneGroup;
import com.l2fprod.common.swing.plaf.basic.BasicTaskPaneGroupUI;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.event.MouseInputAdapter;
import javax.swing.event.MouseInputListener;
import javax.swing.plaf.ComponentUI;
import java.awt.*;
import java.awt.event.MouseEvent;

/**
 *         Date: Feb 6, 2009 11:06:40 AM
 */
public class MTaskPaneGroupUI extends BasicTaskPaneGroupUI {

  private JCheckBox box = new JCheckBox();

  public static ComponentUI createUI(JComponent c) {
    return new MTaskPaneGroupUI();
  }

  protected Border createPaneBorder() {
    return new TaskPaneBorder();
  }

  protected MouseInputListener createMouseInputListener() {
    return new ToggleListener();
  }

  protected boolean isInBorder(MouseEvent event) {
    return event.getY() < getTitleHeight() && event.getX() > box.getWidth();
  }

  class ToggleListener extends MouseInputAdapter {

    public void mouseEntered(MouseEvent e) {

      if (isInBorder(e)) {
        mouseOver = true;
        if (e.getSource() != null) {
          MTaskPaneGroup grp = (MTaskPaneGroup) e.getSource();
          grp.setToolTipText(grp.getTitle());
        }
      } else {
        if (e.getSource() != null) {
          MTaskPaneGroup grp = (MTaskPaneGroup) e.getSource();
          grp.setToolTipText(null);
        }
        mouseOver = false;
        group.repaint();
      }
    }

    public void mouseExited(MouseEvent e) {
      e.getComponent().setCursor(null);
      mouseOver = false;
      group.repaint();
    }

    public void mouseMoved(MouseEvent e) {
      if (isInBorder(e) && group.isCollapsable()) {
        mouseOver = true;
        group.repaint();
      } else {
        e.getComponent().setCursor(null);
        mouseOver = false;
        group.repaint();
      }
    }

    public void mouseReleased(MouseEvent e) {
      if (isInBorder(e) && group.isCollapsable() && e.getButton() == MouseEvent.BUTTON1) {
        group.setExpanded(!group.isExpanded());
      }
    }

    public void mouseClicked(MouseEvent e) {
      int start = box.getLocation().x;
      int width = box.getWidth();
      if (e.getX() >= start && e.getX() <= width) {
        MTaskPaneGroup grp = (MTaskPaneGroup) group;
        grp.setSelected(!grp.isSelected());
      }
    }
  }

  /**
   * Overriden to paint the background of the component but keeping the rounded
   * corners.
   */
  public void update(Graphics g, JComponent c) {
    if (c.isOpaque()) {
      g.setColor(c.getParent().getBackground());
      g.fillRect(0, 0, c.getWidth(), c.getHeight());
      g.setColor(c.getBackground());
      g.fillRect(0, ROUND_HEIGHT, c.getWidth(), c.getHeight() - ROUND_HEIGHT);
    }
    paint(g, c);
  }

  /**
   * The border of the taskpane group paints the "text", the "icon", the
   * "expanded" status and the "special" type.
   */
  class TaskPaneBorder extends PaneBorder {

    TaskPaneBorder() {
      super();
      box.setOpaque(false);
    }

    protected void paintTitleBackground(JTaskPaneGroup group, Graphics g) {

      Paint oldPaint = ((Graphics2D) g).getPaint();
      Paint paint = group.getParent().getBackground();
      /*
      GradientPaint gradient =
              new GradientPaint(
                      0f,
                      0f, //group.getWidth() / 2,
                      titleBackgroundGradientStart,
                      0f, //group.getWidth(),
                      TITLE_HEIGHT,
                      titleBackgroundGradientEnd);
      */

      ((Graphics2D) g).setRenderingHint(
              RenderingHints.KEY_COLOR_RENDERING,
              RenderingHints.VALUE_COLOR_RENDER_QUALITY);
      ((Graphics2D) g).setRenderingHint(
              RenderingHints.KEY_INTERPOLATION,
              RenderingHints.VALUE_INTERPOLATION_BILINEAR);
      ((Graphics2D) g).setRenderingHint(
              RenderingHints.KEY_RENDERING,
              RenderingHints.VALUE_RENDER_QUALITY);

      //((Graphics2D) g).setPaint(gradient);
      ((Graphics2D) g).setPaint(paint);

      /*
      g.fillRoundRect(
              0,
              0,
              group.getWidth(),
              ROUND_HEIGHT * 2,
              ROUND_HEIGHT,
              ROUND_HEIGHT);
              */
      g.fillRect(
              0,
              ROUND_HEIGHT,
              group.getWidth(),
              TITLE_HEIGHT - ROUND_HEIGHT);


      ((Graphics2D) g).setPaint(oldPaint);


      Shape oldClip = g.getClip();
      Rectangle clip = new Rectangle(0, 0, group.getWidth(), TITLE_HEIGHT);
      g.setClip(clip.intersection(oldClip.getBounds()));
      g.setColor(borderColor);
      g.drawRoundRect(
              0,
              0,
              group.getWidth() - 1,
              TITLE_HEIGHT + ROUND_HEIGHT,
              ROUND_HEIGHT,
              ROUND_HEIGHT);
      g.drawLine(0, TITLE_HEIGHT - 1, group.getWidth(), TITLE_HEIGHT - 1);
      g.setClip(oldClip);
    }

    protected void paintTitle(
            JTaskPaneGroup group,
            Graphics g,
            Color textColor,
            int x,
            int y,
            int width,
            int height) {

      configureBox(group);
      configureLabel(group);
      box.setForeground(textColor);
      g.translate(x, y);
      int boxWidth = box.getPreferredSize().width;
      box.setBounds(0, 0, boxWidth, height);
      box.paint(g);

      label.setForeground(textColor);
      g.translate(boxWidth, 0);
      label.setBounds(0, 0, width - 30, height);
      label.paint(g);

      g.translate(-(x + boxWidth), -y);
    }

    protected void configureBox(JTaskPaneGroup group) {
      box.applyComponentOrientation(group.getComponentOrientation());
      box.setSelected(((MTaskPaneGroup) group).isSelected());
    }


    protected void paintExpandedControls(JTaskPaneGroup group, Graphics g, int x,
                                         int y, int width, int height) {
      ((Graphics2D) g).setRenderingHint(
              RenderingHints.KEY_ANTIALIASING,
              RenderingHints.VALUE_ANTIALIAS_ON);

      paintOvalAroundControls(group, g, x, y, width, height);
      g.setColor(getPaintColor(group));
      paintChevronControls(group, g, x, y, width, height);

      ((Graphics2D) g).setRenderingHint(
              RenderingHints.KEY_ANTIALIASING,
              RenderingHints.VALUE_ANTIALIAS_OFF);
    }

    protected boolean isMouseOverBorder() {
      return true;
    }

  }

}


