/*
 * Created by JFormDesigner on Thu Mar 12 11:27:25 EDT 2009
 */

package anl.aida.client.widget;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.EtchedBorder;

import simphony.util.messages.MessageCenter;
import anl.aida.client.net.ConceptGraph;
import anl.aida.client.net.TSConceptGraphs;
import anl.aida.client.net.TermNode;
import anl.aida.client.util.ClientUtils;
import anl.aida.util.DateRange;
import anl.aida.util.DateRangeProducer;
import anl.aida.viz.NetworkDisplay;

import com.jgoodies.forms.factories.FormFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.ColumnSpec;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.layout.FormSpec;
import com.jgoodies.forms.layout.RowSpec;
import com.jgoodies.forms.layout.Sizes;

/**
 * @author User #1
 */
@SuppressWarnings("serial")
public class NetworkTimeSlider2 extends JPanel {

  private static final String FWD_ICON = "agt_forward.png";
  private static final String BACK_ICON = "agt_back.png";
  
  private DateFormat shortFormat = new SimpleDateFormat("MMM. dd, yy");
  private Calendar cal = GregorianCalendar.getInstance();

  private TSConceptGraphs graphs;
  private GraphSwapable graphSwapable;
  
  private int minVal, maxVal;
  private Date minDate, maxDate;
  private boolean triggerOther, locked, swapGraph;
  private DateRange currentRange = new DateRange(new Date(), new Date());
 
  private Animator animator;
 
  public NetworkTimeSlider2() {
    initComponents();
    loadIcons();
    animator = new Animator(this);
    
    triggerOther = true;
    locked = false;
    slider.setMinimum(0);
    slider.setPaintLabels(false);
    slider.setValue(0);
    slider.setEnabled(false);

    lockedBox.setSelected(true);
    lockedBox.setEnabled(false);
    
    prevBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	if (minVal > 0) slider.setSecondValue(slider.getSecondValue() - 1);
      }
    });
    
    nextBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	if (maxVal < graphs.getRanger().getTotalDays()) slider.setValue(slider.getValue() + 1);
      }
    });

    slider.addRangeChangedListener(new RangeChangedListener() {
      public void rangeChanged(RangeChangeEvent evt) {

        int firstVal = slider.getFirstValue();
        DateRangeProducer ranger = graphs.getRanger();

        if (evt.isFirstChanged() && maxVal != firstVal) {
          if (locked && triggerOther) {
            triggerOther = false;
            // increment the min value by the amt the first
            // value has changed -- i.e. the diff between val
            // and max val.
            slider.setSecondValue(slider.getSecondValue() + (firstVal - maxVal));
            triggerOther = true;
          }

          maxVal = firstVal;
          // make sure that the max is always greater than the min
          // by the range length
          if (firstVal - ranger.getRangeLength() < slider.getSecondValue()) {
            slider.setFirstValue(slider.getSecondValue() + ranger.getRangeLength());
          } else {
            maxDate = calcDateFromStart(firstVal);
          }
        }
        fireRangeChanged();
      }
    });

    slider.addRangeChangedListener(new RangeChangedListener() {
      public void rangeChanged(RangeChangeEvent evt) {

        int secondVal = slider.getSecondValue();
        DateRangeProducer ranger = graphs.getRanger();

        if (evt.isSecondChanged() && minVal != secondVal) {
          if (locked && triggerOther) {
            triggerOther = false;
            slider.setValue(slider.getFirstValue() + secondVal - minVal);
            triggerOther = true;
          }
          minVal = secondVal;

          if (secondVal + ranger.getRangeLength() > slider.getFirstValue()) {
            slider.setSecondValue(slider.getFirstValue() - ranger.getRangeLength());
          } else {
            minDate = calcDateFromStart(secondVal);
          }
        }

        fireRangeChanged();
      }
    });

    lockedBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	DateRangeProducer ranger = graphs.getRanger();
        if (!locked && slider.getFirstValue() - slider.getSecondValue() != ranger.getRangeLength()) {
          // about to be locked so set the slider interval appropriately
          slider.setSecondValue(slider.getFirstValue() - ranger.getRangeLength());
        }
        locked = lockedBox.isSelected();
      }
    });
  }
  
  private void fireRangeChanged() {
    boolean isAdjusting = slider.getValueIsAdjusting() || (!triggerOther);
    rangeLbl.setText("Current Slice Range: " + shortFormat.format(minDate) + "  through  " + 
	shortFormat.format(maxDate));
    prevBtn.setEnabled(minVal > 0);
    nextBtn.setEnabled(maxVal < graphs.getRanger().getTotalDays());
    
    DateRange range = new DateRange(minDate, maxDate);
    if (!isAdjusting && swapGraph && !range.equals(currentRange)) {
      currentRange = range;
      ConceptGraph<TermNode> graph = graphs.getGraph(currentRange);
      if (graph != null) graphSwapable.swapGraph(graph, currentRange);
      else {
	MessageCenter.getMessageCenter(getClass()).info("Synthetic range requested -- not yet implemented");
      }
    }
  }
  
  private Date calcDateFromStart(int days) {
    DateRangeProducer ranger = graphs.getRanger();
    cal.setTime(ranger.getStart());
    cal.add(Calendar.DAY_OF_MONTH, days);
    Date date = cal.getTime();
    cal.setTime(ranger.getStart());
    return date;
  }
  
  public DateRange getRange() {
    return new DateRange(minDate, maxDate);
  }
  
  public void setRange(int minValue, int maxValue) {
    slider.setValue(maxValue);
    if (!locked) {
      slider.setSecondValue(minValue);
    }
  }
  
  public void init(GraphSwapable graphSwapable, TSConceptGraphs graphs, DateRange startRange) {
    swapGraph = false;
    animator.resetDefaultRanger(graphs.getRanger());
    this.graphs = graphs;
    this.graphSwapable = graphSwapable;
    rangeLbl.setText(startRange.toString());
    resetSlider(startRange);
    swapGraph = true;
  }
  
  private void resetSlider(DateRange range) {
    DateRangeProducer ranger = graphs.getRanger();
    int totalDays = ranger.getTotalDays();
    minDate = range.getStart();
    maxDate = range.getEnd();
    
    currentRange = new DateRange(minDate, maxDate);
    
    fireRangeChanged();
    
    cal.setTime(minDate);
    int minMonth = cal.get(Calendar.MONTH);
    int minDay= cal.get(Calendar.DAY_OF_MONTH);
    
    cal.setTime(maxDate);
    int maxMonth = cal.get(Calendar.MONTH);
    int maxDay= cal.get(Calendar.DAY_OF_MONTH);
    
    cal.setTime(ranger.getStart());
    for (int i = 0; i <= totalDays; i++) {
      int day = cal.get(Calendar.DAY_OF_MONTH);
      int month = cal.get(Calendar.MONTH);
      if (day == minDay && month == minMonth) minVal = i;
      if (day == maxDay && month == maxMonth) maxVal = i;
      cal.add(Calendar.DAY_OF_MONTH, 1);
    }

    TimeSliderInitializer initter = new TimeSliderInitializer(ranger, shortFormat);
    initter.initSlider(slider, minVal, maxVal, false);
    locked = true;
    initter.initBox(lockedBox, locked);
    
    cal.setTime(ranger.getStart());
  }

  private void loadIcons() {
    prevBtn.setIcon(ClientUtils.loadIcon(BACK_ICON));
    nextBtn.setIcon(ClientUtils.loadIcon(FWD_ICON));
   
    nextBtn.setText("");
    prevBtn.setText("");
  }
  
  /**
   * Gets the animator that can be used to animate this
   * time slider.
   * 
   * @return the animator that can be used to animate this
   * time slider.
   */
  public Animator getAnimator() {
    return animator;
  }

  private void initComponents() {
    // JFormDesigner - Component initialization - DO NOT MODIFY
    // //GEN-BEGIN:initComponents
	// Generated using JFormDesigner non-commercial license
	panel1 = new JPanel();
	rangeLbl = new JLabel();
	prevBtn = new JButton();
	slider = new RangeSlider();
	nextBtn = new JButton();
	lockedBox = new JCheckBox();
	CellConstraints cc = new CellConstraints();

	//======== this ========
	setBorder(new EtchedBorder());
	setLayout(new FormLayout(
		new ColumnSpec[] {
			FormFactory.DEFAULT_COLSPEC,
			FormFactory.LABEL_COMPONENT_GAP_COLSPEC,
			new ColumnSpec(ColumnSpec.FILL, Sizes.DEFAULT, FormSpec.DEFAULT_GROW),
			FormFactory.LABEL_COMPONENT_GAP_COLSPEC,
			FormFactory.DEFAULT_COLSPEC,
			FormFactory.LABEL_COMPONENT_GAP_COLSPEC,
			FormFactory.DEFAULT_COLSPEC
		},
		new RowSpec[] {
			FormFactory.DEFAULT_ROWSPEC,
			FormFactory.LINE_GAP_ROWSPEC,
			FormFactory.DEFAULT_ROWSPEC
		}));

	//======== panel1 ========
	{
		panel1.setLayout(new FlowLayout(FlowLayout.LEFT));

		//---- rangeLbl ----
		rangeLbl.setText("text");
		panel1.add(rangeLbl);
	}
	add(panel1, cc.xy(3, 1));

	//---- prevBtn ----
	prevBtn.setText("prev");
	add(prevBtn, cc.xy(1, 3));
	add(slider, cc.xy(3, 3));

	//---- nextBtn ----
	nextBtn.setText("next");
	add(nextBtn, cc.xy(5, 3));

	//---- lockedBox ----
	lockedBox.setText("Locked");
	add(lockedBox, cc.xy(7, 3));
    // JFormDesigner - End of component initialization //GEN-END:initComponents
  }

  // JFormDesigner - Variables declaration - DO NOT MODIFY //GEN-BEGIN:variables
  // Generated using JFormDesigner non-commercial license
  private JPanel panel1;
  private JLabel rangeLbl;
  private JButton prevBtn;
  private RangeSlider slider;
  private JButton nextBtn;
  private JCheckBox lockedBox;
  // JFormDesigner - End of variables declaration //GEN-END:variables
}
