package anl.aida.client.widget;

import javax.accessibility.AccessibleContext;
import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.util.ArrayList;
import java.util.List;

/**
 *
 */
public class RangeSlider extends JSlider {

  private BoundedRangeModel otherModel;
  private boolean updateUI = false;
  private int cFirst, cSecond;

  private List<RangeChangedListener> listeners = new ArrayList<RangeChangedListener>();

  public RangeSlider() {
    this(0, 100);
  }

  public RangeSlider(int min, int max) {
    super(min, max);
    otherModel = new DefaultBoundedRangeModel(min, 0, min, max);
    updateUI = true;
    updateUI();
    otherModel.addChangeListener(createChangeListener());

    cFirst = getValue();
    cSecond = getSecondValue();

    getModel().addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        fireRangeChanged(true, false);
      }
    });

    otherModel.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        fireRangeChanged(false, true);
      }
    });
  }

  public int getFirstValue() {
    return getValue();
  }

  public void setFirstValue(int firstValue) {
    setValue(firstValue);
  }

  public void addRangeChangedListener(RangeChangedListener listener) {
    listeners.add(listener);
  }

  private void fireRangeChanged(boolean firstChanged, boolean secondChanged) {
    RangeChangeEvent evt = new RangeChangeEvent(this, firstChanged, secondChanged);
    for (RangeChangedListener listener : listeners) {
      listener.rangeChanged(evt);
    }
  }
  
  @Override
  public void setMaximum(int maximum) {
    super.setMaximum(maximum);
    otherModel.setMaximum(maximum);
  }

  @Override
  public void setMinimum(int minimum) {
    super.setMinimum(minimum);
    otherModel.setMinimum(minimum);
  }

  public int getSecondValue() {
    return otherModel.getValue();
  }

  public void setSecondValue(int value) {
    int oldValue = otherModel.getValue();
    if (oldValue == value) {
      return;
    }
    otherModel.setValue(value);

    if (accessibleContext != null) {
      accessibleContext.firePropertyChange(
              AccessibleContext.ACCESSIBLE_VALUE_PROPERTY,
              new Integer(oldValue),
              new Integer(otherModel.getValue()));
    }
  }

  public void updateUI() {
    if (updateUI) {
      setUI(RangeSliderUI.createUI(this));
      // The labels preferred size may be derived from the font
      // of the slider, so we must update the UI of the slider first, then
      // that of labels.  This way when setSize is called the right
      // font is used.
      updateLabelUIs();
    }
  }

  public BoundedRangeModel getSecondModel() {
    return otherModel;
  }
}
