package anl.aida.client.widget;

import javax.swing.*;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicSliderUI;
import javax.swing.plaf.metal.MetalLookAndFeel;
import javax.swing.plaf.metal.OceanTheme;
import java.awt.*;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

/**
 * Based on the MetalSliderUI.
 */
public class RangeSliderUI extends BasicSliderUI {

  protected final int TICK_BUFFER = 4;
  protected boolean filledSlider = false;
  // NOTE: these next three variables are currently unused.
  protected static Color thumbColor;
  protected static Color highlightColor;
  protected static Color darkShadowColor;
  protected static int trackWidth;
  protected static int tickLength;

  /**
   * A default horizontal thumb <code>Icon</code>. This field might not be
   * used. To change the <code>Icon</code> used by this delgate directly set it
   * using the <code>Slider.horizontalThumbIcon</code> UIManager property.
   */
  protected static Icon horizThumbIcon;

  /**
   * A default vertical thumb <code>Icon</code>. This field might not be
   * used. To change the <code>Icon</code> used by this delgate directly set it
   * using the <code>Slider.verticalThumbIcon</code> UIManager property.
   */
  protected static Icon vertThumbIcon;

  private static Icon SAFE_HORIZ_THUMB_ICON;
  private static Icon SAFE_VERT_THUMB_ICON;


  protected final String SLIDER_FILL = "JSlider.isFilled";
  private Color sliderAltTrackColor;

  private Rectangle otherThumbRect;
  private boolean isDragging;

  public static ComponentUI createUI(JComponent c) {
    return new RangeSliderUI();
  }

  public RangeSliderUI() {
    super(null);
  }

  private RangeSlider getSlider() {
    return (RangeSlider) slider;
  }

  private static Icon getHorizThumbIcon() {
    if (System.getSecurityManager() != null) {
      return SAFE_HORIZ_THUMB_ICON;
    } else {
      return horizThumbIcon;
    }
  }

  private static Icon getVertThumbIcon() {
    if (System.getSecurityManager() != null) {
      return SAFE_VERT_THUMB_ICON;
    } else {
      return vertThumbIcon;
    }
  }

  @Override
  protected void uninstallListeners(JSlider slider) {
    getSlider().getSecondModel().removeChangeListener(changeListener);
    super.uninstallListeners(slider);

  }

  @Override
  protected void installListeners(JSlider slider) {
    super.installListeners(slider);
    getSlider().getSecondModel().addChangeListener(changeListener);
  }

  public void installUI(JComponent c) {
    MetalLookAndFeel metal = new MetalLookAndFeel();
    MetalLookAndFeel.setCurrentTheme(new OceanTheme());

    UIDefaults defs = metal.getDefaults();
    UIManager.put("Slider.focusGradient", defs.get("Slider.focusGradient"));
    UIManager.put("Slider.gradient", defs.get("Slider.gradient"));
    trackWidth = ((Integer) defs.get("Slider.trackWidth")).intValue();
    tickLength = ((Integer) defs.get("Slider.majorTickLength")).intValue();
    horizThumbIcon = SAFE_HORIZ_THUMB_ICON =
            defs.getIcon("Slider.horizontalThumbIcon");
    vertThumbIcon = SAFE_VERT_THUMB_ICON =
            defs.getIcon("Slider.verticalThumbIcon");
    sliderAltTrackColor = (Color) defs.get("Slider.altTrackColor");
    otherThumbRect = new Rectangle();

    super.installUI(c);

    thumbColor = defs.getColor("Slider.thumb");
    highlightColor = defs.getColor("Slider.highlight");
    darkShadowColor = defs.getColor("Slider.darkShadow");

    scrollListener.setScrollByBlock(false);

    Object sliderFillProp = c.getClientProperty(SLIDER_FILL);
    if (sliderFillProp != null) {
      filledSlider = ((Boolean) sliderFillProp).booleanValue();
    }
  }

  public void paint(Graphics g, JComponent c) {
    super.paint(g, c);

    if (g.getClipBounds().intersects(otherThumbRect)) {
      paintThumb(g);
    }
  }

  protected void calculateThumbSize() {
    super.calculateThumbSize();
    Dimension size = getThumbSize();
    otherThumbRect.setSize(size.width, size.height);
  }

  private int getSnappedValue(int value) {
    int snappedValue = value;
    int majorTickSpacing = slider.getMajorTickSpacing();
    int minorTickSpacing = slider.getMinorTickSpacing();
    int tickSpacing = 0;

    if (minorTickSpacing > 0) {
      tickSpacing = minorTickSpacing;
    } else if (majorTickSpacing > 0) {
      tickSpacing = majorTickSpacing;
    }

    if (tickSpacing != 0) {
      // If it's not on a tick, change the value
      if ((value - slider.getMinimum()) % tickSpacing != 0) {
        float temp = (float) (value - slider.getMinimum()) / (float) tickSpacing;
        int whichTick = Math.round(temp);
        snappedValue = slider.getMinimum() + (whichTick * tickSpacing);
      }

      if (snappedValue != value) {
        return snappedValue;
      }
    }

    return -1;
  }

  protected void calculateThumbLocation() {
    if (slider.getSnapToTicks()) {
      int val = getSnappedValue(slider.getValue());
      if (val != -1) slider.setValue(val);

      val = getSnappedValue(getSlider().getSecondValue());
      if (val != -1) getSlider().setSecondValue(-1);
    }

    if (slider.getOrientation() == JSlider.HORIZONTAL) {
      int valuePosition = xPositionForValue(slider.getValue());

      thumbRect.x = valuePosition - (thumbRect.width / 2);
      thumbRect.y = trackRect.y;

      valuePosition = xPositionForValue(getSlider().getSecondValue());
      otherThumbRect.x = valuePosition - (otherThumbRect.width / 2);
      otherThumbRect.y = trackRect.y;
    } else {
      int valuePosition = yPositionForValue(slider.getValue());

      thumbRect.x = trackRect.x;
      thumbRect.y = valuePosition - (thumbRect.height / 2);

      valuePosition = yPositionForValue(getSlider().getSecondValue());
      otherThumbRect.y = valuePosition - (otherThumbRect.height / 2);
      otherThumbRect.x = trackRect.x;
    }
  }


  protected PropertyChangeListener createPropertyChangeListener(JSlider slider) {
    return new MetalPropertyListener();
  }

  protected class MetalPropertyListener extends PropertyChangeHandler {
    public void propertyChange(PropertyChangeEvent e) {  // listen for slider fill
      super.propertyChange(e);

      String name = e.getPropertyName();
      if (name.equals(SLIDER_FILL)) {
        if (e.getNewValue() != null) {
          filledSlider = ((Boolean) e.getNewValue()).booleanValue();
        } else {
          filledSlider = false;
        }
      }
    }
  }

  public void paintThumb(Graphics g) {
    Rectangle knobBounds = thumbRect;

    g.translate(knobBounds.x, knobBounds.y);

    if (slider.getOrientation() == JSlider.HORIZONTAL) {
      getHorizThumbIcon().paintIcon(slider, g, 0, 0);
    } else {
      getVertThumbIcon().paintIcon(slider, g, 0, 0);
    }

    g.translate(-knobBounds.x, -knobBounds.y);

    knobBounds = otherThumbRect;
    g.translate(knobBounds.x, knobBounds.y);

    if (slider.getOrientation() == JSlider.HORIZONTAL) {
      getHorizThumbIcon().paintIcon(slider, g, 0, 0);
    } else {
      getVertThumbIcon().paintIcon(slider, g, 0, 0);
    }
    g.translate(-knobBounds.x, -knobBounds.y);

  }

  /**
   * If <code>chooseFirst</code>is true, <code>c1</code> is returned,
   * otherwise <code>c2</code>.
   */
  private Color chooseColor(boolean chooseFirst, Color c1, Color c2) {
    if (chooseFirst) {
      return c2;
    }
    return c1;
  }

  static boolean isLeftToRight(Component c) {
    return c.getComponentOrientation().isLeftToRight();
  }


  /**
   * Returns a rectangle enclosing the track that will be painted.
   */
  private Rectangle getPaintTrackRect() {
    int trackLeft = 0, trackRight = 0, trackTop = 0, trackBottom = 0;
    if (slider.getOrientation() == JSlider.HORIZONTAL) {
      trackBottom = (trackRect.height - 1) - getThumbOverhang();
      trackTop = trackBottom - (getTrackWidth() - 1);
      trackRight = trackRect.width - 1;
    } else {
      if (isLeftToRight(slider)) {
        trackLeft = (trackRect.width - getThumbOverhang()) -
                getTrackWidth();
        trackRight = (trackRect.width - getThumbOverhang()) - 1;
      } else {
        trackLeft = getThumbOverhang();
        trackRight = getThumbOverhang() + getTrackWidth() - 1;
      }
      trackBottom = trackRect.height - 1;
    }
    return new Rectangle(trackRect.x + trackLeft, trackRect.y + trackTop,
            trackRight - trackLeft, trackBottom - trackTop);
  }

  public void paintTrack(Graphics g) {

    oceanPaintTrack(g);

  }

  private void oceanPaintTrack(Graphics g) {
    boolean leftToRight = isLeftToRight(slider);
    boolean drawInverted = drawInverted();

    // Translate to the origin of the painting rectangle
    Rectangle paintRect = getPaintTrackRect();
    g.translate(paintRect.x, paintRect.y);

    // Width and height of the painting rectangle.
    int w = paintRect.width;
    int h = paintRect.height;

    if (!slider.isEnabled()) {
      g.setColor(MetalLookAndFeel.getControlShadow());
      g.drawRect(0, 0, w - 1, h - 1);
    } else if (slider.getOrientation() == JSlider.HORIZONTAL) {
      
      int middleOfThumb = thumbRect.x + (thumbRect.width / 2) -
              paintRect.x;
      
      int otMiddle = otherThumbRect.x + (otherThumbRect.width / 2) - paintRect.x;
      
      int fillMinX;
      int fillMaxX;
      
      int tmin = Math.min(middleOfThumb, otMiddle);
      int tmax = Math.max(middleOfThumb, otMiddle);

      if (tmin > 0) {
        g.setColor(chooseColor(drawInverted,
                MetalLookAndFeel.getPrimaryControlDarkShadow(),
                MetalLookAndFeel.getControlDarkShadow()));
        g.drawRect(0, 0, tmin - 1, h - 1);
      }
      
      if (tmax < w) {
        g.setColor(chooseColor(drawInverted,
            MetalLookAndFeel.getPrimaryControlDarkShadow(),
                MetalLookAndFeel.getControlDarkShadow()));
        g.drawRect(tmax, 0, w - tmax, h - 1);
      }
      
      if (tmax != tmin) {
	g.setColor(chooseColor(drawInverted,
            MetalLookAndFeel.getPrimaryControlDarkShadow(),
                MetalLookAndFeel.getControlDarkShadow()));
	// draws track between thumbs
        g.drawRect(tmin, 0, tmax - tmin, h - 1);
      }
      
      
      g.setColor(MetalLookAndFeel.getPrimaryControlShadow());
      if (drawInverted) {
        fillMinX = middleOfThumb;
        fillMaxX = w - 2;
        g.drawLine(1, 1, middleOfThumb, 1);
      } else {
        fillMinX = tmin;
        fillMaxX = tmax;
        //g.drawLine(middleOfThumb, 1, w - 1, 1);
      }
      if (h == 6) {
        g.setColor(MetalLookAndFeel.getWhite());
        g.drawLine(fillMinX, 1, fillMaxX, 1);
        g.setColor(sliderAltTrackColor);
        g.drawLine(fillMinX, 2, fillMaxX, 2);
        g.setColor(MetalLookAndFeel.getControlShadow());
        g.drawLine(fillMinX, 3, fillMaxX, 3);
        g.setColor(MetalLookAndFeel.getPrimaryControlShadow());
        g.drawLine(fillMinX, 4, fillMaxX, 4);
      }
      
    } else {
      int middleOfThumb = thumbRect.y + (thumbRect.height / 2) -
              paintRect.y;

      int otMiddle = otherThumbRect.y + (otherThumbRect.height / 2) -
              paintRect.y;

      int fillMinY;
      int fillMaxY;

      int tmin = Math.min(middleOfThumb, otMiddle);
      int tmax = Math.max(middleOfThumb, otMiddle);

      if (tmin > 0) {
        g.setColor(chooseColor(drawInverted,
                MetalLookAndFeel.getControlDarkShadow(),
                MetalLookAndFeel.getPrimaryControlDarkShadow()));
        // draws track from top min thumb
        g.drawRect(0, 0, w - 1, tmin - 1);
      }

      if (tmax < h) {
        g.setColor(chooseColor(drawInverted,
                MetalLookAndFeel.getControlDarkShadow(),
                MetalLookAndFeel.getPrimaryControlDarkShadow()));
        // draws track max thumb to bottom
        g.drawRect(0, tmax, w - 1, h - tmax - 1);
      }


      //System.out.println("tmax = " + tmax);
      //System.out.println("tmin = " + tmin);
      //System.out.println("h = " + h);
      if (tmax != tmin) {
        g.setColor(chooseColor(drawInverted,
                MetalLookAndFeel.getPrimaryControlDarkShadow(),
                MetalLookAndFeel.getControlDarkShadow()));
        // draws track between thumbs
        g.drawRect(0, tmin, w - 1, tmax - tmin - 1);
      }


      g.setColor(MetalLookAndFeel.getPrimaryControlShadow());
      if (drawInverted()) {
        fillMinY = 1;
        fillMaxY = middleOfThumb;
        if (leftToRight) {
          g.drawLine(1, middleOfThumb, 1, h - 1);
        } else {
          g.drawLine(w - 2, middleOfThumb, w - 2, h - 1);
        }
      } else {
        fillMinY = tmin; //middleOfThumb;
        fillMaxY = tmax - 2; //h - 2;
      }
      if (w == 6) {
        g.setColor(chooseColor(!leftToRight,
                MetalLookAndFeel.getWhite(),
                MetalLookAndFeel.getPrimaryControlShadow()));
        g.drawLine(1, fillMinY, 1, fillMaxY);
        g.setColor(chooseColor(!leftToRight, sliderAltTrackColor,
                MetalLookAndFeel.getControlShadow()));
        g.drawLine(2, fillMinY, 2, fillMaxY);
        g.setColor(chooseColor(!leftToRight,
                MetalLookAndFeel.getControlShadow(),
                sliderAltTrackColor));
        g.drawLine(3, fillMinY, 3, fillMaxY);
        g.setColor(chooseColor(!leftToRight,
                MetalLookAndFeel.getPrimaryControlShadow(),
                MetalLookAndFeel.getWhite()));
        g.drawLine(4, fillMinY, 4, fillMaxY);
      }
    }

    g.translate(-paintRect.x, -paintRect.y);
  }

  public void paintFocus(Graphics g) {
  }

  protected Dimension getThumbSize() {
    Dimension size = new Dimension();

    if (slider.getOrientation() == JSlider.VERTICAL) {
      size.width = getVertThumbIcon().getIconWidth();
      size.height = getVertThumbIcon().getIconHeight();
    } else {
      size.width = getHorizThumbIcon().getIconWidth();
      size.height = getHorizThumbIcon().getIconHeight();
    }

    return size;
  }

  /**
   * Gets the height of the tick area for horizontal sliders and the width of the
   * tick area for vertical sliders.  BasicSliderUI uses the returned value to
   * determine the tick area rectangle.
   */
  public int getTickLength() {
    return slider.getOrientation() == JSlider.HORIZONTAL ? tickLength + TICK_BUFFER + 1 :
            tickLength + TICK_BUFFER + 3;
  }

  /**
   * Returns the shorter dimension of the track.
   */
  protected int getTrackWidth() {
    // This strange calculation is here to keep the
    // track in proportion to the thumb.
    final double kIdealTrackWidth = 7.0;
    final double kIdealThumbHeight = 16.0;
    final double kWidthScalar = kIdealTrackWidth / kIdealThumbHeight;

    if (slider.getOrientation() == JSlider.HORIZONTAL) {
      return (int) (kWidthScalar * thumbRect.height);
    } else {
      return (int) (kWidthScalar * thumbRect.width);
    }
  }

  /**
   * Returns the longer dimension of the slide bar.  (The slide bar is only the
   * part that runs directly under the thumb)
   */
  protected int getTrackLength() {
    if (slider.getOrientation() == JSlider.HORIZONTAL) {
      return trackRect.width;
    }
    return trackRect.height;
  }

  /**
   * Returns the amount that the thumb goes past the slide bar.
   */
  protected int getThumbOverhang() {
    return (int) (getThumbSize().getHeight() - getTrackWidth()) / 2;
  }

  protected void scrollDueToClickInTrack(int dir) {
    scrollByUnit(dir);
  }

  protected void paintMinorTickForHorizSlider(Graphics g, Rectangle tickBounds, int x) {
    g.setColor(slider.isEnabled() ? slider.getForeground() : MetalLookAndFeel.getControlShadow());
    g.drawLine(x, TICK_BUFFER, x, TICK_BUFFER + (tickLength / 2));
  }

  protected void paintMajorTickForHorizSlider(Graphics g, Rectangle tickBounds, int x) {
    g.setColor(slider.isEnabled() ? slider.getForeground() : MetalLookAndFeel.getControlShadow());
    g.drawLine(x, TICK_BUFFER, x, TICK_BUFFER + (tickLength - 1));
  }

  protected void paintMinorTickForVertSlider(Graphics g, Rectangle tickBounds, int y) {
    g.setColor(slider.isEnabled() ? slider.getForeground() : MetalLookAndFeel.getControlShadow());

    if (isLeftToRight(slider)) {
      g.drawLine(TICK_BUFFER, y, TICK_BUFFER + (tickLength / 2), y);
    } else {
      g.drawLine(0, y, tickLength / 2, y);
    }
  }

  protected void paintMajorTickForVertSlider(Graphics g, Rectangle tickBounds, int y) {
    g.setColor(slider.isEnabled() ? slider.getForeground() : MetalLookAndFeel.getControlShadow());

    if (isLeftToRight(slider)) {
      g.drawLine(TICK_BUFFER, y, TICK_BUFFER + tickLength, y);
    } else {
      g.drawLine(0, y, tickLength, y);
    }
  }

  private static Rectangle unionRect = new Rectangle();

  public void setThumbLocation(int x, int y, Rectangle rect) {
    unionRect.setBounds(rect);
    rect.setLocation(x, y);

    SwingUtilities.computeUnion(rect.x, rect.y, rect.width, rect.height, unionRect);
    slider.repaint(unionRect.x, unionRect.y, unionRect.width, unionRect.height);
  }


  protected TrackListener createTrackListener(JSlider slider) {
    return new RangeTrackListener();
  }

  public class RangeTrackListener extends TrackListener {

    protected transient int offset;
    protected transient int currentMouseX, currentMouseY;
    protected Rectangle rect;
    protected boolean isOther = false;

    /**
     * If the mouse is pressed above the "thumb" component
     * then reduce the scrollbars value by one page ("page up"),
     * otherwise increase it by one page.  If there is no
     * thumb then page up if the mouse is in the upper half
     * of the track.
     */
    public void mousePressed(MouseEvent e) {
      if (!slider.isEnabled()) {
        return;
      }

      currentMouseX = e.getX();
      currentMouseY = e.getY();

      if (slider.isRequestFocusEnabled()) {
        slider.requestFocus();
      }

      // Clicked in the Thumb area?
      if (thumbRect.contains(currentMouseX, currentMouseY)) {
        switch (slider.getOrientation()) {
          case JSlider.VERTICAL:
            offset = currentMouseY - thumbRect.y;
            break;
          case JSlider.HORIZONTAL:
            offset = currentMouseX - thumbRect.x;
            break;
        }
        rect = thumbRect;
        isOther = false;
        isDragging = true;
        return;
      }

      if (otherThumbRect.contains(currentMouseX, currentMouseY)) {
        switch (slider.getOrientation()) {
          case JSlider.VERTICAL:
            offset = currentMouseY - otherThumbRect.y;
            break;
          case JSlider.HORIZONTAL:
            offset = currentMouseX - otherThumbRect.x;
            break;
        }
        rect = otherThumbRect;
        isOther = true;
        isDragging = true;
        return;
      }
      isDragging = false;
    }

    /**
     * Set the models value to the position of the top/left
     * of the thumb relative to the origin of the track.
     */
    public void mouseDragged(MouseEvent e) {
      int thumbMiddle = 0;

      if (!slider.isEnabled()) {
        return;
      }

      currentMouseX = e.getX();
      currentMouseY = e.getY();

      if (!isDragging) {
        return;
      }

      slider.setValueIsAdjusting(true);

      switch (slider.getOrientation()) {
        case JSlider.VERTICAL:
          int halfThumbHeight = rect.height / 2;
          int thumbTop = e.getY() - offset;
          int trackTop = trackRect.y;
          int trackBottom = trackRect.y + (trackRect.height - 1);
          int vMax = yPositionForValue(slider.getMaximum() -
                  slider.getExtent());

          if (drawInverted()) {
            trackBottom = vMax;
          } else {
            trackTop = vMax;
          }
          thumbTop = Math.max(thumbTop, trackTop - halfThumbHeight);
          thumbTop = Math.min(thumbTop, trackBottom - halfThumbHeight);

          setThumbLocation(rect.x, thumbTop, rect);

          thumbMiddle = thumbTop + halfThumbHeight;
          if (isOther) {
            getSlider().setSecondValue(valueForYPosition(thumbMiddle));
          } else {
            slider.setValue(valueForYPosition(thumbMiddle));
          }

          break;
        case JSlider.HORIZONTAL:
          int halfThumbWidth = rect.width / 2;
          int thumbLeft = e.getX() - offset;
          int trackLeft = trackRect.x;
          int trackRight = trackRect.x + (trackRect.width - 1);
          int hMax = xPositionForValue(slider.getMaximum() -
                  slider.getExtent());

          if (drawInverted()) {
            trackLeft = hMax;
          } else {
            trackRight = hMax;
          }
          thumbLeft = Math.max(thumbLeft, trackLeft - halfThumbWidth);
          thumbLeft = Math.min(thumbLeft, trackRight - halfThumbWidth);

          setThumbLocation(thumbLeft, rect.y, rect);

          thumbMiddle = thumbLeft + halfThumbWidth;
          if (isOther) {
            getSlider().setSecondValue(valueForXPosition(thumbMiddle));
          } else {
            slider.setValue(valueForXPosition(thumbMiddle));
          }
          break;
        default:
          return;
      }
    }

    public void mouseMoved(MouseEvent e) {
    }
  }
}