/*
 * Created by JFormDesigner on Fri Mar 06 11:01:49 EST 2009
 */

package anl.aida.client.widget;

import anl.aida.util.DateRange;
import anl.aida.util.DateRangeProducer;

import com.jgoodies.forms.factories.FormFactory;
import com.jgoodies.forms.layout.*;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @author User #1
 */
public class TimeSliceSlider extends JPanel {

  private DateFormat shortFormat = new SimpleDateFormat("MM-dd-yy");
  //private DateFormat monthFormat = new SimpleDateFormat("MMM");
  private Calendar cal = GregorianCalendar.getInstance();
  private boolean locked = true;

  private DateRangeProducer ranger;
  private Date minDate, maxDate;
  private int minVal, maxVal;
  private boolean triggerOther = true;
  private java.util.List<DateRangeChangedListener> listeners = new ArrayList<DateRangeChangedListener>();

  public TimeSliceSlider() {
    initComponents();

    minSlider.setMinimum(0);
    maxSlider.setMinimum(0);
    minSlider.setPaintLabels(false);
    maxSlider.setPaintLabels(false);
    minSlider.setValue(0);
    maxSlider.setValue(0);
    minSlider.setEnabled(false);
    maxSlider.setEnabled(false);

    lockedBox.setSelected(true);
    lockedBox.setEnabled(false);

    minSlider.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        int val = minSlider.getValue();
        if (locked && triggerOther) {
          triggerOther = false;
          maxSlider.setValue(maxSlider.getValue() + val - minVal);
          triggerOther = true;
        }
        minVal = val;

        if (val + ranger.getRangeLength() > maxSlider.getValue()) {
          minSlider.setValue(maxSlider.getValue() - ranger.getRangeLength());
        } else {
          minDate = calcDateFromStart(val);
          fireRangeChanged();
        }
      }
    });

    maxSlider.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        int val = maxSlider.getValue();

        if (locked && triggerOther) {
          triggerOther = false;
          minSlider.setValue(minSlider.getValue() + (val - maxVal));
          triggerOther = true;
        }
        maxVal = val;

        if (val - ranger.getRangeLength() < minSlider.getValue()) {
          maxSlider.setValue(minSlider.getValue() + ranger.getRangeLength());
          return;
        }
        maxDate = calcDateFromStart(val);
        fireRangeChanged();
      }
    });

    lockedBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        locked = lockedBox.isSelected();
      }
    });
  }

  private void fireRangeChanged() {
    boolean isAdjusting = maxSlider.getValueIsAdjusting() || (!triggerOther) || minSlider.getValueIsAdjusting();
    for (DateRangeChangedListener listener : listeners) {
      listener.rangeChanged(new RangeEvent(new DateRange(minDate, maxDate), isAdjusting));
    }
  }

  public void addRangeChangedListener(DateRangeChangedListener listener) {
    listeners.add(listener);
  }

  public void init(DateRangeProducer ranger) {
    maxSlider.setPaintLabels(true);

    minSlider.setEnabled(true);
    maxSlider.setEnabled(true);
    lockedBox.setEnabled(true);

    this.ranger = ranger;
    int totalDays = ranger.getTotalDays();
    int interval = ranger.getRangeLength();
    minDate = ranger.getStart();
    maxDate = ranger.getEnd();
    fireRangeChanged();

    minSlider.setMaximum(totalDays);
    maxSlider.setMaximum(totalDays);

    minSlider.setMajorTickSpacing(interval);
    maxSlider.setMajorTickSpacing(interval);

    minSlider.setValue(totalDays - interval);
    maxSlider.setValue(totalDays);
    minVal = totalDays - interval;
    maxVal = totalDays;

    initLabels(ranger);
    cal.setTime(ranger.getStart());

    lockedBox.setSelected(true);
    lockedBox.setText("Locked (" + interval + " days)");


  }

  private Date calcDateFromStart(int days) {
    cal.setTime(ranger.getStart());
    cal.add(Calendar.DAY_OF_MONTH, days);
    Date date = cal.getTime();
    cal.setTime(ranger.getStart());
    return date;
  }

  private void initLabels(DateRangeProducer ranger) {
    cal.setTime(ranger.getStart());
    int totalDays = ranger.getTotalDays();
    int interval = ranger.getRangeLength();
    Hashtable labels = new Hashtable();
    labels.put(0, getLabel(shortFormat.format(ranger.getStart())));
    labels.put(totalDays, getLabel(shortFormat.format(ranger.getEnd())));

    //Hashtable labels2 = new Hashtable();
    //labels2.put(0, getLabel(shortFormat.format(ranger.getStart())));
    //labels2.put(totalDays, getLabel(shortFormat.format(ranger.getEnd())));

    for (int i = 1; i < totalDays - 1; i++) {
      int month = cal.get(Calendar.MONTH);
      cal.add(Calendar.DAY_OF_MONTH, 1);
      int newMonth = cal.get(Calendar.MONTH);
      if (month != newMonth) {
        //labels.put(i, getLabel(monthFormat.format(cal.getTime())));
        //labels2.put(i, getLabel(monthFormat.format(cal.getTime())));
      } else if (i % interval == 0) {
        labels.put(i, getLabel(String.valueOf(cal.get(Calendar.DAY_OF_MONTH))));
        //labels2.put(i, getLabel(String.valueOf(cal.get(Calendar.DAY_OF_MONTH))));
      }

    }
    //minSlider.setLabelTable(labels2);
    maxSlider.setLabelTable(labels);
  }

  private JLabel getLabel(String text) {
    JLabel label = new JLabel(text);
    label.setFont(new Font("Dialog", Font.PLAIN, 10));
    return label;
  }


  private void initComponents() {
    // JFormDesigner - Component initialization - DO NOT MODIFY  //GEN-BEGIN:initComponents
    // Generated using JFormDesigner non-commercial license
    minSlider = new JSlider();
    maxSlider = new JSlider();
    panel2 = new JPanel();
    lockedBox = new JCheckBox();
    CellConstraints cc = new CellConstraints();

    //======== this ========
    setBorder(new EmptyBorder(5, 5, 5, 5));
    setLayout(new FormLayout(
            ColumnSpec.decodeSpecs("right:default, default:grow"),
            new RowSpec[]{
                    new RowSpec(RowSpec.FILL, Sizes.DEFAULT, FormSpec.DEFAULT_GROW),
                    FormFactory.LINE_GAP_ROWSPEC,
                    FormFactory.DEFAULT_ROWSPEC
            }));

    //---- minSlider ----
    minSlider.setOrientation(SwingConstants.VERTICAL);
    minSlider.setPaintTicks(true);
    minSlider.setSnapToTicks(true);
    minSlider.setMaximum(60);
    minSlider.setPaintLabels(true);
    minSlider.setMajorTickSpacing(5);
    minSlider.setFont(minSlider.getFont().deriveFont(minSlider.getFont().getSize() - 3f));
    minSlider.setMinorTickSpacing(1);
    add(minSlider, cc.xy(1, 1));

    //---- maxSlider ----
    maxSlider.setOrientation(SwingConstants.VERTICAL);
    maxSlider.setMajorTickSpacing(5);
    maxSlider.setMaximum(60);
    maxSlider.setMinorTickSpacing(1);
    maxSlider.setFont(maxSlider.getFont().deriveFont(maxSlider.getFont().getSize() - 3f));
    maxSlider.setSnapToTicks(true);
    maxSlider.setPaintTicks(true);
    maxSlider.setPaintLabels(true);
    add(maxSlider, cc.xy(2, 1));

    //======== panel2 ========
    {
      panel2.setBorder(null);
      panel2.setLayout(new FormLayout(
              "default:grow",
              "fill:default:grow"));

      //---- lockedBox ----
      lockedBox.setText("Locked (0 days)");
      panel2.add(lockedBox, cc.xy(1, 1));
    }
    add(panel2, cc.xywh(1, 3, 2, 1));
    // JFormDesigner - End of component initialization  //GEN-END:initComponents
  }

  // JFormDesigner - Variables declaration - DO NOT MODIFY  //GEN-BEGIN:variables
  // Generated using JFormDesigner non-commercial license
  private JSlider minSlider;
  private JSlider maxSlider;
  private JPanel panel2;
  private JCheckBox lockedBox;
  // JFormDesigner - End of variables declaration  //GEN-END:variables
}
