/*
 * Created by JFormDesigner on Tue Mar 10 09:14:08 EDT 2009
 */

package anl.aida.client.widget;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import anl.aida.util.DateRange;
import anl.aida.util.DateRangeProducer;

import com.jgoodies.forms.factories.FormFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.ColumnSpec;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.layout.FormSpec;
import com.jgoodies.forms.layout.RowSpec;
import com.jgoodies.forms.layout.Sizes;

/**
 * @author User #1
 */
public class TimeSliceSlider2 extends JPanel {

  private DateFormat shortFormat = new SimpleDateFormat("MM-dd-yy");
  // private DateFormat monthFormat = new SimpleDateFormat("MMM");
  private Calendar cal = GregorianCalendar.getInstance();
  private boolean locked = true;

  private DateRangeProducer ranger;
  private Date minDate, maxDate;
  private int minVal, maxVal;
  private boolean triggerOther = true;
  private java.util.List<DateRangeChangedListener> listeners = new ArrayList<DateRangeChangedListener>();

  public TimeSliceSlider2() {
    initComponents();
    initSliderBox();
  }

  protected void initSliderBox() {
    slider.setMinimum(0);
    slider.setPaintLabels(false);
    slider.setValue(0);
    slider.setEnabled(false);

    lockedBox.setSelected(true);
    lockedBox.setEnabled(false);

    slider.addRangeChangedListener(new RangeChangedListener() {
      public void rangeChanged(RangeChangeEvent evt) {

	int firstVal = slider.getFirstValue();

	if (evt.isFirstChanged() && maxVal != firstVal) {
	  if (locked && triggerOther) {
	    triggerOther = false;
	    // increment the min value by the amt the first
	    // value has changed -- i.e. the diff between val
	    // and max val.
	    slider.setSecondValue(slider.getSecondValue() + (firstVal - maxVal));
	    triggerOther = true;
	  }

	  maxVal = firstVal;
	  // make sure that the max is always greater than the min
	  // by the range length
	  if (firstVal - ranger.getRangeLength() < slider.getSecondValue()) {
	    slider.setFirstValue(slider.getSecondValue() + ranger.getRangeLength());
	  } else {
	    maxDate = calcDateFromStart(firstVal);
	  }
	}
	fireRangeChanged();
      }
    });

    slider.addRangeChangedListener(new RangeChangedListener() {
      public void rangeChanged(RangeChangeEvent evt) {

	int secondVal = slider.getSecondValue();

	if (evt.isSecondChanged() && minVal != secondVal) {
	  if (locked && triggerOther) {
	    triggerOther = false;
	    slider.setValue(slider.getFirstValue() + secondVal - minVal);
	    triggerOther = true;
	  }
	  minVal = secondVal;

	  if (secondVal + ranger.getRangeLength() > slider.getFirstValue()) {
	    slider.setSecondValue(slider.getFirstValue() - ranger.getRangeLength());
	  } else {
	    minDate = calcDateFromStart(secondVal);
	  }
	}

	fireRangeChanged();
      }
    });

    lockedBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	if (!locked && slider.getFirstValue() - slider.getSecondValue() != ranger.getRangeLength()) {
	  // about to be locked so set the slider interval appropriately
	  slider.setSecondValue(slider.getFirstValue() - ranger.getRangeLength());
	}
	locked = lockedBox.isSelected();
      }
    });
  }

  private void fireRangeChanged() {
    boolean isAdjusting = slider.getValueIsAdjusting() || (!triggerOther);
    for (DateRangeChangedListener listener : listeners) {
      listener.rangeChanged(new RangeEvent(new DateRange(minDate, maxDate), isAdjusting));
    }
  }

  public void addRangeChangedListener(DateRangeChangedListener listener) {
    listeners.add(listener);
  }

  public void init(DateRangeProducer ranger) {
    this.ranger = ranger;
    int totalDays = ranger.getTotalDays();
    int interval = ranger.getRangeLength();
    maxDate = ranger.getEnd();
    minDate = calcDateFromStart(totalDays - interval);
    
    fireRangeChanged();

    minVal = totalDays - interval;
    maxVal = totalDays;

    TimeSliderInitializer initter = new TimeSliderInitializer(ranger, shortFormat);
    initter.initSlider(slider, minVal, maxVal, true);
    initter.initBox(lockedBox, true);
    
    cal.setTime(ranger.getStart());
  }

  private Date calcDateFromStart(int days) {
    cal.setTime(ranger.getStart());
    cal.add(Calendar.DAY_OF_MONTH, days);
    Date date = cal.getTime();
    cal.setTime(ranger.getStart());
    return date;
  }

  private void initComponents() {
    // JFormDesigner - Component initialization - DO NOT MODIFY
    // //GEN-BEGIN:initComponents
    // Generated using JFormDesigner non-commercial license
    slider = new RangeSlider();
    lockedBox = new JCheckBox();
    CellConstraints cc = new CellConstraints();

    // ======== this ========
    setLayout(new FormLayout(ColumnSpec.decodeSpecs("default"), new RowSpec[] {
	new RowSpec(RowSpec.FILL, Sizes.DEFAULT, FormSpec.DEFAULT_GROW),
	FormFactory.LINE_GAP_ROWSPEC, FormFactory.DEFAULT_ROWSPEC }));

    // ---- slider ----
    slider.setSnapToTicks(true);
    slider.setPaintTicks(true);
    slider.setPaintLabels(true);
    slider.setOrientation(SwingConstants.VERTICAL);
    slider.setMajorTickSpacing(5);
    slider.setMinorTickSpacing(1);
    slider.setFont(slider.getFont().deriveFont(slider.getFont().getSize() - 3f));
    add(slider, cc.xy(1, 1));

    // ---- lockedBox ----
    lockedBox.setText(" Locked (0 Days)");
    add(lockedBox, cc.xy(1, 3));
    // JFormDesigner - End of component initialization //GEN-END:initComponents
  }

  // JFormDesigner - Variables declaration - DO NOT MODIFY //GEN-BEGIN:variables
  // Generated using JFormDesigner non-commercial license
  private RangeSlider slider;
  private JCheckBox lockedBox;
  // JFormDesigner - End of variables declaration //GEN-END:variables
}
