package anl.aida.client.widget;

import java.awt.Font;
import java.text.DateFormat;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Hashtable;

import javax.swing.JCheckBox;
import javax.swing.JLabel;

import anl.aida.util.DateRangeProducer;

public class TimeSliderInitializer {
  
  private Calendar cal = GregorianCalendar.getInstance();
  private DateRangeProducer ranger;
  private DateFormat format;
  
  public TimeSliderInitializer(DateRangeProducer ranger, DateFormat format) {
    this.ranger = ranger;
    this.format = format;
  }

  public void initSlider(RangeSlider slider, int minVal, int maxVal, boolean showEndsAsDates) {
    slider.setPaintLabels(true);
    slider.setEnabled(true);
    slider.setSnapToTicks(true);
    slider.setPaintTicks(true);
    
    int totalDays = ranger.getTotalDays();
    int interval = ranger.getRangeLength();
    
    slider.setMaximum(totalDays);
    slider.setMinimum(0);
    slider.setMajorTickSpacing(interval);
    slider.setMinorTickSpacing(1);
    slider.setSecondValue(totalDays - interval);
    slider.setFirstValue(totalDays);
    
    cal.setTime(ranger.getStart());
    
    Hashtable labels = new Hashtable();
    if (showEndsAsDates) {
      labels.put(0, getLabel(format.format(ranger.getStart())));
      labels.put(totalDays, getLabel(format.format(ranger.getEnd())));
    }  else {
      labels.put(0, getLabel(String.valueOf(cal.get(Calendar.DAY_OF_MONTH))));
      cal.add(Calendar.DAY_OF_MONTH, totalDays);
      labels.put(totalDays, getLabel(String.valueOf(cal.get(Calendar.DAY_OF_MONTH))));
    }
    
    cal.setTime(ranger.getStart());
    for (int i = 1; i < totalDays - 1; i++) {
      //int month = cal.get(Calendar.MONTH);
      cal.add(Calendar.DAY_OF_MONTH, 1);
      //int newMonth = cal.get(Calendar.MONTH);
      if (i % interval == 0) {
	labels.put(i, getLabel(String.valueOf(cal.get(Calendar.DAY_OF_MONTH))));
      }
    }
    // minSlider.setLabelTable(labels2);
    slider.setLabelTable(labels);
    slider.setFirstValue(maxVal);
    slider.setSecondValue(minVal);
  }
  
  public void initBox(JCheckBox box, boolean selected) {
    box.setEnabled(true);
    box.setSelected(selected);
    box.setText("Locked (" + ranger.getRangeLength() + " days)");
  }
  
  private JLabel getLabel(String text) {
    JLabel label = new JLabel(text);
    label.setFont(new Font("Dialog", Font.PLAIN, 10));
    return label;
  }

}
