/**
 * 
 */
package anl.aida.data.mock;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.List;

import javax.measure.unit.NonSI;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;

import ucar.ma2.Array;
import ucar.ma2.ArrayDouble;
import ucar.ma2.ArrayInt;
import anl.aida.data.AbstractDataset;
import anl.aida.data.AxisData;
import anl.aida.data.AxisType;
import anl.aida.data.DataFrameAxis;
import anl.aida.data.DataFrameBuilder;
import anl.aida.data.DefaultStringCoordAxis;
import anl.aida.data.DefaultVariable;
import anl.aida.data.Range;
import anl.aida.data.RegularTimeAxis;
import anl.aida.data.Units;
import anl.aida.data.Variable;

/**
 * Sample dataset containing wisconsin data.
 * 
 */
public class ERDataFeed extends AbstractDataset {

  public ERDataFeed() throws IOException {
    //Variable var = new DefaultVariable("Flu / All", "Flu / All", Units.createUnit("ER Visits"), this);
    //addVariable(var);
    Variable[] vars = new Variable[2];
    Variable var = new DefaultVariable("Fever_Flu", "Fever_Flu", Units.createUnit("ER Visits"), this);
    addVariable(var);
    vars[0] = var;
    var = new DefaultVariable("All", "All", Units.createUnit("ER Visits"), this);
    addVariable(var);
    vars[1] = var;

    FileInputStream in = new FileInputStream("./data/WHIE ED visits 01Oct08_31Jul09.xls");
    HSSFWorkbook wb = new HSSFWorkbook(in);
    List<DataFrameAxis> axes = new ArrayList<DataFrameAxis>();
    createAxes(axes);
    for (int i = 0; i < 2; i++) {
      DataFrameBuilder builder = new DataFrameBuilder();
      for (DataFrameAxis axis : axes) {
	builder.addAxis(axis);
      }
      builder.setVariable(vars[i]);
      Array array;
      boolean isIntType = true;
      int sheetIndex = i + 1;
      //if (i == 0) {
	//array = new ArrayDouble.D2(axes.get(0).getExtent(), axes.get(1).getExtent());
	//isIntType = false;
      //}
      //else 
      array = new ArrayInt.D2(axes.get(0).getExtent(), axes.get(1).getExtent());
      fillArray(wb.getSheetAt(sheetIndex), array, isIntType);

      builder.setArray(array);
      frames.put(vars[i], builder.createDataFrame());
    }

    in.close();
  }

  private void fillArray(HSSFSheet sheet, Array array, boolean isInt) {
    int rowCount = 0;
    for (Iterator<HSSFRow> rit = (Iterator<HSSFRow>) sheet.rowIterator(); rit.hasNext();) {
      HSSFRow row = rit.next();
      int colCount = 0;
      if (rowCount > 1) {
	for (Iterator<HSSFCell> cit = (Iterator<HSSFCell>) row.cellIterator(); cit.hasNext();) {
	  HSSFCell cell = cit.next();
	  if (colCount > 0) {
	    if (isInt) ((ArrayInt.D2)array).set(rowCount - 2, colCount - 1, (int)cell.getNumericCellValue());
	    else ((ArrayDouble.D2)array).set(rowCount - 2, colCount - 1, cell.getNumericCellValue());
	  }
	  colCount++;
	}
      }
      rowCount++;
    }

  }

  private void createAxes(List<DataFrameAxis> axes) {

    Calendar cal = GregorianCalendar.getInstance();
    cal.set(2008, 9, 1, 0, 0, 0);

    AxisData axisData = new AxisData();
    axisData.setUnit(NonSI.DAY);
    axisData.setDescription("Date");
    axisData.setName("Date");
    axisData.setRange(new Range(0, 304));
    axisData.setType(AxisType.TIME);
    RegularTimeAxis tAxis = new RegularTimeAxis(cal.getTime(), 1, axisData);
    axes.add(DataFrameAxis.createDataFrameAxis(tAxis, 0));

    String[] categories = { "00-01", "02-04", "05-12", "13-17", "18-39", "40-64", "65+", "Total" };
    axisData = new AxisData();
    axisData.setUnit(NonSI.YEAR);
    axisData.setDescription("Age Range");
    axisData.setName("Age Range");
    axisData.setType(AxisType.CATEGORY);
    axisData.setRange(new Range(0, categories.length));
    DefaultStringCoordAxis axis = new DefaultStringCoordAxis(categories, axisData);
    axes.add(DataFrameAxis.createDataFrameAxis(axis, 1));
  }


  /*
   * (non-Javadoc)
   * 
   * @see anl.mifs.data.Dataset#getName()
   */
  @Override
  public String getName() {
    return "Emergency Room Visits";
  }
}
