/**
 * 
 */
package anl.aida.data.mock;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import javax.measure.unit.NonSI;
import javax.measure.unit.Unit;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;

import ucar.ma2.ArrayInt;
import anl.aida.data.AbstractDataset;
import anl.aida.data.Axes;
import anl.aida.data.AxisAttributeConstants;
import anl.aida.data.AxisData;
import anl.aida.data.AxisType;
import anl.aida.data.DataFrame;
import anl.aida.data.DataFrameAxis;
import anl.aida.data.DataFrameBuilder;
import anl.aida.data.DefaultStringCoordAxis;
import anl.aida.data.DefaultVariable;
import anl.aida.data.Range;
import anl.aida.data.RegularTimeAxis;
import anl.aida.data.Units;
import anl.aida.data.Variable;

/**
 * Sample dataset containing wisconsin data.
 * 
 */
public class WIDataFeed extends AbstractDataset {
  
  private DataFrame frame;
  private DefaultVariable variable;

  public WIDataFeed() throws IOException {
    variable = new DefaultVariable("H1N1", "H1N1", Units.createUnit("Case Counts"), this);
    addVariable(variable);
    DataFrameBuilder builder = new DataFrameBuilder();
    builder.setVariable(variable);

    FileInputStream in = new FileInputStream(
	"./data/H1N1_Cases_bySpecimenCollectionDate_byCounty_Wisconsin.xls");
    HSSFWorkbook wb = new HSSFWorkbook(in);
    HSSFSheet sheet = wb.getSheetAt(0);

    List<String> counties = new ArrayList<String>();
    List<Date> dates = new ArrayList<Date>();
    List<List<Integer>> data = new ArrayList<List<Integer>>();
    boolean firstRow = true;
    for (Iterator<HSSFRow> rit = (Iterator<HSSFRow>) sheet.rowIterator(); rit.hasNext();) {
      HSSFRow row = rit.next();
      if (firstRow) {
	readCounties(row, counties);
	firstRow = false;
      } else
	readData(row, dates, data);
    }
    in.close();
    
    ArrayInt.D2 array = new ArrayInt.D2(dates.size(), counties.size());
    for (int i = 0; i < data.size(); i++) { 
      List<Integer> row = data.get(i);
      for (int j = 0; j < row.size(); j++) {
	array.set(i, j, row.get(j));
      }
    }
    builder.setArray(array);
    
    AxisData axisData = new AxisData();
    axisData.setUnit(NonSI.DAY);
    axisData.setDescription("Date");
    axisData.setName("Date");
    axisData.setRange(new Range(0, dates.size()));
    axisData.setType(AxisType.TIME);
    //System.out.println("dates.get(0).getTime() = " + dates.get(0).getTime());
    RegularTimeAxis tAxis = new RegularTimeAxis(dates.get(0), 1, axisData);
    builder.addAxis(DataFrameAxis.createDataFrameAxis(tAxis, 0));
    
    axisData = new AxisData();
    axisData.setUnit(Unit.ONE);
    axisData.setDescription("County");
    axisData.setName("County");
    axisData.setType(AxisType.NAMED_LOCATION);
    axisData.setRange(new Range(0, counties.size()));
   
    DefaultStringCoordAxis countyAxis = new DefaultStringCoordAxis(counties.toArray(new String[counties.size()]), axisData);
    countyAxis.setAttribute(AxisAttributeConstants.CRS_CODE, "EPSG:4269");
    countyAxis.setAttribute(AxisAttributeConstants.MAP_FILE, "./shapefiles/tl_2008_55_county/tl_2008_55_county.shp");
    countyAxis.setAttribute(AxisAttributeConstants.NAME_ATTRIBUTE, "NAME");
    builder.addAxis(DataFrameAxis.createDataFrameAxis(countyAxis, 1));
    frame = builder.createDataFrame();
  }

  private void readData(HSSFRow row, List<Date> dates, List<List<Integer>> data) {
    List<Integer> dataRow = new ArrayList<Integer>();
    data.add(dataRow);
    boolean firstCol = true;
    for (Iterator<HSSFCell> cit = (Iterator<HSSFCell>) row.cellIterator(); cit.hasNext();) {
      HSSFCell cell = cit.next();
      if (firstCol) {
	dates.add(cell.getDateCellValue());
	firstCol = false;
      } else {
	dataRow.add((int) cell.getNumericCellValue());
      }
    }

  }

  private List<String> readCounties(HSSFRow row, List<String> list) {
    int index = 0;
    for (Iterator<HSSFCell> cit = (Iterator<HSSFCell>) row.cellIterator(); cit.hasNext();) {
      HSSFCell cell = cit.next();
      if (index > 0) {
	String county = cell.getRichStringCellValue().getString().toLowerCase();
	county = county.replace("_", " ");
	list.add(county);
      }
      index++;
    }

    return list;

  }

  /*
   * (non-Javadoc)
   * 
   * @see anl.mifs.data.Dataset#getDataFrame(anl.mifs.data.Variable)
   */
  @Override
  public DataFrame getDataFrame(Variable var) {
    if (var.equals(variable)) return frame;
    return null;
  }

  /*
   * (non-Javadoc)
   * 
   * @see anl.mifs.data.Dataset#getName()
   */
  @Override
  public String getName() {
    return "Influenza Case Counts";
  }
}
