/**
 * 
 */
package anl.aida.data.mock;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import javax.measure.unit.NonSI;
import javax.measure.unit.Unit;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;

import ucar.ma2.ArrayInt;

import anl.aida.data.AbstractDataset;
import anl.aida.data.AxisAttributeConstants;
import anl.aida.data.AxisData;
import anl.aida.data.AxisType;
import anl.aida.data.DataFrameAxis;
import anl.aida.data.DataFrameBuilder;
import anl.aida.data.DefaultStringCoordAxis;
import anl.aida.data.DefaultVariable;
import anl.aida.data.Range;
import anl.aida.data.RegularTimeAxis;
import anl.aida.data.Units;
import anl.aida.data.Variable;

/**
 * Data for population and health employee counts.
 * 
 */
public class WIHealthEmployeeFeed extends AbstractDataset {

  private void createVariable(String name, List<Variable> vars) {
    Variable var = new DefaultVariable(name, name, Units.createUnit("Persons"), this);
    vars.add(var);
    addVariable(var);
  }

  public WIHealthEmployeeFeed() throws IOException {
    List<Variable> vars = new ArrayList<Variable>();
    createVariable("Population", vars);
    createVariable("AHCS_Employees", vars);
    createVariable("AHCS_Facilities", vars);
    createVariable("Hospital_Employees", vars);
    createVariable("Hospital_Facilities", vars);

    List<DataFrameAxis> axes = new ArrayList<DataFrameAxis>();
    FileInputStream in = new FileInputStream("./data/WI NAICS 62 Summary - text.xls");
    HSSFWorkbook wb = new HSSFWorkbook(in);
    HSSFSheet sheet = wb.getSheetAt(0);
    createAxes(axes, sheet);

    int tcount = axes.get(0).getExtent();
    int cCount = axes.get(1).getExtent();

    int column = 1;
    for (Variable var : vars) {
      //System.out.println("col: " + column);
      DataFrameBuilder builder = new DataFrameBuilder();
      builder.setVariable(var);
      builder.addAxis(axes.get(0));
      builder.addAxis(axes.get(1));

      ArrayInt.D2 array = new ArrayInt.D2(tcount, cCount);

      // same data for each time step
      for (int tIndex = 0; tIndex < tcount; tIndex++) {
	int countyIndex = 0;
	int rowCount = 0;
	for (Iterator<HSSFRow> rit = (Iterator<HSSFRow>) sheet.rowIterator(); rit.hasNext();) {
	  HSSFRow row = rit.next();
	  if (rowCount > 2 && rowCount < 76) {
	    HSSFCell cell = row.getCell(column);
	    array.set(tIndex, countyIndex++, getCellValue(cell));
	  }
	  rowCount++;
	}
      }
      
      builder.setArray(array);
      frames.put(var, builder.createDataFrame());
      column++;
    }

    in.close();
  }

  private int getCellValue(HSSFCell cell) {
    int num = 0;
    if (cell.getCellType() == HSSFCell.CELL_TYPE_STRING) {
      String val = cell.getRichStringCellValue().getString();
      val = val.replace(",", "");
      try {
	num = Integer.parseInt(val);
      } catch (NumberFormatException ex) {
	System.out.println("bad val: " + val);
      }
      return num;
    } else if (cell.getCellType() == HSSFCell.CELL_TYPE_NUMERIC) {
      num = (int)cell.getNumericCellValue();
    }
    return num;

  }

  private void createAxes(List<DataFrameAxis> list, HSSFSheet sheet) {
    AxisData axisData = new AxisData();
    axisData.setUnit(NonSI.DAY);
    axisData.setDescription("Date");
    axisData.setName("Date");
    axisData.setRange(new Range(0, 108));
    axisData.setType(AxisType.TIME);
    RegularTimeAxis tAxis = new RegularTimeAxis(new Date(1239768000000L), 1, axisData);
    list.add(DataFrameAxis.createDataFrameAxis(tAxis, 0));

    List<String> counties = new ArrayList<String>();
    int rowCount = 0;
    for (Iterator<HSSFRow> rit = (Iterator<HSSFRow>) sheet.rowIterator(); rit.hasNext();) {
      HSSFRow row = rit.next();
      if (rowCount > 2 && rowCount < 76) {
	HSSFCell cell = row.getCell(0);
	if (cell != null) {
	  //System.out.printf("%d - %s%n", rowCount, cell.getRichStringCellValue());
	  String county = cell.getRichStringCellValue().getString().toLowerCase();
	  counties.add(county.substring(0, county.lastIndexOf(" ")));
	}
      }
      rowCount++;
    }

    axisData = new AxisData();
    axisData.setUnit(Unit.ONE);
    axisData.setDescription("County");
    axisData.setName("County");
    axisData.setType(AxisType.NAMED_LOCATION);
    axisData.setRange(new Range(0, counties.size()));

    DefaultStringCoordAxis countyAxis = new DefaultStringCoordAxis(counties
	.toArray(new String[counties.size()]), axisData);
    countyAxis.setAttribute(AxisAttributeConstants.CRS_CODE, "EPSG:4269");
    countyAxis.setAttribute(AxisAttributeConstants.MAP_FILE,
	"./shapefiles/tl_2008_55_county/tl_2008_55_county.shp");
    countyAxis.setAttribute(AxisAttributeConstants.NAME_ATTRIBUTE, "NAME");
    list.add(DataFrameAxis.createDataFrameAxis(countyAxis, 1));
  }

  @Override
  public String getName() {
    return "WI NAICS Summary";
  }
}
