/**
 * 
 */
package anl.aida.data.mock;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.measure.unit.NonSI;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;

import ucar.ma2.Array;
import ucar.ma2.ArrayDouble;
import ucar.ma2.ArrayInt;
import anl.aida.data.AbstractDataset;
import anl.aida.data.AxisData;
import anl.aida.data.AxisType;
import anl.aida.data.DataFrame;
import anl.aida.data.DataFrameAxis;
import anl.aida.data.DataFrameBuilder;
import anl.aida.data.DefaultVariable;
import anl.aida.data.Range;
import anl.aida.data.RegularTimeAxis;
import anl.aida.data.Units;
import anl.aida.data.Variable;

/**
 * Sample dataset containing wisconsin data.
 * 
 */
public class WIPCRFeed extends AbstractDataset {

  private void createVariable(String name, List<Variable> vars) {
    Variable var = new DefaultVariable(name, name, Units.createUnit("Test Count"), this);
    vars.add(var);
    addVariable(var);
  }

  public WIPCRFeed() throws IOException {
    List<Variable> vars = new ArrayList<Variable>();
    createVariable("Total", vars);
    createVariable("A_H1", vars);
    createVariable("AH3", vars);
    createVariable("B", vars);
    createVariable("Novel_H1", vars);
    createVariable("A_Not_Subtyped", vars);

    DataFrameAxis axis = createAxis();
    

    FileInputStream in = new FileInputStream("./data/pcr influenza types.xls");
    HSSFWorkbook wb = new HSSFWorkbook(in);

    for (int i = 0; i < vars.size(); i++) {
      DataFrameBuilder builder = new DataFrameBuilder();
      builder.setVariable(vars.get(i));
      builder.addAxis(axis);
      Array array;
      boolean isIntType = true;
      if (i == 7) {
	array = new ArrayDouble.D1(axis.getExtent());
	isIntType = false;
      } else array = new ArrayInt.D1(axis.getExtent());
      fillArray(wb.getSheetAt(0), array, isIntType, i + 2);

      builder.setArray(array);
      frames.put(vars.get(i), builder.createDataFrame());
    }

    in.close();
  }

  private void fillArray(HSSFSheet sheet, Array array, boolean isInt, int colIndex) {
    boolean read = false;
    int index = 0;
    for (Iterator<HSSFRow> rit = (Iterator<HSSFRow>) sheet.rowIterator(); rit.hasNext();) {
      HSSFRow row = rit.next();
      // skip first row
      if (read) {
	HSSFCell cell = row.getCell(colIndex);
	if (isInt)
	  ((ArrayInt.D1) array).set(index++, (int) cell.getNumericCellValue());
	else
	  ((ArrayDouble.D1) array).set(index++, cell.getNumericCellValue());
      }
      read = true;
    }
    
  }

  private DataFrameAxis createAxis() {

    Calendar cal = GregorianCalendar.getInstance();
    cal.set(2008, 9, 4, 0, 0, 0);

    AxisData axisData = new AxisData();
    axisData.setUnit(NonSI.WEEK);
    axisData.setDescription("Week Ending");
    axisData.setName("Week Ending");
    axisData.setRange(new Range(0, 44));
    axisData.setType(AxisType.TIME);
    RegularTimeAxis tAxis = new RegularTimeAxis(cal.getTime(), 1, axisData);
    return DataFrameAxis.createDataFrameAxis(tAxis, 0);
  }

  /*
   * (non-Javadoc)
   * 
   * @see anl.mifs.data.Dataset#getName()
   */
  @Override
  public String getName() {
    return "Wisconsin PCR Influenza Types";
  }
  
  
}
