package anl.aida.gis;

import java.io.File;
import java.util.List;

import org.geotools.feature.FeatureCollection;
import org.opengis.feature.simple.SimpleFeature;
import org.opengis.feature.simple.SimpleFeatureType;

import com.vividsolutions.jts.geom.Point;

/**
 * Finds a attribute of a feature given a location.
 */
public class FeaturePropertyFinder implements LocationChangedListener {
  
  private FeatureSpatialMap map;
  private String value = null;
  private String name;
  
  public FeaturePropertyFinder(String attributeName, File shapeFile) {
    map = new FeatureSpatialMap(shapeFile);
    name = attributeName;
  }
  
  public FeaturePropertyFinder(String attributeName, FeatureCollection<SimpleFeatureType, SimpleFeature> features) {
    map = new FeatureSpatialMap(features);
    name = attributeName;
  }
  
  /**
   * Gets the name of the attribute this finder finds.
   * 
   * @return the name of the attribute this finder finds.
   * 
   */
  public String getAttributeName() {
    return name;
  }
  
  /**
   * Gets the value of the current feature's attribute or
   * null if no feature is found.
   * 
   * @return
   */
  public String getValue() {
    return value;
  }
  
  @Override
  public void locationChanged(Point pt) {
    if (pt == null) value = null;
    else {
      List<SimpleFeature> features = map.findFeature(pt);
      if (features.size() == 0) {
	value = null;
      } else {
	SimpleFeature feature = features.get(0);
	value = feature.getAttribute(name).toString();
      }
    }
  }
}
