package anl.aida.gis;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.util.Map;
import java.util.Map.Entry;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JToolBar;

import org.geotools.map.DefaultMapContext;
import org.geotools.map.MapContext;
import org.geotools.referencing.crs.DefaultGeographicCRS;

import edu.umd.cs.piccolo.event.PInputEventListener;

/**
 * This is a very basic panel that holds a canvas and a toolbar.
 *
 */
public class PiccoloMapPanel extends JPanel {
  
  private static final long serialVersionUID = -3638545744874130251L;

  private PGISCanvas canvas;
  private JToolBar toolBar;
  private MapContext context;
  private ToolManager toolMan = new ToolManager();

  /**
   * Create a new panel with no MapContext. This will create a new
   * DefaultMapContext.
   */
  public PiccoloMapPanel() {
    context = new DefaultMapContext(DefaultGeographicCRS.WGS84);
    init();
  }

  /**
   * Create a new panel based on an existing MapContext.
   *
   * @param context The map context to display
   */
  public PiccoloMapPanel(MapContext context) {
    this.context = context;
    init();
  }

  private void init() {
    canvas = new PGISCanvas(context);
    toolBar = toolMan.getToolBar();
    setLayout(new BorderLayout());
    add(canvas, BorderLayout.CENTER);
    canvas.setBorder(BorderFactory.createLineBorder(Color.BLACK));
    //add(toolBar, BorderLayout.NORTH);
  }

  /**
   * Add a new tool to the toolBar to interact with the canvas.
   *
   * @param listener   The PInputListener to add
   * @param properties Action Properties
   */
  public void addTool(final PInputEventListener listener,
                      Map<String, Object> properties) {

    Action action = new AbstractAction() {
      public void actionPerformed(ActionEvent ev) {
        canvas.setEventHandler(listener);
      }
    };

    for (Entry<String, Object> propEntry : properties.entrySet()) {
      action.putValue(propEntry.getKey(), propEntry.getValue());
    }

    toolMan.addTool(action);
    Boolean defaultTool = (Boolean) properties.get("DEFAULT");
    if (defaultTool != null && defaultTool) {
      toolMan.setTool(action);
      canvas.setEventHandler(listener);
    }
  }

  public ToolManager getToolManager() {
    return toolMan;
  }

  public JToolBar getToolBar() {
    return toolBar;
  }

  public PGISCanvas getCanvas() {
    return canvas;
  }
}
