package anl.aida.gis;

import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import org.geotools.data.DataStore;
import org.geotools.data.DataStoreFinder;
import org.geotools.data.DataUtilities;
import org.geotools.data.FeatureSource;
import org.geotools.feature.FeatureCollection;
import org.geotools.feature.FeatureIterator;
import org.geotools.geometry.jts.JTS;
import org.geotools.referencing.CRS;
import org.geotools.referencing.crs.DefaultGeographicCRS;
import org.opengis.feature.simple.SimpleFeature;
import org.opengis.feature.simple.SimpleFeatureType;
import org.opengis.geometry.MismatchedDimensionException;
import org.opengis.referencing.FactoryException;
import org.opengis.referencing.crs.CoordinateReferenceSystem;
import org.opengis.referencing.operation.MathTransform;
import org.opengis.referencing.operation.TransformException;

import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.simplify.TopologyPreservingSimplifier;

/**
 * Simplifies the geometry in a shapefile and converts
 * it to WGS84.
 * 
 */
public class ShapefileSimplifier {

  private FeatureSource<SimpleFeatureType, SimpleFeature> openShapefile(File shapefile)
      throws IOException {
    Map<String, Serializable> params = new HashMap<String, Serializable>();
    params.put("url", shapefile.toURI().toURL());
    params.put("create spatial index", true);
    DataStore dataStore = DataStoreFinder.getDataStore(params);

    String typeName = dataStore.getTypeNames()[0];
    return dataStore.getFeatureSource(typeName);
  }

  public FeatureCollection<SimpleFeatureType, SimpleFeature> simplify(File shapefile,
      double tolerance) throws IOException, FactoryException, MismatchedDimensionException, TransformException {
    FeatureCollection<SimpleFeatureType, SimpleFeature> features = openShapefile(shapefile)
	.getFeatures();
    FeatureCollection<SimpleFeatureType, SimpleFeature> newCollection = DataUtilities
	.collection(features);
    
    CoordinateReferenceSystem sourceCRS = features.getSchema().getCoordinateReferenceSystem();
    MathTransform transform = CRS.findMathTransform(sourceCRS, DefaultGeographicCRS.WGS84);
    

    FeatureIterator<SimpleFeature> fIter = newCollection.features();
    try {
      while (fIter.hasNext()) {
	SimpleFeature feature = fIter.next();
	Geometry geom = (Geometry)feature.getDefaultGeometry();
	//System.out.printf("%s: %d%n", feature.getAttribute("NAME"), geom.getNumPoints());
	Geometry newGeom = TopologyPreservingSimplifier.simplify(geom, tolerance);
	//System.out.printf("%s: %d%n", feature.getAttribute("NAME"), newGeom.getNumPoints());
	
	feature.setDefaultGeometry(JTS.transform(newGeom, transform));
      }
    } finally {
      newCollection.close(fIter);
    }

    return newCollection;
  }

}
