package anl.aida.gis3D;

import gov.nasa.worldwind.Model;
import gov.nasa.worldwind.avlist.AVKey;
import gov.nasa.worldwind.event.PositionEvent;
import gov.nasa.worldwind.event.PositionListener;
import gov.nasa.worldwind.geom.Position;
import gov.nasa.worldwind.layers.AnnotationLayer;
import gov.nasa.worldwind.render.Annotation;
import gov.nasa.worldwind.render.AnnotationAttributes;
import gov.nasa.worldwind.render.DrawContext;
import gov.nasa.worldwind.render.ScreenAnnotation;

import java.awt.Color;
import java.awt.Font;
import java.awt.Insets;
import java.awt.Rectangle;
import java.util.HashMap;
import java.util.Map;

import anl.aida.client.util.ClientUtils;
import anl.aida.data.AxisType;
import anl.aida.data.DataFrame;
import anl.aida.data.DataFrameIndex;
import anl.aida.data.DataFrameStringAxis;
import anl.aida.data.Units;
import anl.aida.data.Variable;
import anl.aida.gis.FeaturePropertyFinder;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.Point;

/**
 * Combines feature name data and data frame values and presents them as a
 * screen annotation.
 * 
 */
public class FeatureOverlay implements PositionListener {

  private FeaturePropertyFinder finder;
  private Point pt = new GeometryFactory().createPoint(new Coordinate(0, 0));
  private String currentName;
  private int timestamp = 0;
  private DataFrame frame;
  private Map<String, String> infoStrings = new HashMap<String, String>();
  private String valueHeading;

  private AnnotationLayer annotations;

  /**
   * @param finder
   * @param model
   */
  public FeatureOverlay(FeaturePropertyFinder finder, DataFrame frame, Model model) {
    this.finder = finder;
    this.frame = frame;
    Variable var = frame.getVariable();
    valueHeading = var.getName() + " (" + Units.getFormattedName(var.getUnit()) + "): ";

    annotations = new AnnotationLayer();
    model.getLayers().add(annotations);
  }

  private void createAnnotation(String text) {
    InfoAnnotation annotation = new InfoAnnotation(text, new java.awt.Point(0, 0));
    int alpha = (int) (255 * .7);
    Color transGray = new Color(Color.GRAY.getRed(), Color.GRAY.getGreen(), Color.GRAY.getBlue(),
	alpha);
    AnnotationAttributes attributes = annotation.getAttributes();
    attributes.setBackgroundColor(transGray);
    attributes.setTextColor(Color.WHITE);
    attributes.setInsets(new Insets(4, 4, 4, 4));
    attributes.setFont(Font.decode("Arial-BOLD-11"));
    attributes.setAdjustWidthToText(Annotation.SIZE_FIT_TEXT);
    attributes.setTextAlign(AVKey.LEFT);
    attributes.setCornerRadius(4);
    annotations.removeAllAnnotations();
    annotations.addAnnotation(annotation);
  }

  private void update() {
    String text = infoStrings.get(currentName);
    if (text == null) {
      DataFrameStringAxis axis = (DataFrameStringAxis) frame.getAxes().getAxis(
	  AxisType.NAMED_LOCATION);
      DataFrameIndex index = frame.getIndex();
      int catIndex = axis.getIndex(currentName.toLowerCase());
      String val = "N/A";
      if (catIndex != -1) {
	index.set(timestamp, catIndex);
	val = String.valueOf(frame.getDouble(index));
      }

      StringBuilder buf = new StringBuilder(currentName);
      buf.append("\nDate: ");
      buf.append(ClientUtils.formatOverlayDate(frame.getAxes().getDate(timestamp)));
      buf.append("\n");
      buf.append(valueHeading);
      buf.append(val);
      text = buf.toString();
      infoStrings.put(currentName, text);
    }

    createAnnotation(text);
  }

  public void timestepChanged(int timestep) {
    this.timestamp = timestep;
    infoStrings.clear();
  }

  @Override
  public void moved(PositionEvent event) {
    Position position = event.getPosition();
    if (position != null) {
      pt.getCoordinate().x = position.getLongitude().getDegrees();
      pt.getCoordinate().y = position.getLatitude().getDegrees();
      finder.locationChanged(pt);
      String name = finder.getValue();
      if (name == null) {
	currentName = null;
	annotations.removeAllAnnotations();
      } else {
	if (currentName == null || !currentName.equals(name)) {
	  currentName = name;
	  update();
	}
      }
    }
  }

  class InfoAnnotation extends ScreenAnnotation {

    public InfoAnnotation(String text, java.awt.Point position, AnnotationAttributes defaults) {
      super(text, position, defaults);
    }

    public InfoAnnotation(String text, java.awt.Point position, Font font, Color textColor) {
      super(text, position, font, textColor);
    }

    public InfoAnnotation(String text, java.awt.Point position, Font font) {
      super(text, position, font);
    }

    public InfoAnnotation(String text, java.awt.Point point) {
      super(text, point);
    }

    @Override
    protected void doRenderNow(DrawContext dc) {
      Rectangle bounds = this.getTextBounds(dc, getText(), getAttributes().getFont(),
	  getAttributes().getTextAlign());
      int y = dc.getDrawableHeight() - (int) bounds.getCenterY();

      setScreenPoint(new java.awt.Point((int) bounds.getCenterX() + 10, y - 25));
      super.doRenderNow(dc);
    }
  }
}
