package anl.aida.gis3D;

import gov.nasa.worldwind.Model;
import gov.nasa.worldwind.layers.Layer;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JPanel;

import com.jidesoft.popup.JidePopup;

public class OptionsWidget {
  
  public static class LayerEnableAction extends AbstractAction {
    private Layer layer;
    
    public LayerEnableAction(Layer layer) {
      super(layer.getName());
      this.layer = layer;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
      layer.setEnabled(!layer.isEnabled());
    }
  }
  
  private JidePopup popup = new JidePopup();
  private JPanel panel;
  private JButton button;
  
  public OptionsWidget(Model model) {
    List<Layer> layers = new ArrayList<Layer>();
    for (Layer layer : model.getLayers()) {
      layers.add(layer);
    }
    
    Collections.sort(layers, new Comparator<Layer>() {
      @Override
      public int compare(Layer o1, Layer o2) {
	return o1.getName().toLowerCase().compareTo(o2.getName().toLowerCase());
      }
    });
    
    
    panel = new JPanel();
    panel.setBorder(BorderFactory.createTitledBorder("Layers"));
    panel.setLayout(new BoxLayout(panel, BoxLayout.PAGE_AXIS));
    for (Layer layer : layers) {
      JCheckBox box = new JCheckBox(new LayerEnableAction(layer));
      box.setSelected(layer.isEnabled());
      panel.add(box);
    }
    
    popup.getContentPane().setLayout(new BorderLayout());
    popup.getContentPane().add(panel, BorderLayout.CENTER);
    popup.setMovable(false);
  }
  
  public JButton getButton() {
    if (button == null) createButton();
    return button;
  }
  
  private void createButton() {
    button = new JButton("Options");
    button.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	popup.setOwner(button);
	if (popup.isPopupVisible()) {
	  popup.hidePopup();
	} else {
	  popup.showPopup();
	}
      }
    });
  }
}
