package anl.aida.palette;

import java.awt.Color;

import org.geotools.brewer.color.BrewerPalette;
import org.geotools.brewer.color.ColorBrewer;

/**
 * @version $Revision$ $Date$
 */
public class Palette {

  private Color[] colors;
  private String description = "";

  public Palette(Palette pal) {
    colors = new Color[pal.colors.length];
    System.arraycopy(pal.colors, 0, colors, 0, pal.colors.length);
    this.description = pal.description;
  }

  public Palette(Color[] colors, String description) {
    this.colors = colors;
    this.description = description;
  }

  public String getDescription() {
    return description;
  }

  public Color[] getColors() {
    return colors;
  }

  public void setColor(int index, Color color) {
    colors[index] = color;
  }

  public int getColorCount() {
    return colors.length;
  }

  public Color getColor(int index) {
    return colors[index];
  }
  
  /*
   * YlOrRd
Grays
PuBuGn
RdPu
BuPu
YlOrBr
Greens
BuGn
GnBu
PuRd
Purples
Blues
Oranges
PuBu
OrRd
Reds
YlGn
YlGnBu
   */
  
  public static Palette getPalette(String name, int colorCount) {
    ColorBrewer brewer = ColorBrewer.instance();
    /*
    BrewerPalette[] palettes = brewer.getPalettes(ColorBrewer.SEQUENTIAL);
    for (BrewerPalette pal : palettes) {
      System.out.println(pal.getName());
    }
    */
    BrewerPalette palette = brewer.getPalette(name);
    return new Palette(palette.getColors(colorCount), palette.getDescription());
  }

  public static Palette getDefaultPalette() {
    ColorBrewer brewer = new ColorBrewer();
    brewer.loadPalettes();
    BrewerPalette palette = brewer.getPalettes(ColorBrewer.QUALITATIVE)[0];
    return new Palette(palette.getColors(8), palette.getDescription());
  }
}
