package anl.aida.palette;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListModel;
import javax.swing.UIManager;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableCellRenderer;

import org.geotools.brewer.color.ColorBrewer;
import org.geotools.brewer.color.PaletteType;

import com.jgoodies.forms.factories.DefaultComponentFactory;
import com.jgoodies.forms.factories.FormFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.ColumnSpec;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.layout.FormSpec;
import com.jgoodies.forms.layout.RowSpec;
import com.jgoodies.forms.layout.Sizes;

/*
 * Created by JFormDesigner on Wed Mar 28 14:11:11 EDT 2007
 */

/**
 * @author User #2
 */
public class PalettePanel extends JPanel {
  private static final long serialVersionUID = -8109206936055484044L;
  private ColorMapTableModel model;
  private boolean autoInterval = true;
  private PaletteType paletteType;

  public PalettePanel() {
    initComponents();
    model = new ColorMapTableModel();
    paletteTable.setModel(model);
    paletteTable.getColumnModel().getColumn(0).setCellRenderer(new CellRenderer());
    paletteTable.getColumnModel().getColumn(1).setCellRenderer(new NumberCellRenderer());

    paletteTable.putClientProperty("terminateEditOnFocusLost", Boolean.TRUE);

    paletteList.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent e) {
	Palette palette = (Palette) paletteList.getSelectedValue();
	if (palette != null) {
	  model.resetPalette(palette);
	}
      }
    });

    paletteTable.setCellSelectionEnabled(true);
    paletteTable.addMouseListener(new MouseAdapter() {
      public void mouseClicked(MouseEvent e) {
	int col = paletteTable.getSelectedColumn();

	if (col == 0) {
	  int row = paletteTable.getSelectedRow();
	  if (row != -1) {
	    Color color = (Color) model.getValueAt(row, 0);
	    color = JColorChooser.showDialog(paletteTable, "Select Palette Color", color);
	    if (color != null) {
	      model.setValueAt(color, row, 0);
	    }
	  }
	}

	if (col == 1) {
	  int row = paletteTable.getSelectedRow();
	  if (row != -1) {
	    String avalue = model.getValueAt(row, 1).toString();
	    if (avalue != null) {
	      try {
		model.setValueAt(avalue, row, 1);
	      } catch (NumberFormatException e1) {
		JOptionPane.showMessageDialog(PalettePanel.this, e1.getMessage(),
		    "Color Map Error", JOptionPane.ERROR_MESSAGE);
	      }
	    }
	  }
	}
      }
    });

    intervalType.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	autoInterval = intervalType.getSelectedIndex() == 0;
	model.setIntervalEditEnabled(!autoInterval);

	if (autoInterval) {
	  minFld.setEnabled(true);
	  maxFld.setEnabled(true);
	  rebuildBtn.setEnabled(true);
	} else {
	  minFld.setEnabled(false);
	  maxFld.setEnabled(false);
	  rebuildBtn.setEnabled(false);
	}
      }
    });

    rebuildBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	String text = minFld.getText().trim();
	double min, max;
	try {
	  min = Double.valueOf(text);
	} catch (NumberFormatException ex) {
	  JOptionPane.showMessageDialog(PalettePanel.this, "'" + text + " is not a valid number",
	      "Color Map Error", JOptionPane.ERROR_MESSAGE);
	  return;
	}

	text = maxFld.getText().trim();

	try {
	  max = Double.valueOf(text);
	} catch (NumberFormatException ex) {
	  JOptionPane.showMessageDialog(PalettePanel.this, "'" + text + " is not a valid number",
	      "Color Map Error", JOptionPane.ERROR_MESSAGE);
	  return;
	}

	if (min > max) {
	  double tmp = max;
	  max = min;
	  min = tmp;
	}
	Palette palette = (Palette) paletteList.getSelectedValue();
	ColorMap map = new ColorMap(palette, min, max);
	map.setPaletteType(getPaletteType());
	model.resetColorMap(map);
	paletteTable.revalidate();
      }
    });
  }

  public void setPalettes(List<Palette> palettes) {
    paletteList.setData(palettes);
    int index = findPaletteIndex(model.getColorMap().getPalette());
    if (palettes.size() > 0) {
      if (index == -1)
	paletteList.setSelectedIndex(0);
      else
	paletteList.setSelectedIndex(index);
      Rectangle bounds = paletteList.getCellBounds(index, index);
      if (bounds != null)
	paletteList.scrollRectToVisible(bounds);
    } else {
      model.resetColorMap(new ColorMap());
    }
  }

  private int findPaletteIndex(Palette palette) {
    ListModel model = paletteList.getModel();
    int index = -1;
    for (int i = 0; i < model.getSize(); i++) {
      Palette p = (Palette) model.getElementAt(i);
      if (p.getDescription().equals(palette.getDescription())) {
	index = i;
	break;
      }
    }

    return index;
  }

  public void initMap(ColorMap colorMap, double min, double max) {
    ColorMap.IntervalType iType = colorMap.getIntervalType();
    if (iType == ColorMap.IntervalType.AUTOMATIC)
      intervalType.setSelectedItem("Automatic");
    else
      intervalType.setSelectedItem("Custom");
    int index = findPaletteIndex(colorMap.getPalette());
    if (index == -1)
      index = 0;

    paletteList.setSelectedIndex(index);
    paletteList.scrollRectToVisible(paletteList.getCellBounds(index, index));

    String minConv = "f";
    String maxConv = "f";

    if (Math.abs(min) < 0.001)
      minConv = "e";

    if (Math.abs(max) < 0.001)
      maxConv = "e";

    minFld.setText(String.format("%.6" + minConv, min));
    maxFld.setText(String.format("%.6" + maxConv, max));
    minFld.setCaretPosition(0);
    maxFld.setCaretPosition(0);

    maxFld.addMouseListener(new MouseAdapter() {
      public void mouseClicked(MouseEvent e) {
	maxFld.selectAll();
	maxFld.requestFocusInWindow();
      }
    });

    minFld.addMouseListener(new MouseAdapter() {
      public void mouseClicked(MouseEvent e) {
	minFld.selectAll();
	minFld.requestFocusInWindow();
      }
    });

    formatFld.setText("%.3f");

    if (minConv.equals("e") || maxConv.equals("e"))
      formatFld.setText("%.3e");

    if (colorMap.getNumberFormat() != null)
      formatFld.setText(getFormat(colorMap.getNumberFormat()));

    model.resetColorMap(colorMap);
    paletteTable.revalidate();
  }

  private void initComponents() {
    // JFormDesigner - Component initialization - DO NOT MODIFY
    // //GEN-BEGIN:initComponents
    // Generated using JFormDesigner non-commercial license
    DefaultComponentFactory compFactory = DefaultComponentFactory.getInstance();
    separator1 = compFactory.createSeparator("Palette");
    scrollPane1 = new JScrollPane();
    paletteList = new PaletteList();
    separator2 = compFactory.createSeparator("");
    label1 = compFactory.createLabel("Interval:");
    intervalType = new JComboBox();
    panel0 = new JPanel();
    panel1 = new JPanel();
    label2 = new JLabel();
    minFld = new JTextField();
    label3 = new JLabel();
    maxFld = new JTextField();
    labelFormat = new JLabel();
    formatFld = new JTextField("1", 6);

    rebuildBtn = new JButton();
    scrollPane2 = new JScrollPane();
    paletteTable = new JTable();
    CellConstraints cc = new CellConstraints();

    // ======== this ========
    setLayout(new FormLayout(new ColumnSpec[] { FormFactory.PREF_COLSPEC,
	FormFactory.LABEL_COMPONENT_GAP_COLSPEC, FormFactory.PREF_COLSPEC,
	FormFactory.LABEL_COMPONENT_GAP_COLSPEC, new ColumnSpec("min(default;150dlu):grow") },
	new RowSpec[] { FormFactory.DEFAULT_ROWSPEC, FormFactory.LINE_GAP_ROWSPEC,
	    new RowSpec(RowSpec.CENTER, Sizes.DEFAULT, 0.5), FormFactory.RELATED_GAP_ROWSPEC,
	    FormFactory.MIN_ROWSPEC, FormFactory.LINE_GAP_ROWSPEC, FormFactory.PREF_ROWSPEC,
	    FormFactory.LINE_GAP_ROWSPEC, FormFactory.PREF_ROWSPEC, FormFactory.LINE_GAP_ROWSPEC,
	    new RowSpec(RowSpec.FILL, Sizes.DEFAULT, 0.5) }));
    add(separator1, cc.xywh(1, 1, 5, 1));

    // ======== scrollPane1 ========
    {
      scrollPane1.setPreferredSize(new Dimension(2, 200));
      scrollPane1.setBackground(UIManager.getColor("List.background"));

      // ---- paletteList ----
      paletteList.setVisibleRowCount(-1);
      paletteList.setOpaque(false);
      scrollPane1.setViewportView(paletteList);
    }
    add(scrollPane1, cc.xywh(1, 3, 5, 1));
    add(separator2, cc.xywh(1, 5, 5, 1));

    // ======== panel0 ========
    {
      panel0.setLayout(new FormLayout(new ColumnSpec[] { FormFactory.DEFAULT_COLSPEC,
	  FormFactory.LABEL_COMPONENT_GAP_COLSPEC,
	  new ColumnSpec(ColumnSpec.FILL, Sizes.DEFAULT, FormSpec.DEFAULT_GROW),
	  FormFactory.LABEL_COMPONENT_GAP_COLSPEC, FormFactory.DEFAULT_COLSPEC,
	  FormFactory.LABEL_COMPONENT_GAP_COLSPEC,
	  new ColumnSpec(ColumnSpec.FILL, Sizes.DEFAULT, FormSpec.DEFAULT_GROW) }, RowSpec
	  .decodeSpecs("default")));

      panel0.add(label1, cc.xy(1, 1));

      intervalType.setPrototypeDisplayValue("Automatic");
      intervalType.setModel(new DefaultComboBoxModel(new String[] { "Automatic", "Custom" }));

      panel0.add(intervalType, cc.xy(3, 1));
      labelFormat.setText("Number Format:");
      panel0.add(labelFormat, cc.xy(5, 1));
      panel0.add(formatFld, cc.xy(7, 1));
    }
    add(panel0, cc.xy(1, 7));

    // ======== panel1 ========
    {
      panel1.setLayout(new FormLayout(new ColumnSpec[] { FormFactory.DEFAULT_COLSPEC,
	  FormFactory.LABEL_COMPONENT_GAP_COLSPEC,
	  new ColumnSpec(ColumnSpec.FILL, Sizes.DEFAULT, FormSpec.DEFAULT_GROW),
	  FormFactory.LABEL_COMPONENT_GAP_COLSPEC, FormFactory.DEFAULT_COLSPEC,
	  FormFactory.LABEL_COMPONENT_GAP_COLSPEC,
	  new ColumnSpec(ColumnSpec.FILL, Sizes.DEFAULT, FormSpec.DEFAULT_GROW) }, RowSpec
	  .decodeSpecs("default")));

      // ---- label2 ----
      label2.setText("Min:");
      panel1.add(label2, cc.xy(1, 1));
      panel1.add(minFld, cc.xy(3, 1));

      // ---- label3 ----
      label3.setText("Max:");
      panel1.add(label3, cc.xy(5, 1));
      panel1.add(maxFld, cc.xy(7, 1));
    }
    add(panel1, cc.xy(1, 9));

    // ---- rebuildBtn ----
    rebuildBtn.setText("Rebuild");
    add(rebuildBtn, cc.xy(5, 9));

    // ======== scrollPane2 ========
    {

      // ---- paletteTable ----
      paletteTable.setPreferredScrollableViewportSize(new Dimension(450, 150));
      scrollPane2.setViewportView(paletteTable);
    }
    add(scrollPane2, cc.xywh(1, 11, 5, 1));
    // JFormDesigner - End of component initialization
    // //GEN-END:initComponents
  }

  // JFormDesigner - Variables declaration - DO NOT MODIFY
  // //GEN-BEGIN:variables
  // Generated using JFormDesigner non-commercial license
  private JComponent separator1;
  private JScrollPane scrollPane1;
  private PaletteList paletteList;
  private JComponent separator2;
  private JLabel label1;
  private JComboBox intervalType;
  private JPanel panel1;
  private JLabel label2;
  private JPanel panel0;
  private JLabel labelFormat;
  private JTextField formatFld;
  private JTextField minFld;
  private JLabel label3;
  private JTextField maxFld;
  private JButton rebuildBtn;
  private JScrollPane scrollPane2;
  private JTable paletteTable;

  // JFormDesigner - End of variables declaration //GEN-END:variables

  public ColorMap getColorMap() throws Exception {
    ColorMap map = model.getColorMap();
    String format = formatFld.getText();

    if (format != null && !format.trim().isEmpty())
      return resetNumberFormat(map, format);

    return map;
  }

  private ColorMap resetNumberFormat(ColorMap map, String format) throws Exception {
    if (format == null)
      return map;

    String temp = format.trim();
    Pattern p = Pattern.compile("^%\\d*\\.\\d*[eEfFgGdD]\\d*$");
    Matcher m = p.matcher(temp);

    if (!m.matches())
      throw new Exception("Number format '" + temp + "'is not valid.\n"
	  + "Example: 0.00123456 formatted with \"%2.3E\" yields \"12.345E-4\"");

    int exponent = temp.toUpperCase().indexOf("E");
    int start = 0;

    while (!Character.isDigit(temp.charAt(start)) && temp.charAt(start) != '.')
      start++;

    int dot = temp.indexOf(".");
    int end = (dot == -1 ? start + 1 : dot + 1);

    while (Character.isDigit(temp.charAt(end)))
      end++;

    String intStr = temp.substring(start, dot);
    String decStr = temp.substring(dot + 1, end);
    int integer = Integer.valueOf(intStr == null || intStr.isEmpty() ? 0 + "" : intStr);
    int decimal = Integer.valueOf(decStr == null || decStr.isEmpty() ? 0 + "" : decStr);

    String fmtStr = "";

    for (int i = 0; i < integer - 1; i++)
      fmtStr += "#";

    fmtStr += "0.";

    for (int i = 0; i < decimal; i++)
      fmtStr += "0";

    if (exponent > 0)
      fmtStr += "E0";

    map.setNumberFormat(new DecimalFormat(fmtStr));

    return map;
  }

  private String getFormat(NumberFormat numberFormat) {
    if (numberFormat == null)
      return " %.3f";

    String format = ((DecimalFormat) numberFormat).toPattern().toUpperCase();
    int exp = format.indexOf("E");
    int dot = format.indexOf(".");
    String forStr = "%" + dot + ".";

    if (exp > 0)
      forStr += (exp - dot) + "E";
    else
      forStr += (format.length() - 1 - dot) + "f";

    return forStr;
  }

  private class CellRenderer extends DefaultTableCellRenderer {
    private static final long serialVersionUID = -4766649744060292439L;

    @Override
    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected,
	boolean hasFocus, int row, int column) {
      JLabel label = (JLabel) super.getTableCellRendererComponent(table, value, isSelected,
	  hasFocus, row, column);
      label.setText("");
      SquareIcon icon = new SquareIcon((Color) value, 12);
      label.setIcon(icon);
      label.setHorizontalAlignment(JLabel.CENTER);
      return label;
    }
  }

  private class NumberCellRenderer extends DefaultTableCellRenderer {
    private static final long serialVersionUID = 4651016553448777932L;

    NumberFormat format;

    public NumberCellRenderer() {
      format = new DecimalFormat();
      format.setMaximumFractionDigits(3);
    }

    public void setValue(Object value) {
      setText((value == null) ? "" : format.format(value));
      setEnabled(!autoInterval);
    }
  }

  public anl.aida.palette.ColorMap.PaletteType getPaletteType() {
    if (paletteType == ColorBrewer.SEQUENTIAL)
      return ColorMap.PaletteType.SEQUENTIAL;

    if (paletteType == ColorBrewer.QUALITATIVE)
      return ColorMap.PaletteType.QUALITATIVE;

    return ColorMap.PaletteType.DIVERGING;
  }

  public void setPaletteType(PaletteType paletteType) {
    this.paletteType = paletteType;
    model.resetPaletteType(getPaletteType());
  }
}
