/**
 * 
 */
package anl.aida.plot;

import static javax.media.opengl.GL.GL_COLOR_BUFFER_BIT;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.media.opengl.DebugGL;
import javax.media.opengl.GL;
import javax.media.opengl.GLAutoDrawable;
import javax.media.opengl.GLCanvas;
import javax.media.opengl.GLCapabilities;
import javax.media.opengl.GLEventListener;
import javax.media.opengl.glu.GLU;
import javax.swing.ButtonGroup;
import javax.swing.JPanel;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;

import anl.aida.client.util.ClientUtils;
import anl.aida.data.DataFrame;
import anl.aida.viz3d.Box;
import anl.aida.viz3d.CanvasListener;
import anl.aida.viz3d.MouseRotate;
import anl.aida.viz3d.MouseTranslate;
import anl.aida.viz3d.MouseWheelZoom;
import anl.aida.viz3d.MouseZoom;
import anl.aida.viz3d.Utils3D;
import anl.aida.viz3d.VNode;
import anl.aida.viz3d.VShapeFactory;
import anl.aida.viz3d.VSpatial;

/**
 * 
 */
public class Canvas3D implements GLEventListener {
  
  private static final String PAN_ICON = "move.png";
  private static final String SELECT_ICON = "select.png";
  
  private enum Mode {PAN, SELECT}
  
  private Mode mode = Mode.SELECT;
  private GLAutoDrawable drawable;
  private JPanel panel;
  
  private GLU glu = new GLU();
  private MouseRotate rotator;
  private MouseTranslate translator;
  private MouseWheelZoom wheelZoomer;
  private MouseZoom zoomer;
  private Color background;
  private NodeSelector selector;
  private InfoDragger infoDragger;
  private Point3f viewLocation = new Point3f();
  
  private List<CanvasListener> listeners = new ArrayList<CanvasListener>();
  private Box worldSizeAtZero;
  private VNode root;
  private boolean initialized = false;
  
  private JToolBar bar;
  
  public Canvas3D(DataFrame frame) {
    VShapeFactory.init();
    GLCapabilities caps = new GLCapabilities();
    caps.setSampleBuffers(true);
    caps.setNumSamples(4);
    drawable = new GLCanvas(caps);
    drawable.addGLEventListener(this);
    
    rotator = new MouseRotate(drawable, 0, 0);
    translator = new MouseTranslate(drawable);
    wheelZoomer = new MouseWheelZoom(drawable);
    zoomer = new MouseZoom(drawable);
    selector = new NodeSelector(this, frame);
    infoDragger = new InfoDragger(frame, this, drawable);
    
    root = new VNode();
    root.id = "ROOT";
    
    setMode(Mode.SELECT);
  }
  
  public NodeSelector getNodeSelector() {
    return selector;
  }
  
  public void display() {
    drawable.display();
  }
  
  /**
   * Called when a VSpatial has been moved using the mouse.
   * 
   * @param spatial the spatial that has been moved
   * @param localTrans the new local translation of the spatial
   */
  public void vSpatialMoved(VSpatial spatial, Point3f localTrans) {
    for (CanvasListener listener : listeners) {
      listener.vSpatialMoved(spatial, localTrans);
    }
  }
  
  /**
   * Adds the specified canvas listener to this canvas.
   * 
   * @param listener the listener to add
   */
  public void addCanvasListener(CanvasListener listener) {
    listeners.add(listener);
  }
  
  /**
   * Clears the selected items.
   */
  public void clearSelections() {
    for (CanvasListener listener : listeners) {
      listener.selectionsCleared();
    }
  }
  
  public void resetSelection() {
    selector.clearSelection();
  }

  /**
   * Adds the selected item.
   * 
   * @param obj the selected item
   */
  public void addSelection(Object obj) {
    for (CanvasListener listener : listeners) {
      listener.itemSelected(obj);
    }
  }
  
  private void createToolBar() {
    bar = new JToolBar();
    bar.setFloatable(false);
    ButtonGroup grp = new ButtonGroup();
    bar.putClientProperty("BTN_GROUP", grp);

    JToggleButton btn = createButton(Mode.PAN);
    btn.setIcon(ClientUtils.loadIcon(PAN_ICON));
    btn.setToolTipText("Enable mouse display panning");
    bar.add(btn);
    grp.add(btn);
    
    btn = createButton(Mode.SELECT);
    btn.setIcon(ClientUtils.loadIcon(SELECT_ICON));
    btn.setToolTipText("Enable mouse node and link selection");
    bar.add(btn);
    grp.add(btn);
    btn.setSelected(true);
  }
  
  private JToggleButton createButton(final Mode mode) {
    JToggleButton button = new JToggleButton();
    button.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	setMode(mode);
      }
    });

    return button;
  }
  
  private void setMode(Mode mode) {
    boolean panEnabled = mode == Mode.PAN;
    this.mode = mode;

    rotator.setEnabled(panEnabled);
    translator.setEnabled(panEnabled);
    //wheelZoomer.setEnabled(panEnabled);
    zoomer.setEnabled(panEnabled);
  }
  
  public JToolBar getToolBar() {
     if (bar == null) createToolBar();
     return bar;
  }

  
  public JPanel getPanel() {
    if (panel == null) {
      panel = new JPanel(new BorderLayout());
      //panel.add(getToolBar(), BorderLayout.NORTH);
      panel.add((Component)drawable, BorderLayout.CENTER);
    }
    return panel;
  }
  
  public void init(GLAutoDrawable drawable) {
    
    if (Boolean.getBoolean("anl.aida.jogl.debug")) {
      System.out.println("jogl debug on");
      drawable.setGL(new DebugGL(drawable.getGL()));
    }
    
    //System.err.println("Chosen GLCapabilities: " + drawable.getChosenGLCapabilities());
    
    GL gl = drawable.getGL();
    background = Color.decode("#99CCFF").darker();
    // Color c = Color.BLACK;
    gl.glClearColor(background.getRed() / 255f, background.getGreen() / 255f,
	background.getBlue() / 255f, 0f);
    gl.setSwapInterval(1);
    gl.glShadeModel(GL.GL_SMOOTH);
    gl.glEnable(GL.GL_DEPTH_TEST);
    gl.glEnable(GL.GL_RESCALE_NORMAL);
    gl.glHint(GL.GL_PERSPECTIVE_CORRECTION_HINT, GL.GL_NICEST);
    gl.glLineWidth(2);
    
    
    float[] lightPosition = {-2, 2, 2, 1};
    float[] lightAmbient = {0, 0, 0, 1};
    float[] lightDiffuse = {1, 1, 1, 1};
    float[] lightSpecular = {1, 1, 1, 1};
    
    gl.glLightfv(GL.GL_LIGHT0, GL.GL_POSITION, lightPosition, 0);
    gl.glLightfv(GL.GL_LIGHT0, GL.GL_AMBIENT, lightAmbient, 0);
    gl.glLightfv(GL.GL_LIGHT0, GL.GL_SPECULAR, lightSpecular, 0);
    gl.glLightfv(GL.GL_LIGHT0, GL.GL_DIFFUSE, lightDiffuse, 0);
    
    gl.glLightModelfv(GL.GL_LIGHT_MODEL_AMBIENT, lightSpecular, 0);
    
    gl.glEnable(GL.GL_LIGHTING);
    gl.glEnable(GL.GL_LIGHT0);
    
    for (CanvasListener listener : listeners) {
      listener.init(drawable, root);
    }
    
    if (!initialized) {
      drawable.addMouseListener(rotator);
      drawable.addMouseMotionListener(rotator);
      drawable.addMouseListener(translator);
      drawable.addMouseMotionListener(translator);
      drawable.addMouseListener(zoomer);
      drawable.addMouseMotionListener(zoomer);
      drawable.addMouseWheelListener(wheelZoomer);
      drawable.addMouseListener(selector);
      drawable.addMouseListener(infoDragger);
      drawable.addMouseMotionListener(infoDragger);
    }
    
    VShapeFactory.getInstance().glInit();
    initialized = true;
  }
  
  @Override
  public void display(GLAutoDrawable drawable) {
    
    GL gl = drawable.getGL();
    gl.glClear(GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
    // anti aliasing
    gl.glEnable(GL.GL_MULTISAMPLE);

    gl.glPushMatrix();
    
    
    Vector3f translation = root.getTranslation();
    translation.set(translator.getTranslation());
    translation.add(wheelZoomer.getTranslation());
    translation.add(zoomer.getTranslation());
    rotator.process(root);
   
    root.draw(gl);

    gl.glPopMatrix();
    
    /*
    BoundingSphere sphere = root.get(4).getBoundingSphere();
    //System.out.println("sphere = " + sphere);
    gl.glPushMatrix();
    gl.glColor3f(0, 0, 0);
    gl.glTranslatef(sphere.getCenterRef().x, sphere.getCenterRef().y, sphere.getCenterRef().z);
    AxisAngle4f angle = new AxisAngle4f();
    angle.set(root.get(3).worldRotation);
    gl.glRotatef((float)Math.toDegrees(angle.angle), angle.x, angle.y, angle.z);
    new GLUT().glutWireSphere(sphere.getRadius(), 32, 12);
    gl.glPopMatrix();
    */
   
    gl.glDisable(GL.GL_MULTISAMPLE);
   
    if (mode == Mode.SELECT) {
      infoDragger.process(gl, viewLocation, root);
      infoDragger.render();
      selector.process(gl, viewLocation, root);
    }
  }

  @Override
  public void displayChanged(GLAutoDrawable drawable, boolean modeChanged, boolean deviceChanged) {
  }
  
  /**
   * Gets a Box representing the dimensions of the viewable world when z is 0.
   * 
   * @return a Box representing the dimensions of the viewable world when z is 0.
   */
  public Box getWorldSizeAtZero() {
    return worldSizeAtZero == null ? new Box() : worldSizeAtZero; 
  }

  @Override
  public void reshape(GLAutoDrawable drawable, int x, int y, int width, int height) {
    GL gl = drawable.getGL();
    float aspect = (float)width / height;

    gl.glMatrixMode(GL.GL_PROJECTION);
    gl.glLoadIdentity();
    glu.gluPerspective(35.0f, aspect, 1.0, 20.0);
    gl.glMatrixMode(GL.GL_MODELVIEW);
    gl.glLoadIdentity();
    glu.gluLookAt(0, 0, 1.3, 0, 0, -1, 0, 1, 0);
    viewLocation.set(0, 0, 1.3f);
    
    worldSizeAtZero = Utils3D.getWorldSizeAtZ(gl, .3f, -18.3f, 0);
    System.out.println("worldSizeAtZero = " + worldSizeAtZero);
    
    for (CanvasListener display : listeners) {
      display.reshape(drawable, worldSizeAtZero.getWidth(), worldSizeAtZero.getHeight(), root);
    }
    root.updateWorldTransformation();
  }
}
