/**
 * 
 */
package anl.aida.plot;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Font;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;

import saf.core.ui.dock.DockableFrame;
import anl.aida.client.gui.AIDAGui;
import anl.aida.client.gui.StructuredDataPanel;
import anl.aida.client.util.ClientUtils;
import anl.aida.plot.io.PlotData;
import anl.aida.plot.io.Project;

/**
 * GUI table contents for saved charts.
 * 
 */
public class Dashboard {
  
  private static final String ICON = "kwikdisk.png";
  private static final String ENTRY_INFO_KEY = "ENTRY_INFO_KEY";
  
  private class MouseForLabel extends MouseAdapter {
    public void mouseClicked(MouseEvent e) {
      JLabel label = (JLabel) e.getSource();
      EntryInfo info = (EntryInfo) label.getClientProperty(ENTRY_INFO_KEY);
      if (info.dockableId == null) {
	info.dockableId = structData.createPlot(info.data);
      } else {
	DockableFrame frame = gui.getDockable(info.dockableId);
	if (frame == null) {
	  // assume that the plot has been closed since it was last created
	  // so make it again.
	  info.dockableId = structData.createPlot(info.data);
	} else {
	  frame.toFront();
	}
      }
    }

    public void mouseEntered(MouseEvent e) {
      JLabel label = (JLabel) e.getSource();
      label.setForeground(Color.RED);
      label.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
    }
    
    public void mouseExited(MouseEvent e) {
      JLabel label = (JLabel) e.getSource();
      label.setForeground(Color.BLACK);
      label.setCursor(null);
    }
  }
  
  private class EntryInfo {
    private PlotData data;
    private String dockableId = null;
  }
  
  private JPanel panel;
  private Font font = new JLabel().getFont().deriveFont(Font.BOLD, 12f);
  private AIDAGui gui;
  private StructuredDataPanel structData;
  
  
  public Dashboard(AIDAGui gui, StructuredDataPanel structData) {
    panel = new JPanel();
    panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
    this.gui = gui;
    this.structData = structData;
  }
  
  public void addProject(Project project) {
    for (PlotData data : project.plots()) {
      addLabel(data);
    }
  }
  
  private void addLabel(PlotData data) {
    JLabel label = new JLabel(data.getTitle(), ClientUtils.loadIcon(ICON), JLabel.LEFT);
    EntryInfo info = new EntryInfo();
    info.data = data;
    label.putClientProperty(ENTRY_INFO_KEY, info);
    label.setFont(font);
    label.setBorder(BorderFactory.createEmptyBorder(2, 5, 0, 0));
    label.addMouseListener(new MouseForLabel());
    
    panel.add(label);
    
  }
  
  public JPanel getPanel() {
    return panel;
  }

}
