package anl.aida.plot;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.jfree.data.category.CategoryDataset;
import org.jfree.data.general.AbstractDataset;

import anl.aida.data.AxisType;
import anl.aida.data.DataFrame;
import anl.aida.data.DataFrameAxis;
import anl.aida.data.DataFrameIndex;
import anl.aida.data.DataFrameTimeAxis;

/**
 * DataFrame adapted to a category data set.
 * 
 */
public class DataFrameCategoryDataset extends AbstractDataset implements CategoryDataset {

  private DataFrame frame;
  private List<DateColumnKey> colKeys = new ArrayList<DateColumnKey>();
  private List<PlotRowKey> rowKeys = new ArrayList<PlotRowKey>();

  /**
   * @param frame
   */
  public DataFrameCategoryDataset(DataFrame frame) {
    this.frame = frame;

    DataFrameTimeAxis timeAxis = (DataFrameTimeAxis) frame.getAxes().getTimeAxis();
    for (int i = timeAxis.getOrigin(), n = timeAxis.getOrigin() + timeAxis.getExtent(); i < n; i++) {
      DateColumnKey key = new DateColumnKey(timeAxis.getDate(i), i);
      colKeys.add(key);
    }

    DataFrameAxis axis = frame.getAxes().getAxis(AxisType.NAMED_LOCATION);
    for (int i = axis.getOrigin(), n = axis.getOrigin() + axis.getExtent(); i < n; i++) {
      PlotRowKey rowKey = new PlotRowKey(axis.getValueAsString(i), i);
      rowKeys.add(rowKey);
    }

    // set up the min, max values on the row keys
    DataFrameIndex index = frame.getIndex();
    for (int i = timeAxis.getOrigin(), n = timeAxis.getOrigin() + timeAxis.getExtent(); i < n; i++) {
      for (int j = axis.getOrigin(), k = axis.getOrigin() + axis.getExtent(); j < k; j++) {
	index.set(i, j);
	rowKeys.get(j).testMinMax(frame.getDouble(index));
      }
    }
  }

  @Override
  public int getColumnIndex(Comparable col) {
    return ((DateColumnKey) col).index;
  }

  @Override
  public Comparable getColumnKey(int index) {
    return colKeys.get(index);
  }

  @Override
  public List getColumnKeys() {
    return colKeys;
  }

  @Override
  public int getRowIndex(Comparable row) {
    return ((PlotRowKey) row).getIndex();
  }

  @Override
  public Comparable getRowKey(int index) {
    return rowKeys.get(index);
  }

  @Override
  public List getRowKeys() {
    return rowKeys;
  }

  @Override
  public Number getValue(Comparable row, Comparable col) {
    int timeIndex = getColumnIndex(col);
    int otherIndex = getRowIndex(row);

    return getValue(otherIndex, timeIndex);
  }

  @Override
  public int getColumnCount() {
    return colKeys.size();
  }

  @Override
  public int getRowCount() {
    return rowKeys.size();
  }

  @Override
  public Number getValue(int row, int col) {
    DataFrameIndex index = frame.getIndex();
    index.set(col, row);
    return frame.getDouble(index);
  }

  public static class DateColumnKey implements Comparable<DateColumnKey> {

    private static DateFormat formatter = new SimpleDateFormat("MM/dd");

    Date date;
    String str;
    int index;

    public DateColumnKey(Date date, int index) {
      this.date = date;
      str = formatter.format(date);
      this.index = index;
    }

    @Override
    public int compareTo(DateColumnKey o) {
      return date.before(o.date) ? -1 : date.equals(o.date) ? 0 : 1;
    }

    public String toString() {
      return str;
    }

  }

}
