/**
 * 
 */
package anl.aida.plot;

import java.awt.Color;
import java.util.ArrayList;
import java.util.List;

import org.geotools.filter.text.cql2.CQL;
import org.geotools.filter.text.cql2.CQLException;
import org.geotools.styling.FeatureTypeStyle;
import org.geotools.styling.LabelPlacement;
import org.geotools.styling.PointPlacement;
import org.geotools.styling.Rule;
import org.geotools.styling.Style;
import org.geotools.styling.StyleBuilder;
import org.geotools.styling.Symbolizer;
import org.geotools.styling.TextSymbolizer;
import org.opengis.filter.Filter;

import anl.aida.data.DataFrame;
import anl.aida.data.DataFrameAxis;
import anl.aida.data.DataFrameIndex;
import anl.aida.gis.Styler;
import anl.aida.palette.ColorMap;

/**
 * 
 */
public class GISTimeStepStyler implements Styler {

  private DataFrame frame;
  private ColorMap map;
  private Style[] styles;

  /**
   * Creates a GISTimeStepStyler from the frame and map.
   * 
   * @param frame
   * @param map
   */
  public GISTimeStepStyler(DataFrame frame, ColorMap map) {
    this.frame = frame;
    this.map = map;

    styles = new Style[frame.getAxes().getTimeAxis().getExtent()];
  }

  @Override
  public Style getStyle(int index) throws PlotCreationException {
    Style style = styles[index];
    if (style == null) {
      try {
	style = createPolygonStyle(map, frame, index);
      } catch (CQLException ex) {
	throw new PlotCreationException(ex);
      }
      styles[index] = style;
    }
    return style;
  }

  private Style createPolygonStyle(ColorMap map, DataFrame frame, int timestep) throws CQLException {
    StyleBuilder builder = new StyleBuilder();

    DataFrameIndex index = frame.getIndex();
    DataFrameAxis axis = frame.getAxes().getNonTimeAxis();
    List<Rule> rules = new ArrayList<Rule>();
    for (int i = 0; i < axis.getExtent(); i++) {
      String county = axis.getValueAsString(i);
      // county = county.substring(0, 1).toUpperCase() + county.substring(1);
      index.set(timestep, i);
      int val = frame.getInt(index);

      TextSymbolizer label = builder.createTextSymbolizer(Color.BLACK, builder.createFont("Arial",
	  10), "NAME");
      PointPlacement placement = builder.createPointPlacement(0.5, 0, 0);
      label.setPlacement(placement);

      TextSymbolizer count = builder.createStaticTextSymbolizer(Color.BLACK, builder.createFont(
	  "Arial", 10), String.valueOf(val));
      placement = builder.createPointPlacement(0.5, 0, 0);
      placement.setDisplacement(builder.createDisplacement(0, -12));
      count.setPlacement(placement);

      Rule rule = builder.getStyleFactory().createRule();
      rule.setSymbolizers(new Symbolizer[] {
	  builder.createPolygonSymbolizer(map.getColorForValue(val), Color.BLACK, 1.0), label,
	  count });

      Filter filter = CQL.toFilter(String.format("NAME like '%s'", county));
      // Filter filter =
      // CQL.toFilter(String.format("strEqualsIgnoreCase(NAME, '%s'", county));
      rule.setFilter(filter);
      rules.add(rule);
    }

    TextSymbolizer label = builder.createTextSymbolizer(Color.BLACK, builder
	.createFont("Arial", 10), "NAME");
    PointPlacement placement = builder.createPointPlacement(0.5, 0, 0);
    label.setPlacement(placement);

    TextSymbolizer count = builder.createStaticTextSymbolizer(Color.BLACK, builder.createFont(
	"Arial", 10), "N/A");
    placement = builder.createPointPlacement(0.5, 0, 0);
    placement.setDisplacement(builder.createDisplacement(0, -12));
    count.setPlacement(placement);

    Rule rule2 = builder.getStyleFactory().createRule();
    Symbolizer poly = builder.createPolygonSymbolizer(Color.GRAY, Color.BLACK, 1.0);

    rule2.setSymbolizers(new Symbolizer[] { poly, label, count });
    rule2.setIsElseFilter(true);
    rules.add(rule2);

    FeatureTypeStyle fts = builder.createFeatureTypeStyle("tl_2008_55_county", rules
	.toArray(new Rule[rules.size()]));
    Style style = builder.createStyle();
    style.addFeatureTypeStyle(fts);
    return style;
  }

}
