package anl.aida.plot;

import java.awt.BorderLayout;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JToolBar;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import anl.aida.client.net.ValueGraphCreator;
import anl.aida.client.net.ValueNode;
import anl.aida.data.DataFrame;
import edu.uci.ics.jung.graph.Graph;

public class GraphPlotRenderer {
  
  private Canvas3D canvas;
  private TSGraphPlotCanvasListener listener;
  private JPanel panel;
  private Map<Integer, Graph<ValueNode, Object>> graphs = new HashMap<Integer, Graph<ValueNode, Object>>();
  private DataFrame frame;
  
  
  public GraphPlotRenderer(DataFrame frame, Graph<ValueNode, Object> graph) {
    canvas = new Canvas3D(frame);
    this.frame = frame;
    graphs.put(0, graph);
    listener = new TSGraphPlotCanvasListener(frame, graph, 0);
    canvas.addCanvasListener(listener);
    TimeStepWidget widget = listener.initToolBar(canvas.getToolBar());
    widget.addSpinnerListener(new ChangeListener() {
      @Override
      public void stateChanged(ChangeEvent e) {
	JSpinner spinner = (JSpinner) e.getSource();
	int timestep = ((Number)spinner.getValue()).intValue();
	// timestep widget adds 1 so the first time step is 
	// user friendly 1
	swapGraph(getGraph(timestep), timestep - 1);
	
      } 
    });
  }
  
  public Graph<ValueNode, Object> getGraph(int timestep) {
    Graph<ValueNode, Object> graph = graphs.get(timestep);
    if (graph == null) {
      graph = new ValueGraphCreator().createGraph(frame, timestep);
      graphs.put(timestep, graph);
    }
    return graph;
  }
  
  public JPanel getPanel() {
    if (panel == null) {
      panel = new JPanel(new BorderLayout());
      panel.add(canvas.getPanel(), BorderLayout.CENTER);
      JToolBar bar = canvas.getToolBar();
      panel.add(bar, BorderLayout.NORTH);
    }
    return panel;
  }
 
  public Canvas3D getCanvas() {
    return canvas;
  }

 /**
  * Swaps the current graph for the specified one.
  * 
  * @param graph
  * @param timestep
  */
  public void swapGraph(Graph<ValueNode, Object> graph, int timestep) {
    // listener needs to add the graph, but if it already has it then
    // tell StackRenderer to jump to that graph
    canvas.clearSelections();
    canvas.resetSelection();
    listener.addGraph(graph, timestep);
  }
}
