package anl.aida.plot;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;

import javax.swing.JComponent;
import javax.swing.JFrame;

import org.jfree.chart.axis.AxisLocation;
import org.jfree.chart.block.BlockBorder;
import org.jfree.chart.renderer.LookupPaintScale;
import org.jfree.chart.title.PaintScaleLegend;
import org.jfree.ui.RectangleEdge;
import org.jfree.ui.RectangleInsets;

import anl.aida.palette.ColorMap;
import anl.aida.palette.PaletteCreator;

/**
 * @version $Revision$ $Date$
 */
public class LegendPanel extends JComponent {

  private PaintScaleLegend legend;

  public LegendPanel(ColorMap map, double min, double max, String units) {
    initLegend(map, min, max, units);
  }

  @Override
  protected void paintComponent(Graphics g) {
    if (isOpaque()) { // paint background
      g.setColor(getBackground());
      g.fillRect(0, 0, getWidth(), getHeight());
    }
    Graphics2D g2d = (Graphics2D) g.create();
    legend.draw(g2d, g.getClipBounds());
    g2d.dispose();
  }

  public Dimension getPreferredSize() {
    return new Dimension(110, 100);
  }

  public Dimension getMinimumSize() {
    return new Dimension(110, 100);
  }

  public void initLegend(ColorMap map, double min, double max, String units) {
    //LookupPaintScale scale = createPaintScale(map, min, max);
    LegendAxis scaleAxis = new LegendAxis(units, min, max, map);
    scaleAxis.setTickMarkPaint(Color.BLACK);
    scaleAxis.setTickLabelFont(new Font("Dialog", Font.PLAIN, 9));
 
    if (legend == null) {
      legend = new PaintScaleLegend(scaleAxis.getPaintScale(), scaleAxis);
      legend.setAxisLocation(AxisLocation.BOTTOM_OR_LEFT);
      legend.setAxisOffset(5.0);
      legend.setMargin(new RectangleInsets(5, 5, 5, 5));
      legend.setFrame(new BlockBorder(Color.red));
      legend.setPadding(new RectangleInsets(10, 10, 10, 10));
      legend.setStripWidth(10);
      legend.setPosition(RectangleEdge.RIGHT);
      legend.setBackgroundPaint(Color.WHITE);
    } else {
      legend.setScale(scaleAxis.getPaintScale());
      legend.setAxis(scaleAxis);
    }
  }

  public PaintScaleLegend getLegend() {
    return legend;
  }

  // creates the legend scale from the color map
  protected LookupPaintScale createPaintScale(ColorMap map, double min, double max) {
    int colorCount = map.getColorCount();
    LookupPaintScale paintScale = new LookupPaintScale(min, max, Color.GRAY);
    if (map.getIntervalType() == ColorMap.IntervalType.AUTOMATIC) {
      double interval = (max - min) / colorCount;
      if (max == min) {
	paintScale.add(new Double(min), map.getColor(0));
      } else {
	for (int i = 0; i < colorCount; i++) {
	  paintScale.add(new Double(min + (i * interval)), map.getColor(i));
	}
      }
    } else {
      // use the values directly from the color map itself
      for (int i = 0; i < colorCount; i++) {
	paintScale.add(new Double(map.getIntervalStart(i)), map.getColor(i));
      }
    }

    return paintScale;
  }

  public static void main(String[] args) {
    ColorMap map = new ColorMap(new PaletteCreator().createPalettes(8).get(0), 0, 10);
    LegendPanel panel = new LegendPanel(map, 0, 10, "Units");
    JFrame frame = new JFrame();
    frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
    frame.setLayout(new BorderLayout());
    frame.add(panel, BorderLayout.CENTER);
    frame.pack();
    frame.setVisible(true);
  }
}
