package anl.aida.plot;

import java.awt.BorderLayout;
import java.awt.Color;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutionException;

import javax.swing.JPanel;
import javax.swing.SwingWorker;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.geotools.filter.text.cql2.CQL;
import org.geotools.filter.text.cql2.CQLException;
import org.geotools.styling.FeatureTypeStyle;
import org.geotools.styling.Rule;
import org.geotools.styling.Style;
import org.geotools.styling.StyleBuilder;
import org.geotools.styling.Symbolizer;
import org.geotools.styling.TextSymbolizer;
import org.opengis.filter.Filter;
import org.opengis.referencing.crs.CoordinateReferenceSystem;

import simphony.util.messages.MessageCenter;
import anl.aida.client.gui.AIDAGui;
import anl.aida.client.gui.ProgressLayer;
import anl.aida.data.Axes;
import anl.aida.data.AxisAttributeConstants;
import anl.aida.data.AxisType;
import anl.aida.data.DataFrame;
import anl.aida.data.DataFrameAxis;
import anl.aida.data.DataFrameIndex;
import anl.aida.data.Units;
import anl.aida.gis.FeaturePropertyFinder;
import anl.aida.gis.GISDisplay;
import anl.aida.gis3D.GIS3DDisplay;
import anl.aida.palette.ColorMap;

public class Map3DPlotCreator extends SwingWorker<Object, Object> {
  
  private static MessageCenter center = MessageCenter.getMessageCenter(Map3DPlotCreator.class);
  
  private JPanel panel;
  private DataFrame dataFrame;
  private AIDAGui gui;
  private GIS3DDisplay display;
  
  public Map3DPlotCreator(DataFrame frame, JPanel panel, AIDAGui gui) {
    this.dataFrame = frame;
    this.panel = panel;
    this.gui = gui;
  }
  
  static {
    System.setProperty("org.geotools.referencing.forceXY", "true");
  }
  
  @Override
  protected Object doInBackground() throws Exception {
    display = new GIS3DDisplay();
    display.init(dataFrame);
    return null;
  }

  @Override
  protected void done() {
    try {
      get();
      
      panel.add(display.getPanel(), BorderLayout.CENTER);
      panel.validate();
      
    } catch (InterruptedException e) {
      center.error("Error while creating map plot", e);
    } catch (ExecutionException e) {
      center.error("Error while creating map plot", e);
    } finally {
     ((ProgressLayer)panel.getClientProperty(ProgressLayer.CLIENT_KEY)).setEnabled(true);
    }
  }

  
  
  private static class TimeStepListener implements ChangeListener {
    
    private TimeStepWidget widget;
    private MapToolTipper tipper;
    /**
     * @param widget
     * @param tipper
     */
    public TimeStepListener(TimeStepWidget widget, MapToolTipper tipper) {
      this.widget = widget;
      this.tipper = tipper;
    }
    
    /* (non-Javadoc)
     * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
     */
    @Override
    public void stateChanged(ChangeEvent e) {
      int tsIndex = widget.getTime();
      tipper.setTimeStepIndex(tsIndex);
    }
  }
}
