package anl.aida.plot;

import java.awt.Component;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import anl.aida.client.gui.AIDAGui;
import anl.aida.data.AxisType;
import anl.aida.data.DataFrame;
import anl.aida.data.DataFrameStringAxis;
import anl.aida.data.StringCoordAxis;
import anl.aida.gis.FeaturePropertyFinder;

/**
 * Makes right click menus over the map.
 * 
 */
public class MapMenuMaker extends MouseAdapter {

  private FeaturePropertyFinder finder;
  private Component parent;
  private DataFrame frame;
  private AIDAGui gui;

  /**
   * @param finder
   */
  public MapMenuMaker(Component parent, DataFrame frame, FeaturePropertyFinder finder, AIDAGui gui) {
    this.finder = finder;
    this.parent = parent;
    this.frame = frame;
    this.gui = gui;
  }

  /*
   * (non-Javadoc)
   * 
   * @see java.awt.event.MouseAdapter#mousePressed(java.awt.event.MouseEvent)
   */
  @Override
  public void mousePressed(MouseEvent e) {
    if (e.isPopupTrigger())
      showPopup(e.getPoint());
  }

  /*
   * (non-Javadoc)
   * 
   * @see java.awt.event.MouseAdapter#mouseReleased(java.awt.event.MouseEvent)
   */
  @Override
  public void mouseReleased(MouseEvent e) {
    if (e.isPopupTrigger())
      showPopup(e.getPoint());
  }

  private void showPopup(Point pt) {
    String attribute = finder.getValue();
    if (attribute != null) {
      DataFrameStringAxis axis = (DataFrameStringAxis) frame.getAxes().getAxis(AxisType.NAMED_LOCATION);
      int index = axis.getIndex(attribute.toLowerCase());
      if (index != StringCoordAxis.INDEX_NOT_FOUND) {
	JPopupMenu menu = new JPopupMenu(attribute);
	JMenuItem item = menu.add("Time Series Chart");
	item.setAction(new ShowTimeSeries(index));
	menu.show(parent, pt.x, pt.y);
      }
    }
  }

  
  @SuppressWarnings("serial")
  private class ShowTimeSeries extends AbstractAction {

    int axisIndex;

    public ShowTimeSeries(int index) {
      super("Time Series Chart");
      axisIndex = index;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    @Override
    public void actionPerformed(ActionEvent e) {
      List<DataFrame> frames = new ArrayList<DataFrame>();
      frames.add(frame);
      gui.addTimeSeries(frames, axisIndex);
    }
  }
}
