package anl.aida.plot;

import java.awt.Color;
import java.awt.Font;
import java.awt.GradientPaint;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.text.SimpleDateFormat;
import java.util.Date;

import ucar.ma2.InvalidRangeException;
import anl.aida.data.AxisType;
import anl.aida.data.DataFrame;
import anl.aida.data.DataFrameIndex;
import anl.aida.data.DataFrameStringAxis;
import anl.aida.data.DataUtilities;
import anl.aida.data.StringCoordAxis;
import anl.aida.data.DataUtilities.MinMax;
import anl.aida.gis.FeaturePropertyFinder;
import edu.umd.cs.piccolo.PCamera;
import edu.umd.cs.piccolo.PCanvas;
import edu.umd.cs.piccolo.event.PBasicInputEventHandler;
import edu.umd.cs.piccolo.event.PInputEvent;
import edu.umd.cs.piccolo.nodes.PPath;
import edu.umd.cs.piccolo.nodes.PText;
import edu.umd.cs.piccolo.util.PBounds;

/**
 * Displays tooltip over a map.
 * 
 */
public class MapToolTipper extends PBasicInputEventHandler implements MouseMotionListener {

  private FeaturePropertyFinder finder;
  private DataFrame frame;
  private int timeIndex = 0;
  private float mpX, mpY;
  private PCamera camera;
  private PCanvas canvas;
  private Font font;

  private PText text;
  private PPath path;
  private boolean added = false;

  private SimpleDateFormat format = new SimpleDateFormat("MM/dd/yyyy");

  /**
   * @param finder
   * @param frame
   */
  public MapToolTipper(DataFrame frame, FeaturePropertyFinder finder, PCanvas canvas) {
    this.finder = finder;
    this.frame = frame;
    this.camera = canvas.getCamera();
    this.canvas = canvas;

    font = new PText().getFont();
    font = font.deriveFont(Font.BOLD, 12f);
  }

  public void showTip(String ttext) {
    // Point2D pt = camera.localToView(new Point2D.Double(x, y));
    // x = (float) pt.getX();
    // y = (float) pt.getY();
    removeTip();
    added = true;
    text = new PText(ttext);
    text.setFont(font);
    text.setConstrainHeightToTextHeight(true);
    text.setConstrainWidthToTextWidth(true);
    text.setTextPaint(Color.WHITE);

    PBounds tBounds = text.getFullBounds();
    float height = (float) tBounds.getHeight() + 20;
    float width = (float) tBounds.getWidth() + 20;

    float x = mpX;
    float y = mpY;
    
    x = x + width < canvas.getWidth() ? x : x - width;
    y = y - height < 0 ? y : y - height;
    path = PPath.createRoundRectangle(x, y, width, height, 20, 20);
    PBounds pBounds = path.getFullBounds();
    path.setPaint(new GradientPaint((float) pBounds.getMinX(), (float) pBounds.getMinY(),
	Color.GRAY, (float) pBounds.getMaxX(), (float) pBounds.getMaxY(), Color.BLACK));
    path.setTransparency(.7f);
    path.setStroke(null);
    text.offset(pBounds.getOrigin().getX() + 10, pBounds.getOrigin().getY() + 10);

    camera.addChild(path);
    camera.addChild(text);
    // camera.addLayer(layer);
  }

  public void removeTip() {
    if (added) {
      camera.removeChild(text);
      camera.removeChild(path);
      // camera.removeLayer(layer);
      // layer.removeAllChildren();
      // camera.removeChild()
      added = false;
    }
  }

  public void mouseDragged(MouseEvent e) {
    mpX = e.getX();
    mpY = e.getY();
    removeTip();
  }

  public void mouseMoved(MouseEvent e) {
    mpX = e.getX();
    mpY = e.getY();
    removeTip();
  }
  
  public void setTimeStepIndex(int index) {
    timeIndex = index;
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umd.cs.piccolo.event.PBasicInputEventHandler#mousePressed(edu.umd.cs
   * .piccolo.event.PInputEvent)
   */
  @Override
  public void mousePressed(PInputEvent event) {
    super.mousePressed(event);
    if (event.getButton() == MouseEvent.BUTTON1) {
      String val = finder.getValue();
      if (val != null) {
	StringBuilder buf = new StringBuilder(val);
	buf.append("\n");
	Date date = frame.getAxes().getDate(timeIndex);
	buf.append(format.format(date));
	buf.append("\n");
	buf.append(frame.getVariable().getName());
	buf.append(": ");

	DataFrameStringAxis axis = (DataFrameStringAxis) frame.getAxes().getAxis(
	    AxisType.NAMED_LOCATION);
	int axisIndex = axis.getIndex(val.toLowerCase());
	if (axisIndex == StringCoordAxis.INDEX_NOT_FOUND) {
	  buf.append("No Data\nMin: No Data\nMax: No Data");
	} else {

	  DataFrameIndex index = frame.getIndex();
	  index.set(timeIndex, axisIndex);
	  double dataVal = frame.getDouble(index);
	  buf.append(dataVal);
	  buf.append("\n");
	  try {
	    MinMax minMax = DataUtilities.minMaxTimeOther(frame, axisIndex);
	    buf.append("Min: ");
	    buf.append(minMax.getMin());
	    buf.append("\n");
	    buf.append("Max: ");
	    buf.append(minMax.getMax());

	  } catch (InvalidRangeException ex) {
	    // this should only happen if something is really wrong
	    // with the data and that would have surfaced long before this
	    ex.printStackTrace();
	  }
	}

	showTip(buf.toString());
      }
    }
  }
}
