/**
 * 
 */
package anl.aida.plot;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.media.opengl.GL;
import javax.vecmath.Point3f;

import anl.aida.client.ShowDocViewOp;
import anl.aida.client.gui.AIDAGui;
import anl.aida.client.net.ValueNode;
import anl.aida.data.DataFrame;
import anl.aida.util.DateRange;
import anl.aida.viz.Colors;
import anl.aida.viz3d.Accumulator;
import anl.aida.viz3d.DefaultAccumulator;
import anl.aida.viz3d.NodeUtils;
import anl.aida.viz3d.Ray3f;
import anl.aida.viz3d.RayCreator;
import anl.aida.viz3d.SelectionManager;
import anl.aida.viz3d.VNode;
import anl.aida.viz3d.VSpatial;

/**
 * Performs node selection via a mouse click.
 * 
 */
public class NodeSelector extends MouseAdapter {
  
  private boolean mousePressed = false;
  private RayCreator rayCreator = new RayCreator();
  private int x, y;
  private Canvas3D canvas;
  private SelectionManager<ValueNode> manager;
  private boolean ctrlDown = false;
  
  private DataFrame frame;
  private ShowDocViewOp docViewOp;
  
  public NodeSelector(Canvas3D canvas, DataFrame frame) {
    this.canvas = canvas;
    manager = new SelectionManager<ValueNode>(Colors.SELECTED_COLOR);
    this.frame = frame;
  }
  
  public void clearSelection() {
    manager.unselectAll();
  }

  /* (non-Javadoc)
   * @see java.awt.event.MouseAdapter#mouseClicked(java.awt.event.MouseEvent)
   */
  @Override
  public void mouseClicked(MouseEvent e) {
    mousePressed = true;
    ctrlDown = e.isControlDown();
    x = e.getX();
    y = e.getY();
    canvas.display();
  }
  
  private void addSelectionsToCanvas() {
    canvas.clearSelections();
    /*
    Set<DocumentID> ids = new HashSet<DocumentID>();
    for (DocumentContainer docs : manager.getSelectedObjects()) {
      for (DocumentID id : docs.documents()) {
        ids.add(id);
      }
    }

    for (DocumentID id : ids) {
      canvas.addSelection(id);
    }
    */
  }
  
  public void process(GL gl, Point3f viewLocation, VNode root) {
    if (mousePressed) {
      Ray3f ray = rayCreator.createRay(gl, viewLocation, x, y);
      Accumulator items = new DefaultAccumulator();
      root.intersects(ray.getOrigin(), ray.getDirection(), items);
      
      DateRange range = null;

      if (!ctrlDown)
	manager.unselectAll();
      // add nodes, if any nodes are added then don't add the edges
      // because those edges are probably the edges terminating "inside" the
      // node
      boolean addEdges = true;
      for (VSpatial item : items.items()) {
	if (NodeUtils.isNode(item)) {
	  manager.addSelectedItem(item);
	  range = NodeUtils.getGraphDate(item);
	  addEdges = false;
	  break;
	}
      }

      if (addEdges) {
	for (VSpatial item : items.items()) {
	  if (NodeUtils.isEdge(item)) {
	    range = NodeUtils.getGraphDate(item);
	    manager.addSelectedItem(item);
	    break;
	  }
	}
      }
      
      mousePressed = false;
      addSelectionsToCanvas();
      canvas.display();
     
      /*
      if (range != null) {
	docViewOp.setElement(cache.getElement(range));
      }
      */
    }
  }
}
