/**
 * 
 */
package anl.aida.plot;

import java.util.HashSet;
import java.util.Set;

import javax.swing.event.TableModelEvent;
import javax.swing.table.AbstractTableModel;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.event.ChartProgressEvent;
import org.jfree.chart.event.ChartProgressListener;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.data.category.CategoryDataset;
import org.jfree.data.category.DefaultCategoryDataset;

/**
 * Wraps a JFreeChart dataset with some useful methods
 * and adapts it to a TableModel.
 * 
 */
public class PlotModel extends AbstractTableModel implements ChartProgressListener {
  
  public static final int NAME_COL = 1;
  public static final int COUNT_COL = 2;
  public static final int MIN_COL = 3;
  public static final int MAX_COL = 4;
  public static final int SELECTED_COL = 0;
 
  private static Class<?>[] COL_CLASSES = {Boolean.class, String.class, Double.class, Double.class, Double.class};
  private String[] columnNames = {" ", "Name", "Current ", "Min. ", "Max. "};
  
  private CategoryDataset dataset;
  private Set<PlotRowKey> selected = new HashSet<PlotRowKey>();
  private int colKey = 0;

  /**
   * @param dataset
   */
  public PlotModel(CategoryDataset dataset, String variableName) {
    this.dataset = dataset;
    selected.addAll(dataset.getRowKeys());
    columnNames[2] = columnNames[2] + variableName;
    columnNames[3] = columnNames[3] + variableName;
    columnNames[4] = columnNames[4] + variableName;
  }
  
  public CategoryDataset getChartDataset() {
    return dataset;
  }

  @Override
  public void chartProgress(ChartProgressEvent evt) {
    if (evt.getType() != ChartProgressEvent.DRAWING_FINISHED) return;
    
    JFreeChart chart = (JFreeChart) evt.getSource();
    CategoryPlot plot = (CategoryPlot) chart.getPlot();
    Comparable key = plot.getDomainCrosshairColumnKey();
    if (key != null) {
      int tmp = dataset.getColumnIndex(key);
      if (tmp != colKey) {
	colKey = tmp;
	fireTableChanged(new TableModelEvent(this, 0, dataset.getRowCount() - 1, COUNT_COL));
      }
    }
  }
  
  public void selectAll() {
    selected.addAll(dataset.getRowKeys());
    fireTableChanged(new TableModelEvent(this, 0, dataset.getRowCount() - 1, SELECTED_COL));
  }
  
  public void unselectAll(){
    selected.clear();
    fireTableChanged(new TableModelEvent(this, 0, dataset.getRowCount() - 1, SELECTED_COL));
  }
  
  public void flipSelectionRows(int[] rows) {
    for (int row : rows) {
      setValueAt(!selected.contains(dataset.getRowKey(row)), row, SELECTED_COL);
    }
  }

  @Override
  public Class<?> getColumnClass(int columnIndex) {
    return COL_CLASSES[columnIndex];
  }

  @Override
  public String getColumnName(int column) {
    return columnNames[column];
  }

  @Override
  public boolean isCellEditable(int rowIndex, int columnIndex) {
    return columnIndex == 0;
  }

  @Override
  public int getColumnCount() {
    return COL_CLASSES.length;
  }

  @Override
  public int getRowCount() {
    return dataset.getRowCount();
  }
  
  public void setValueAt(Object value, int rowIndex, int columnIndex) {
    if (columnIndex == 0) {
      PlotRowKey key = (PlotRowKey) dataset.getRowKey(rowIndex);
      Boolean selected = (Boolean)value;
      if (selected) {
	this.selected.add(key);
      } else {
	this.selected.remove(key);
      }
      fireTableChanged(new TableModelEvent(this, rowIndex, rowIndex, SELECTED_COL));
    }
  }

  @Override
  public Object getValueAt(int rowIndex, int columnIndex) {
    PlotRowKey key = (PlotRowKey) dataset.getRowKey(rowIndex);
    switch (columnIndex) {
    case SELECTED_COL:
      return selected.contains(key);
    case NAME_COL:
      return key.getName();
    case COUNT_COL:
      return dataset.getValue(rowIndex, colKey).doubleValue();
    case MIN_COL:
      return key.getMin();
    case MAX_COL: 
      return key.getMax();
    default:
      return null;
    }
  }
}
