/**
 * 
 */
package anl.aida.plot;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.Map;

import javax.media.opengl.GLAutoDrawable;
import javax.swing.JCheckBox;
import javax.swing.JToolBar;
import javax.vecmath.Point3f;

import anl.aida.client.net.ValueNode;
import anl.aida.data.Axes;
import anl.aida.data.DataFrame;
import anl.aida.data.DataFrameAxis;
import anl.aida.data.DataFrameIndex;
import anl.aida.viz3d.CanvasListener;
import anl.aida.viz3d.StackItem2;
import anl.aida.viz3d.StackItemTextFormatter;
import anl.aida.viz3d.StackKeyHandler;
import anl.aida.viz3d.StackRenderer2;
import anl.aida.viz3d.TSGraphVNode;
import anl.aida.viz3d.VNode;
import anl.aida.viz3d.VSpatial;
import edu.uci.ics.jung.graph.Graph;

/**
 *
 */
public class TSGraphPlotCanvasListener implements CanvasListener {

  private static class ItemFormatter implements StackItemTextFormatter {

    private static DateFormat formatter = new SimpleDateFormat("MMMMM dd, yyyy");

    private Axes<DataFrameAxis> axes;

    /**
     * @param axes
     */
    public ItemFormatter(Axes<DataFrameAxis> axes) {
      this.axes = axes;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * anl.mifs.viz3d.StackItemTextFormatter#format(anl.mifs.viz3d.StackItem2)
     */
    @Override
    public String format(StackItem2 item) {
      int timestep = (int) item.getSortingIndex();
      return formatter.format(axes.getDate(timestep));
    }
  }

  private StackRenderer2 stack;
  private Graph<ValueNode, Object> firstGraph;
  private int timestep;
  private NetworkPlotCreator creator;
  private GLAutoDrawable drawable;
  private Map<Integer, StackItem2> itemMap = new HashMap<Integer, StackItem2>();
  private DataFrame frame;

  private boolean initialized = false;

  /**
   * @param graphs
   */
  public TSGraphPlotCanvasListener(DataFrame frame, Graph<ValueNode, Object> graph, int timestep) {
    this.firstGraph = graph;
    this.timestep = timestep;
    this.frame = frame;
  }

  private float calculateAvg() {
    float sum = 0;
    int count = 0;
    DataFrameAxis timeAxis = frame.getAxes().getTimeAxis();
    DataFrameIndex index = frame.getIndex();
    if (frame.getAxes().getNonTimeAxis() == null) {
      for (int t = 0; t < timeAxis.getExtent(); t++) {
	index.setTime(t);
	float val = frame.getFloat(index);
	if (val != 0) {
	  sum += val;
	  count++;
	}
      }
    } else {
      DataFrameAxis axis = frame.getAxes().getNonTimeAxis();
      for (int t = 0; t < timeAxis.getExtent(); t++) {
	for (int i = 0; i < axis.getExtent(); i++) {
	  index.set(t, i);
	  float val = frame.getFloat(index);
	  if (val != 0) {
	    sum += val;
	    count++;
	  }
	}
      }
    }
    
    return count == 0 ? 0 : sum / count;
  }

  public TimeStepWidget initToolBar(JToolBar bar) {
    bar.addSeparator();
    JCheckBox box = new JCheckBox("Hide Empty Nodes");
    box.setSelected(true);
    box.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	creator.setEmptyNodesVisible(!((JCheckBox) evt.getSource()).isSelected());
	drawable.display();
      }
    });
    bar.add(box);

    bar.addSeparator();
    TimeStepWidget widget = new TimeStepWidget();
    widget.init(frame.getAxes(), timestep);
    bar.add(widget);
    return widget;
  }

  public void addGraph(Graph<ValueNode, Object> graph, int timestep) {
    StackItem2 item = itemMap.get(timestep);
    if (item == null) {
      TSGraphVNode node = new TSGraphVNode();
      creator.addTo(graph, node, timestep);
      item = new StackItem2(node, timestep);
      itemMap.put(timestep, item);
      stack.addStackItem(item);
    } else {
      stack.jumpTo(item, true);
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see anl.mifs.viz3d.CanvasListener#vSpatialMoved(anl.mifs.viz3d.VSpatial,
   * javax.vecmath.Point3f)
   */
  @Override
  public void vSpatialMoved(VSpatial spatial, Point3f localTrans) {
    creator.updateLayout(spatial, localTrans);
  }

  /*
   * (non-Javadoc)
   * 
   * @see anl.mifs.viz3d.CanvasListener#init(javax.media.opengl.GLAutoDrawable,
   * anl.mifs.viz3d.VNode)
   */
  @Override
  public void init(GLAutoDrawable drawable, VNode root) {
    if (!initialized) {
      this.drawable = drawable;
      stack = new StackRenderer2(drawable, new ItemFormatter(frame.getAxes()));
      drawable.addKeyListener(new StackKeyHandler(stack));
      root.addChild(stack);
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * anl.mifs.viz3d.CanvasListener#reshape(javax.media.opengl.GLAutoDrawable,
   * float, float, anl.mifs.viz3d.VNode)
   */
  @Override
  public void reshape(GLAutoDrawable drawable, float width, float height, VNode root) {
    if (!initialized) {
      // KKLayout layout = new KKLayout(firstGraph, new Dimension(10, 10));
      CircleLayout<ValueNode> layout = new CircleLayout<ValueNode>(firstGraph.getVertices(),
	  new Dimension(10, 10));
      creator = new NetworkPlotCreator(width, height, layout, calculateAvg());
      TSGraphVNode node = new TSGraphVNode();
      creator.addTo(firstGraph, node, timestep);
      StackItem2 item = new StackItem2(node, timestep);
      itemMap.put(timestep, item);
      stack.addStackItem(item);
      // root.addChild(item);

      initialized = true;
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see anl.mifs.viz3d.CanvasListener#itemSelected(java.lang.Object)
   */
  @Override
  public void itemSelected(Object obj) {
  }

  /*
   * (non-Javadoc)
   * 
   * @see anl.mifs.viz3d.CanvasListener#selectionsCleared()
   */
  @Override
  public void selectionsCleared() {
  }

}
