/**
 * 
 */
package anl.aida.plot;

import org.jfree.data.time.RegularTimePeriod;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesDataItem;

/**
 * Extends jfreechart's time series to store min and max values for the series.
 * 
 */
public class TimeSeriesMinMax extends TimeSeries {
  
  private double min, max;

  public TimeSeriesMinMax(Comparable name) {
    super(name);
    min = Double.POSITIVE_INFINITY;
    max = Double.NEGATIVE_INFINITY;
  }
  
  private void checkMinMax(double val) {
    if (val < min) min = val;
    if (val > max) max = val;
  }

  /* (non-Javadoc)
   * @see org.jfree.data.time.TimeSeries#add(org.jfree.data.time.RegularTimePeriod, double, boolean)
   */
  @Override
  public void add(RegularTimePeriod period, double value, boolean notify) {
    super.add(period, value, notify);
    checkMinMax(value);
  }

  /* (non-Javadoc)
   * @see org.jfree.data.time.TimeSeries#add(org.jfree.data.time.RegularTimePeriod, double)
   */
  @Override
  public void add(RegularTimePeriod period, double value) {
    super.add(period, value);
    checkMinMax(value);
  }

  /* (non-Javadoc)
   * @see org.jfree.data.time.TimeSeries#add(org.jfree.data.time.RegularTimePeriod, java.lang.Number, boolean)
   */
  @Override
  public void add(RegularTimePeriod period, Number value, boolean notify) {
    super.add(period, value, notify);
    checkMinMax(value.doubleValue());
  }

  /* (non-Javadoc)
   * @see org.jfree.data.time.TimeSeries#add(org.jfree.data.time.RegularTimePeriod, java.lang.Number)
   */
  @Override
  public void add(RegularTimePeriod period, Number value) {
    super.add(period, value);
    checkMinMax(value.doubleValue());
  }

  /* (non-Javadoc)
   * @see org.jfree.data.time.TimeSeries#add(org.jfree.data.time.TimeSeriesDataItem, boolean)
   */
  @Override
  public void add(TimeSeriesDataItem item, boolean notify) {
    super.add(item, notify);
    checkMinMax(item.getValue().doubleValue());
  }

  /* (non-Javadoc)
   * @see org.jfree.data.time.TimeSeries#add(org.jfree.data.time.TimeSeriesDataItem)
   */
  @Override
  public void add(TimeSeriesDataItem item) {
    super.add(item);
    checkMinMax(item.getValue().doubleValue());
  }

  /**
   * Gets the min value of this series.
   * 
   * @return the min value of this series.
   */
  public double getMin() {
    return min;
  }

  /**
   * Gets the max value of this series.
   * 
   * @return the max value of this series.
   */
  public double getMax() {
    return max;
  }
 
  
  
  
  
  

}
