/**
 * 
 */
package anl.aida.plot;

import org.jfree.data.time.TimeSeriesCollection;

/**
 * Parameters for the creation of a time series widget.
 * 
 */
public interface TimeSeriesParameters {
  
  /**
   * Gets the heading for the series.
   * 
   * @return the heading for the series.
   */
  String getSeriesHeading();
  
  
  /**
   * Gets the dataset for the time series.
   * 
   * @return the dataset for the time series.
   */
  TimeSeriesCollection getDataset();
  
  /**
   * Gets the minimum value of the time series.
   * 
   * @return the minimum value of the time series.
   */
  double getMin();
  
  /**
   * Gets the maximum value of the time series.
   * 
   * @return the maximum value of the time series.
   */
  double getMax();
  
  /**
   * Gets the title of the time series.
   * 
   * @return the title of the time series.
   */
  String getTitle();
  
  /**
   * Gets the label for the x-axis.
   * 
   * @return the label for the x-axis.
   */
  String getXAxisLabel();
  
  /**
   * Gets the label for the y-axis.
   * 
   * @return the label for the y-axis.
   */
  String getYAxisLabel();
  
  /**
   * Gets the label for the units, if any.
   * 
   * @return the label for the units, if any.
   */
  String getUnitsLabel();
  

}
