/**
 * 
 */
package anl.aida.plot;

import java.awt.Color;
import java.text.SimpleDateFormat;
import java.util.TimeZone;

import javax.swing.JPanel;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.ui.RectangleInsets;

/**
 * 
 */
public class TimeSeriesPlot {

  private TimeSeriesCollection dataset;
  private JFreeChart chart;
  private ChartPanel chartPanel;
  private double min, max;

  public TimeSeriesPlot(TimeSeriesParameters params) {
    this.dataset = params.getDataset();
    max = params.getMax();
    min = params.getMin();
    createChart(params);
    chartPanel = new ChartPanel(chart);
    chartPanel.setMouseWheelEnabled(true);
    chartPanel.setFillZoomRectangle(true);
  }

  public JPanel getPanel() {
    return chartPanel;
  }

  public TimeSeriesCollection getDataset() {
    return dataset;
  }

  public XYLineAndShapeRenderer getRenderer() {
    return (XYLineAndShapeRenderer) ((XYPlot) chart.getPlot()).getRenderer();
  }

  public JFreeChart getChart() {
    return chart;
  }

  private void createChart(TimeSeriesParameters params) {
    chart = ChartFactory.createTimeSeriesChart(params.getTitle(), // title
	params.getXAxisLabel(), // x-axis label
	params.getYAxisLabel(), // y-axis label
	dataset, // data
	false, // create legend?
	true, // generate tooltips?
	false // generate URLs?
	);

    chart.setBackgroundPaint(Color.white);
    // chart.addProgressListener(this);

    XYPlot plot = (XYPlot) chart.getPlot();
    plot.setBackgroundPaint(Color.WHITE);
    plot.setDomainGridlinePaint(Color.GRAY);
    plot.setRangeGridlinePaint(Color.GRAY);
    plot.setAxisOffset(new RectangleInsets(5.0, 5.0, 5.0, 5.0));
    plot.setDomainCrosshairVisible(true);
    plot.setRangeCrosshairVisible(true);

    NumberAxis rAxis = (NumberAxis) plot.getRangeAxis();
    double tenth = (max - min) / 10;
    if (tenth == 0) {
      rAxis.setRangeAboutValue(min, .01);
    } else {
      rAxis.setRange(min - tenth, max + tenth);
    }
    rAxis.setAutoRange(false);

    XYLineAndShapeRenderer renderer = (XYLineAndShapeRenderer) plot.getRenderer();
    renderer.setBaseShapesVisible(true);
    renderer.setBaseShapesFilled(true);

    DateAxis axis = (DateAxis) plot.getDomainAxis();
    axis.setTimeZone(TimeZone.getTimeZone("UTC"));
    // new SimpleDateFormat("MMMMM dd, yyyy HH:mm:ss z");
    SimpleDateFormat dateFormat = new SimpleDateFormat("MM/dd/yyyy");
    dateFormat.setTimeZone(TimeZone.getTimeZone("UTC"));
    axis.setDateFormatOverride(dateFormat);
    axis.setVerticalTickLabels(true);
  }
}
