/**
 * 
 */
package anl.aida.plot;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyEvent;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.LayoutFocusTraversalPolicy;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableRowSorter;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.LegendItem;
import org.jfree.chart.LegendItemCollection;
import org.jfree.chart.event.ChartProgressEvent;
import org.jfree.chart.event.ChartProgressListener;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.time.FixedMillisecond;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesDataItem;

import anl.aida.client.util.ClientUtils;
import anl.aida.table.TableCreator;

/**
 * GUI widget that combines a time series plot with a table showing the series
 * values.
 * 
 */
public class TimeSeriesPlotWidget implements ChartProgressListener {

  @SuppressWarnings("serial")
  private class LegendRenderer extends DefaultTableCellRenderer {

    @Override
    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected,
	boolean hasFocus, int row, int column) {
      JLabel comp = (JLabel) super.getTableCellRendererComponent(table, value, isSelected,
	  hasFocus, row, column);
      Icon icon = iconMap.get(comp.getText());
      comp.setIcon(icon);

      return comp;
    }
  }

  private static final String SPACE_SELECTION_KEY = "SPACE_SELECTION_KEY";

  private JPanel panel;
  private Map<String, Icon> iconMap = new HashMap<String, Icon>();
  private XYLineAndShapeRenderer renderer;
  private TimeSeriesTableModel model;
  private JTable table;
  private JTable dataTable;
  private JLabel rangeLabel = new JLabel("Current Date: N/A");
  private JSplitPane pane;
  private JFreeChart chart;


  public TimeSeriesPlotWidget(TimeSeriesParameters params) {
    TimeSeriesPlot plot = new TimeSeriesPlot(params);
    chart = plot.getChart();
    
    model = new TimeSeriesTableModel(params);

    renderer = (XYLineAndShapeRenderer) plot.getRenderer();
    plot.getChart().addProgressListener(model);
    plot.getChart().addProgressListener(this);

    LegendItemCollection items = renderer.getLegendItems();
    for (int i = 0; i < items.getItemCount(); i++) {
      LegendItem item = items.get(i);
      iconMap.put(item.getLabel(), new ShapeIcon(item.getShape(), item.getFillPaint()));
    }

    panel = new JPanel(new BorderLayout());
    panel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
    createTable();

    pane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
    pane.setTopComponent(plot.getPanel());

    JPanel tablePanel = new JPanel(new BorderLayout());
    final JScrollPane scrollPane = new JScrollPane(table);

    JTabbedPane tabs = new JTabbedPane();
    tabs.addTab("Current Timestep", tablePanel);

    JTable dataTable = createDataTable(params);
    tabs.addTab("All Timesteps", new JScrollPane(dataTable));
    tabs.setTabPlacement(JTabbedPane.RIGHT);
    pane.setBottomComponent(tabs);
    pane.setDividerLocation(400);

    tablePanel.add(scrollPane, BorderLayout.CENTER);
    JToolBar bar = new JToolBar();
    bar.setBorder(BorderFactory.createEmptyBorder(0, 5, 5, 5));
    bar.setFloatable(false);

    JButton unselectBtn = new JButton();
    unselectBtn.setIcon(ClientUtils.loadIcon("deselect_multiple.png"));
    unselectBtn.setToolTipText("Hide all " + params.getSeriesHeading() + " in Graph");
    unselectBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	TimeSeriesPlotWidget.this.model.unselectAll();
      }
    });

    JButton selectBtn = new JButton();
    selectBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	TimeSeriesPlotWidget.this.model.selectAll();
      }
    });
    selectBtn.setToolTipText("Display all " + params.getSeriesHeading() + " in Graph");
    selectBtn.setIcon(ClientUtils.loadIcon("select_multiple.png"));

    bar.add(selectBtn);
    bar.add(unselectBtn);
    bar.addSeparator();
    bar.add(rangeLabel);
    tablePanel.add(bar, BorderLayout.NORTH);
    panel.add(pane, BorderLayout.CENTER);

    TimeSeries series = model.getChartDataset().getSeries(0);
    TimeSeriesDataItem item = series.getDataItem(0);
    ((XYPlot) plot.getChart().getPlot()).setDomainCrosshairValue(item.getPeriod()
	.getFirstMillisecond(), true);
    model.addTableModelListener(new TableModelListener() {
      @Override
      public void tableChanged(TableModelEvent e) {
	if (e.getColumn() == TimeSeriesTableModel.SELECTED_COL) {
	  for (int row = e.getFirstRow(); row <= e.getLastRow(); row++) {
	    Boolean isVisible = (Boolean) TimeSeriesPlotWidget.this.model.getValueAt(row,
		TimeSeriesTableModel.SELECTED_COL);
	    renderer.setSeriesVisible(row, isVisible);
	  }
	}
      }
    });
  }

  private JTable createDataTable(TimeSeriesParameters params) {
    dataTable = new TableCreator().createTable(params);
    dataTable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {

      private boolean isAdjusting = false;

      @Override
      public void valueChanged(ListSelectionEvent e) {
	if (!isAdjusting) {
	  isAdjusting = true;
	  if (!e.getValueIsAdjusting()) {
	    int row = dataTable.getSelectedRow();
	    if (row != -1) {
	      Date date = (Date) dataTable.getValueAt(row, 0);
	      // set the chart's crosshair to that date
	      ((XYPlot) chart.getPlot()).setDomainCrosshairValue(date.getTime());
	    }
	  }
	  isAdjusting = false;
	}
      }
    });
    return dataTable;
  }

  public void setDividerLocation(double loc) {
    pane.setDividerLocation(loc);
  }

  public void chartProgress(ChartProgressEvent evt) {
    if (evt.getType() != ChartProgressEvent.DRAWING_FINISHED)
      return;

    JFreeChart chart = (JFreeChart) evt.getSource();
    XYPlot plot = (XYPlot) chart.getPlot();
    double key = plot.getDomainCrosshairValue();
    TimeSeries series = model.getChartDataset().getSeries(0);
    Date date = new Date((long) key);
    TimeSeriesDataItem item = series.getDataItem(new FixedMillisecond(date));
    if (item == null) {
      rangeLabel.setText("Current Date: N/A");
    } else {
      rangeLabel.setText("Current Date: " + ClientUtils.formatShortDate(date));
    }

    // some machine don't show the visual updates to the table
    // when cross hair changes, this will hopefully force it.
    table.validate();
    table.repaint();

    for (int i = 0, n = dataTable.getRowCount(); i < n; i++) {
      date = (Date) dataTable.getValueAt(i, 0);
      if (date.getTime() == key) {
	dataTable.setRowSelectionInterval(i, i);
	dataTable.scrollRectToVisible(dataTable.getCellRect(i, 0, true));
	break;
      }
    }
  }

  @SuppressWarnings("serial")
  private void createTable() {
    table = new JTable(model);
    table.setRowHeight(table.getRowHeight() + 2);
    table.getColumnModel().getColumn(TimeSeriesTableModel.NAME_COL).setCellRenderer(
	new LegendRenderer());
    table.addComponentListener(new ComponentAdapter() {
      public void componentResized(ComponentEvent e) {
	table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
	table.getColumnModel().getColumn(0).setPreferredWidth(40);
	table.getColumnModel().getColumn(1).setPreferredWidth(180);
	table.getColumnModel().getColumn(2).setPreferredWidth(120);
	table.getColumnModel().getColumn(3).setPreferredWidth(120);
	table.getColumnModel().getColumn(4).setPreferredWidth(120);

	table.removeComponentListener(this);
      }
    });
    table.setFocusTraversalPolicy(new LayoutFocusTraversalPolicy());
    table.setFocusTraversalPolicyProvider(true);

    KeyStroke keystroke = KeyStroke.getKeyStroke(KeyEvent.VK_SPACE, 0);

    table.getActionMap().put(SPACE_SELECTION_KEY, new AbstractAction() {
      public void actionPerformed(ActionEvent evt) {
	flipSelectedRows();
      }
    });
    table.getInputMap(JComponent.WHEN_FOCUSED).put(keystroke, SPACE_SELECTION_KEY);

    TableRowSorter<TimeSeriesTableModel> sorter = new TableRowSorter<TimeSeriesTableModel>(model);
    sorter.setSortsOnUpdates(true);
    // sorter.setSortable(PlotModel.SELECTED_COL, false);
    table.setRowSorter(sorter);
    sorter.toggleSortOrder(TimeSeriesTableModel.NAME_COL);
  }

  private int[] viewRowsToModelRows(int[] rows) {
    int[] modelRows = new int[rows.length];
    for (int i = 0; i < rows.length; i++) {
      modelRows[i] = table.convertRowIndexToModel(rows[i]);
    }
    return modelRows;

  }

  private void flipSelectedRows() {
    int[] rows = table.getSelectedRows();
    rows = viewRowsToModelRows(rows);
    model.flipSelectionRows(rows);
  }

  public JPanel getPanel() {
    return panel;
  }
}
