/**
 * 
 */
package anl.aida.plot;

import java.util.Date;
import java.util.HashSet;
import java.util.Set;

import javax.swing.event.TableModelEvent;
import javax.swing.table.AbstractTableModel;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.event.ChartProgressEvent;
import org.jfree.chart.event.ChartProgressListener;
import org.jfree.chart.plot.XYPlot;
import org.jfree.data.time.FixedMillisecond;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;

/**
 * Wraps a JFreeChart dataset with some useful methods and adapts it to a
 * TableModel.
 * 
 */
@SuppressWarnings("serial")
public class TimeSeriesTableModel extends AbstractTableModel implements ChartProgressListener {

  public static final int NAME_COL = 1;
  public static final int COUNT_COL = 2;
  public static final int MIN_COL = 3;
  public static final int MAX_COL = 4;
  public static final int SELECTED_COL = 0;

  private static Class<?>[] COL_CLASSES = { Boolean.class, String.class, Double.class,
      Double.class, Double.class };
  private String[] columnNames = { " ", "Name", "Current ", "Min. ", "Max. " };

  private TimeSeriesCollection dataset;
  private Set<TimeSeriesMinMax> selected = new HashSet<TimeSeriesMinMax>();
  private int colKey = 0;

  /**
   * @param dataset
   */
  public TimeSeriesTableModel(TimeSeriesParameters params) {
    this.dataset = params.getDataset();
    selected.addAll(dataset.getSeries());
    columnNames[1] = params.getSeriesHeading();
    columnNames[2] = columnNames[2] + " " + params.getUnitsLabel();
    columnNames[3] = columnNames[3] + " " + params.getUnitsLabel();
    columnNames[4] = columnNames[4] + " " + params.getUnitsLabel();
  }

  public TimeSeriesCollection getChartDataset() {
    return dataset;
  }

  @Override
  public void chartProgress(ChartProgressEvent evt) {
    if (evt.getType() != ChartProgressEvent.DRAWING_FINISHED)
      return;

    JFreeChart chart = (JFreeChart) evt.getSource();
    XYPlot plot = (XYPlot) chart.getPlot();
    double key = plot.getDomainCrosshairValue();
    TimeSeries series = dataset.getSeries(0);
    int index = series.getIndex(new FixedMillisecond(new Date((long) key)));
    //System.out.println("key = " + key);
    //System.out.println(new java.util.Date((long) key));

    if (index != -1 && index != colKey) {
      colKey = index;
      fireTableChanged(new TableModelEvent(this, 0, dataset.getSeriesCount() - 1, COUNT_COL));
    }
  }

  public void selectAll() {
    selected.addAll(dataset.getSeries());
    fireTableChanged(new TableModelEvent(this, 0, dataset.getSeriesCount() - 1, SELECTED_COL));
  }

  public void unselectAll() {
    selected.clear();
    fireTableChanged(new TableModelEvent(this, 0, dataset.getSeriesCount() - 1, SELECTED_COL));
  }

  public void flipSelectionRows(int[] rows) {
    for (int row : rows) {
      setValueAt(!selected.contains(dataset.getSeries(row)), row, SELECTED_COL);
    }
  }

  @Override
  public Class<?> getColumnClass(int columnIndex) {
    return COL_CLASSES[columnIndex];
  }

  @Override
  public String getColumnName(int column) {
    return columnNames[column];
  }

  @Override
  public boolean isCellEditable(int rowIndex, int columnIndex) {
    return columnIndex == 0;
  }

  @Override
  public int getColumnCount() {
    return COL_CLASSES.length;
  }

  @Override
  public int getRowCount() {
    return dataset.getSeriesCount();
  }

  public void setValueAt(Object value, int rowIndex, int columnIndex) {
    if (columnIndex == 0) {
      TimeSeriesMinMax key = (TimeSeriesMinMax) dataset.getSeries(rowIndex);
      Boolean selected = (Boolean) value;
      if (selected) {
	this.selected.add(key);
      } else {
	this.selected.remove(key);
      }
      fireTableChanged(new TableModelEvent(this, rowIndex, rowIndex, SELECTED_COL));
    }
  }

  @Override
  public Object getValueAt(int rowIndex, int columnIndex) {
    TimeSeriesMinMax key = (TimeSeriesMinMax) dataset.getSeries(rowIndex);
    switch (columnIndex) {
    case SELECTED_COL:
      return selected.contains(key);
    case NAME_COL:
      return key.getKey();
    case COUNT_COL:
      return key.getValue(colKey).doubleValue();
    case MIN_COL:
      return key.getMin();
    case MAX_COL:
      return key.getMax();
    default:
      return null;
    }
  }
}
