package anl.aida.plot;

import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Date;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.Timer;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import anl.aida.client.util.ClientUtils;
import anl.aida.data.Axes;
import anl.aida.data.CoordAxis;
import anl.aida.data.DataFrameAxis;

import com.jgoodies.forms.factories.FormFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.ColumnSpec;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.layout.RowSpec;

/*
 * Created by JFormDesigner on Tue Mar 20 14:20:17 EDT 2007
 */

/**
 * @author User #2
 */
@SuppressWarnings("serial")
public class TimeStepWidget extends JPanel {

  private static final String PLAY_ICON = "player_play.png";
  private static final String PAUSE_ICON = "player_pause.png";
  private static final String STOP_ICON = "player_stop.png";

  private class TimerAction implements ActionListener {

    int min, max, current;

    public TimerAction(int min, int max) {
      this.min = min;
      this.max = max;
      current = min;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
      if (current > max) {
	stop();
	current = min;
      } else {
	timeSpinner.setValue(current++);
      }
    }
  }

  private class SpinnerListener implements ChangeListener {
    public void stateChanged(ChangeEvent e) {
      JSpinner source = (JSpinner) e.getSource();
      if (spinnersOn) {
	int val = ((Number) source.getValue()).intValue() - 1;
	if (axes != null) {
	  Date date = axes.getDate(val);
	  timeLabel.setText(ClientUtils.formatShortDate(date));
	}
      }
    }
  }

  private Axes<DataFrameAxis> axes;
  private boolean spinnersOn = false;
  private Icon playIcon, pauseIcon;
  private boolean playing = false;
  private Timer timer;
  private int timerDelay = 1;

  public TimeStepWidget() {
    setBorder(BorderFactory.createEmptyBorder(4, 0, 0, 0));
    initComponents();
    timeSpinner.addChangeListener(new SpinnerListener());
   

    pauseIcon = ClientUtils.loadIcon(PAUSE_ICON);
    playIcon = ClientUtils.loadIcon(PLAY_ICON);
    playBtn.setText("");
    playBtn.setIcon(playIcon);
    playBtn.setBorderPainted(false);
    playBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	if (playing)
	  pause();
	else
	  play();

      }
    });

    stopBtn.setText("");
    stopBtn.setIcon(ClientUtils.loadIcon(STOP_ICON));
    stopBtn.setBorderPainted(false);
    stopBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	stop();
      }
    });
    stopBtn.setEnabled(false);
  }
  
  public int getTimerDelay() {
    return timerDelay;
  }
  
  public void setTimerDelay(int timerDelay) {
    this.timerDelay = timerDelay;
  }

  private void configTimer() {
    SpinnerNumberModel model = (SpinnerNumberModel) timeSpinner.getModel();
    int min = (Integer) model.getMinimum();
    int max = (Integer) model.getMaximum();
    timer = new Timer(0, new TimerAction(min, max));
    timer.setDelay(timerDelay);
  }

  private void play() {
    playing = true;
    playBtn.setToolTipText("Pause the map animation");
    playBtn.setIcon(pauseIcon);
    if (!stopBtn.isEnabled()) {
      configTimer();
    }
    stopBtn.setEnabled(true);
    timer.start();
  }

  private void pause() {
    playing = false;
    playBtn.setToolTipText("Start the map animation");
    playBtn.setIcon(playIcon);
    timer.stop();

  }

  private void stop() {
    playing = false;
    timer.stop();
    playBtn.setIcon(playIcon);
    playBtn.setToolTipText("Animate the map over time");
    stopBtn.setEnabled(false);
  }

  public void addSpinnerListener(ChangeListener time) {
    timeSpinner.addChangeListener(time);
  }

  public int getTime() {
    return ((Integer) timeSpinner.getValue()).intValue() - 1;
  }

  public int[] getAllTimeSteps() {
    if (noTimeSteps)
      return new int[0];

    return getAllSteps(timeSpinner);
  }

  private int[] getAllSteps(JSpinner spinner) {
    SpinnerNumberModel model = (SpinnerNumberModel) spinner.getModel();
    int min = (Integer) model.getMinimum();
    int max = (Integer) model.getMaximum();
    int step = model.getStepSize().intValue();

    int[] steps = new int[(max - min) / step + 1];
    int len = steps.length;
    steps[0] = min - 1;
    steps[len - 1] = max - 1;

    for (int i = 1; i < len - 1; i++)
      steps[i] = steps[i - 1] + step;

    return steps;
  }

  public void setTime(int step) {
    timeSpinner.setValue(new Integer(step) + 1);
  }

  public boolean hasNoTimeSteps() {
    return noTimeSteps;
  }

  public void init(Axes<DataFrameAxis> axes, int timeStep) {
    this.axes = axes;
    spinnersOn = false;
    CoordAxis time = axes.getTimeAxis();
    int min = time.getRange().getOrigin() + 1;
    int max = min + (int) time.getRange().getExtent() - 1;
    SpinnerNumberModel model = (SpinnerNumberModel) timeSpinner.getModel();
    model.setMinimum(min);
    model.setMaximum(max);
    model.setValue(new Integer(timeStep + 1));
    Date date = axes.getDate(timeStep);
    timeLabel.setText(ClientUtils.formatShortDate(date));

    timeSpinner.setEnabled(min != max);
    noTimeSteps = (min == max);
    spinnersOn = true;
  }

  private void initComponents() {
    // JFormDesigner - Component initialization - DO NOT MODIFY
    // //GEN-BEGIN:initComponents
    // Generated using JFormDesigner non-commercial license
    label1 = new JLabel();
    timeSpinner = new JSpinner();
    timeLabel = new JLabel();
    playBtn = new JButton("Play");
    stopBtn = new JButton("Stop");
    CellConstraints cc = new CellConstraints();

    // ======== this ========
    setLayout(new FormLayout(new ColumnSpec[] { FormFactory.PREF_COLSPEC,
	FormFactory.LABEL_COMPONENT_GAP_COLSPEC, new ColumnSpec("max(min;40dlu)"),
	FormFactory.LABEL_COMPONENT_GAP_COLSPEC, FormFactory.DEFAULT_COLSPEC /*
									      * new
									      * ColumnSpec
									      * (
									      * "max(default;50dlu)"
									      * )
									      */,
	FormFactory.UNRELATED_GAP_COLSPEC, FormFactory.DEFAULT_COLSPEC,
	FormFactory.RELATED_GAP_COLSPEC, FormFactory.DEFAULT_COLSPEC }, RowSpec
	.decodeSpecs("default")));

    // ---- label1 ----
    label1.setText("Time Step:");
    add(label1, cc.xy(1, 1));
    add(timeSpinner, cc.xy(3, 1));

    // ---- timeLabel ----
    timeLabel.setFont(new Font("Tahoma", Font.BOLD, 11));
    add(timeLabel, cc.xy(5, 1));

    add(playBtn, cc.xy(7, 1));
    add(stopBtn, cc.xy(9, 1));

    // ---- label2 ----
    // JFormDesigner - End of component initialization //GEN-END:initComponents
  }

  // JFormDesigner - Variables declaration - DO NOT MODIFY //GEN-BEGIN:variables
  // Generated using JFormDesigner non-commercial license
  private JLabel label1;
  private JSpinner timeSpinner;
  private JLabel timeLabel;
  private JButton playBtn;
  private JButton stopBtn;
  private boolean noTimeSteps = false;
  // JFormDesigner - End of variables declaration //GEN-END:variables
}
