package anl.aida.plot.io;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;

import javax.xml.namespace.QName;
import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.events.Attribute;
import javax.xml.stream.events.EndElement;
import javax.xml.stream.events.StartElement;
import javax.xml.stream.events.XMLEvent;

import anl.aida.plot.PlotType;
import anl.aida.plot.io.PlotData;
import anl.aida.plot.io.Project;

/**
 * Reads a project file.
 * 
 */
public class ProjectReader {

  /*
   * <chart type="MAP"> 
   * <formula>AH3[3]</formula>
   */

  private static final String E_CHART = "plot";
  private static final String E_FORMULA = "formula";
  private static final String E_DATASET = "dataset";
  private static final QName A_TYPE = new QName("type");
  private static final QName A_TITLE = new QName("title");
  private static final QName A_CLASSNAME = new QName("classname");

  public Project read(File file) throws IOException, XMLStreamException {
    XMLInputFactory factory = XMLInputFactory.newInstance();
    XMLEventReader reader = factory.createXMLEventReader(new BufferedReader(new FileReader(file)));
    Project project = new Project();

    PlotData plotData = null;
    while (reader.hasNext()) {
      XMLEvent evt = reader.nextEvent();
      if (evt.isStartElement()) {
	StartElement elmt = evt.asStartElement();
	String qName = elmt.getName().getLocalPart();
	if (qName.equals(E_CHART)) {
	  Attribute chartType = elmt.getAttributeByName(A_TYPE);
	  PlotType plotType = PlotType.getPlotType(chartType.getValue());
	  if (plotType == null) {
	    throw new IOException("Invalid plot type");
	  }
	  
	  Attribute chartTitle = elmt.getAttributeByName(A_TITLE);
	  plotData = new PlotData(plotType, chartTitle.getValue());
	  
	} else if (qName.equals(E_FORMULA)) {
	  String formula = reader.getElementText();
	  project.addFormula(formula);
	  if (plotData != null) plotData.addFormula(formula);
	} else if (qName.equals(E_DATASET)) {
	  Attribute aClassName = elmt.getAttributeByName(A_CLASSNAME);
	  project.addDatasetClass(aClassName.getValue());
	}
      } else if (evt.isEndElement()) {
	EndElement elmt = evt.asEndElement();
	String qName = elmt.getName().getLocalPart();
	if (qName.equals(E_CHART)) {
	  project.addPlotData(plotData);
	  plotData = null;
	}
      }
    }
    return project;
  }
}
