/**
 * 
 */
package anl.aida.query;

import java.io.StringReader;

import anl.aida.client.Cache;
import anl.aida.client.CacheElement;
import anl.aida.util.DateRange;
import anl.aida.util.DateRangeProducer;

/**
 * Runs a TriggerOp over a Cache.
 * 
 * @author Nick Collier
 */
public class Trigger {
  
  private String name, query, email;
  
  public Trigger() {
    this("", "", "");
  }
  
  /**
   * @param name
   * @param query
   * @param email
   */
  public Trigger(String name, String query, String email) {
    this.name = name;
    this.query = query;
    this.email = email;
  }

  /**
   * @return the name
   */
  public String getName() {
    return name;
  }

  /**
   * @param name the name to set
   */
  public void setName(String name) {
    this.name = name;
  }

  /**
   * @return the query
   */
  public String getQuery() {
    return query;
  }

  /**
   * @param query the query to set
   */
  public void setQuery(String query) {
    this.query = query;
  }

  /**
   * @return the email
   */
  public String getEmail() {
    return email;
  }

  /**
   * @param email the email to set
   */
  public void setEmail(String email) {
    this.email = email;
  }

  public TriggerResult run(Cache cache, TriggerUpdateListener listener) throws ParseException {
    DateRangeProducer ranger = cache.getRanger();
    DateRange rng = ranger.ranges().iterator().next();
    Frame frame = new Frame(cache, cache.getElement(rng));
    String input = query;
    if (!input.endsWith(";")) input += ";";
    Parser parser = new Parser(new StringReader(input));
    ASTstart start = parser.start();
    
    TriggerResult result = null;
    for (DateRange range : cache.getRanger().ranges()) {
      listener.rangeProcessing(range);
      CacheElement element = cache.getElement(range);
      frame.setCurrentElement(element);
      double val = start.evaluate(frame);
      if (val != 0) {
	if (result == null) {
	  result = new TriggerResult(true, "", range);
	} else{
	  result.addRange(range);
	}
      }
      element.release();
    }
    
    if (result == null) result = new TriggerResult(false, "");
    return result;
  }
}
