package anl.aida.table;

import java.util.Date;

import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableModel;

import org.jdesktop.swingx.JXTable;
import org.jdesktop.swingx.decorator.HighlighterFactory;

import anl.aida.client.util.ClientUtils;
import anl.aida.data.DataFrame;
import anl.aida.plot.TimeSeriesParameters;

/**
 * Creates Data Tables.
 * 
 */
public class TableCreator {

  @SuppressWarnings("serial")
  private static class DateRenderer extends DefaultTableCellRenderer {

    public void setValue(Object value) {
      String text = (value == null) ? "" : ClientUtils.formatShortDate((Date) value);
      setText(text);
    }
  }
  
  public JTable createTable(TimeSeriesParameters params) {
    TableModel model = new TimeSeriesTableModel(params);
    return createTable(model);
  }
  
  private JXTable createTable(TableModel model) {
    JXTable table = new JXTable(model);
    table.getColumnModel().getColumn(0).setCellRenderer(new DateRenderer());
    table.setColumnControlVisible(true);
    table.setHorizontalScrollEnabled(true);
    table.setHighlighters(HighlighterFactory.createAlternateStriping());
    table.setRolloverEnabled(true);
    return table;
    
  }

  public JTable createTable(DataFrame frame) {
    TableModel model = null;
    if (frame.getAxes().size() == 1) {
      model = new TVDataFrameTableModel(frame);
    } else if (frame.getAxes().size() == 2) {
      model = new TCategoryDataFrameTableModel(frame);
    }

    return createTable(model);
  }
}
