/**
 * 
 */
package anl.aida.table;

import javax.swing.table.AbstractTableModel;

import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;

import anl.aida.plot.TimeSeriesParameters;

/**
 * TableModel that wraps a time series collection. Each series
 * is a column in the model.
 * 
 */
@SuppressWarnings("serial")
public class TimeSeriesTableModel extends AbstractTableModel {
  
  private TimeSeriesCollection dataset;
  private String[] colNames;
  
  
  public TimeSeriesTableModel(TimeSeriesParameters params) {
    dataset = params.getDataset();
    colNames = new String[dataset.getSeriesCount() + 1];
    colNames[0] = params.getXAxisLabel();
    
    for (int i = 1; i < colNames.length; i++) {
      colNames[i] = dataset.getSeries(i - 1).getKey().toString();
    }
  }
  
  @Override
  public String getColumnName(int colIndex) {
    return colNames[colIndex];
  }

  /* (non-Javadoc)
   * @see javax.swing.table.TableModel#getColumnCount()
   */
  @Override
  public int getColumnCount() {
    return colNames.length;
  }

  /* (non-Javadoc)
   * @see javax.swing.table.TableModel#getRowCount()
   */
  @Override
  public int getRowCount() {
    return dataset.getItemCount(0);
  }

  /* (non-Javadoc)
   * @see javax.swing.table.TableModel#getValueAt(int, int)
   */
  @Override
  public Object getValueAt(int rowIndex, int columnIndex) {
    if (columnIndex == 0) {
      TimeSeries series = dataset.getSeries(0);
      return series.getTimePeriod(rowIndex).getStart();
    } else {
      return dataset.getSeries(columnIndex - 1).getDataItem(rowIndex).getValue();
    }
  }
}
