package anl.aida.viz;

import anl.aida.client.net.TermNode;
import edu.umd.cs.piccolo.nodes.PPath;
import edu.umd.cs.piccolo.nodes.PText;
import edu.umd.cs.piccolox.util.PBoundsLocator;

import java.awt.*;
import java.awt.geom.Point2D;

/**
 *         Date: Mar 11, 2009 2:15:59 PM
 */
public abstract class AbstractCircleNodeStyler implements NodeStyler {

  protected static final int BASE_NODE_SIZE = 14;

  private Paint nodePaint, textPaint;
  private Font font = new PText().getFont().deriveFont(Font.BOLD);

  public AbstractCircleNodeStyler(Paint nodePaint, Paint textPaint) {
    this.nodePaint = nodePaint;
    this.textPaint = textPaint;
  }

  /**
   * Creates a PPath to represent the specified TermNode.
   *
   * @param tNode    the term node to reprsent.
   * @param location the location of the
   * @return a PNode to represent the specified TermNode.
   */
  public PPath createNode(TermNode tNode, Point2D location) {
    float nodeSize = getSize(tNode);
    float nodeOffset = nodeSize / 2;
    PPath node = PPath.createEllipse((float) location.getX() - nodeOffset, (float) location.getY() - nodeOffset, 
	nodeSize, nodeSize);
    node.setPaint(nodePaint);

    PText text = new PText(tNode.getLabel());
    text.setFont(font);
    text.setPickable(false);
    PBoundsLocator loc = PBoundsLocator.createNorthEastLocator(node);
    text.setBounds(loc.locateX(), loc.locateY(), 20, 20);
    text.setTextPaint(textPaint);
    node.addChild(text);
    
    return node;
  }

  /**
   * Gets the paint appropriate for the specified term node.
   *
   * @param node the term node we want the paint for
   * @return the paint appropriate for the specified term node.
   */
  public Paint getPaint(TermNode node) {
    return nodePaint;
  }
}
