package anl.aida.viz;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.Timer;

import anl.aida.client.doc.DocSetOperation;
import anl.aida.client.gui.DocOpsWidget;
import anl.aida.client.util.ClientUtils;
import anl.aida.core.DocumentID;
import edu.umd.cs.piccolo.PCamera;
import edu.umd.cs.piccolo.PCanvas;
import edu.umd.cs.piccolo.event.PInputEventListener;
import edu.umd.cs.piccolo.util.PBounds;
import edu.umd.cs.piccolo.util.PPaintContext;

/**
 *         Date: Feb 11, 2009 10:37:13 AM
 */
public class Canvas2D extends PCanvas {

  private static final String PAN_ICON = "move.png";
  private static final String SELECT_ICON = "select.png";
  //private static final String LENS_ICON = "free_icon.png";

  private static final String PAN = "pan";
  private static final String LENS = "lens";
  private static final String SELECT = "select";

  private double mpX;
  private double mpY;

  private PInputEventListener currentListener;
  private java.util.List<AIDA2DLayer> layers = new ArrayList<AIDA2DLayer>();

  private Map<String, PInputEventListener> handlers = new HashMap<String, PInputEventListener>();
  private JToolBar bar;
  private ToolTipper tipper;
  private DocOpsWidget docOps = new DocOpsWidget();

  public Canvas2D(int width, int height) {
    this(new Dimension(width, height));
  }

  public Canvas2D(Dimension size) {
    setPreferredSize(size);
    handlers.put(PAN, getPanEventHandler());
    handlers.put(LENS, new LensHandler());
    handlers.put(SELECT, new SelectionHandler());

    currentListener = getPanEventHandler();
    removeInputEventListener(getZoomEventHandler());

    // add mouse wheel zoom control
    this.addMouseWheelListener(new repastPiccoloMouseWheelListener());
    this.addMouseMotionListener(new repastPiccoloMouseMotionListener());

    setDefaultRenderQuality(PPaintContext.HIGH_QUALITY_RENDERING);
    setInteractingRenderQuality(PPaintContext.HIGH_QUALITY_RENDERING);
    setAnimatingRenderQuality(PPaintContext.HIGH_QUALITY_RENDERING);

    // #D9D9F3
    // #6C7B8B
    // #99CCFF
    getCamera().setPaint(Colors.BACKGROUND_COLOR);
    createToolBar();
    tipper = new ToolTipper(getCamera());

    // timer is so we only resize the layout (which may be costly)
    // when it looks like the component has stopped resizing.
    addComponentListener(new ComponentAdapter() {

      Timer timer = null;

      @Override
      public void componentResized(ComponentEvent e) {
        if (timer == null) {
          timer = new Timer(50, new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
              Dimension d = new Dimension(getSize().width - 100, getSize().height - 100);
              for (AIDA2DLayer layer : layers) {
                layer.resized(d);
              }
              //System.out.println("ping");
              timer.stop();
            }
          });
          timer.stop();
          Dimension d = new Dimension(getSize().width - 100, getSize().height - 100);
          for (AIDA2DLayer layer : layers) {
            layer.resized(d);
          }
        } else {
          if (!timer.isRunning()) {
            timer.start();
          } else {
            timer.restart();
          }
        }
      }
    });
    
    switchEventHandler(SELECT);
  }

  private void switchEventHandler(String key) {
    if (currentListener != null) removeInputEventListener(currentListener);

    for (AIDA2DLayer layer : layers) {
      layer.selectionEnabled(key.equals(SELECT));
    }

    currentListener = handlers.get(key);
    if (currentListener != null) addInputEventListener(currentListener);

  }

  public void showToolTip(String ttext) {
    tipper.showTip(ttext, (float) mpX, (float) mpY);
  }

  private JToggleButton createButton(final String key) {
    JToggleButton button = new JToggleButton();
    button.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        switchEventHandler(key);
      }
    });

    return button;
  }

  public JToolBar getToolBar() {
    return bar;
  }

  public void clearSelection() {
    docOps.clearDocuments();
  }

  public void addSelection(Object obj) {
    docOps.addDocument((DocumentID) obj);
  }

  private void createToolBar() {
    bar = new JToolBar();
    bar.setFloatable(false);
    ButtonGroup grp = new ButtonGroup();
    bar.putClientProperty("BTN_GROUP", grp);

    JToggleButton btn = createButton(PAN);
    btn.setIcon(ClientUtils.loadIcon(PAN_ICON));
    btn.setToolTipText("Enable mouse display panning");
    bar.add(btn);
    grp.add(btn);
    
    btn = createButton(SELECT);
    btn.setIcon(ClientUtils.loadIcon(SELECT_ICON));
    btn.setToolTipText("Enable mouse node and link selection");
    bar.add(btn);
    grp.add(btn);
    btn.setSelected(true);

    /*
    btn = createButton(LENS);
    btn.setIcon(ClientUtils.loadIcon(LENS_ICON));
    btn.setToolTipText("Enable mouse keyword lens");
    bar.add(btn);
    grp.add(btn);
    */
  }

  public void lensOver(PBounds bounds, double radius) {
    for (AIDA2DLayer layer : layers) {
      layer.lensOver(bounds, radius, this);
    }
  }

  public JButton addDocOp(DocSetOperation op, String iconFileName) {
    return docOps.addOp(op, iconFileName);
  }

  public void addLayer(AIDA2DLayer mifsLayer) {
    layers.add(mifsLayer);
    mifsLayer.addToCanvas(this);
    
    bar.addSeparator();
    for (JButton button : docOps.getButtons()) {
      bar.add(button);
    }
    
    mifsLayer.initToolBar(bar, (ButtonGroup) bar.getClientProperty("BTN_GRP"));
    mifsLayer.selectionEnabled(currentListener instanceof SelectionHandler);
  }

  public void setViewBounds(Rectangle2D bounds) {
    getCamera().animateViewToCenterBounds(bounds, true, 0);
  }

  public void marqueeOver(PBounds rect, boolean ctrlDown) {
    for (AIDA2DLayer layer : layers) {
      layer.marqueeOver(rect, ctrlDown);
    }
  }
  
  /**
   * Zoom control with mouse wheel
   */
  public class repastPiccoloMouseWheelListener implements MouseWheelListener {
    private double minScale = 0;
    private double maxScale = Double.MAX_VALUE;

    public void mouseWheelMoved(MouseWheelEvent e) {
      PCamera camera = getCamera();

      double dx = -e.getWheelRotation();

      double scaleDelta = (1.0 + (0.1 * dx));

      double currentScale = camera.getViewScale();
      double newScale = currentScale * scaleDelta;

      if (newScale < minScale)
        scaleDelta = minScale / currentScale;

      if ((maxScale > 0) && (newScale > maxScale))
        scaleDelta = maxScale / currentScale;

      Point2D point = camera.localToView(new Point2D.Double(mpX, mpY));

      camera.scaleViewAboutPoint(scaleDelta, point.getX(), point.getY());
    }
  }

  /**
   * Assists mouse wheel zoom control by updating the center coord
   */
  public class repastPiccoloMouseMotionListener implements MouseMotionListener {

    public void mouseDragged(MouseEvent e) {
      mpX = e.getX();
      mpY = e.getY();
      tipper.removeTip();
    }

    public void mouseMoved(MouseEvent e) {
      mpX = e.getX();
      mpY = e.getY();
      tipper.removeTip();
    }
  }

  /**
   * Resets the display view to the original zoom and camera bounds
   */
  public void resetHomeView() {

    getCamera().animateViewToCenterBounds(new Rectangle2D.Float(0, 0, getPreferredSize().width,
            getPreferredSize().height), true, 0);
    //PBounds bounds = getRoot().getChild(0).getFullBounds();
    //System.out.println("bounds = " + bounds);
    //getCamera().animateViewToCenterBounds(bounds, true, 0);
  }
}
