package anl.aida.viz;

import anl.aida.client.net.ConceptGraph;
import anl.aida.client.net.TermNode;

import java.awt.*;

/**
 * Node style implementation that scales the size of the ellipse
 * with respect the document count average.
 *
 */
public class DefaultNodeStyler extends AbstractCircleNodeStyler {

  private float avg = 0;

  public DefaultNodeStyler(Paint nodePaint, Paint textPaint, ConceptGraph<TermNode> graph) {
    super(nodePaint, textPaint);
    // nodes with doc > 0
    int nodeCount = 0;
    for (TermNode tNode : graph.getVertices()) {
      int docCount = tNode.getDocumentCount();
      if (docCount > 0) {
	avg += docCount;
	nodeCount++;
      }
    }
    if (avg == 0 || nodeCount == 0) avg = 1;
    else avg /= nodeCount; //graph.getVertexCount();
  }

  /**
   * Gets the size appropriate for the specified term node.
   *
   * @param tNode the node to get the size for
   * @return the size appropriate for the specified term node.
   */
  public float getSize(TermNode tNode) {
    float factor = tNode.getDocumentCount() / avg;
    float size = factor * BASE_NODE_SIZE;
    //System.out.println("size = " + size);
    return size < 2f ? 2f : size;
    
  }
}
