package anl.aida.viz;

import anl.aida.client.doc.DocumentContainer;
import anl.aida.client.net.CGEdge;
import anl.aida.client.net.TermNode;
import edu.umd.cs.piccolo.PLayer;
import edu.umd.cs.piccolo.event.PBasicInputEventHandler;
import edu.umd.cs.piccolo.event.PInputEvent;
import edu.umd.cs.piccolo.nodes.PPath;
import edu.umd.cs.piccolo.util.PStack;

import java.awt.*;

/**
 *         Date: Mar 2, 2009 10:23:57 AM
 */
public class EdgeSelectionHandler extends PBasicInputEventHandler {

  private PPath edge;
  private Paint selectedEdgePaint = Colors.SELECTED_COLOR;
  private Paint selectedNodePaint = Colors.SELECTED_COLOR;
  private PLayer nodeLayer, edgeLayer, selectionLayer;
  private Paint defaultNodePaint;
  private SelectionManager<DocumentContainer> selection;

  public EdgeSelectionHandler(Paint defaultNodePaint, PLayer nodeLayer, PLayer edgeLayer) {
    getEventFilter().setMarksAcceptedEventsAsHandled(true);
    this.defaultNodePaint = defaultNodePaint;
    this.edgeLayer = edgeLayer;
    this.nodeLayer = nodeLayer;

    selectionLayer = new PLayer();
  }

  @Override
  public void mousePressed(PInputEvent evt) {
    edge = null;
    PStack stack = evt.getPath().getNodeStackReference();
    for (Object obj : stack) {
      if (obj instanceof PPath) {
        edge = (PPath) obj;
        break;
      }
    }

    if (edge != null) {
      edge.setStrokePaint(selectedEdgePaint);
      java.util.List<PPath> nodes = (java.util.List<PPath>) edge.getAttribute(NetworkDisplay.NODES);

      PPath node1 = nodes.get(0);
      node1.setPaint(selectedNodePaint);
      PPath node2 = nodes.get(1);
      node2.setPaint(selectedNodePaint);
      selectionLayer.addChild(edge);
      selectionLayer.addChild(node1);
      selectionLayer.addChild(node2);

      if (selectionLayer.getParent() == null) {
        Canvas2D canvas = (Canvas2D) evt.getCamera().getComponent();
        canvas.getCamera().addLayer(selectionLayer);
        canvas.getRoot().addChild(selectionLayer);

        canvas.showToolTip(createTooltipText(edge, node1, node2));
      }
    }
  }

  private String createTooltipText(PPath edge, PPath node1, PPath node2) {
    TermNode tNode1 = (TermNode) NetworkDisplay.getObject(node1);
    TermNode tNode2 = (TermNode) NetworkDisplay.getObject(node2);
    CGEdge cgEdge = (CGEdge) NetworkDisplay.getObject(edge);

    StringBuilder buf = new StringBuilder(tNode1.getLabel());
    buf.append(" <-> ");
    buf.append(tNode2.getLabel());
    buf.append("\n");
    buf.append("Document Count: ");
    buf.append(cgEdge.getDocumentCount());
    buf.append("\n'");
    buf.append(tNode1.getLabel());
    buf.append("' Document Count: ");
    buf.append(tNode1.getDocumentCount());
    buf.append("\n'");
    buf.append(tNode2.getLabel());
    buf.append("' Document Count: ");
    buf.append(tNode2.getDocumentCount());
    return buf.toString();
  }

  public void mouseReleased(PInputEvent evt) {
    if (edge != null) {
      edge.setStrokePaint(NodeUtils.getPaint(edge));
      java.util.List<PPath> nodes = (java.util.List<PPath>) edge.getAttribute(NetworkDisplay.NODES);
      PPath node1 = nodes.get(0);
      PPath node2 = nodes.get(1);
      node1.setPaint(defaultNodePaint);
      node2.setPaint(defaultNodePaint);

      if (selectionLayer.getParent() != null) {
        Canvas2D canvas = (Canvas2D) evt.getCamera().getComponent();
        canvas.getCamera().removeLayer(selectionLayer);
        canvas.getRoot().removeChild(selectionLayer);
      }

      edgeLayer.addChild(edge);
      nodeLayer.addChild(node1);
      nodeLayer.addChild(node2);

      if (selection != null) selection.paintSelection();

    }
  }

  public void addSelectionManager(SelectionManager<DocumentContainer> selection) {
    this.selection = selection;
  }

}
