/**
 * 
 */
package anl.aida.viz;

import java.awt.Color;
import java.awt.Paint;
import java.util.Collection;

import anl.aida.client.net.CGEdge;
import anl.aida.client.net.ConceptGraph;
import anl.aida.client.net.TermNode;

/**
 * Styles edges such that the edges are drawn on a gray scale
 * such that edges with more docs are darker than those with
 * less.
 * 
 */
public class GrayScaleEdgeStyler extends AbstractEdgeStyler {
  
  //private float avgVal = 0;
  float min = Float.POSITIVE_INFINITY, max = Float.NEGATIVE_INFINITY;
  float diff = 0f;
  
  public GrayScaleEdgeStyler(ConceptGraph<TermNode> graph) {
    update(graph.getEdges());
  }
  
  public void update(Collection<CGEdge> edges) {
    for (CGEdge edge : edges) {
      int count = edge.getDocumentCount();
      min = Math.min(min, count);
      max = Math.max(max, count);
    }
    diff = max - min;
  }
  
  /* (non-Javadoc)
   * @see anl.mifs.viz.EdgeStyler#getPaint(anl.mifs.client.net.CGEdge)
   */
  @Override
  public Paint getPaint(CGEdge edge) {
    float scale = scale(edge);
    return new Color(scale, scale, scale);
  }
  
  private float scale(CGEdge edge) {
    if (diff == 0) return .25f;
    float val = (edge.getDocumentCount() - min) / diff;
    // divide by 2 because the scale runs
    // from 0 to .5f
    return 1f - (val * .75f + .25f);
  }
}
