package anl.aida.viz;

import anl.aida.core.DocumentID;
import anl.aida.core.KeywordGenerator;
import anl.aida.util.HighestTermScores;
import anl.aida.util.AIDAException;
import anl.aida.util.TermScore;

import java.util.List;

/**
 * Displays a keyword tooltip for selected items.
 *
 */
public class KeywordTipper implements SelectionListener<DocumentID> {

  private KeywordGenerator generator;
  private int count;

  public KeywordTipper(KeywordGenerator generator, int count) {
    this.generator = generator;
    this.count = count;
  }

  /**
   * Called when the specified list of items has been selected.
   *
   * @param selectedItems the selected items
   * @param canvas        the visualization canvas
   */
  public void selectionOccurred(List<DocumentID> selectedItems, Canvas2D canvas) {
    try {
      HighestTermScores total = new HighestTermScores(count);
      for (DocumentID id : selectedItems) {
        HighestTermScores scores = generator.generate(id.getPath(), 30);
        for (TermScore score : scores.termScores()) {
          total.add(score.getTerm(), score.getScore());
        }
      }

      StringBuilder buf = new StringBuilder();
      int i = 1;
      for (TermScore score : total.termScores()) {
        if (i % 3 == 0) {
          buf.append("\n");
          i = 0;
        }
        buf.append(score.getTerm());
        buf.append(" ");
        i++;
      }

      canvas.showToolTip(buf.toString());

    } catch (AIDAException e) {
      e.printStackTrace();
    }
  }
}
