package anl.aida.viz;

import edu.umd.cs.piccolo.PLayer;
import edu.umd.cs.piccolo.event.PDragSequenceEventHandler;
import edu.umd.cs.piccolo.event.PInputEvent;
import edu.umd.cs.piccolo.nodes.PPath;
import edu.umd.cs.piccolo.util.PBounds;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Point2D;

/**
 *
 *         Date: Feb 12, 2009 1:22:55 PM
 */
public class LensHandler extends PDragSequenceEventHandler {

  private PPath lens;
  private boolean lensAdded = false;
  private int lensSize = 80;
  private Timer timer;
  private Point2D mousePt;
  private PLayer layer = new PLayer();
  private Canvas2D canvas;

  public LensHandler() {
    getEventFilter().setMarksAcceptedEventsAsHandled(true);
    lens = PPath.createEllipse(0, 0, lensSize, lensSize);
    lens.setPaint(Color.WHITE);
    lens.setTransparency(.3f);
    lens.setStroke(null);
    timer = new Timer(500, new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        PBounds bounds = new PBounds();
        int radius = lensSize / 2;
        bounds.setRect(mousePt.getX() - radius, mousePt.getY() - radius,
                lensSize, lensSize);
        canvas.lensOver(bounds, radius);
      }
    });

    timer.setRepeats(false);
  }

  public void startDrag(PInputEvent evt) {
    super.startDrag(evt);
    canvas = (Canvas2D) evt.getComponent();
    if (!lensAdded) {
      Point2D pt = evt.getPosition();
      lens.setOffset(pt.getX() - lensSize / 2, pt.getY() - lensSize / 2);
      layer.addChild(lens);
      canvas.getCamera().addLayer(layer);
      lensAdded = true;
    }
    timer.restart();
    mousePt = new Point2D.Double(evt.getPosition().getX(), evt.getPosition().getY());
  }

  public void endDrag(PInputEvent evt) {
    super.endDrag(evt);
    //System.out.println("drag ended");
    layer.removeChild(lens);
    canvas.getCamera().removeLayer(layer);
    lensAdded = false;
    timer.stop();
  }

  @Override
  public void drag(PInputEvent evt) {
    lens.translate(evt.getDelta().width, evt.getDelta().height);
    lens.repaint();
    timer.restart();
    mousePt = new Point2D.Double(evt.getPosition().getX(), evt.getPosition().getY());
  }
}
