package anl.aida.viz;

import anl.aida.client.doc.DocumentContainer;
import anl.aida.client.net.TermNode;
import anl.aida.viz.layout.Layout2D;
import edu.umd.cs.piccolo.PLayer;
import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.event.PDragSequenceEventHandler;
import edu.umd.cs.piccolo.event.PInputEvent;
import edu.umd.cs.piccolo.nodes.PPath;
import edu.umd.cs.piccolo.util.PBounds;

import java.awt.*;
import java.awt.geom.Point2D;
import java.util.ArrayList;

/**
 * 
 */
public class NodeDragSelectionHandler extends PDragSequenceEventHandler {

  private Paint paint;
  private Paint selectedNodePaint = Colors.SELECTED_COLOR;
  private Paint selectedPathPaint = Colors.SELECTED_COLOR;

  private java.util.List<PPath> selectedNodes = new ArrayList<PPath>();
  private java.util.List<PPath> selectedEdges = new ArrayList<PPath>();
  private PLayer nodeLayer, edgeLayer;

  private PPath node;
  private Paint defaultNodePaint;

  private PLayer selectionLayer = new PLayer();
  private Canvas2D canvas;
  private SelectionManager<DocumentContainer> selection = null;
  private Layout2D<TermNode> layout;

  public NodeDragSelectionHandler(Paint defNodePaint, PLayer nodeLayer, PLayer edgeLayer,
                                  Layout2D<TermNode> layout) {
    getEventFilter().setMarksAcceptedEventsAsHandled(true);
    this.defaultNodePaint = defNodePaint;
    this.nodeLayer = nodeLayer;
    this.edgeLayer = edgeLayer;
    this.layout = layout;
  }

  @Override
  public void mousePressed(PInputEvent evt) {
    super.mousePressed(evt);
    canvas = (Canvas2D) evt.getCamera().getComponent();
    if (evt.getPickedNode() instanceof PPath) {
      if (node == null) {
        node = (PPath) evt.getPickedNode();
        paint = node.getPaint();
        node.setPaint(selectedNodePaint);
      }
      selectPaths();
      nodeLayer.removeChild(node);
      selectionLayer.addChild(node);
      selectedNodes.add(node);
      canvas.getCamera().addLayer(selectionLayer);
      canvas.getRoot().addChild(selectionLayer);

      canvas.showToolTip(createTooltipText(node));
    }
  }

  private void selectPaths() {
    java.util.List<PPath> edges = (java.util.List<PPath>) node.getAttribute(NetworkDisplay.EDGES);

    for (PPath edge : edges) {
      edge.setStrokePaint(selectedPathPaint);
      selectedEdges.add(edge);
      selectionLayer.addChild(edge);
      java.util.List<PPath> nodes = (java.util.List<PPath>) edge.getAttribute(NetworkDisplay.NODES);
      // == equality test should be appropriate here
      PPath otherNode;
      if (nodes.get(0) == node) otherNode = nodes.get(1);
      else otherNode = nodes.get(0);

      otherNode.setPaint(selectedPathPaint);
      selectedNodes.add(otherNode);
      selectionLayer.addChild(otherNode);
    }
  }

  private String createTooltipText(PPath node1) {
    TermNode tNode1 = (TermNode) NetworkDisplay.getObject(node1);
    int edgeCount = ((java.util.List) node1.getAttribute(NetworkDisplay.EDGES)).size();

    StringBuilder buf = new StringBuilder(tNode1.getLabel());
    buf.append("\n");
    buf.append("Document Count: ");
    buf.append(tNode1.getDocumentCount());
    buf.append("\nEdge Count: ");
    buf.append(edgeCount);
    return buf.toString();
  }


  @Override
  public void mouseReleased(PInputEvent evt) {
    super.mouseReleased(evt);
    if (selectionLayer.getParent() != null) {
      canvas.getCamera().removeLayer(selectionLayer);
      canvas.getRoot().removeChild(selectionLayer);
    }

    for (PPath path : selectedNodes) {
      path.setPaint(defaultNodePaint);
      nodeLayer.addChild(path);
    }

    for (PPath edge : selectedEdges) {
      edge.setStrokePaint(NodeUtils.getPaint(edge));
      edgeLayer.addChild(edge);
    }

    selectedNodes.clear();
    selectedEdges.clear();

    if (selection != null) selection.paintSelection();
  }

  public void startDrag(PInputEvent e) {
    super.startDrag(e);
    if (e.getPickedNode() instanceof PPath) {
      node = (PPath) e.getPickedNode();
      paint = node.getPaint();

      node.setPaint(selectedNodePaint);
    }
  }

  public void endDrag(PInputEvent e) {
    super.endDrag(e);
    if (node != null) {
      node.setPaint(paint);
      node = null;
    }
  }

  @SuppressWarnings("unchecked")
  public void drag(PInputEvent e) {
    if (node != null) {
      node.translate(e.getDelta().width, e.getDelta().height);
      
      // get bounds gives us the bounds of the node without the
      // text label child
      Point2D pt = node.getBounds().getCenter2D();
      // the location is relative to the parent though, so we need
      // to transform.
      node.localToParent(pt);
      layout.setLocation((TermNode) NodeUtils.getObject(node), pt.getX(), pt.getY());

      java.util.List<PPath> edges = (java.util.List<PPath>) node.getAttribute(NetworkDisplay.EDGES);

      for (PPath edge : edges) {
        java.util.List<PPath> nodes = (java.util.List<PPath>) edge.getAttribute(NetworkDisplay.NODES);
        
       
        PNode node1 = nodes.get(0);
        PNode node2 = nodes.get(1);
        
        Point2D other;
        if (node1 == node) {
          other = node2.getBounds().getCenter2D();
          node2.localToParent(other);
        } else {
          other = node1.getBounds().getCenter2D();
          node1.localToParent(other);
        }
        
        edge.reset();
        edge.moveTo((float) pt.getX(), (float) pt.getY());
        edge.lineTo((float) other.getX(), (float) other.getY());

        /*
        This was the old way of doing it -- kept here for reference
        // Note that the node's "FullBounds" must be used (instead of just the "Bound")
        // because the nodes have non-identity transforms which must be included when
        // determining their position.
        PNode c1 = node1.removeChild(0);
        PNode c2 = node2.removeChild(0);

        Point2D.Double bound1 = (Point2D.Double) node1.getFullBounds().getCenter2D();
        Point2D.Double bound2 = (Point2D.Double) node2.getFullBounds().getCenter2D();

        node1.addChild(c1);
        node2.addChild(c2);

        edge.moveTo((float) bound1.getX(), (float) bound1.getY());
        edge.lineTo((float) bound2.getX(), (float) bound2.getY());
        */
      }
    }
  }

  public void addSelectionManager(SelectionManager<DocumentContainer> selection) {
    this.selection = selection;
  }
}

