package anl.aida.viz;

import java.awt.Paint;

import edu.umd.cs.piccolo.PNode;

/**
 * 
 */
public class NodeUtils {

  public static final String TYPE_LABEL = "TYPE";
  public static final String NODE = "NODE";
  public static final String EDGE = "EDGE";
  public static final String OBJ = "OBJ";
  public static final String UNKNOWN = "__UNKNOWN__";
  public static final String PAINT = "PAINT";
  
  /**
   * Sets the paint attribute / property of the node. This
   * does not set the nodes paint.
   * 
   * @param node the node 
   * @param paint the paint
   */
  public static void setPaint(PNode node, Paint paint) {
    node.addAttribute(PAINT, paint);
  }
  
  /**
   * Gets the specified node's paint attribute.
   * 
   * @param node the node whose paint attribute we want
   * @return the specified node's paint attribute.
   */
  public static Paint getPaint(PNode node) {
    return (Paint)node.getAttribute(PAINT);
  }

  public static boolean isNode(PNode node) {
    Object obj = node.getAttribute(TYPE_LABEL);
    return obj != null && obj.equals(NODE);
  }

  public static boolean isEdge(PNode node) {
    Object obj = node.getAttribute(TYPE_LABEL);
    return obj != null && obj.equals(EDGE);
  }

  public static void setAsEdge(PNode node) {
    node.addAttribute(TYPE_LABEL, EDGE);
  }

  public static void setAsNode(PNode node) {
    node.addAttribute(TYPE_LABEL, NODE);
  }

  public static void setType(PNode node, Object type) {
    node.addAttribute(TYPE_LABEL, type);
  }

  public static Object getType(PNode node) {
    Object type = node.getAttribute(TYPE_LABEL);
    if (type == null) return UNKNOWN;
    return type;
  }

  public static void setObject(PNode node, Object obj) {
    node.addAttribute(OBJ, obj);
  }

  public static Object getObject(PNode node) {
    return node.getAttribute(OBJ);
  }
}
