package anl.aida.viz;

import anl.aida.client.net.TermNode;
import anl.aida.util.Pair;
import edu.umd.cs.piccolo.PLayer;
import edu.umd.cs.piccolo.PNode;

import java.awt.*;
import java.util.HashMap;
import java.util.Map;

/**
 * Node style implementation that scales the size of the ellipse
 * with respect to some intial size and document count. If a term node's
 * document count is equal to the base initial count then the node
 * will be the same as the original size. If the count is less than
 * the initial count then the size will be smaller than the initial
 * size and so on.
 *
 */
public class ScaledNodeStyler extends AbstractCircleNodeStyler {

  private Map<String, Pair<Float, Float>> sizeMap = new HashMap<String, Pair<Float, Float>>();


  /**
   * Layer is expected to contain PNodes with TermNode objects.
   *
   * @param nodePaint the node paint
   * @param textPaint the text paint
   * @param layer     the layer containing the PNodes whose sizes will be
   *                  used as a base.
   */
  public ScaledNodeStyler(Paint nodePaint, Paint textPaint, PLayer layer) {
    super(nodePaint, textPaint);
    for (int i = 0, n = layer.getChildrenCount(); i < n; i++) {
      PNode node = layer.getChild(i);
      TermNode tNode = (TermNode) NodeUtils.getObject(node);
      sizeMap.put(tNode.getLabel(), new Pair<Float, Float>((float) node.getWidth(), (float) tNode.getDocumentCount()));
    }
  }

  /**
   * Gets the size appropriate for the specified term node.
   *
   * @param tNode the node to get the size for
   * @return the size appropriate for the specified term node.
   */
  public float getSize(TermNode tNode) {
    Pair<Float, Float> vals = sizeMap.get(tNode.getLabel());
    float baseCount = vals.getSecond().floatValue();
    if (baseCount == 0) baseCount = 1f;
    float per = tNode.getDocumentCount() / baseCount;
    /*
    if (tNode.getTerm().equalsIgnoreCase("nile")) {
      System.out.println("baseCount = " + baseCount);
      System.out.println("tNode.getDocumentCount() = " + tNode.getDocumentCount());
      System.out.println("per = " + per);
      System.out.println("vals.getFirst() * per = " + vals.getFirst() * per);
    }
    */
    return vals.getFirst() * per;
  }
}