package anl.aida.viz;

import edu.umd.cs.piccolo.nodes.PPath;

import java.awt.*;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 * 
 */
public class SelectionManager<T> {

  private Paint selectionColor, nodeColor;
  private Set<PPath> selectedNodes = new HashSet<PPath>();

  public SelectionManager(Paint nodeColor, Paint selectionColor) {
    this.nodeColor = nodeColor;
    this.selectionColor = selectionColor;
  }

  public void unselectAll() {
    for (PPath node : selectedNodes) {
      if (NodeUtils.isEdge(node)) node.setStrokePaint(NodeUtils.getPaint(node));
      else if (NodeUtils.isNode(node)) node.setPaint(nodeColor);
    }
    selectedNodes.clear();
  }

  public void addSelectedItem(PPath node) {
    selectedNodes.add(node);
    if (NodeUtils.isEdge(node)) node.setStrokePaint(selectionColor);
    else if (NodeUtils.isNode(node)) node.setPaint(selectionColor);
  }

  public void unselect(PPath node) {
    selectedNodes.remove(node);
    if (NodeUtils.isEdge(node)) node.setStrokePaint(NodeUtils.getPaint(node));
    else if (NodeUtils.isNode(node)) node.setPaint(nodeColor);
  }

  public Iterable<T> getSelectedObjects() {
    return new Iter<T>(selectedNodes.iterator());
  }

  public void paintSelection() {
    for (PPath node : selectedNodes) {
      if (NodeUtils.isEdge(node)) node.setStrokePaint(selectionColor);
      else if (NodeUtils.isNode(node)) node.setPaint(selectionColor);
    }
  }

  private static class Iter<T> implements Iterable<T>, Iterator<T> {

    Iterator<PPath> iter;

    private Iter(Iterator<PPath> iter) {
      this.iter = iter;
    }

    public Iterator<T> iterator() {
      return this;
    }

    public boolean hasNext() {
      return iter.hasNext();
    }

    public T next() {
      return (T) NodeUtils.getObject(iter.next());
    }

    public void remove() {
      throw new UnsupportedOperationException();
    }
  }


}
