package anl.aida.viz;

import anl.aida.algo.Cluster;
import anl.aida.algo.Clusters;
import anl.aida.algo.ClustersBuilder;
import edu.umd.cs.piccolo.PLayer;
import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.nodes.PPath;
import edu.umd.cs.piccolo.nodes.PText;
import edu.umd.cs.piccolo.util.PBounds;
import edu.umd.cs.piccolo.util.PNodeFilter;
import edu.umd.cs.piccolox.util.PBoundsLocator;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.List;

/**
 * 
 *         Date: Feb 11, 2009 11:43:03 AM
 */
public class SimilarityClusterDisplay<T> extends PLayer implements AIDA2DLayer {


  public static final String CENTROID = "CLUSTER";
  public static final String CENTER = "CENTER";

  private SimilarityClusterLayout<T> layout;
  private Clusters<T> clusters;
  private SelectionListener<T> lensListener;
  private static final Color NODE_COLOR = Color.YELLOW;

  private SelectionManager<T> selection;
  private ItemSelectionHandler selectionHandler;
  private Canvas2D canvas;


  public SimilarityClusterDisplay(Clusters<T> clusters, Dimension size) {
    this.clusters = clusters;
    layout = new SimilarityClusterLayout<T>(clusters, size);
    addInputEventListener(new ClusterCentroidDragHandler(layout));
    selection = new SelectionManager<T>(NODE_COLOR, Colors.SELECTED_COLOR);
    selectionHandler = new ItemSelectionHandler(this);
  }

  public void setLensListener(SelectionListener<T> listener) {
    lensListener = listener;
  }

  public void initialize() {
    layout.initialize();
    reset();
  }

  private void create() {
    for (int i = 0; i < clusters.size(); i++) {
      Point2D coord = layout.getCentroidLocation(i);
      float x = (float) coord.getX() - 10;
      float y = (float) coord.getY() - 10;
      PPath node = PPath.createEllipse(x, y, 20, 20);
      NodeUtils.setObject(node, i);
      NodeUtils.setType(node, CENTROID);
      Cluster<T> cluster = clusters.getCluster(i);
      PText text = new PText(cluster.getCentroidLabel());
      PBoundsLocator loc = PBoundsLocator.createNorthEastLocator(node);
      text.setBounds(loc.locateX(), loc.locateY(), 20, 20);
      node.addChild(text);

      addChild(node);
    }

    // add center
    Point2D pt = layout.getClusterCenter();
    PPath center = PPath.createEllipse((float) pt.getX() - 5, (float) pt.getY() - 5, 10, 10);
    NodeUtils.setType(center, CENTER);
    center.setPaint(Color.RED);
    addChild(center);

    // add the clustered items.
    int size = 8;
    float offset = size / 2f;
    for (T obj : clusters.items()) {
      Point2D coord = layout.getLocation(obj);
      PPath node = PPath.createEllipse((float) coord.getX() - offset, (float) coord.getY() - offset,
              size, size);
      NodeUtils.setAsNode(node);
      NodeUtils.setObject(node, obj);
      node.setPaint(NODE_COLOR);
      addChild(node);
    }
  }

  public void reset() {
    layout.reset();

    int size = 8;
    float offset = size / 2f;
    if (getChildrenCount() == 0) {
      create();
    } else {
      for (int i = 0, n = getChildrenCount(); i < n; i++) {
        PNode node = getChild(i);
        Object type = NodeUtils.getType(node);
        if (type.equals(CENTROID)) {
          Point2D coord = layout.getCentroidLocation((Integer) NodeUtils.getObject(node));
          float x = (float) coord.getX() - 10;
          float y = (float) coord.getY() - 10;
          Point2D pt = new Point2D.Double(x, y);
          pt = node.globalToLocal(pt);
          node.setBounds(pt.getX(), pt.getY(), 20, 20);
          PBoundsLocator loc = PBoundsLocator.createNorthEastLocator(node);
          PNode child = node.getChild(0);
          child.setBounds(loc.locateX(), loc.locateY(), child.getBounds().width, child.getBounds().height);
        } else if (type.equals(CENTER)) {
          Point2D pt = layout.getClusterCenter();
          node.setBounds((float) pt.getX() - 5, (float) pt.getY() - 5, 10, 10);
        } else {
          T obj = (T) NodeUtils.getObject(node);
          Point2D coord = layout.getLocation(obj);
          node.setBounds((float) coord.getX() - offset, (float) coord.getY() - offset, size, size);
        }
      }
    }

    repaint();
  }

  @SuppressWarnings("unchecked")
  public void lensOver(PBounds bounds, double radius, Canvas2D canvas) {
    if (lensListener != null) {
      List nodes = new ArrayList();
      PNodeFilter filter = new BoundsFilter(bounds, radius);
      this.getAllNodes(filter, nodes);

      java.util.List<T> items = new ArrayList<T>();
      //StringBuilder buf = new StringBuilder();
      for (Object node : nodes) {
        PNode pnode = (PNode) node;
        if (NodeUtils.isNode(pnode)) {
          T obj = (T) NodeUtils.getObject(pnode);
          items.add(obj);
        }
      }
      if (items.size() > 0) lensListener.selectionOccurred(items, canvas);
    }
  }

  public void marqueeOver(PBounds bounds, boolean ctrlDown) {
    if (!ctrlDown) selection.unselectAll();
    List nodes = new ArrayList();
    PNodeFilter filter = new BoundsFilter(bounds, 0);
    this.getAllNodes(filter, nodes);
    for (Object node : nodes) {
      PNode pnode = (PNode) node;
      if (NodeUtils.isNode(pnode)) {
        selection.addSelectedItem((PPath) pnode);
      }
    }

    canvas.clearSelection();
    for (T id : selection.getSelectedObjects()) {
      canvas.addSelection(id);
    }

  }

  public void addToCanvas(Canvas2D canvas) {
    canvas.getRoot().addChild(this);
    canvas.getCamera().addLayer(this);
    this.canvas = canvas;
  }

  /**
   * Called when selection mode is enabled or disabled.
   *
   * @param enabled state of selection mode.
   */
  public void selectionEnabled(boolean enabled) {
    if (enabled) addInputEventListener(selectionHandler);
    else {
      selection.unselectAll();
      removeInputEventListener(selectionHandler);
    }
  }

  /**
   * Called when an individual node has been selected.
   *
   * @param node     the selected node
   * @param ctrlDown whether or not the the ctrl key was down
   *                 when the node was selected
   */
  public void nodeSelected(PNode node, boolean ctrlDown) {
    if (NodeUtils.isNode(node)) {
      if (!ctrlDown) selection.unselectAll();
      selection.addSelectedItem((PPath) node);
    }

    canvas.clearSelection();
    for (T id : selection.getSelectedObjects()) {
      canvas.addSelection(id);
    }

  }

  public void initToolBar(JToolBar bar, ButtonGroup grp) {
    bar.addSeparator();
    bar.add(new AbstractAction("Reset") {
      public void actionPerformed(ActionEvent e) {
        reset();
      }
    });
  }

  public void resized(Dimension size) {
    layout.resetSize(size);
    initialize();
    repaint();
  }

  private class BoundsFilter implements PNodeFilter {

    PBounds bounds;
    PBounds localBounds = new PBounds();
    double radius2 = 0;

    private BoundsFilter(PBounds bounds, double radius) {
      this.bounds = bounds;
      this.radius2 = radius * radius;
    }

    public boolean accept(PNode pNode) {
      //localBounds.setRect(bounds);
      //pNode.globalToLocal(localBounds);
      //System.out.println("pNode.getBounds() = " + pNode.getBounds());
      //System.out.println("bounds = " + localBounds);
      boolean intersect = pNode.intersects(bounds);
      if (radius2 == 0) return intersect;
      return radius2 >= bounds.getCenter2D().distanceSq(pNode.getBounds().getCenter2D());


    }

    public boolean acceptChildrenOf(PNode pNode) {
      return true;
    }
  }

  public static void main(String[] args) {
    //float[] sims = {.5f, .25f, .75f};
    //float[] sims = {0f, .1f, .75f};
    ClustersBuilder<Integer> cBuilder = new ClustersBuilder<Integer>();
    for (int i = 0; i < 3; i++) {
      cBuilder.addCluster(new float[]{}, String.valueOf(i));
    }

    for (int i = 0; i < 3; i++) {
      for (int j = 0; j < 20; j++) {
        cBuilder.addItemToCluster(j, i, (float) Math.random());
      }
    }

    SimilarityClusterDisplay<Integer> display = new SimilarityClusterDisplay<Integer>(cBuilder.buildClusters(),
            new Dimension(700, 500));
    display.initialize();

    display.setLensListener(new SelectionListener<Integer>() {
      public void selectionOccurred(List<Integer> selectedItems, Canvas2D canvas) {
        canvas.showToolTip("Lens Over");
      }
    });

    Canvas2D canvas = new Canvas2D(700, 500);
    canvas.addLayer(display);

    JFrame frame = new JFrame();
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame.setLayout(new BorderLayout());
    frame.add(canvas, BorderLayout.CENTER);
    frame.add(canvas.getToolBar(), BorderLayout.NORTH);
    frame.setSize(800, 600);
    frame.pack();
    frame.setVisible(true);

  }
}
