package anl.aida.viz.layout;

import anl.aida.client.net.CGEdge;
import anl.aida.client.net.ConceptGraph;
import anl.aida.client.net.TermNode;

import java.awt.*;
import java.awt.geom.Point2D;
import java.util.HashMap;
import java.util.Map;

/**
 * Layout2D that wraps the jung KK layout.
 *
 */
public class KKLayout implements Layout2D<TermNode> {

  private edu.uci.ics.jung.algorithms.layout.KKLayout<TermNode, CGEdge> layout;

  public KKLayout(ConceptGraph graph, Dimension size) {
    initLayout(graph, size);
  }

  private void initLayout(ConceptGraph graph, Dimension size) {
    layout = new edu.uci.ics.jung.algorithms.layout.KKLayout<TermNode, CGEdge>(graph.getGraph());
    layout.setSize(size);
    // > .5 spreads it out vs. default
    //layout.setDisconnectedDistanceMultiplier(.6);
    layout.setLengthFactor(1);
  }


  /**
   * Swaps the layouts current graph with the specified one. This assumes the
   * graphs have equivalent term nodes and it preserves the locations.
   * 
   * @param graph
   *          the graph to swap in.
   */
  public void swapGraph(ConceptGraph<TermNode> graph) {
    if (!layout.getGraph().equals(graph)) {
      Map<String, Point2D> locations = new HashMap<String, Point2D>();
      for (TermNode node : layout.getVertices()) {
	locations.put(node.getLabel(), getLocation(node));
      }
      Dimension size = layout.getSize();
      initLayout(graph, size);

      for (TermNode node : graph.getVertices()) {
	Point2D pt = locations.get(node.getLabel());
	layout.setLocation(node, pt);
      }
    }
  }

  /**
   * Sets the location of the specified node.
   *
   * @param node the node whose location we want to set
   * @param x    the x coordinate
   * @param y    the y coordinate
   */
  public void setLocation(TermNode node, double x, double y) {
    layout.setLocation(node, x, y);
  }

  /**
   * Initializes the layout. This should be called prior to calling getLocation.
   */
  public void initialize() {
    layout.initialize();
    while (!layout.done()) {
      //System.out.println("layout.getStatus() = " + layout.getStatus());
      layout.step();
    }
  }
  
  /**
   * Initializes the layout with the specified size.
   * 
   * @param size the size
   */
  public void initialize(Dimension size) {
    layout.setSize(size);
    initialize();
  }


  /**
   * Gets the location of the object.
   *
   * @param obj the object whose location we want
   * @return the location of the object.
   */
  public Point2D getLocation(TermNode obj) {
    return layout.transform(obj);
  }

  /**
   * Called when the display area size has changed.
   *
   * @param size the new size
   */
  public void sizeChanged(Dimension size) {
    layout.setSize(size);
    while (!layout.done()) {
      //System.out.println("layout.getStatus() = " + layout.getStatus());
      layout.step();
    }
  }
}
