package anl.aida.viz.layout;

import anl.aida.client.net.ConceptGraph;

import java.awt.*;
import java.awt.geom.Point2D;

/**
 * Interface for 2D layouts.
 *
 */
public interface Layout2D<T> {

  /**
   * Initializes the layout. This should be called prior to calling getLocation.
   */
  void initialize();
  
  /**
   * Initializes the layout with the specified size.
   * 
   * @param size the size
   */
  void initialize(Dimension size);

  /**
   * Gets the location of the object.
   *
   * @param obj the object whose location we want
   * @return the location of the object.
   */
  Point2D getLocation(T obj);

  /**
   * Called when the display area size has changed.
   *
   * @param size the new size
   */
  void sizeChanged(Dimension size);

  /**
   * Swaps the layouts current graph with the specified one.
   * This assumes the graphs have equivalent term nodes and
   * it preserves the locations.
   *
   * @param graph the graph to swap in.
   */
  void swapGraph(ConceptGraph<T> graph);

  /**
   * Sets the location of the specified node.
   *
   * @param node the node whose location we want to set
   * @param x    the x coordinate
   * @param y    the y coordinate
   */
  void setLocation(T node, double x, double y);
}
