package anl.aida.viz.layout;

import anl.aida.client.net.CGEdge;
import anl.aida.client.net.ConceptGraph;
import edu.uci.ics.jung.algorithms.layout.SpringLayout2;
import edu.uci.ics.jung.graph.Graph;

import java.awt.*;
import java.awt.geom.Point2D;

/**
 * Layout2D that wraps the jung spring layout.
 *
 */
public class SpringLayout<T> implements Layout2D<T> {

  private SpringLayout2<T, CGEdge> layout;

  public SpringLayout(Graph<T, CGEdge> graph, Dimension size) {
    layout = new SpringLayout2<T, CGEdge>(graph, new SpringLayout2.UnitLengthFunction(100));
    layout.setSize(size);
  }

  /**
   * Initializes the layout. This should be called prior to calling getLocation.
   */
  public void initialize() {
    layout.initialize();
    for (int i = 0; i < 300; i++) {
      //System.out.println("layout.getStatus() = " + layout.getStatus());
      layout.step();
    }
  }

  /**
   * Sets the location of the specified node.
   *
   * @param node the node whose location we want to set
   * @param x    the x coordinate
   * @param y    the y coordinate
   */
  public void setLocation(T node, double x, double y) {
    layout.setLocation(node, x, y);
  }
  
  /**
   * Initializes the layout with the specified size.
   * 
   * @param size the size
   */
  public void initialize(Dimension size) {
    layout.setSize(size);
    initialize();
  }

  /**
   * Swaps the layouts current graph with the specified one.
   * This assumes the graphs have equivalent term nodes and
   * it preserves the locations.
   *
   * @param graph the graph to swap in.
   */
  public void swapGraph(ConceptGraph<T> graph) {
    throw new UnsupportedOperationException();
  }

  /**
   * Gets the location of the object.
   *
   * @param obj the object whose location we want
   * @return the location of the object.
   */
  public Point2D getLocation(T obj) {
    return layout.transform(obj);
  }

  /**
   * Called when the display area size has changed.
   *
   * @param size the new size
   */
  public void sizeChanged(Dimension size) {
    layout.setSize(size);
    for (int i = 0; i < 300; i++) {
      //System.out.println("layout.getStatus() = " + layout.getStatus());
      layout.step();
    }
  }
}