/**
 * 
 */
package anl.aida.viz3d;

import javax.media.opengl.GLAutoDrawable;
import javax.vecmath.Point3f;

/**
 * Interface for classes that will listen for Canvas reshape and init events.
 * 
 */
public interface CanvasListener {
  
  /**
   * Called when all the selected items in a canvas 
   * are unselected.
   */
  void selectionsCleared();
  
  /**
   * Called when an item has been selected in the canvas.
   * 
   * @param obj the selected item.
   */
  void itemSelected(Object obj);
  
  /**
   * Called when a VSpatial has been moved using the mouse.
   * 
   * @param spatial the spatial that has been moved
   * @param localTrans the new local translation of the spatial
   */
  void vSpatialMoved(VSpatial spatial, Point3f localTrans);
  
  /**
   * Called when the the canvas initializes itself in its GLEventListener.init.
   * 
   * @param drawable the canvas drawable
   * @param root the root node of the canvas to which other scene elements can be
   * added.
   */
  void init(GLAutoDrawable drawable, VNode root);
  
  /**
   * Called when canvas reshapes in its GLEventListener.reshape 
   * 
   * @param drawable drawable the canvas drawable
   * @param width the width at z = 0 in world coordinates
   * @param height the height at z = in world coordinates
   * @param root the root node of the canvas to which other scene elements can be
   * added.
   */
  void reshape(GLAutoDrawable drawable, float width, float height, VNode root);
}
