/**
 * 
 */
package anl.aida.viz3d;

import java.awt.Color;

import javax.media.opengl.GL;
import javax.vecmath.Point3f;
import javax.vecmath.Quat4f;
import javax.vecmath.Vector3f;

/**
 *
 */
public class CompositeGeometry extends VSpatial {
  
  private VGeometry[] geometries;
  
  public CompositeGeometry(BoundingSphere sphere, VGeometry...geoms) {
    this.geometries = new VGeometry[geoms.length];
    for (int i = 0; i < geoms.length; i++) {
      this.geometries[i] = geoms[i];
      this.geometries[i].parent = this;
    }
    
    this.boundingSphere = new BoundingSphere(sphere);
  }

  /* (non-Javadoc)
   * @see anl.mifs.viz3d.VSpatial#markAsDirty()
   */
  @Override
  protected void markAsDirty() {
    super.markAsDirty();
    for (VGeometry geom : geometries) {
      geom.markAsDirty();
    }
  }
  
  /*
   * (non-Javadoc)
   * 
   * @see anl.mifs.viz3d.VSpatial#updateWorldTransformation()
   */
  @Override
  public void updateWorldTransformation() {
    super.updateWorldTransformation();
    for (VGeometry geom : geometries) {
      geom.updateWorldTransformation();
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see anl.mifs.viz3d.AbstractVNode#doDraw(javax.media.opengl.GL)
   */
  @Override
  protected void doDraw(GL gl) {
    if (visible) {
      for (VGeometry geom : geometries) {
        geom.draw(gl);
      }
    }
  }
  
  

  /* (non-Javadoc)
   * @see anl.mifs.viz3d.VSpatial#setColor(java.awt.Color, float)
   */
  @Override
  public void setMaterialAppearence(MaterialAppearence app) {
    super.setMaterialAppearence(app);
    for (VGeometry geom : geometries) {
      geom.setMaterialAppearence(app);
    }
  }


  /*
   * (non-Javadoc)
   * 
   * @see anl.mifs.viz3d.VSpatial#intersects(javax.vecmath.Point3f,
   * javax.vecmath.Vector3f, anl.mifs.viz3d.Accumulator)
   */
  @Override
  public void intersects(Point3f rayOrigin, Vector3f rayDirection, Accumulator accumulator) {
    BoundingSphere sphere = getBoundingSphere();
    if (sphere.intersects(rayOrigin, rayDirection)) {
      // if it intersects the children geometry then add this.
      DefaultAccumulator acc = new DefaultAccumulator();
      for (VGeometry geom : geometries) {
	geom.intersects(rayOrigin, rayDirection, acc);
	if (acc.size() > 0) {
	  accumulator.add(this);
	  break;
	}
      }
    }
  }
}
