/**
 * 
 */
package anl.aida.viz3d;

import java.awt.Color;
import java.util.Collection;

import anl.aida.client.net.CGEdge;
import anl.aida.client.net.ConceptGraph;
import anl.aida.client.net.TermNode;

/**
 * Styles edges such that each edge is drawn in a gray scale.
 * Edges with more docs are darker than those with less.
 * 
 */
public class EdgeStyler {
  
  float min = Float.POSITIVE_INFINITY, max = Float.NEGATIVE_INFINITY;
  float diff = 0f;
  
  /**
   * Create an EdgeStyler based on the edge values in the graph.
   * 
   * @param graph 
   */
  public EdgeStyler(ConceptGraph<TermNode> graph) {
    update(graph.getEdges());
  }
  
  /**
   * Styles the specified line. This will calculate and then
   * set the color of the line.
   * 
   * @param line the line to style.
   */
  public void style(VLine line) {
    CGEdge edge = (CGEdge)NodeUtils.getObject(line);
    float scale = calcScale(edge);
    Color color = new Color(scale, scale, scale);
    line.setColor(color);
    NodeUtils.setColor(line, color);
  }
  
  private float calcScale(CGEdge edge) {
    if (diff == 0) return .25f;
    float val = (edge.getDocumentCount() - min) / diff;
    // scale runs from 0 (full black) to .75f (light gray)
    return 1f - (val * .75f + .25f);
  }
  
  public void update(Collection<CGEdge> edges) {
    for (CGEdge edge : edges) {
      int count = edge.getDocumentCount();
      min = Math.min(min, count);
      max = Math.max(max, count);
    }
    diff = max - min;
  }
}
