/**
 * 
 */
package anl.aida.viz3d;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.List;

import javax.media.opengl.GL;
import javax.media.opengl.GLDrawable;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;

import anl.aida.client.net.CGEdge;
import anl.aida.client.net.TermNode;
import anl.aida.util.DateRange;
import anl.aida.viz.Colors;

/**
 * Moves the nodes in reponse to drag events as well as
 * displaying node and edge info in an overlay pane.
 * 
 */
public class InfoDragger extends MouseAdapter {
  
  private static DateFormat formatter = new SimpleDateFormat("MMMMM dd, yyyy");
  
  private boolean mousePressed = false, mouseDragged = false;
  private RayCreator rayCreator = new RayCreator();
  private int x, y;
  private Canvas3D canvas;
  private InfoOverlay overlay;
  private VSpatial node;
  
  private SelectionManager<TermNode> manager;
  
  public InfoDragger(Canvas3D canvas, GLDrawable drawable) {
    this.canvas = canvas;
    manager = new SelectionManager<TermNode>(Colors.SELECTED_COLOR);
    overlay = new InfoOverlay(drawable);
  }
  
  /* (non-Javadoc)
   * @see java.awt.event.MouseAdapter#mouseClicked(java.awt.event.MouseEvent)
   */
  @Override
  public void mousePressed(MouseEvent evt) {
    mousePressed = true;
    x = evt.getX();
    y = evt.getY();
    canvas.display();
  }
  
  public void mouseDragged(MouseEvent evt) {
    if (node != null) {
      mouseDragged = true;
      x = evt.getX();
      y = evt.getY();
      canvas.display();
    }
  }
  
  public void mouseReleased(MouseEvent evt) {
   
    manager.unselectAll();
    overlay.clearLines();
    
    canvas.display();
    node = null;
  }
  
  @SuppressWarnings("unchecked")
  private void colorNode(VSpatial node) {
    manager.unselectAll();
    manager.addSelectedItem(node);
    List<VSpatial> edges = (java.util.List<VSpatial>) node.getProperty(NodeUtils.NODE_EDGES);
    int edgeCount = 0;
    for (VSpatial edge : edges) {
      manager.addSelectedItem(edge);
      java.util.List<VSpatial> nodes = (java.util.List<VSpatial>) edge.getProperty(NodeUtils.EDGE_ENDS);
      // == equality test should be appropriate here
      VSpatial otherNode;
      if (nodes.get(0) == node) otherNode = nodes.get(1);
      else otherNode = nodes.get(0);
      manager.addSelectedItem(otherNode);
      edgeCount++;
    }
    
    TermNode tNode = (TermNode) NodeUtils.getObject(node);
    overlay.setTitle(tNode.getLabel());
   
    DateRange date = NodeUtils.getGraphDate(node);
    if (date != null) {
      String dt = formatter.format(date.getStart());
      overlay.addLine("Date: " + dt);
    }
    overlay.addLine("Document Count: " + tNode.getDocumentCount());
    overlay.addLine("Edge Count: " + edgeCount);
  }
  
  @SuppressWarnings("unchecked")
  private void colorEdge(VSpatial edge) {
    node = null;
    manager.unselectAll();
    manager.addSelectedItem(edge);
    List<VSpatial> ends = (java.util.List<VSpatial>) edge.getProperty(NodeUtils.EDGE_ENDS);
    manager.addSelectedItem(ends.get(0));
    manager.addSelectedItem(ends.get(1));
    
    TermNode tNode1 = (TermNode)NodeUtils.getObject(ends.get(0));
    TermNode tNode2 = (TermNode)NodeUtils.getObject(ends.get(1));
    CGEdge tEdge = (CGEdge) NodeUtils.getObject(edge);
    overlay.setTitle(tNode1.getLabel() + " <-> " + tNode2.getLabel());
    DateRange date = NodeUtils.getGraphDate(edge);
    if (date != null) {
      String dt = formatter.format(date.getStart());
      overlay.addLine("Date: " + dt);
    }
    overlay.addLine("Document Count: " + tEdge.getDocumentCount());
    overlay.addLine("'" + tNode1.getLabel() + "' Document Count: " + tNode1.getDocumentCount());
    overlay.addLine("'" + tNode2.getLabel() + "' Document Count: " + tNode1.getDocumentCount());
  }
  
  public void render() {
    overlay.render();
  }
  
  public void process(GL gl, Point3f viewLocation, VNode root) {
    if (mousePressed || mouseDragged) {
      Ray3f ray = rayCreator.createRay(gl, viewLocation, x, y);
      if (mousePressed) processMousePressed(root, ray);
      if (mouseDragged) processMouseDragged(ray);
    }
  }
  
  @SuppressWarnings("unchecked")
  private synchronized void processMouseDragged(Ray3f ray) {
    //System.out.println();
    mouseDragged = false;
    // use the ray to get the world x, y at the node's z
    Point3f center;
    if (node instanceof LabeledShape) {
      center = ((LabeledShape)node).getNodeBoundingSphere().getCenterRef();
    } else {
      center = node.getBoundingSphere().getCenterRef();
    }
    
    Vector3f planeNormal = new Vector3f(0, 0, 1);
    Vector3f ptOnPlane = new Vector3f(0, 0, center.z);
    ptOnPlane.sub(ray.getOrigin());
    
    float d1 = planeNormal.dot(ptOnPlane);
    float d2 = ray.getDirection().dot(planeNormal);
    
    // t is distance from origin to intersection
    // point along the ray
    float t = d1 / d2;
    
    Point3f pt = new Point3f(ray.getDirection());
    pt.scale(t);
    pt.add(ray.getOrigin());
    
    //transform from world coordinates into local coords.
    node.getWorldToLocalTransform().transform(pt);
    
    //System.out.println("old trans = " + node.localTrans);
    //System.out.println("new trans = " + pt);
    
    Point3f trans = new Point3f(node.localTrans);
    trans.add(pt);
    node.translate(trans);
    
    canvas.vSpatialMoved(node, trans);
    
    // update the edges
    if (node instanceof LabeledShape) {
      center = ((LabeledShape)node).getNodeBoundingSphere().getCenterRef();
    } else {
      center = node.getBoundingSphere().getCenterRef();
    }
    Point3f newCenter = new Point3f(center);
    node.getParent().getWorldToLocalTransform().transform(newCenter);
    List<VSpatial> edges = (java.util.List<VSpatial>) node.getProperty(NodeUtils.NODE_EDGES);
    for (VSpatial edge : edges) {
      VLine line = (VLine)edge;
      List<VSpatial> nodes =  (java.util.List<VSpatial>)line.getProperty(NodeUtils.EDGE_ENDS);
      if (node.equals(nodes.get(0))) {
	// is the starting node
	line.resetStartPt(newCenter);
      } else {
	// is the ending node
	line.resetEndPt(newCenter);
      }
    }
    canvas.display();
  }
  
  private void processMousePressed(VNode root, Ray3f ray) {
    overlay.clearLines();
    Accumulator items = new DefaultAccumulator();
    root.intersects(ray.getOrigin(), ray.getDirection(), items);

    boolean doEdge = true;
    for (VSpatial item : items.items()) {
      if (NodeUtils.isNode(item)) {
	doEdge = false;
	this.node = item;
	colorNode(item);
	break;
      }
    }

    if (doEdge) {
      for (VSpatial item : items.items()) {
	if (NodeUtils.isEdge(item)) {
	  colorEdge(item);
	  break;
	}
      }
    }

    mousePressed = false;
    canvas.display();
  }
}
