/**
 * 
 */
package anl.aida.viz3d;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Font;
import java.awt.GradientPaint;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.font.FontRenderContext;
import java.awt.font.GlyphVector;
import java.util.ArrayList;
import java.util.List;

import javax.media.opengl.GLDrawable;

import anl.aida.viz.Colors;

import com.sun.opengl.util.j2d.Overlay;
import edu.umd.cs.piccolo.nodes.PText;

/**
 * 
 */
public class InfoOverlay {
  
  private Overlay overlay;
  private Font font = new PText().getFont().deriveFont(Font.BOLD, 12f);
  private Font lineFont = font.deriveFont(11f);
  private String title = "";
  private List<String> lines = new ArrayList<String>();
  private int[] ys;
  private Color transGray, transBlack;
  private Rectangle rect = new Rectangle(0, 0, 0, 0);
  //private Color TRANSPARENT_BLACK = new Color(0.0f, 0.0f, 0.0f, 0.0f);

  public InfoOverlay(GLDrawable drawable) {
    overlay = new Overlay(drawable);
  
    int alpha = (int)(255 * .7);
    transGray = new Color(Color.GRAY.getRed(), Color.GRAY.getGreen(), Color.GRAY.getBlue(), alpha);
    transBlack = new Color(Color.BLACK.getRed(), Color.BLACK.getGreen(), Color.BLACK.getBlue(), alpha);
  }
  
  public void setTitle(String title) {
    this.title = title;
  }
  
  public void addLine(String line) {
    lines.add(line);
  }
  
  public void clearLines() {
    lines.clear();
  }
  
  private void createRect(Graphics2D g2) {
    
    int width = 0;
    FontRenderContext frc = g2.getFontRenderContext();
    ys = new int[lines.size() + 2];
    Rectangle rect = font.createGlyphVector(frc, title).getPixelBounds(frc, 0, 0);
    ys[0] = rect.height + 15;
    ys[1] = ys[0] + 4;
    int i = 2;
    for (String line : lines) {
      GlyphVector gv = font.createGlyphVector(frc, line);
      rect = gv.getPixelBounds(frc, 0, 0);
      int last = ys[i - 1];
      ys[i++] = rect.height + last + 4;
      if (rect.width > width) width = rect.width + 10;
      
    }
    
    this.rect = new Rectangle(10, 10, width + 10, ys[i - 1]);
  }
  
  
  public void render() {
    if (lines.size() > 0) {
      Graphics2D g2 = overlay.createGraphics();
      g2.setColor(Colors.BACKGROUND_COLOR);
      g2.fillRect(rect.x, rect.y, rect.width, rect.height);
      Rectangle dirtyRect = new Rectangle(rect);
      createRect(g2);
      g2.setPaint(new GradientPaint(rect.x, rect.y, transGray, rect.x + rect.width,
	  rect.y + rect.height, transBlack));
      g2.setComposite(AlphaComposite.Src);
      g2.setFont(font);
      
      g2.fillRoundRect(rect.x, rect.y, rect.width, rect.height, 10, 10);
      g2.setPaint(Color.WHITE);
      
      g2.drawString(title, rect.x + 10, ys[0]);
      g2.drawLine(rect.x + 10, ys[1], rect.width - 10, ys[1]);
      
      int i = 2;
      g2.setFont(lineFont);
      for (String line : lines) {
	g2.drawString(line, rect.x + 10, ys[i++]);
      }
      dirtyRect = dirtyRect.union(rect);
      
      overlay.markDirty(dirtyRect.x, dirtyRect.y, dirtyRect.width, dirtyRect.height);
      overlay.drawAll();
      g2.dispose();
    }
  }
}
