package anl.aida.viz3d;

import java.awt.Color;

import javax.media.opengl.GL;
import javax.vecmath.Color4f;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;

import anl.aida.viz3d.Utils3D.Location;

public class LabeledShape extends VSpatial {
  
  private VSpatial node;
  private VLabel label;
  private float[] offset;
  
  public LabeledShape (VSpatial node, String label, float[] offset) {
    this.node = node;
    this.label = VShapeFactory.getInstance().createLabel(label);
    this.offset = offset;
    node.setParent(this);
    this.label.setParent(this);
    resetLabelLocation();
  }
  
  
  
  /* (non-Javadoc)
   * @see anl.mifs.viz3d.VSpatial#markAsDirty()
   */
  @Override
  protected void markAsDirty() {
    super.markAsDirty();
    node.markAsDirty();
    label.markAsDirty();
  }

  /* (non-Javadoc)
   * @see anl.mifs.viz3d.VSpatial#updateWorldTransformation()
   */
  @Override
  public void updateWorldTransformation() {
    super.updateWorldTransformation();
    node.updateWorldTransformation();
    label.updateWorldTransformation();
  }

  /**
   * Scales the node part of this node label, leaving the label
   * as is.
   * 
   * @param scale the amount to scale
   */
  public void scaleNode(float scale) {
    node.scale(scale);
    resetLabelLocation();
  }
  
  public void setNodeMaterialAppearence(MaterialAppearence app) {
    node.setMaterialAppearence(app);
  }
  
  
  /* (non-Javadoc)
   * @see anl.mifs.viz3d.VSpatial#getMaterialAppearence()
   */
  @Override
  public MaterialAppearence getMaterialAppearence() {
    return node.getMaterialAppearence();
  }

  /* (non-Javadoc)
   * @see anl.mifs.viz3d.VSpatial#setMaterialAppearence(anl.mifs.viz3d.MaterialAppearence)
   */
  @Override
  public void setMaterialAppearence(MaterialAppearence app) {
    node.setMaterialAppearence(app);
  }

  /**
   * Sets the color of this LabeledNode's label.
   * 
   * @param color the new color
   */
  public void setLabelColor(Color color) {
    label.setColor(color);
  }
  
  /**
   * Gets the color of this LabeledNode's label.
   * 
   * @return  the color of this LabeledNode's label.
   */
  public Color4f getLabelColor() {
    return label.getColor();
  }
  
  
  /**
   * Sets the visibility of this LabeledNode's node.
   *  
   * @param visible the new visibility property
   */
  public void setNodeVisible(boolean visible) {
    node.setVisible(visible);
  }
  
  /**
   * Gets whether or not this LabeledNode's node is visible.
   * 
   * @return true if this LabeledNode's node is visible, otherwise false.
   */
  public boolean isNodeVisible() {
    return node.isVisible();
  }
  
  
  /**
   * Sets the visibility of this LabeledNode's label.
   *  
   * @param visible the new visibility property
   */
  public void setLabelVisible(boolean visible) {
    label.setVisible(visible);
  }
  
  /**
   * Gets whether or not this LabeledNode's label. is visible.
   * 
   * @return true if this LabeledNode's label.is visible, otherwise false.
   */
  public boolean isLabelVisible() {
    return label.isVisible();
  }
  
  /*
   * (non-Javadoc)
   * 
   * @see anl.mifs.viz3d.AbstractVNode#doDraw(javax.media.opengl.GL)
   */
  @Override
  protected void doDraw(GL gl) {
    node.draw(gl);
    label.draw(gl);
  }
  
  /**
   * Gets the node (non-label) bounding Sphere.
   * 
   * @return the node bounding sphere
   */
  public BoundingSphere getNodeBoundingSphere() {
    return node.getBoundingSphere();
  }
  
  protected BoundingSphere doGetBoundingSphere() {
    BoundingSphere sphere = new BoundingSphere(node.doGetBoundingSphere());
    sphere.merge(label.doGetBoundingSphere());
    return sphere;
  }

  /* (non-Javadoc)
   * @see anl.mifs.viz3d.VSpatial#intersects(javax.vecmath.Point3f, javax.vecmath.Vector3f, anl.mifs.viz3d.Accumulator)
   */
  @Override
  public void intersects(Point3f rayOrigin, Vector3f rayDirection, Accumulator accumulator) {
    DefaultAccumulator acc = new DefaultAccumulator();
    node.intersects(rayOrigin, rayDirection, acc);
    if (acc.size() > 0) accumulator.add(this);
  }

  private void resetLabelLocation() {
    Point3f offset = Utils3D.getRelativeLocation(node, Location.NORTH_EAST);
    offset.x += this.offset[0];
    offset.y += this.offset[1];
    offset.z += this.offset[2];
    label.translate(offset);
  }
}
