/**
 * 
 */
package anl.aida.viz3d;

import java.awt.event.MouseEvent;

import javax.media.opengl.GLAutoDrawable;
import javax.vecmath.Vector3f;

/**
 * Adds translate to a matrix via a mouse (third button drag). This will 
 * call canvas.display() when at the end of each drag event.
 * 
 * 
 */
public class MouseTranslate extends InputHandler {
  
  private float  lastX, lastY, xFactor, yFactor;
  private Vector3f translation = new Vector3f();
  private GLAutoDrawable canvas;
  
  public MouseTranslate(GLAutoDrawable canvas) {
    this.canvas = canvas;
    xFactor = .01f;
    yFactor = .01f;
  }
  
  public Vector3f getTranslation() {
    return translation;
  }
  
  public void reset() {
    translation.set(0, 0, 0);
  }
  
  public void mousePressed(MouseEvent evt) {
    if (((evt.getModifiers() & MouseEvent.BUTTON3_MASK) != 0) && isEnabled) {
      lastX = evt.getX();
      lastY = evt.getY();
    }
  }

  // Methods required for the implementation of MouseMotionListener
  public void mouseDragged(MouseEvent evt) {
    if (((evt.getModifiers() & MouseEvent.BUTTON3_MASK) != 0) && isEnabled) {

      int x = evt.getX();
      int y = evt.getY();

      float dx = x - lastX;
      float dy = y - lastY;
      
      translation.x += dx * xFactor;
      translation.y += -dy * yFactor;

      lastX = x;
      lastY = y;

      canvas.display();
    }
  }
  
  public String toString() {
    return String.format("Translate: tx: %f, ty: %f, tz: %f", translation.x, 
	translation.y, 0f);
  }
}
