package anl.aida.viz3d;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.vecmath.Point3f;

import anl.aida.client.net.CGEdge;
import anl.aida.client.net.ConceptGraph;
import anl.aida.client.net.TermNode;
import anl.aida.util.DateRange;
import anl.aida.viz.Colors;
import anl.aida.viz.layout.Layout2D;
import edu.uci.ics.jung.graph.util.Pair;

public class NetworkVizCreator  {
  
  private static int SCALE_FACTOR = 1000;
  
  private Layout2D<TermNode> layout;
  private Color nodePaint;
  private float width, height;
  private EdgeStyler edgeStyler;
  private List<VSpatial> emptyNodes = new ArrayList<VSpatial>();
  private boolean emptyNodesVisible = false;
  
  public NetworkVizCreator(float width, float height, Layout2D<TermNode> layout) {
    this.layout = layout;
    this.width = width;
    this.height = height;
    layout.initialize(new Dimension((int)(width * SCALE_FACTOR), (int)(height * SCALE_FACTOR)));
    nodePaint = new Color(Colors.NODE_COLOR.getRed(), Colors.NODE_COLOR.getGreen(), Colors.NODE_COLOR.getBlue(),
	Colors.NODE_ALPHA);
  }
  
  /**
   * Gets whether or not the empty nodes are visible.
   * 
   * @return whether or not the empty nodes are visible.
   */
  public boolean isEmptyNodesVisible() {
    return emptyNodesVisible;
  }
  
  /**
   * Sets the visibility of empty nodes.
   * 
   * @param visible
   */
  public void setEmptyNodesVisible(boolean visible) {
    if (emptyNodesVisible != visible) {
      emptyNodesVisible = visible;
      for (VSpatial node : emptyNodes) {
	node.setVisible(visible);
      }
    }
  }
  
  /**
   * Updates the location of the specified spatial to the
   * local transform.
   * 
   * @param spatial the spatial whose location we should update
   * @param localTrans the new location
   */
  public void updateLayout(VSpatial spatial, Point3f localTrans) {
    TermNode node = (TermNode)NodeUtils.getObject(spatial);
    if (node != null) layout.setLocation(node, localTrans.x * SCALE_FACTOR, localTrans.y * SCALE_FACTOR);
  }
  
  @SuppressWarnings("unchecked")
  public void addTo(ConceptGraph<TermNode> graph, VNode parent, DateRange date) {
    NodeStyler nodeStyler = new NodeStyler(nodePaint, Colors.TEXT_COLOR, graph.getVertices());
    VNode nodeLayer = new VNode();
    
    nodeLayer.id = "NODE LAYER";
    
    VNode edgeLayer = new VNode();
    edgeLayer.id = "EDGE LAYER";
    
    nodeLayer.translate(-width / 2, -height / 2, 0);
    edgeLayer.translate(-width / 2, -height / 2, 0);
    parent.addChild(nodeLayer);
    parent.addChild(edgeLayer);
    layout.swapGraph(graph);
    
    Map<TermNode, LabeledShape> nodeMap = new HashMap<TermNode, LabeledShape>();

    for (TermNode tNode : graph.getVertices()) {
      Point2D coord = layout.getLocation(tNode);
      LabeledShape node = nodeStyler.createNode(tNode, coord);
      nodeLayer.addChild(node);
      nodeMap.put(tNode, node);
      if (date != null) NodeUtils.putGraphDate(node, date);
      boolean empty = tNode.getDocumentCount() == 0;
      node.setVisible(!empty || emptyNodesVisible);
      if (empty) {
	emptyNodes.add(node);
      }
      
      if (date != null) NodeUtils.putGraphDate(node, date);
      NodeUtils.setObject(node, tNode);
      NodeUtils.setAsNode(node);
    }

    if (edgeStyler == null) edgeStyler = new EdgeStyler(graph);
    else {
      edgeStyler.update(graph.getEdges());
    }
    for (CGEdge tEdge : graph.getEdges()) {
      Pair<TermNode> nodes = graph.getEndpoints(tEdge);
      LabeledShape node1 = nodeMap.get(nodes.getFirst());
      LabeledShape node2 = nodeMap.get(nodes.getSecond());

      Point3f source = new Point3f(node1.localTrans);
      Point3f target = new Point3f(node2.localTrans);
      
      VLine edge = new VLine(source, target);
      NodeUtils.setObject(edge, tEdge);
      if (date != null) NodeUtils.putGraphDate(edge, date);
      NodeUtils.setAsEdge(edge, node1, node2);
      edgeStyler.style(edge);

      List<VSpatial> list = (java.util.List<VSpatial>) node1.getProperty(NodeUtils.NODE_EDGES);
      list.add(edge);
      list = (java.util.List<VSpatial>) node2.getProperty(NodeUtils.NODE_EDGES);
      list.add(edge);

      edgeLayer.addChild(edge);
    }
  }
}
