/**
 * 
 */
package anl.aida.viz3d;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.HashSet;
import java.util.Set;

import javax.media.opengl.GL;
import javax.vecmath.Point3f;

import anl.aida.client.Cache;
import anl.aida.client.ShowDocViewOp;
import anl.aida.client.doc.DocumentContainer;
import anl.aida.client.gui.AIDAGui;
import anl.aida.client.net.TermNode;
import anl.aida.core.DocumentID;
import anl.aida.util.DateRange;
import anl.aida.viz.Colors;

/**
 * Performs node selection via a mouse click.
 * 
 *
 */
public class NodeSelector extends MouseAdapter {
  
  private boolean mousePressed = false;
  private RayCreator rayCreator = new RayCreator();
  private int x, y;
  private Canvas3D canvas;
  private SelectionManager<TermNode> manager;
  private boolean ctrlDown = false;
  
  private Cache cache;
  private ShowDocViewOp docViewOp;
  
  public NodeSelector(Canvas3D canvas, Cache cache) {
    this.canvas = canvas;
    manager = new SelectionManager<TermNode>(Colors.SELECTED_COLOR);
    this.cache = cache;
  }
  
  public void clearSelection() {
    manager.unselectAll();
  }
  
  public ShowDocViewOp getDocViewOp(AIDAGui gui) {
    if (docViewOp == null) {
      docViewOp = new ShowDocViewOp(gui, cache.getElement(1));
    }
    return docViewOp;
  }

  /* (non-Javadoc)
   * @see java.awt.event.MouseAdapter#mouseClicked(java.awt.event.MouseEvent)
   */
  @Override
  public void mouseClicked(MouseEvent e) {
    mousePressed = true;
    ctrlDown = e.isControlDown();
    x = e.getX();
    y = e.getY();
    canvas.display();
  }
  
  private void addSelectionsToCanvas() {
    canvas.clearSelections();
    Set<DocumentID> ids = new HashSet<DocumentID>();
    for (DocumentContainer docs : manager.getSelectedObjects()) {
      for (DocumentID id : docs.documents()) {
        ids.add(id);
      }
    }

    for (DocumentID id : ids) {
      canvas.addSelection(id);
    }
  }
  
  public void process(GL gl, Point3f viewLocation, VNode root) {
    if (mousePressed) {
      Ray3f ray = rayCreator.createRay(gl, viewLocation, x, y);
      Accumulator items = new DefaultAccumulator();
      root.intersects(ray.getOrigin(), ray.getDirection(), items);
      
      DateRange range = null;

      if (!ctrlDown)
	manager.unselectAll();
      // add nodes, if any nodes are added then don't add the edges
      // because those edges are probably the edges terminating "inside" the
      // node
      boolean addEdges = true;
      for (VSpatial item : items.items()) {
	if (NodeUtils.isNode(item)) {
	  manager.addSelectedItem(item);
	  range = NodeUtils.getGraphDate(item);
	  addEdges = false;
	  break;
	}
      }

      if (addEdges) {
	for (VSpatial item : items.items()) {
	  if (NodeUtils.isEdge(item)) {
	    range = NodeUtils.getGraphDate(item);
	    manager.addSelectedItem(item);
	    break;
	  }
	}
      }
      
      mousePressed = false;
      addSelectionsToCanvas();
      canvas.display();
      
      if (range != null) {
	docViewOp.setElement(cache.getElement(range));
      }
    }
  }
}
