package anl.aida.viz3d;

import java.awt.Color;
import java.awt.geom.Point2D;
import java.util.Collection;

import anl.aida.client.net.ValueNode;

public class NodeStyler<T extends ValueNode> {
  
  private static float BASE_NODE_SIZE = .01f;
  
  private Color textColor;
  private MaterialAppearence nodeApp;
  private float avg;
  
  public NodeStyler(Color nodeColor, Color textColor, float avg) {
    this.nodeApp = AppearenceFactory.createAppearence(nodeColor);
    this.textColor = textColor;
    this.avg = avg;
  }
  
  public NodeStyler(Color nodeColor, Color textColor, Collection<T> vertices) {
    this.nodeApp = AppearenceFactory.createAppearence(nodeColor);
    this.textColor = textColor;
    
 // nodes with doc > 0
    int nodeCount = 0;
    for (T tNode : vertices) {
      double val = tNode.getValue();
      if (val != 0) {
	avg += val;
	nodeCount++;
      }
    }
    if (avg == 0 || nodeCount == 0) avg = 1;
    else avg /= nodeCount;
  }
  
  public LabeledShape createNode(T tNode, Point2D coord) {
    VSpatial item = VShapeFactory.getInstance().createDisk(getSize(tNode), .02f, 32);
    LabeledShape node = new LabeledShape(item, tNode.getLabel(), new float[] { .001f, .001f, .002f });
    node.setNodeMaterialAppearence(nodeApp);
    NodeUtils.setAppearence(node, nodeApp);
    node.setLabelColor(textColor);
    node.translate((float) coord.getX() * .001f, (float) coord.getY() * .001f, 0);
    
    return node;
  }
  
  /**
   * Gets the size appropriate for the specified term node.
   *
   * @param tNode the node to get the size for
   * @return the size appropriate for the specified term node.
   */
  public float getSize(T tNode) {
    float factor = (float)(tNode.getValue() / avg);
    float size = factor * BASE_NODE_SIZE;
    //System.out.println("size = " + size);
    return size < .0025f ? .0025f : size;
    
  }

}
