package anl.aida.viz3d;

import java.awt.Color;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 * 
 */
public class SelectionManager<T> {

  private Color selectionColor;
  private MaterialAppearence selectionApp;
  private Set<VSpatial> selectedNodes = new HashSet<VSpatial>();

  public SelectionManager(Color selectionColor) {
    selectionApp = AppearenceFactory.createAppearence(selectionColor);
    //selectionApp.setEmissiveColor(new Color4f(selectionColor));
    this.selectionColor = selectionColor;
  }

  public void unselectAll() {
    for (VSpatial node : selectedNodes) {
      if (NodeUtils.isEdge(node)) ((VLine)node).setColor(NodeUtils.getColor(node));
      else if (NodeUtils.isNode(node)) node.setMaterialAppearence(NodeUtils.getAppearance(node));
    }
    selectedNodes.clear();
  }

  public void addSelectedItem(VSpatial node) {
    selectedNodes.add(node);
    if (NodeUtils.isEdge(node)) ((VLine)node).setColor(selectionColor);
    else if (NodeUtils.isNode(node)) node.setMaterialAppearence(selectionApp);
  }

  public void unselect(VSpatial node) {
    selectedNodes.remove(node);
    if (NodeUtils.isEdge(node)) ((VLine)node).setColor(NodeUtils.getColor(node));
    else if (NodeUtils.isNode(node)) node.setMaterialAppearence(NodeUtils.getAppearance(node));
  }

  public Iterable<T> getSelectedObjects() {
    return new Iter<T>(selectedNodes.iterator());
  }

  public void paintSelection() {
    for (VSpatial node : selectedNodes) {
      if (NodeUtils.isEdge(node)) ((VLine)node).setColor(selectionColor);
      else if (NodeUtils.isNode(node)) node.setMaterialAppearence(selectionApp);
    }
  }

  private static class Iter<T> implements Iterable<T>, Iterator<T> {

    Iterator<VSpatial> iter;

    private Iter(Iterator<VSpatial> iter) {
      this.iter = iter;
    }

    public Iterator<T> iterator() {
      return this;
    }

    public boolean hasNext() {
      return iter.hasNext();
    }

    @SuppressWarnings("unchecked")
    public T next() {
      return (T) NodeUtils.getObject(iter.next());
    }

    public void remove() {
      throw new UnsupportedOperationException();
    }
  }


}
