/**
 * 
 */
package anl.aida.viz3d;

import javax.media.opengl.GL;

import com.sun.opengl.util.j2d.TextRenderer;

/**
 * An item in a StackDisplay.
 * 
 */
public class StackItem implements Comparable<StackItem> {
  
  private static float MAX_X = .9f;
  private static float MAX_Z = .7f;
  private static float MAX_ROT = 90f;
  
  private float rotation, x, z;
  private StackDrawable drawable;

  /**
   * Creates a stack item that will draw the specified drawable.
   * 
   * @param drawable
   *          the drawable to draw
   */
  public StackItem(StackDrawable drawable) {
    this.drawable = drawable;
  }
  
  /**
   * Initializes this StackItem. This may be called multiple times
   * during the life of this stack item in response to OpenGL context 
   * changes.
   */
  public void init() {
    drawable.init();
  }
  
  /**
   * Dispose of any resources this StackItem may be holding.
   */
  public void dispose() {
    drawable.dispose();
  }
  
  /**
   * Gets an index for this StackItem that can be used
   * to sort this item with respect to other items.
   * 
   * @return an sorting index for this StackItem
   */
  public long getSortingIndex() {
    return drawable.getSortIndex();
  }

  /**
   * Puts this StackItem completely off the stack.
   */
  public void putOffStack() {
    this.x = MAX_X;
    this.rotation = MAX_ROT;
    this.z = MAX_Z;
  }
  
  /**
   * Puts this StackItem completely on the stack.
   */
  public void putOnStack() {
    rotation = x = z = 0;
  }

  /**
   * Gets whether or not this slice is currently moving. A moving
   * StackItem is neither fully on or off the stack.
   * 
   * @return whether or not this slice is currently moving.
   */
  public boolean isMoving() {
    return rotation > .001f && rotation < 90;
  }

  /**
   * Moves this StackItem off the stack.
   * 
   * @return true if this item is completely off the stack
   * and will move no further, otherwise false.
   */
  public synchronized boolean moveOff() {
    if (rotation < 90) {
      rotation += 5f;
      if (rotation > 20f)
	z += .05f;
      x += .05f;
    }

    return rotation >= 90;
  }

  /**
   * Moves this StackItem onto the stack.
   * 
   * @return true if this item is completely on the stack
   * and will move no further, otherwise false.
   */
  public synchronized boolean moveOn() {
    if (rotation > 0) {
      if (rotation > 20f)
	z -= .05f;
      rotation -= 5f;
      x -= 0.05f;
    }

    return rotation <= 0.001;
  }
  
  public String toString() {
    return String.format("StackItem: x: %f, z: %f, rotation: %f", x, z, rotation);
  }
  
  /**
   * Draws this StackItem.
   * 
   * @param gl
   * @param zOff the z offset for this StackItem
   * @param xOff the x offset for this StackItem
   */
  public synchronized void draw(GL gl, float xOff, float zOff) {
    gl.glPushMatrix();
    // gl.glColor4fv(color, 0);
    gl.glTranslatef(-x + xOff, 0f, z + zOff);
    if (rotation != 0)
      gl.glRotatef(rotation, 0f, .5f, 0f);
    drawable.draw(gl);
    gl.glPopMatrix();
  }

  @Override
  public int compareTo(StackItem o) {
    long i1 = getSortingIndex();
    long i2 = o.getSortingIndex();
    
    return i1 < i2 ? -1 : i1 == i2 ? 0 : 1;
  }
}
