package anl.aida.viz3d;

import static javax.media.opengl.GL.GL_COLOR_BUFFER_BIT;

import java.awt.Color;
import java.awt.Font;
import java.util.ArrayList;
import java.util.List;

import javax.media.opengl.DebugGL;
import javax.media.opengl.GL;
import javax.media.opengl.GLAutoDrawable;
import javax.media.opengl.GLCanvas;
import javax.media.opengl.GLEventListener;
import javax.media.opengl.glu.GLU;

import com.sun.opengl.util.j2d.TextRenderer;

/**
 * Renders a 3D stack of stack items. Items can be rotated on and off the
 * display.
 * 
 */
public class StackRenderer implements GLEventListener {
  
  private static final String DEBUG_KEY = "anl.aida.jogl.debug";

  private static float Z_INTERVAL = .15f;
  private static float X_INTERVAL = .1f;

  private GLU glu = new GLU();

  private Color background;

  private GLCanvas canvas;
  private MouseRotate rotator;
  private MouseTranslate translator;
  private MouseWheelZoom wheelZoomer;
  private MouseZoom zoomer;
  private boolean addKeyListener, initialized;
  private TextRenderer textRenderer;

  private List<StackItem> items = new ArrayList<StackItem>();
  private int stackIndex = 0;
  private boolean atTop, atBottom;
  //private Axis axis = new Axis();
  
  /*
  private StackItemTextFormatter formatter = new StackItemTextFormatter() {
    @Override
    public String format(StackItem item) {
      return String.valueOf(item.getSortingIndex());
    }
  };
  */

  /**
   * Creates a StackRenderer that will render to the specified canvas.
   * 
   * @param canvas
   *          the canvas to render to
   */
  public StackRenderer(GLCanvas canvas, Color background) {
    this(canvas, background, false);
  }
  
  
  /**
   * Gets the sorter used to sort the stack items.
   * 
   * @return the sorter used to sort the stack items.
   *
  public StackSorter getStackSorter() {
    return sorter;
  }*/

  /**
   * Creates a StackRenderer that will render to the specified canvas.
   * 
   * @param canvas
   *          the canvas to render to
   */
  public StackRenderer(GLCanvas canvas, Color background, boolean addKeyListener) {
    this.canvas = canvas;
    rotator = new MouseRotate(canvas, -4f, 11f);
    translator = new MouseTranslate(canvas);
    wheelZoomer = new MouseWheelZoom(canvas);
    zoomer = new MouseZoom(canvas);
    this.background = background;
    atTop = true;
    atBottom = false;
    this.addKeyListener = addKeyListener;
  }
  
  /**
   * Gets the StackItemFormatter used by this StackRenderer. 
   * 
   * @return the StackItemFormatter used by this StackRenderer. 
   *
  public StackItemTextFormatter getFormatter() {
    return formatter;
  }

  /**
   * Sets the StackItemFormatter used by this StackRenderer. 
   * 
   * @return the StackItemFormatter used by this StackRenderer. 
   *
  public void setFormatter(StackItemTextFormatter formatter) {
    this.formatter = formatter;
  }
  */

  /**
   * Gets the canvas associated with this drawable.
   * 
   * @return the canvas associated with this drawable.
   */
  GLCanvas getCanvas() {
    return canvas;
  }

  /**
   * Adds a StackItem to this StackRenderer.
   * 
   * @param item the item to add
   */
  public void addStackItem(StackItem item) {
    items.add(item);
    //sorter.sort(items);
    if (initialized) {
      int index = items.indexOf(item);
      resetItems();
      jumpTo(index, false);
      canvas.display();
    }
  }

  private void glInit(GL gl) {
    gl.glClearColor(background.getRed() / 255f, background.getGreen() / 255f,
	background.getBlue() / 255f, 0f);
    gl.setSwapInterval(1);
    gl.glShadeModel(GL.GL_SMOOTH);
    gl.glEnable(GL.GL_DEPTH_TEST);
    gl.glHint(GL.GL_PERSPECTIVE_CORRECTION_HINT, GL.GL_NICEST);
    textRenderer = new TextRenderer(new Font("SanSerif", Font.BOLD, 14), true, false);
  }

  private void listenerInit(GLAutoDrawable drawable) {
    if (!initialized) {
      drawable.addMouseListener(rotator);
      drawable.addMouseMotionListener(rotator);
      drawable.addMouseListener(translator);
      drawable.addMouseMotionListener(translator);
      drawable.addMouseListener(zoomer);
      drawable.addMouseMotionListener(zoomer);
      drawable.addMouseWheelListener(wheelZoomer);

      //if (addKeyListener)
	//drawable.addKeyListener(new StackKeyHandler(this));
    }
  }

  @Override
  public void init(GLAutoDrawable drawable) {
    if (Boolean.getBoolean(DEBUG_KEY)) {
      drawable.setGL(new DebugGL(drawable.getGL()));
    }
    glInit(drawable.getGL());
    listenerInit(drawable);
    for (StackItem item : items) {
      item.init();
    }
    initialized = true;
  }

  @Override
  public void reshape(GLAutoDrawable drawable, int x, int y, int width, int height) {
    GL gl = drawable.getGL();
    float aspect = (float) width / (float) height;

    gl.glMatrixMode(GL.GL_PROJECTION);
    gl.glLoadIdentity();
    glu.gluPerspective(40f, aspect, 1.0, 20.0);
    gl.glMatrixMode(GL.GL_MODELVIEW);
    gl.glLoadIdentity();
    //gl.glTranslatef(0.0f, 0.0f, -1.3f);
    glu.gluLookAt(0, 0, 1.5, 0, 0, -1, 0, 1, 0);
  }

  @Override
  public void displayChanged(GLAutoDrawable drawable, boolean modeChanged, boolean deviceChanged) {
  }

  @Override
  public void display(GLAutoDrawable drawable) {
    GL gl = drawable.getGL();
    gl.glClear(GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);

    gl.glPushMatrix();
    //wheelZoomer.process(gl);
    //zoomer.process(gl);
    //translator.process(gl);
    //rotator.process(gl);

    gl.glEnable(GL.GL_BLEND);
    gl.glBlendFunc(GL.GL_ONE, GL.GL_ONE_MINUS_SRC_ALPHA);

    gl.glEnable(GL.GL_TEXTURE_2D);
    gl.glTexEnvi(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_REPLACE);

    // draws the off stack panels; draw in
    // z buffer order for proper transparency
    float xOff = -X_INTERVAL * (stackIndex - 1) - .5f;
    for (int i = 0; i < stackIndex; i++) {
      //System.out.printf("item %d at xoff %f%n", i, xOff);
      items.get(i).draw(gl, xOff, 0);
      xOff += X_INTERVAL;
    }

    // draws on stack panels; draw in
    // z buffer order for proper transparency
    float zOff = -(Z_INTERVAL * (items.size() - stackIndex));
    for (int i = items.size() - 1; i >= stackIndex; i--) {
      items.get(i).draw(gl, 0, zOff);
      zOff += Z_INTERVAL;
    }

    gl.glDisable(GL.GL_TEXTURE_2D);
    gl.glDisable(GL.GL_BLEND);
    
    /*
    zOff = -(Z_INTERVAL * (items.size() - stackIndex));
    for (int i = items.size() - 1; i >= stackIndex; i--) {
      BoundingBox box = items.get(i).getBoundingBox();
      Point3f lower = box.getLower();
      float zStart = zOff;
      zOff += Z_INTERVAL;
      axis.draw(gl, lower, zStart, zOff);
    }
    */

    if (items.size() > 0) {
      textRenderer.beginRendering(drawable.getWidth(), drawable.getHeight());
      textRenderer.setColor(0, 0, 0, 1);
      StackItem item = items.get(stackIndex);
      //textRenderer.draw(formatter.format(item), 20, 20);
      textRenderer.endRendering();
    }

    gl.glPopMatrix();
  }
  
  private void resetItems() {
    for (StackItem item : items) {
      item.putOnStack();
    }
    stackIndex = 0;
  }

  /**
   * Resets this StackRenderer such that all the items are on the stack and
   * visual position of the stack returns to its default.
   */
  public void reset() {
    resetItems();
    zoomer.reset();
    wheelZoomer.reset();
    rotator.reset();
    translator.reset();
  }

  /**
   * Pops the current stack item completely off the stack and make the current
   * stack item the next one.
   */
  public void next() {
    if (stackIndex < items.size() - 1) {
      items.get(stackIndex).putOffStack();
      stackIndex++;
    }
  }

  /**
   * If the current stack item is current moving onto / off the stack, then this
   * pushes it completely onto the stack. If it is not moving, then the previous
   * item is pushed onto the stack.
   */
  public void prev() {
    if (stackIndex > 0) {
      boolean isMoving = items.get(stackIndex).isMoving();
      items.get(stackIndex).putOnStack();
      if (!isMoving) {
	stackIndex--;
	items.get(stackIndex).putOnStack();
      }
    } else if (stackIndex == 0) {
      items.get(0).putOnStack();
    }
  }

  /**
   * Moves the current stack item incrementaly off the stack. If the movement
   * takes the item completely off the stack then this increments the stack
   * index.
   */
  public void moveOff() {
    if (!atBottom) {
      atTop = false;
      boolean doneMoving = items.get(stackIndex).moveOff();
      if (doneMoving) {
	atBottom = stackIndex == items.size() - 1;
	if (!atBottom)
	  stackIndex++;
      }
    }
  }

  /**
   * Moves the current stack item incrementaly on to the stack. If the movement
   * takes the item completely on to the stack then this decrements the stack
   * index.
   */
  public void moveOn() {
    if (!atTop) {
      atBottom = false;
      boolean doneMoving = items.get(stackIndex).moveOn();
      if (doneMoving) {
	atTop = stackIndex == 0;
	if (!atTop)
	  --stackIndex;
      }
    }
  }
  
  /**
   * Jumps the top of the stack to the StackItem
   * 
   * @param item the item to jump to
   * @param updateDisplay whether or not to update the display as the stack 
   * pops / pushes to the specified index
   */
  public void jumpTo(StackItem item, boolean updateDisplay) {
    int index = items.indexOf(item);
    if (index != -1) {
      jumpTo(index, updateDisplay);
    }
  }
  
  /**
   * Disposes of any resources this StackRenderer or its items 
   * may be holding.
   */
  public void dispose() {
    for (StackItem item : items) {
      item.dispose();
    }
  }
  
  /**
   * Jumps the top of the stack to the specified index.
   * 
   * @param index the index to jump to.
   * @param updateDisplay whether or not to update the display as the stack 
   * pops / pushes to the specified index
   */
  public void jumpTo(int index, boolean updateDisplay) {
    if (index == stackIndex) {
      items.get(stackIndex).putOnStack();
      canvas.display();
    } else if (index >= 0 && index < items.size()) {

      if (index < stackIndex) {
	while (stackIndex != index) {
	  prev();
	  if (updateDisplay) canvas.display();
	}
      } else {
	while (stackIndex != index) {
	  next();
	  if (updateDisplay) canvas.display();
	}
      }
      canvas.display();
    }
  }

  /**
   * Gets a formatted descriptive string about this StackRenderer.
   * 
   * @return a descriptive string about this StackRenderer.
   */
  public String getInfo() {
    StringBuilder buf = new StringBuilder("StackRenderer:\n\t");
    buf.append(rotator.toString());
    buf.append("\n\t");
    buf.append(translator.toString());
    buf.append("\n\t");
    buf.append(wheelZoomer.toString());
    buf.append("\n\t");
    buf.append(zoomer.toString());

    for (int i = 0; i < items.size(); i++) {
      buf.append("\n\t");
      buf.append(i + " - " + items.get(i).toString());
    }

    return buf.toString();
  }
}
