/**
 * 
 */
package anl.aida.viz3d;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;

/**
 * Sorts a stack.
 * 
 */
public class StackSorter {
  
  private static class Asc implements Comparator<StackItem2> {

    @Override
    public int compare(StackItem2 o1, StackItem2 o2) {
      long i1 = o1.getSortingIndex();
      long i2 = o2.getSortingIndex();
      
      return i1 < i2 ? -1 : i1 == i2 ? 0 : 1;
    }
  }
  
  private static class Desc implements Comparator<StackItem2> {

    @Override
    public int compare(StackItem2 o1, StackItem2 o2) {
      long i1 = o1.getSortingIndex();
      long i2 = o2.getSortingIndex();
      
      return i1 < i2 ? 1 : i1 == i2 ? 0 : -1;
    }
    
  }
  
  public enum SortOrder {ASCENDING, DESCENDING};
  
  private SortOrder sortOrder = SortOrder.ASCENDING;
  private Comparator<StackItem2> comparator = new Asc();

  /**
   * Gets the sort order of this sorter.
   * 
   * @return the sort order of this sorter.
   */
  public SortOrder getSortOrder() {
    return sortOrder;
  }

  /**
   * Sets the sort order of this sorter.
   * 
   * @param sortOrder the new sort order
   */
  public void setSortOrder(SortOrder sortOrder) {
    this.sortOrder = sortOrder;
    if (sortOrder == SortOrder.ASCENDING) {
      comparator = new Asc();
    } else if (sortOrder == SortOrder.DESCENDING) {
      comparator = new Desc();
    }
  }
  
  /**
   * Sorts a list of stack items.
   * 
   * @param items the items to sort.
   */
  public void sort(List<StackItem2> items) {
    Collections.sort(items, comparator);
  }
}
