package anl.aida.viz3d;

import java.awt.AlphaComposite;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.imageio.ImageIO;
import javax.media.opengl.GLCanvas;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JToolBar;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

public class StackTester {
  
  StackRenderer renderer;
  int stackCount = 67;
  int[] imgIndices = {3, 19, 2, 12, 16, 1, 17, 2, 9, 15, 4, 8, 0};
  int imgIndex = 0;
  
  List<BufferedImage> imgs = new ArrayList<BufferedImage>();
  
  public void run() {
    GLCanvas canvas = new GLCanvas();
    try {
    createRenderer(canvas);
    } catch (IOException ex) {
      ex.printStackTrace();
      System.exit(-1);
    }
    canvas.addGLEventListener(renderer);
    
    JFrame frame = new JFrame();
    frame.setLayout(new BorderLayout());
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame.add(canvas, BorderLayout.CENTER);
    frame.add(createToolBar(), BorderLayout.NORTH);
    frame.setSize(900, 775);
    frame.setVisible(true);
  }
  
  private JToolBar createToolBar() {
    JToolBar bar = new JToolBar();
    SpinnerNumberModel model = new SpinnerNumberModel();
    model.setMaximum(20);
    model.setMinimum(1);
    model.setValue(1);
    final JSpinner spinner = new JSpinner(model);
    model.addChangeListener(new ChangeListener() {

      @Override
      public void stateChanged(ChangeEvent e) {
	int val = (Integer)spinner.getValue();
	renderer.jumpTo(val - 1, true);
      }
    });
    
    bar.add(spinner);
    
    JButton button = new JButton("Add");
    button.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	for (int i = 0; i < imgs.size() - 1; i++) {
	  renderer.addStackItem(new StackItem(new TextureDrawable(imgs.get(i), i)));
	}
	/*
	int index = imgIndices[imgIndex];
	renderer.addStackItem(new StackItem(new TextureDrawable(imgs.get(index), index)));
	imgIndex++;
	if (imgIndex == imgIndices.length) ((JButton)evt.getSource()).setEnabled(false);
	*/
      }
    });
    //bar.add(button);
    return bar;
  }
  
  private BufferedImage createTransImage(BufferedImage img) {
    BufferedImage dest = new BufferedImage(img.getWidth(), img.getHeight(),
	BufferedImage.TYPE_INT_ARGB);
    Graphics2D g2 = dest.createGraphics();
    g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, .9f));
    g2.drawImage(img, null, 0, 0);
    g2.dispose();
    return dest;
  }

  private BufferedImage createAlphaColorImage(BufferedImage img, Color color) {
    BufferedImage dest = new BufferedImage(img.getWidth(), img.getHeight(),
	BufferedImage.TYPE_INT_ARGB);
    Graphics2D g2 = dest.createGraphics();
    g2.setComposite(AlphaComposite.Src);
    g2.drawImage(img, null, 0, 0);
    g2.dispose();
    int rgb = color.getRGB();
    for (int y = 0, n = dest.getHeight(); y < n; y++) {
      for (int x = 0, k = dest.getWidth(); x < k; x++) {
	if (dest.getRGB(x, y) == rgb) {
	  // changes rgb to alpha color
	  dest.setRGB(x, y, 0x8F1C1C);
	}
      }
    }
    return dest;
  }

  private void createRenderer(GLCanvas canvas) throws IOException {
    final Color background = Color.decode("#99CCFF").darker();
    renderer = new StackRenderer(canvas, background, true);
    for (int i = 0; i < stackCount; i++) {
      String imgPath = "./pics/image_" + i + ".png";
      BufferedImage img = ImageIO.read(new File(imgPath));
      // BufferedImage dest = createTransImage(img);
      BufferedImage dest = createAlphaColorImage(img, background);
      dest = createTransImage(dest);
      imgs.add(dest);
    }
    
    //renderer.addStackItem(new StackItem(new TextureDrawable(imgs.get(imgs.size() - 1), imgs.size() - 1)));
    for (int i = 0; i < 15; i++) {
	  renderer.addStackItem(new StackItem(new TextureDrawable(imgs.get(i), i)));
	}
    //renderer.addStackItem(new StackItem(new TextureDrawable(imgs.get(5), 5)));
  }
  
  public static void main(String[] args) {
    StackTester tester = new StackTester();
    tester.run();
  }
}
