/**
 * 
 */
package anl.aida.viz3d;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.Map;

import javax.media.opengl.GLAutoDrawable;
import javax.swing.JCheckBox;
import javax.swing.JToolBar;
import javax.vecmath.Point3f;

import anl.aida.client.net.ConceptGraph;
import anl.aida.client.net.TermNode;
import anl.aida.util.DateRange;
import anl.aida.viz.layout.FRLayout;

/**
 * 
 */
public class TSGraphCanvasListener implements CanvasListener {
  
  private StackRenderer2 stack;
  private ConceptGraph<TermNode> firstGraph;
  private DateRange range;
  private NetworkVizCreator creator;
  private GLAutoDrawable drawable;
  private Map<DateRange, StackItem2> itemMap = new HashMap<DateRange, StackItem2>();
  
  private boolean initialized = false;

  /**
   * @param graphs
   */
  public TSGraphCanvasListener(ConceptGraph<TermNode> graph, DateRange range) {
    this.firstGraph = graph;
    this.range = range;
  }
  
  public void initToolBar(JToolBar bar) {
    bar.addSeparator();
    JCheckBox box = new JCheckBox("Hide Empty Nodes");
    box.setSelected(true);
    box.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
	creator.setEmptyNodesVisible(!((JCheckBox)evt.getSource()).isSelected());
	drawable.display();
      }
    });
    bar.add(box);
  }
  
  public void addGraph(ConceptGraph<TermNode> graph, DateRange range) {
    StackItem2 item = itemMap.get(range);
    if (item == null) {
      TSGraphVNode node = new TSGraphVNode();
      creator.addTo(graph, node, range);
      item = new StackItem2(node, range.getStart().getTime());
      itemMap.put(range, item);
      stack.addStackItem(item);
    } else {
      stack.jumpTo(item, true);
    }
  }
  
  /* (non-Javadoc)
   * @see anl.mifs.viz3d.CanvasListener#vSpatialMoved(anl.mifs.viz3d.VSpatial, javax.vecmath.Point3f)
   */
  @Override
  public void vSpatialMoved(VSpatial spatial, Point3f localTrans) {
    creator.updateLayout(spatial, localTrans);
  }

  /*
   * (non-Javadoc)
   * 
   * @see anl.mifs.viz3d.CanvasListener#init(javax.media.opengl.GLAutoDrawable,
   * anl.mifs.viz3d.VNode)
   */
  @Override
  public void init(GLAutoDrawable drawable, VNode root) {
    if (!initialized) {
      this.drawable = drawable;
      stack = new StackRenderer2(drawable);
      drawable.addKeyListener(new StackKeyHandler(stack));
      root.addChild(stack);
    }
  }
  
  /*
   * (non-Javadoc)
   * 
   * @see
   * anl.mifs.viz3d.CanvasListener#reshape(javax.media.opengl.GLAutoDrawable,
   * float, float, anl.mifs.viz3d.VNode)
   */
  @Override
  public void reshape(GLAutoDrawable drawable, float width, float height, VNode root) {
    if (!initialized) {
      //KKLayout layout = new KKLayout(firstGraph, new Dimension(10, 10));
      FRLayout layout = new FRLayout(firstGraph, new Dimension(10, 10));
      creator = new NetworkVizCreator(width, height, layout);
      TSGraphVNode node = new TSGraphVNode();
      creator.addTo(firstGraph, node, range);
      StackItem2 item = new StackItem2(node, range.getStart().getTime());
      itemMap.put(range, item);
      stack.addStackItem(item);
      // root.addChild(item);

      initialized = true;
    }
  }

  /* (non-Javadoc)
   * @see anl.mifs.viz3d.CanvasListener#itemSelected(java.lang.Object)
   */
  @Override
  public void itemSelected(Object obj) {}

  /* (non-Javadoc)
   * @see anl.mifs.viz3d.CanvasListener#selectionsCleared()
   */
  @Override
  public void selectionsCleared() {}
  
  
}
