/**
 * 
 */
package anl.aida.viz3d;

import java.awt.image.BufferedImage;

import javax.media.opengl.GL;
import javax.vecmath.Point3f;

import com.sun.opengl.util.texture.Texture;
import com.sun.opengl.util.texture.TextureIO;

/**
 * Draws a texture on to a unit sized rectangle.
 */
public class TextureDrawable implements StackDrawable {
  
  private Texture texture;
  private BufferedImage img;
  private long sortIndex;
  // x,y pairs
  private float[] coords = new float[8];
  
  public TextureDrawable(BufferedImage img, long sortIndex) {
    this.img = img;
    this.sortIndex = sortIndex;
  }
  
  public void init() {
    if (texture != null ) {
      texture.dispose();
      texture = null;
    }
    createTexture();
  }
  
  @Override
  public void dispose() {
    texture = null;
    img = null;
  }

  private void createTexture() {
    texture = TextureIO.newTexture(img, false);
    texture.setTexParameteri(GL.GL_TEXTURE_MAG_FILTER, GL.GL_LINEAR);
    texture.setTexParameteri(GL.GL_TEXTURE_MIN_FILTER, GL.GL_LINEAR);
    
    // rectangle coordinates should preserve the aspect ratio
    // of the texture
    float width = texture.getAspectRatio();
    float halfWidth = width / 2f;
    
    // lower left
    coords[0] = -halfWidth;
    coords[1] = -.5f;
    // lower right
    coords[2] = halfWidth;
    coords[3] = -.5f;
    // upper right
    coords[4] = halfWidth;
    coords[5] = .5f;
    // upper left
    coords[6] = -halfWidth;
    coords[7] = .5f;
    
    //box = new BoundingSphere(new Point3f(coords[0], coords[1], 0), new Point3f(coords[4], coords[5], 0));
    
  }

  /* (non-Javadoc)
   * @see anl.mifs.viz3d.StackDrawable#getSortIndex()
   */
  @Override
  public long getSortIndex() {
    return sortIndex;
  }

  /*
   * (non-Javadoc)
   * 
   * @see anl.mifs.viz3d.StackDrawable#draw(javax.media.opengl.GL)
   */
  @Override
  public void draw(GL gl) {
    if (texture == null) createTexture();
    texture.bind();
    // todo single array versions of these
    gl.glBegin(GL.GL_QUADS);

    gl.glTexCoord2f(0, 1);
    gl.glVertex2f(coords[0], coords[1]);

    gl.glTexCoord2f(1, 1);
    gl.glVertex2f(coords[2], coords[3]);

    gl.glTexCoord2f(1, 0);
    gl.glVertex2f(coords[4], coords[5]);

    gl.glTexCoord2f(0, 0);
    gl.glVertex2f(coords[6], coords[7]);
    gl.glEnd();
  }
}
