/**
 * 
 */
package anl.aida.viz3d;

import javax.media.opengl.GL;
import javax.vecmath.Matrix4f;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;

/**
 * Abstract base class for the VSpatials that contain geometry.
 * 
 *
 */
public abstract class VGeometry extends VSpatial {
  
  protected GeometryData geom;
  protected int vertexCount;
  protected int mode;
  
  /*
   * (non-Javadoc)
   * 
   * @see anl.mifs.viz3d.AbstractVNode#doDraw(javax.media.opengl.GL)
   */
  @Override
  protected void doDraw(GL gl) {
    geom.rewind();
    gl.glEnableClientState(GL.GL_VERTEX_ARRAY);
    gl.glEnableClientState(GL.GL_NORMAL_ARRAY);
    
    gl.glNormalPointer(GL.GL_FLOAT, 0, geom.getNormals());
    gl.glVertexPointer(3, GL.GL_FLOAT, 0, geom.getVertices());
    gl.glDrawArrays(mode, 0, vertexCount);
    
    gl.glDisableClientState(GL.GL_VERTEX_ARRAY);
    gl.glDisableClientState(GL.GL_NORMAL_ARRAY);
  }
  
  
  /* (non-Javadoc)
   * @see anl.mifs.viz3d.VNode#intersects(javax.vecmath.Point3f, javax.vecmath.Vector3f, anl.mifs.viz3d.Accumulator)
   */
  @Override
  public void intersects(Point3f rayOrigin, Vector3f rayDirection, Accumulator accumulator) {
    BoundingSphere sphere = getBoundingSphere();
    if (sphere.intersects(rayOrigin, rayDirection)) {
      
      // transform the ray to local frame from world frame using the
      // inverse of the local to world matrix. The ray can then
      // be tested against the geometry to see if it intersects
      Point3f transOrigin = new Point3f(rayOrigin);
      Vector3f transDir = new Vector3f(rayDirection);
      
      Matrix4f matrix = new Matrix4f();
      matrix.setIdentity();
      
      matrix.setRotation(worldRotation);
      matrix.setScale(worldScale);
      
      Matrix4f invert = new Matrix4f();
      invert.setIdentity();
      invert.invert(matrix);
      invert.transform(transDir);
      transDir.normalize();
      
      matrix.setTranslation(worldTrans);
      invert.setIdentity();
      invert.invert(matrix);
      invert.transform(transOrigin);
      
      if (intersects(transOrigin, transDir)) {
	accumulator.add(this);
      }
    }
  }
  
  /**
   * Gets whether or not the specified array intersects
   * this VGeometry's geometry.
   * 
   * @param rayOrigin the ray's origin
   * @param rayDirection the ray's direction
   * 
   * @return whether or not the specified array intersects
   * this VGeometry's geometry.
   */
  protected abstract boolean intersects(Point3f rayOrigin, Vector3f rayDirection);

}
