/**
 * 
 */
package anl.aida.viz3d;

import java.awt.Color;
import java.nio.FloatBuffer;

import javax.media.opengl.GL;
import javax.vecmath.Color4f;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;

import com.sun.opengl.util.BufferUtil;

/**
 * VSpatial that draws a line between two points.
 * 
 * 
 */
public class VLine extends VGeometry {
  
  private static final float K_EPSILON = 1.0e-5f;
  private Color4f color = new Color4f(Color.white);

  /**
   * @param start
   * @param end
   */
  public VLine(Point3f start, Point3f end) {
    FloatBuffer buf = BufferUtil.newFloatBuffer(6);
  
    buf.put(start.x);
    buf.put(start.y);
    buf.put(start.z);
    buf.put(end.x);
    buf.put(end.y);
    buf.put(end.z);
   
    Point3f center = new Point3f(start);
    center.add(end);
    center.scale(.5f);
    boundingSphere = new BoundingSphere(center, start.distance(end) / 2f);
    geom = new GeometryData(buf, null);
  }
  
  private void resetBoundingSphere() {
    float[] pt = new float[3];
    FloatBuffer buf = geom.getVertices();
    buf.rewind();
    buf.get(pt);
    Point3f start = new Point3f(pt);
    buf.get(pt);
    Point3f end = new Point3f(pt);
    
    Point3f center = new Point3f(start);
    center.add(end);
    center.scale(.5f);
    boundingSphere = new BoundingSphere(center, start.distance(end) / 2f);
  }
  
  public void setColor(Color color) {
    this.color.set(color);
  }
  
  public Color4f getColor() {
    return color;
  }
  
  public void resetStartPt(Point3f start) {
    FloatBuffer buf = geom.getVertices();
    buf.rewind();
    buf.put(start.x);
    buf.put(start.y);
    buf.put(start.z);
    resetBoundingSphere();
  }
  
  public void resetEndPt(Point3f end) {
    FloatBuffer buf = geom.getVertices();
    buf.put(3, end.x);
    buf.put(4, end.y);
    buf.put(5, end.z);
    resetBoundingSphere();
  }

  /* (non-Javadoc)
   * @see anl.mifs.viz3d.AbstractVNode#doDraw(javax.media.opengl.GL)
   */
  @Override
  protected void doDraw(GL gl) {
    gl.glDisable(GL.GL_LIGHTING);
    gl.glColor4f(color.x, color.y, color.z, color.w);
    
    gl.glEnableClientState(GL.GL_VERTEX_ARRAY);
    FloatBuffer buf = geom.getVertices();
    buf.rewind();
    gl.glVertexPointer(3, GL.GL_FLOAT, 0, buf);
    gl.glDrawArrays(GL.GL_LINES, 0, 2);
    gl.glDisableClientState(GL.GL_VERTEX_ARRAY);
   
    gl.glEnable(GL.GL_LIGHTING);
  }

  /* (non-Javadoc)
   * @see anl.mifs.viz3d.VGeometry#intersects(javax.vecmath.Point3f, javax.vecmath.Vector3f)
   */
  // from Van Verth and Bishop, "Essential Mathematics for games"
  @Override
  protected boolean intersects(Point3f rayOrigin, Vector3f rayDirection) {
    // get distance between ray and this line
    FloatBuffer buf = geom.getVertices();
    buf.rewind();
    Point3f orig = new Point3f();
    float[] coord = new float[3];
    buf.get(coord);
    orig.set(coord);
    Point3f dest = new Point3f();
    buf.get(coord);
    dest.set(coord);
    Vector3f dir = new Vector3f(dest);
    dir.sub(orig);
    
    Vector3f w0 = new Vector3f(orig);
    w0.sub(rayOrigin);
    
    float a = dir.dot(dir);
    float b = dir.dot(rayDirection);
    float c = rayDirection.dot(rayDirection);
    float d = dir.dot(w0);
    float e = rayDirection.dot(w0);

    float denom = a*c - b*b;

    // parameters to compute s_c, t_c
    float sn, sd, tn, td;
    
    // if denom is zero, try finding closest point on segment1 to origin0
    if (Utils3D.isZero(denom)) {
        // clamp s_c to 0
        sd = td = c;
        sn = 0.0f;
        tn = e;
    } else {
        // clamp s_c within [0,1]
        sd = td = denom;
        sn = b*e - c*d;
        tn = a*e - b*d;
        
        // clamp s_c to 0
        if (sn < 0.0f) {
            sn = 0.0f;
            tn = e;
            td = c;
        }

        // clamp s_c to 1
        else if (sn > sd) {
            sn = sd;
            tn = e + b;
            td = c;
        }
    }
    
    float t_c, s_c;

    // clamp t_c within [0,+inf]
    // clamp t_c to 0
    if (tn < 0.0f) {

        t_c = 0.0f;
        // clamp s_c to 0
        if ( -d < 0.0f ){
            s_c = 0.0f;
         }

        // clamp s_c to 1
        else if ( -d > a ) {
            s_c = 1.0f;
        } else {
            s_c = -d/a;
        }

    } else {
        t_c = tn/td;
        s_c = sn/sd;
    }



    // compute difference vector and distance squared

    Vector3f sDir = new Vector3f(dir);
    sDir.scale(s_c);
    Vector3f tDir = new Vector3f(rayDirection);
    tDir.scale(t_c);
    sDir.add(w0);
    sDir.sub(tDir);
    
    float distSq = sDir.dot(sDir);

    return distSq < K_EPSILON;
  }
}
