package anl.aida.viz3d;

import java.nio.FloatBuffer;

import javax.media.opengl.GL;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;

/**
 * Encapsulates a vertex array in a VSpatial. This assumes the
 * vertex array is in GL.GL_TRIANGLE_STRIP format.
 * 
 * 
 */
public class VShapeTS extends VGeometry {
  
  public VShapeTS(GeometryData geom) {
    this.geom = geom;
    this.vertexCount = geom.getVertices().limit() / 3;
    boundingSphere = Utils3D.createBoundingSphere(geom.getVertices());
    mode = GL.GL_TRIANGLE_STRIP;
  }

  /**
   * Tests whether the ray intersects with the geometry (the triangles) of this
   * shape.
   * 
   * @param rayOrigin the ray's origin
   * @param rayDirection the ray's shape
   * 
   * @return true if the ray does intersect with this geometry
   */
  protected boolean intersects(Point3f rayOrigin, Vector3f rayDirection) {
    // rewind the buffer so we are the start
    FloatBuffer buf = geom.getVertices();
    buf.rewind();
   
    // get the first point
    float[] pt = new float[3];
    Point3f p0 = new Point3f();
    buf.get(pt);
    p0.set(pt);
    
    // get the second point
    Point3f p1 = new Point3f();
    buf.get(pt);
    p1.set(pt);
    
    Point3f p2 = new Point3f();
    // iterate through the remaining points
    // with triangle of p0, p1 and p2
    // and after check p1 is set to p2.
    // and p2 is then the next point
    while (buf.hasRemaining()) {
      buf.get(pt);
      p2.set(pt);
      if (Utils3D.triangleIntersect(p0, p1, p2, rayOrigin, rayDirection)) return true;
      p0.set(p1);
      p1.set(p2);
    }
    return false;
  }
}
