package anl.aida.client.io;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.io.IOException;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import javax.xml.stream.XMLStreamException;

import org.junit.Test;

import anl.aida.client.Cache;
import anl.aida.client.CacheElement;
import anl.aida.client.Keyword;
import anl.aida.client.Keywords;
import anl.aida.client.io.CacheFileReader;
import anl.aida.util.AIDAUtils;
import anl.aida.util.DateRangeProducer;

public class CacheReaderTests {

  /*
   * <cache lucene_index="index"> <range> <start>1230786000007</start>
   * <length>60</length> <interval>5</interval> </range> <cache_items>
   * <cache_item>stc_090101_090302.bin</cache_item>
   * <cache_item>stc_090101_090106.bin</cache_item>
   * <cache_item>stc_090102_090107.bin</cache_item>
   * <cache_item>stc_090103_090108.bin</cache_item>
   * <cache_item>stc_090104_090109.bin</cache_item> </cache_items> <keywords>
   */

  long startDate = 1230786000007L;
  int rangeLength = 5;
  int totalDays = 60;

  String[] elements = { "stc_090101_090302.bin", "stc_090101_090106.bin", "stc_090102_090107.bin",
      "stc_090103_090108.bin", "stc_090104_090109.bin" };
  String[] bioAgents = { "Typhoid Fever", "Plague", "Marburg Virus", "Melioidosis" };
  String[] folks = { "xxx" };

  private Set<String> createSet(String[]... strings) {
    Set<String> items = new HashSet<String>();
    for (String[] vals : strings) {
      for (String item : vals) {
	items.add(item);
	items.add(AIDAUtils.toInternalFormat(item));
      }
    }

    return items;
  }

  @Test
  public void testReader() throws IOException, XMLStreamException {
    CacheFileReader reader = new CacheFileReader();
    Cache cache = reader.read(new File("./test_data/cache.xml"));

    DateRangeProducer ranger = cache.getRanger();
    assertEquals(startDate, ranger.getStart().getTime());
    assertEquals(rangeLength, ranger.getRangeLength());
    assertEquals(totalDays, ranger.getTotalDays());

    Set<String> items = createSet(elements);

    assertEquals(5, cache.size());
    for (CacheElement item : cache.elements()) {
      File f = item.getSTCLocation();
      assertTrue(items.remove(f.getName()));
    }
    assertEquals(0, items.size());

    Keywords keywords = cache.getKeywords();
    
    Collection<Keyword> words = keywords.getKeywords("BioAgent");
    assertEquals(4, words.size());
    items = createSet(bioAgents);
    for (Keyword word : words) {
      assertTrue(items.remove(word.getLabel()));
      assertTrue(items.remove(word.getTerm()));
    }
    assertEquals(0, items.size());
    
    words = keywords.getKeywords("DangerousFolks");
    assertEquals(4, words.size());
    items = createSet(folks);
    for (Keyword word : words) {
      assertTrue(items.remove(word.getLabel()));
      assertTrue(items.remove(word.getTerm()));
    }
    assertEquals(0, items.size());
    
    words = keywords.getAllKeywords();
    assertEquals(8, words.size());
    items = createSet(bioAgents, folks);
    for (Keyword word : words) {
      assertTrue(items.remove(word.getTerm()));
      assertTrue(items.remove(word.getLabel()));
    }
    assertEquals(0, items.size());
  }
}
