package anl.aida.client.net;

import static org.junit.Assert.assertEquals;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.swing.JFrame;

import org.junit.Test;

import anl.aida.client.CacheElement;
import anl.aida.client.DefaultKeyword;
import anl.aida.client.Keyword;
import anl.aida.client.net.CGEdge;
import anl.aida.client.net.ConceptGraph;
import anl.aida.client.net.ConceptGraphBuilder;
import anl.aida.client.net.SVEdgeCC;
import anl.aida.client.net.SimilarityFilterCreator;
import anl.aida.client.net.TermNode;
import anl.aida.core.MIFSTerm;
import anl.aida.core.STCHeader;
import anl.aida.core.STCReader;
import anl.aida.score.TermScoreFilter;
import anl.aida.util.HighestTermScores;
import anl.aida.util.TermScore;
import anl.aida.viz.Canvas2D;
import anl.aida.viz.NetworkDisplay;
import anl.aida.viz.layout.KKLayout;
import edu.uci.ics.jung.graph.Graph;

/**
 *
 *         Date: Feb 26, 2009 11:30:09 AM
 */
public class NetTests {

  private CacheElement element;
  private List<Keyword> concepts = new ArrayList<Keyword>();

  public NetTests() throws IOException {
    STCReader reader = new STCReader();
    STCHeader header = reader.readHeader("./cache/stc_090101_090106.bin");
    element = new CacheElement(0, new File("./cache/stc_090101_090106.bin"), header);

    String[] excludeA = {"from", "were", "we", "have", "has", "which", "been",
            "all", "between", "also", "using", "than", "more", "two", "other",
            "may", "also", "can", "had", "among", "based", "when"};
    Set<String> exclude = new HashSet<String>(Arrays.asList(excludeA));

    HighestTermScores scores = new HighestTermScores(120);
    for (int i = 0; i < element.getSTC().getTermCount(); i++) {
      MIFSTerm term = element.getSTC().getTerm(i);
      scores.add(term.getText(), term.getDocumentCount());
    }

    for (TermScore ts : scores.termScores()) {
      if (!exclude.contains(ts.getTerm())) concepts.add(new DefaultKeyword(ts.getTerm(), ts.getTerm()));
    }

  }

  @Test
  public void testGraph() {
    TermScoreFilter filter = new SimilarityFilterCreator().createFilter(element, .02f);
    ConceptGraphBuilder builder = new ConceptGraphBuilder(element.documentSets().iterator().next().documents());

    builder.buildNodes(filter, concepts);
    builder.buildEdges(new SVEdgeCC().createEdgeCreator(element, 2, .4f));
    Graph<TermNode, CGEdge> graph = builder.getResult();

    System.out.println("graph.getEdgeCount() = " + graph.getEdgeCount());
    assertEquals(concepts.size(), graph.getVertexCount());
  }

  public void testDisplay() {
    TermScoreFilter filter = new SimilarityFilterCreator().createFilter(element, .3f);
    ConceptGraphBuilder builder = new ConceptGraphBuilder(element.documentSets().iterator().next().documents());

    builder.buildNodes(filter, concepts);
    builder.buildEdges(new SVEdgeCC().createEdgeCreator(element, 2, .4f));
    Graph<TermNode, CGEdge> graph = builder.getResult();
    ConceptGraph<TermNode> cGraph = new ConceptGraph<TermNode>("foo", element.getDocumentSet(0).getId(),
            graph);

    System.out.println("graph.getEdgeCount() = " + graph.getEdgeCount());
    Dimension size = new Dimension(900, 800);

    //CircleLayout<TermNode> layout = new CircleLayout<TermNode>(graph.getVertices(), size);
    KKLayout layout = new KKLayout(cGraph, size);
    //FRLayout<TermNode> layout = new FRLayout<TermNode>(graph, size);
    //ISOMLayout<TermNode> layout = new ISOMLayout<TermNode>(graph, size);
    //anl.mifs.viz.layout.SpringLayout<TermNode> layout = new anl.mifs.viz.layout.SpringLayout<TermNode>(graph, size);

    NetworkDisplay display = new NetworkDisplay(cGraph, layout);
    display.initialize();

    /*
    display.setLensListener(new SelectionListener<Integer>() {
      public void selectionOccurred(List<Integer> selectedItems, Canvas2D canvas) {
        canvas.showToolTip("Lens Over");
      }
    });
    */

    Canvas2D canvas = new Canvas2D(size);
    canvas.addLayer(display);

    JFrame frame = new JFrame();
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame.setLayout(new BorderLayout());
    frame.add(canvas, BorderLayout.CENTER);
    frame.add(canvas.getToolBar(), BorderLayout.NORTH);
    frame.setSize(800, 600);
    frame.pack();
    frame.setVisible(true);
  }


  public static void main(String[] args) throws IOException {
    NetTests tests = new NetTests();
    //tests.testDisplay3D();
    tests.testDisplay();
  }

}
