package anl.aida.client.search;

import anl.aida.ae.lucene.LuceneConstants;
import anl.aida.client.search.LuceneSearcher;
import anl.aida.core.DocumentID;
import anl.aida.core.STCReader;
import anl.aida.core.ScoredTermsCollection;
import anl.aida.util.AIDAException;
import org.apache.log4j.BasicConfigurator;
import org.apache.lucene.document.DateTools;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.Term;
import org.apache.lucene.index.TermDocs;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.search.RangeFilter;
import static org.junit.Assert.*;
import org.junit.Test;

import java.io.IOException;
import java.util.*;

/**
 * 
 *         Date: Feb 24, 2009 2:08:36 PM
 */
public class LuceneSearchTest {

  static {
    BasicConfigurator.configure();
  }

  private ScoredTermsCollection stc;

  public LuceneSearchTest() throws IOException, AIDAException {
    STCReader reader = new STCReader();
    stc = reader.readCollection("./cache/stc_02092009.bin");
  }

  @Test
  public void testSearch() throws IOException, ParseException {
    int hits = 10;
    LuceneSearcher search = new LuceneSearcher("./cache/index", stc);
    List<DocumentID> results = search.search("bioterrorism", hits);
    assertEquals(hits, results.size());

    for (DocumentID id : results) {
      assertNotNull(id);
    }
  }

  @Test
  public void testFilteredSearch() throws IOException, ParseException {
    int hits = 100;
    LuceneSearcher search = new LuceneSearcher("./cache/index", stc);
    List<DocumentID> results = search.search("outbreak", hits);

    Set<DocumentID> expected = new HashSet<DocumentID>(results);

    results = search.search(results, "food", 10);
    assertTrue(results.size() > 0);
    for (DocumentID id : results) {
      assertNotNull(id);
      assertTrue(expected.contains(id));
    }
  }

  @Test
  public void testDateFilteredSearch() throws IOException, ParseException, java.text.ParseException {
    int hits = 10;
    LuceneSearcher search = new LuceneSearcher("./cache/index", stc);
    Calendar cal = GregorianCalendar.getInstance();
    // jan is 0, so 8 is sept.
    cal.set(2008, 8, 01);
    Date lowerDate = cal.getTime();
    String lower = DateTools.dateToString(lowerDate, DateTools.Resolution.DAY);
    cal.set(2009, 2, 01);
    Date upperDate = cal.getTime();
    String upper = DateTools.dateToString(upperDate, DateTools.Resolution.DAY);

    RangeFilter filter = new RangeFilter(LuceneConstants.TIMESTAMP_FIELD, lower, upper, true, true);

    List<DocumentID> results = search.search("bioterrorism", hits, filter);
    assertTrue(results.size() > 0);

    IndexReader reader = IndexReader.open("./cache/index");
    for (DocumentID id : results) {
      Term term = new Term(LuceneConstants.PATH_FIELD, id.getPath());
      TermDocs docs = reader.termDocs(term);
      docs.next();
      Document doc = reader.document(docs.doc());
      String timestamp = doc.get(LuceneConstants.TIMESTAMP_FIELD);
      Date date = DateTools.stringToDate(timestamp);
      assertTrue(date.after(lowerDate) || date.equals(lowerDate));
      assertTrue(date.before(upperDate) || date.equals(upperDate));
    }

    reader.close();
  }
}
