package anl.aida.client.widget;

import java.awt.BorderLayout;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;

import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.xml.stream.XMLStreamException;

import anl.aida.client.Cache;
import anl.aida.client.CacheElement;
import anl.aida.client.Keyword;
import anl.aida.client.gui.AnimationConfigDialog;
import anl.aida.client.gui.CacheTSPanel;
import anl.aida.client.gui.VizPropertyBean;
import anl.aida.client.io.CacheFileReader;
import anl.aida.client.net.TSConceptGraphs;
import anl.aida.client.widget.AnimatorPropertyBean;
import anl.aida.client.widget.NetworkTimeSlider2;
import anl.aida.client.widget.RangeChangeEvent;
import anl.aida.client.widget.RangeChangedListener;
import anl.aida.client.widget.RangeSlider;
import anl.aida.client.widget.TimeSliceSlider;
import anl.aida.core.STCHeader;
import anl.aida.util.DateRange;
import anl.aida.util.DateRangeProducer;

/**
 *
 *         Date: Mar 6, 2009 11:23:52 AM
 */
public class TimeSliderTest {

  Calendar cal = GregorianCalendar.getInstance();

  public void runSlider() {
    JFrame frame = new JFrame();
    frame.setLayout(new BorderLayout());
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

    TimeSliceSlider slider = new TimeSliceSlider();
    cal.set(2008, 9, 1, 0, 0, 0);
    cal.set(Calendar.MILLISECOND, 0);
    DateRangeProducer prod = new DateRangeProducer(cal.getTime(), 60, 5);
    slider.init(prod);

    frame.add(slider, BorderLayout.CENTER);
    frame.pack();
    frame.setVisible(true);
  }
  
  public void runNetSlider() throws IOException, XMLStreamException {
    JFrame frame = new JFrame();
    frame.setLayout(new BorderLayout());
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

    NetworkTimeSlider2 slider = new NetworkTimeSlider2();
    CacheFileReader reader = new CacheFileReader();
    Cache cache = reader.read(new File("./cache/cache.xml"));
    TSConceptGraphs graphs = new TSConceptGraphs(cache, new VizPropertyBean("Test", new ArrayList<Keyword>()));
    slider.init(null, graphs, cache.getRanger().ranges().iterator().next());
    

    frame.add(slider, BorderLayout.CENTER);
    frame.pack();
    frame.setVisible(true);
  }
  
  public void runAnimationDialog() throws IOException, XMLStreamException {
    //JFrame frame = new JFrame();
    //frame.setLayout(new BorderLayout());
    //frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

    CacheFileReader reader = new CacheFileReader();
    Cache cache = reader.read(new File("./cache/cache.xml"));
    AnimationConfigDialog dialog = new AnimationConfigDialog((JFrame)null);
    dialog.init(new AnimatorPropertyBean(cache.getRanger()));
    
    dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
    //dialog.pack();
    dialog.setVisible(true);
  }

  public void runCacheReader() throws IOException, XMLStreamException {
    CacheFileReader reader = new CacheFileReader();
    Cache cache = reader.read(new File("./cache/cache.xml"));
    DateRangeProducer ranger = cache.getRanger();
    System.out.printf("Full Range: %s - %s%n", ranger.getStart().toString(), ranger.getEnd().toString());
    for (DateRange range : ranger.ranges()) {
      CacheElement item = cache.getElement(range);
      STCHeader header = item.getHeader();
      DateRange dateRange = header.getDateRange();
      System.out.printf("item: %s - %s, %d%n", dateRange.getStart().toString(),
              dateRange.getEnd().toString(), header.getDocCount());
    }
  }

  public void runCachePanel() throws IOException, XMLStreamException {
    CacheFileReader reader = new CacheFileReader();
    Cache cache = reader.read(new File("./cache/cache.xml"));

    JFrame frame = new JFrame();
    frame.setLayout(new BorderLayout());
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

    CacheTSPanel panel = new CacheTSPanel();
    panel.reset(cache);

    frame.add(panel, BorderLayout.CENTER);
    frame.pack();
    frame.setVisible(true);
  }

  public void runRangeSlider() {
    JPanel panel = new JPanel(new BorderLayout());
    final RangeSlider slider = new RangeSlider(0, 60);
    slider.setPaintLabels(true);
    slider.setMajorTickSpacing(10);
    slider.setMinorTickSpacing(1);
    slider.setPaintTicks(true);
    //slider.setPaintTrack(false);
    slider.setSnapToTicks(true);
    slider.setOrientation(JSlider.VERTICAL);

    slider.addRangeChangedListener(new RangeChangedListener() {
      public void rangeChanged(RangeChangeEvent evt) {
        System.out.println("first changed: " + evt.isFirstChanged());
        System.out.println("second changed: " + evt.isSecondChanged());

        System.out.println("slider.getFirstValue() = " + slider.getFirstValue());
        System.out.println("slider.getSecondValue() = " + slider.getSecondValue());

      }
    });

    panel.add(slider, BorderLayout.CENTER);

    JFrame frame = new JFrame();
    frame.setLayout(new BorderLayout());
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame.add(panel, BorderLayout.CENTER);

    frame.pack();
    frame.setVisible(true);

  }


  public static void main(String[] args) throws IOException, XMLStreamException {
    //new TimeSliderTest().runSlider();
    //new TimeSliderTest().runCachePanel();
    //new TimeSliderTest().runNetSlider();
    new TimeSliderTest().runAnimationDialog();
    //new TimeSliderTest().runRangeSlider();
    //new TimeSliderTest().runCacheReader();
  }
}
