/**
 * 
 */
package anl.aida.gis;

import java.awt.Color;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.BasicConfigurator;
import org.geotools.filter.text.cql2.CQL;
import org.geotools.filter.text.cql2.CQLException;
import org.geotools.referencing.CRS;
import org.geotools.styling.FeatureTypeStyle;
import org.geotools.styling.Rule;
import org.geotools.styling.Style;
import org.geotools.styling.StyleBuilder;
import org.geotools.styling.Symbolizer;
import org.geotools.styling.TextSymbolizer;
import org.opengis.filter.Filter;
import org.opengis.referencing.FactoryException;

import anl.aida.data.DataFrame;
import anl.aida.data.DataFrameAxis;
import anl.aida.data.DataFrameIndex;
import anl.aida.gis.GISDisplay;
import anl.aida.palette.ColorMap;

/**
 * 
 */
public class GISPanelTest {

  static {
    System.setProperty("org.geotools.referencing.forceXY", "true");
    BasicConfigurator.configure();
  }

  public void run() throws FactoryException, CQLException {

    GISDisplay display = new GISDisplay(CRS.decode("EPSG:4269"));
    File file = new File("./shapefiles/tl_2008_55_county/tl_2008_55_county.shp");

    /*
    MockWIFeed feed = new MockWIFeed();
    DataFrame dataFrame = feed.getDataFrame(feed.getVariables().get(2));
    MinMax minMax = DataUtilities.minMax(dataFrame);
    ColorMap map = new ColorMap(new PavePaletteCreator().createPalettes(8).get(0), minMax.getMin(),
	minMax.getMax());

    display.addLayer(file, createPolygonStyle(map, dataFrame));
    display.addLocationListener(new FeaturePropertyFinder("NAME", file));
   
    JFrame frame = new JFrame();
    frame.setLayout(new BorderLayout());
    frame.add(display.getPanel(), BorderLayout.CENTER);
    frame.add(new LegendPanel(map, minMax.getMin(), minMax.getMax(), "Cases"), BorderLayout.EAST);
    frame.setSize(800, 800);
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame.setVisible(true);
    */
    
  }

  private Style createPolygonStyle(ColorMap map, DataFrame frame) throws CQLException {
    StyleBuilder builder = new StyleBuilder();
    TextSymbolizer label = builder.createTextSymbolizer(Color.BLACK, builder
	.createFont("Arial", 10), "NAME");

    DataFrameIndex index = frame.getIndex();
    DataFrameAxis axis = frame.getAxes().getAxis("County");
    List<Rule> rules = new ArrayList<Rule>();
    for (int i = 0; i < axis.getExtent(); i++) {
      String county = axis.getValueAsString(i);
      index.set(0, i);
      int val = frame.getInt(index);

      Rule rule = builder.getStyleFactory().createRule();
      rule.setSymbolizers(new Symbolizer[] {
	  builder.createPolygonSymbolizer(map.getColorForValue(val), Color.BLACK, 1.0), label });
      Filter filter = CQL.toFilter(String.format("NAME = '%s'", county));
      rule.setFilter(filter);
      rules.add(rule);
    }

    Rule rule2 = builder.getStyleFactory().createRule();
    Symbolizer poly = builder.createPolygonSymbolizer(Color.GRAY, Color.BLACK, 1.0);
    rule2.setSymbolizers(new Symbolizer[] { poly, label });
    rule2.setIsElseFilter(true);
    rules.add(rule2);

    FeatureTypeStyle fts = builder.createFeatureTypeStyle("tl_2008_55_county", rules
	.toArray(new Rule[rules.size()]));
    Style style = builder.createStyle();
    style.addFeatureTypeStyle(fts);
    return style;
  }

  public static void main(String[] args) throws IOException, FactoryException, CQLException {
    new GISPanelTest().run();
  }
}
