package anl.aida.gis;

import java.awt.BorderLayout;
import java.io.File;
import java.io.IOException;

import javax.swing.JFrame;

import org.apache.log4j.BasicConfigurator;
import org.geotools.feature.FeatureCollection;
import org.geotools.map.DefaultMapContext;
import org.geotools.map.DefaultMapLayer;
import org.geotools.referencing.crs.DefaultGeographicCRS;
import org.geotools.styling.SLDParser;
import org.geotools.styling.StyleFactoryImpl;
import org.opengis.feature.simple.SimpleFeature;
import org.opengis.feature.simple.SimpleFeatureType;
import org.opengis.geometry.MismatchedDimensionException;
import org.opengis.referencing.FactoryException;
import org.opengis.referencing.NoSuchAuthorityCodeException;
import org.opengis.referencing.operation.TransformException;

import anl.aida.gis.GISDisplay;
import anl.aida.gis.ShapefileSimplifier;

public class SimplifierTest {
  
  static {
    System.setProperty("org.geotools.referencing.forceXY", "true");
    BasicConfigurator.configure();
  }
  
  public void run() throws IOException, NoSuchAuthorityCodeException, FactoryException, MismatchedDimensionException, 
  TransformException {
    
    File file = new File("./shapefiles/tl_2008_55_county/tl_2008_55_county.shp");
    ShapefileSimplifier simplifier = new ShapefileSimplifier();
    FeatureCollection<SimpleFeatureType, SimpleFeature> features = simplifier.simplify(file, .005f);
    
    DefaultMapContext context = new DefaultMapContext(DefaultGeographicCRS.WGS84);
    SLDParser parser = new SLDParser(new StyleFactoryImpl() , new File("./shapefiles/tl_2008_55_county/default_line.sld"));
    context.addLayer(new DefaultMapLayer(features, parser.readXML()[0]));
    
    GISDisplay display = new GISDisplay(context);
    JFrame frame = new JFrame();
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame.setLayout(new BorderLayout());
    frame.add(display.getPanel(), BorderLayout.CENTER);
    frame.setSize(800, 800);
    frame.setVisible(true);
  }
  
  public static void main(String[] args) {
    try {
      new SimplifierTest().run();
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }

}
