package anl.aida.plot;

import java.awt.BorderLayout;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import javax.measure.unit.NonSI;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import anl.aida.AIDADatasetProducer;
import anl.aida.bubble.BubbleChart;
import anl.aida.bubble.BubbleDataset;
import anl.aida.bubble.DefaultBubbleDataset;
import anl.aida.bubble.LogDomainAxis;
import anl.aida.bubble.NumericRange;
import anl.aida.data.AxisType;
import anl.aida.data.DataFrame;
import anl.aida.data.Dataset;

public class BubbleExperiments {

  private BubbleChart chart;

  class Data implements BubbleDataset {

    private List<String> categories = new ArrayList<String>();
    // timestep, category, col
    private float[][][] data;

    public Data() {
      categories.add("Ajax");
      categories.add("Barcelona");
      categories.add("Chelsea");
      categories.add("Bayern Munich");
      categories.add("Everton");
      categories.add("Fulham");
      categories.add("Real Madrid");
      categories.add("Juventus");
      categories.add("Inter Milan");
      categories.add("A.C. Milan");

      // for (int i = 0; i < 100; i++) {
      // categories.add("Item " + i);
      // }

      // make the arrays
      Random random = new Random();
      data = new float[10][categories.size()][3];
      for (int i = 0; i < 10; i++) {
	for (int j = 0; j < categories.size(); j++) {
	  for (int k = 0; k < 3; k++) {
	    data[i][j][k] = random.nextFloat();// (float)(100 * Math.random());
	  }
	}
      }
    }

    @Override
    public Iterable<String> categories() {
      return categories;
    }

    /*
     * (non-Javadoc)
     * 
     * @see anl.aida.bubble.BubbleDataset#getFormattedTimeStep(int)
     */
    @Override
    public String getFormattedTimeStep(int timestep) {
      return String.valueOf(timestep);
    }

    /*
     * (non-Javadoc)
     * 
     * @see anl.aida.bubble.BubbleDataset#getCategoryIndex(java.lang.String)
     */
    @Override
    public int getCategoryIndex(String category) {
      return categories.indexOf(category);
    }

    @Override
    public int getCategoryCount() {
      return categories.size();
    }

    @Override
    public int getTimeStepCount() {
      return 10;
    }

    @Override
    public double getXValue(int rowIndex, int timestep) {
      return data[timestep][rowIndex][0];
    }

    public double getYValue(int rowIndex, int timestep) {
      return data[timestep][rowIndex][1];
    }

    public double getZValue(int rowIndex, int timestep) {
      return data[timestep][rowIndex][2];
    }

    /*
     * (non-Javadoc)
     * 
     * @see anl.aida.bubble.BubbleDataset#getXRange()
     */
    @Override
    public NumericRange getXRange() {
      // TODO Auto-generated method stub
      return new NumericRange(0, 2);
    }

    /*
     * (non-Javadoc)
     * 
     * @see anl.aida.bubble.BubbleDataset#getYRange()
     */
    @Override
    public NumericRange getYRange() {
      // TODO Auto-generated method stub
      return new NumericRange(-1, 1);
    }

    /*
     * (non-Javadoc)
     * 
     * @see anl.aida.bubble.BubbleDataset#getZRange()
     */
    @Override
    public NumericRange getZRange() {
      // TODO Auto-generated method stub
      return new NumericRange(0, 1);
    }
  }

  private void initialize() {

    try {

      AIDADatasetProducer producer = new AIDADatasetProducer(AxisType.NAMED_LOCATION, "Country",
	  NonSI.YEAR);
      Dataset dataset = producer.createDataset("./data/gap_minder_data.bin", "GAP Minder Data");
      DataFrame yFrame = dataset.getDataFrame(dataset.getVariable("Life_expectancy_at_birth"));
      DataFrame xFrame = dataset.getDataFrame(dataset.getVariable("Income_per_person"));
      DataFrame zFrame = dataset.getDataFrame(dataset.getVariable("Population_total"));
      DefaultBubbleDataset data = new DefaultBubbleDataset(xFrame, yFrame, zFrame);

      /*
       * AIDADatasetProducer producer = new
       * AIDADatasetProducer(AxisType.CATEGORY, "Driver", NonSI.YEAR); Dataset
       * dataset = producer.createDataset("./data/H2CAS.bin",
       * "H2CAS Model Data"); DataFrame yFrame =
       * dataset.getDataFrame(dataset.getVariable("Hydrogen_Utility"));
       * DataFrame xFrame =
       * dataset.getDataFrame(dataset.getVariable("dTotDist")); DataFrame zFrame
       * = dataset.getDataFrame(dataset.getVariable("q")); DefaultBubbleDataset
       * data = new DefaultBubbleDataset(xFrame, yFrame, zFrame, 150);
       */

      chart = new BubbleChart(data);
      /*
       * chart.setCategoryComparator(new Comparator<String>() {
       * 
       * @Override public int compare(String o1, String o2) { int i1 =
       * Integer.parseInt(o1); int i2 = Integer.parseInt(o2); return i1 < i2 ?
       * -1 : i1 == i2 ? 0 : 1; }
       * 
       * });
       */
      // chart.getPlot().getRangeAxis().setLabel("Hydrogen Utility");
      chart.getPlot().setDomainAxis(new LogDomainAxis(data.getXRange(), "Income per Person ($)"));
    } catch (IOException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
  }

  public void run() {
    final JFrame frame = new JFrame("Bubble Experiments");
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

    frame.getContentPane().setLayout(new BorderLayout());
    frame.setBounds(100, 100, 1000, 600);

    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
	initialize();
	JPanel panel = chart.getPanel();
	frame.getContentPane().add(panel, BorderLayout.CENTER);
	panel.validate();
	panel.repaint();

	frame.validate();
	frame.setVisible(true);
      }
    });
  }

  public static void main(String[] args) {
    new BubbleExperiments().run();
  }

}
