package anl.aida.viz3d;

import static org.junit.Assert.assertEquals;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.vecmath.Matrix4f;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;
import javax.xml.stream.XMLStreamException;

import org.junit.Test;

import anl.aida.client.Cache;
import anl.aida.client.Keyword;
import anl.aida.client.gui.DocOpsWidget;
import anl.aida.client.gui.VizPropertyBean;
import anl.aida.client.io.CacheFileReader;
import anl.aida.client.net.ConceptGraph;
import anl.aida.client.net.OccurenceEdgeCC;
import anl.aida.client.net.OccurrenceFilterCreator;
import anl.aida.client.net.TSConceptGraphs;
import anl.aida.client.net.TermNode;
import anl.aida.util.DateRange;
import anl.aida.viz3d.Canvas3D;
import anl.aida.viz3d.DocOpsCanvasListener;
import anl.aida.viz3d.TSGraphCanvasListener;

public class Viz3DTests {
  
  private Cache cache;
  
  @Test
  public void bBoxTest() {
    Point3f lower = new Point3f(2, -2, 1);
    Point3f upper = new Point3f(6, 2, 3);
    //BoundingSphere box = new BoundingSphere(lower, upper);
    
    //assertEquals(lower, box.getLower());
    //assertEquals(upper, box.getUpper());
    //assertEquals(new Point3f(4, 0, 2), box.getCenter());
  }
  
  @Test 
  public void transformTests() {
    Point3f pt = new Point3f(2, 1, 3);
    
    Matrix4f matrix = new Matrix4f();
    matrix.setIdentity();
    matrix.setTranslation(new Vector3f(-1, 4, 2));
    
    matrix.transform(pt);
    assertEquals(new Point3f(1, 5, 5), pt);
  }
  
  private TSConceptGraphs createGraphs() throws IOException, XMLStreamException {
    CacheFileReader reader = new CacheFileReader();
    cache = reader.read(new File("./cache/cache.xml"));
    Collection<Keyword> keywords = cache.getKeywords().getKeywords("Diseases");
    VizPropertyBean bean = new VizPropertyBean("", new ArrayList<Keyword>(keywords));
    bean.setNodeFilter(new OccurrenceFilterCreator());
    bean.setEdgeFilter(new OccurenceEdgeCC());
    
    TSConceptGraphs graphs = new TSConceptGraphs(cache, bean);
    return graphs;
    //DateRange dateRange = cache.getElement(cache.size() - 1).getHeader().getDateRange();
    //return graphs.getGraph(dateRange);
  }
  
  public void testDisplay() throws IOException, XMLStreamException {
    JFrame frame = new JFrame();
    frame.setLayout(new BorderLayout());
    Canvas3D canvas = new Canvas3D(cache);
   
    TSConceptGraphs graphs = createGraphs();
    DateRange range = cache.getElement(cache.size() - 1).getHeader().getDateRange();
    ConceptGraph<TermNode> graph = graphs.getGraph(range);
    final TSGraphCanvasListener listener = new TSGraphCanvasListener(graph, range);
    canvas.addCanvasListener(listener);
    frame.add(canvas.getPanel(), BorderLayout.CENTER);
    
    canvas.getToolBar().add(new AbstractAction("Add") {
      @Override
      public void actionPerformed(ActionEvent e) {
	//listener.add();
      }
    });
    
    DocOpsWidget widget = new DocOpsWidget();
    JButton button = widget.addOp(graphs.getDocSetOp(null), "page_white_stack.png");
    canvas.addCanvasListener(new DocOpsCanvasListener(widget));
    canvas.getToolBar().add(button);
    listener.initToolBar(canvas.getToolBar());

    frame.setSize(900, 775);
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame.setVisible(true);
  }
  
  public static void main(String[] args) throws IOException, XMLStreamException {
    Viz3DTests tests = new Viz3DTests();
    tests.testDisplay();
  }
}
