/**
 * 
 */
package anl.aida;

import java.io.IOException;

import org.apache.uima.UIMAFramework;
import org.apache.uima.cas.CAS;
import org.apache.uima.collection.CollectionProcessingEngine;
import org.apache.uima.collection.EntityProcessStatus;
import org.apache.uima.collection.StatusCallbackListener;
import org.apache.uima.collection.metadata.CpeDescription;
import org.apache.uima.collection.metadata.CpeDescriptorException;
import org.apache.uima.resource.ResourceInitializationException;
import org.apache.uima.util.InvalidXMLException;
import org.apache.uima.util.Level;
import org.apache.uima.util.XMLInputSource;

/**
 * Runs a CPE.
 * 
 */
public class CPERunner {
  
  
  class StatusCallbackListenerImpl implements StatusCallbackListener {

    boolean wait = true;

    public void entityProcessComplete(CAS cas, EntityProcessStatus entityProcessStatus) {
    }

    public void initializationComplete() {
    }

    public void batchProcessComplete() {
    }

    public void collectionProcessComplete() {
      synchronized (this) {
	UIMAFramework.getLogger().log(Level.INFO, "CPE Runner Finished");
        notifyAll();
      }
    }

    public void paused() {
    }

    public void resumed() {
    }

    public void aborted() {
      synchronized (this) {
        notifyAll();
      }
    }
  }
  
  private CollectionProcessingEngine engine;
  private StatusCallbackListenerImpl listener;
  
  /*
  private static final String GAPP_FILE = "./gate_app/TestApplication090422_v1/application.xgapp";
  private static final String GATE_MAPPING_FILE = "./gate_app/TestApplication090422_v1/application_map.xml";
  private static final String GATE_HOME = "./gate_app";
  
  private static final String LUCENE_INDEX = "./indices/tmp";
  private static final String LUCENE_MAPPING = "./config/sample_lucene_mapping.xml";
  //private static final String LUCENE_MAPPING = "./config/msc_lucene_mapping.xml";
  private static final Boolean OVERWRITE_INDEX = Boolean.TRUE;
  private static final Boolean CREATE_CACHE = Boolean.TRUE;
  private static final String CACHE_START_DATE = "20090401";
  //private static final String CACHE_START_DATE = "19920901";
   */

  public CPERunner(String cpePath, String configFile) throws IOException, InvalidXMLException, ResourceInitializationException, CpeDescriptorException {
    XMLInputSource xml = new XMLInputSource(cpePath);
    CpeDescription desc = UIMAFramework.getXMLParser().parseCpeDescription(xml);
    Configurator config = Configurator.init(configFile);
    config.configure(desc);
    engine = UIMAFramework.produceCollectionProcessingEngine(desc);
    listener = new StatusCallbackListenerImpl();
    engine.addStatusCallbackListener(listener);
  }
  
  public void run() throws ResourceInitializationException {
    engine.process();
    try {
      // engine.process occurs in different thread
      // so we wait here for it to finish. The StatusCallbackListener
      // does a notifyAll() when the collection processing is complete
      // and that stops the wait.
      synchronized (listener) {
        listener.wait();
      }
    } catch (InterruptedException e) {
      e.printStackTrace();
    }
  }
  
  public static void usage() {
    System.out.println("usage: java anl.aida.CPERunner -config config_file");
  }
  
  public static void main(String[] args) {
    try {
      if (args.length != 2 || !args[0].equals("-config")) {
	usage();
	return;
      }
      
      CPERunner runner = new CPERunner("./config/sample_cpe.xml", args[1]);
      runner.run();
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }
}
