package anl.aida.ae.gate;

import org.apache.uima.cas.Type;

import java.util.List;
import java.util.ArrayList;

/**
 *         Date: Jan 16, 2009 3:57:44 PM
 */
public abstract class AbstractUGAnnotationBuilder {

  protected String gateType;
  protected Type type;
  protected String annotationSet;
  protected List<FeatureBuilder> fBuilders = new ArrayList<FeatureBuilder>();

  public AbstractUGAnnotationBuilder(Type type, String gateType) {
    this.type = type;
    this.gateType = gateType;
  }

  /**
   * Gets the UIMA name of this annotation builder.
   *
   * @return the UIMA name of this annotation builder.
   */
  public String getUIMAName() {
    return type.getName();
  }

  /**
   * Gets the name of GATE annotation set, if any, used by this
   * builder.
   *
   * @return the name of GATE annotation set used by this
   *         builder, or null if not set is specified.
   */

  public String getAnnotationSet() {
    return annotationSet;
  }

  /**
   * Sets the name of GATE annotation set, if any, used by this
   * builder.
   *
   * @param annotationSet the name of the annotation set.
   */
  public void setAnnotationSet(String annotationSet) {
    this.annotationSet = annotationSet;
  }

  /**
   * Adds the specified feature builder.
   *
   * @param builder the builder to add
   */
  public void addFeatureBuilder(FeatureBuilder builder) {
    fBuilders.add(builder);
  }
}
