package anl.aida.ae.gate;

import org.apache.uima.cas.CAS;
import org.apache.uima.cas.FeatureStructure;
import gate.Document;

import java.util.Map;

/**
 * Interface for classes that port Annotations between UIMA representations
 * and GATE.
 *
 */
public interface AnnotationBuilder {

  /**
   * Runs the annotation builder using the supplied CAS and Document.
   * Annotations will be transfered between the two.
   *
   * @param cas the UIMA CAS
   * @param doc the gate Document
   * @param index Map of GATE Annotation id ints to UIMA AnnotationFS.
   *
   * @throws UGMappingException if there is an error while building the annotations.
   */
  void run(CAS cas, Document doc, Map<Integer, FeatureStructure> index) throws UGMappingException;

  /**
   * Returns whether or not this AnnotationBuilder will map the gate and uima annotations
   * to each other.
   * @return true if the anntotations are mapped otherwise false.
   */
  boolean isIndexed();

  /**
   * Gets the UIMA name of this annotation builder.
   *
   * @return the UIMA name of this annotation builder.
   */
  String getUIMAName();

  /**
   * Gets the name of GATE annotation set, if any, used by this
   * builder.
   *
   * @return the name of GATE annotation set used by this
   * builder, or null if not set is specified.
   */
  String getAnnotationSet();

  /**
   * Sets the name of GATE annotation set, if any, used by this
   * builder.
   *
   * @param annotationSet the name of the annotation set.
   */
  void setAnnotationSet(String annotationSet);


  /**
   * Adds the specified feature builder.
   *
   * @param builder the builder to add
   */
  void addFeatureBuilder(FeatureBuilder builder);
  
  /**
   * Sets the specified AnnotationBuilderFilter.
   * 
   * @param filter the filter to set
   */
  void setFilter(AnnotationBuilderFilter filter);
}
