package anl.aida.ae.gate;

import gate.AnnotationSet;
import gate.Document;
import gate.Factory;
import gate.FeatureMap;
import gate.util.InvalidOffsetException;
import org.apache.uima.cas.CAS;
import org.apache.uima.cas.FeatureStructure;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationIndex;
import org.apache.uima.cas.text.AnnotationFS;

import java.util.*;

/**
 * Builds GATE style annotations from UIMA ones.
 */
public class InputAnnotationBuilder extends AbstractUGAnnotationBuilder implements AnnotationBuilder {

  private boolean indexed;

  public InputAnnotationBuilder(String gateType, Type type, boolean indexed) {
    super(type, gateType);
    this.indexed = indexed;
  }


  /**
   * Runs the annotation builder using the supplied CAS and Document.
   * Annotations will be transfered between the two.
   *
   * @param cas   the UIMA CAS
   * @param doc   the gate Document
   * @param indexMap map of GATE Annotation id ints to UIMA AnnotationFS.
   * @throws UGMappingException if there is an error while building the annotations.
   */
  public void run(CAS cas, Document doc, Map<Integer, FeatureStructure> indexMap) throws UGMappingException {
    AnnotationSet set;
    if (annotationSet == null) set = doc.getAnnotations();
    else set = doc.getAnnotations(annotationSet);

    AnnotationIndex index = cas.getAnnotationIndex(type);
    for (Iterator iter = index.iterator(false); iter.hasNext();) {
      AnnotationFS annot = (AnnotationFS) iter.next();
      FeatureMap map = Factory.newFeatureMap();

      for (FeatureBuilder builder : fBuilders) {
        builder.build(annot, map);
      }

      try {
       Integer id = set.add(new Long(annot.getBegin()), new Long(annot.getEnd()), gateType, map);
       if (indexed) indexMap.put(id, annot);
      } catch (InvalidOffsetException ex) {
        throw new UGMappingException("Error while building input annotation", ex);
      }
    }
  }

  /**
   * Returns whether or not this AnnotationBuilder will map the gate and uima annotations
   * to each other.
   *
   * @return true if the anntotations are mapped otherwise false.
   */
  public boolean isIndexed() {
    return indexed;
  }
  
  /**
   * Adds the specified AnnotationBuilderFilter.
   * 
   * @param filter the filter to add
   */
  public void setFilter(AnnotationBuilderFilter filter) {
    throw new UnsupportedOperationException();
  }
}
