package anl.aida.ae.gate;

import gate.AnnotationSet;
import gate.Document;
import gate.FeatureMap;

import org.apache.uima.cas.CAS;
import org.apache.uima.cas.FeatureStructure;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationFS;

import java.util.Map;

/**
 * Builds UIMA style annotations from GATE ones.
 *
 */
public class OutputAnnotationBuilder extends AbstractUGAnnotationBuilder implements AnnotationBuilder {

  private boolean indexed;
  private AnnotationBuilderFilter filter = new AnnotationBuilderFilter() {
    @Override
    public boolean buildAnnotation(FeatureMap map) {
      return true;
    }
  };

  public OutputAnnotationBuilder(String gateType, Type type, boolean indexed) {
    super(type, gateType);
    this.indexed = indexed;
  }

  /**
   * Runs the annotation builder using the supplied CAS and Document.
   * Annotations will be transfered between the two.
   * 
   * @param cas
   *          the UIMA CAS
   * @param doc
   *          the gate Document
   * @param indexMap
   *          map of GATE Annotation id ints to UIMA AnnotationFS.
   * @throws UGMappingException
   *           if there is an error while building the annotations.
   */
  public void run(CAS cas, Document doc, Map<Integer, FeatureStructure> indexMap)
      throws UGMappingException {
    AnnotationSet set;
    if (annotationSet == null)
      set = doc.getAnnotations().get(gateType);
    else
      set = doc.getAnnotations(annotationSet).get(gateType);

    if (indexed) {
      for (Integer id : indexMap.keySet()) {
	gate.Annotation gAnt = set.get(id);
	AnnotationFS uAnt = (AnnotationFS) indexMap.get(id);
	buildAnnotation(cas, uAnt, gAnt);
      }

    } else {
      for (gate.Annotation gAnt : set) {
	if (filter.buildAnnotation(gAnt.getFeatures())) {
	  AnnotationFS uAnt = cas.createAnnotation(type,
	      gAnt.getStartNode().getOffset().intValue(), gAnt.getEndNode().getOffset().intValue());
	  buildAnnotation(cas, uAnt, gAnt);
	}
      }
    }
  }

  private void buildAnnotation(CAS cas, AnnotationFS uAnt, gate.Annotation gAnt) {
    FeatureMap features = gAnt.getFeatures();
    for (FeatureBuilder builder : fBuilders) {
      builder.build(uAnt, features);
    }
    cas.addFsToIndexes(uAnt);
  }

  /**
   * Returns whether or not this AnnotationBuilder will map the gate and uima annotations
   * to each other.
   *
   * @return true if the anntotations are mapped otherwise false.
   */
  public boolean isIndexed() {
    return indexed;
  }

  /**
   * Adds the specified AnnotationBuilderFilter.
   * 
   * @param filter the filter to add
   */
  public void setFilter(AnnotationBuilderFilter filter) {
    this.filter = filter;
  }
}