package anl.aida.ae.gate;

import org.apache.uima.cas.FeatureStructure;
import org.apache.uima.cas.Feature;
import gate.FeatureMap;

/**
 * Builds GATE / UIMA features from a GATE / UIMA source.
 * 
 */
public class OutputFeatureBuilder extends AbstractFeatureBuilder {

	public OutputFeatureBuilder(String gateName, Feature feature,
			FeatureType fType) {
		super(gateName, fType, feature);
	}

	/**
	 * Transfers UIMA features to a FeatureMap or vis-a-versa.
	 * 
	 * @param parent
	 *            the UIMA featureStructure containing the feature
	 * @param map
	 *            the feature map to put the feature in
	 */
	public void build(FeatureStructure parent, FeatureMap map) {
		Object val = map.get(gateName);
		// may legitimately be null as GATE annotations of the
		// same type do not necessarily have the same keys their
		// feature map. e.g. a Token may not have orth entry.
		if (val != null) {
			switch (fType) {
			case STRING:
				parent.setStringValue(feature, val.toString());
				break;
			case FLOAT:
				parent.setFloatValue(feature, (Float) val);
				break;
			case DOUBLE:
				parent.setDoubleValue(feature, (Double) val);
				break;
			case LONG:
				parent.setLongValue(feature, (Long) val);
				break;
			case INT:
				parent.setIntValue(feature, (Integer) val);
				break;
			case BOOLEAN:
				parent.setBooleanValue(feature, (Boolean) val);
				break;
			default:
				// todo throw some exp
			}
		}
	}
}