package anl.aida.ae.lucene;

import org.apache.lucene.document.DateTools;
import org.apache.lucene.document.Document;

import java.text.ParseException;
import java.util.Date;


/**
 * Passes documents whose date field value is within
 * some specified range.
 *
 */
public class DateRangeFieldFilter {

  private long start, end;

  /**
   * Creates a DateFieldFilter with the specified range. The
   * filter will be inclusive of start and end.
   *
   * @param start the start of the range
   * @param end   the end of the range
   */
  public DateRangeFieldFilter(Date start, Date end) {
    this.start = DateTools.round(start.getTime(), DateTools.Resolution.DAY);
    this.end = DateTools.round(end.getTime(), DateTools.Resolution.DAY);
  }


  /**
   * Evaluates an item against the date range
   * criteria and returns true if the item passes the filter.
   *
   * @param item the item to evaluate
   * @return true if the item passes the filter, otherwise false.
   * @throws ParseException if there is an error getting the timestamp from the
   *                        document.
   */
  public boolean evaluate(Document item) throws ParseException {
    long time = DateTools.stringToTime(item.get(LuceneConstants.TIMESTAMP_FIELD));
    return evaluate(time);
  }

  /**
   * Evaluates an item against the date range
   * criteria and returns true if the item passes the filter.
   * The time must be rounded using DateTools.round(time, DateTools.Resolution.DAY).
   *
   * @param time the item to evaluate. It must be rounded using
   *             DateTools.round(time, DateTools.Resolution.DAY)
   * @return true if the item passes the filter, otherwise false.
   */
  public boolean evaluate(long time) {
    return time >= start && time <= end;
  }
}
