package anl.aida.ae.lucene;

import anl.aida.util.Filter;

import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.Term;
import org.apache.lucene.index.TermDocs;

import java.io.IOException;

/**
 * Filters a Term based on how many documents in an index it appears in.
 *
 */
public class DocumentFrequencyFilter implements Filter<Term> {

  private IndexReader reader;
  private int freq;

  /**
   * Creates a DocFrequencyFilter that will use the
   * specified index.
   *
   * @param reader    the reader to read the index with
   * @param frequency the minimum number of times the terms must appear in the indexed corpus.
   * @throws IOException if there is an exception opening the index.
   */
  public DocumentFrequencyFilter(IndexReader reader, int frequency) throws IOException {
    this.reader = reader;
    this.freq = frequency;
  }

  /**
   * Evaluates an item against some implementor specific
   * criteria and returns true if the item passes the filter.
   *
   * @param term the item to evaluate
   * @return true if the item passes the filter, otherwise false.
   */
  public boolean evaluate(Term term) {
    TermDocs tDocs = null;
    boolean retVal = false;
    try {
      int count = 0;
      tDocs = reader.termDocs(term);
      while (tDocs.next()) {
        count += tDocs.freq();
        if (count >= freq) {
          retVal = true;
          break;
        }
      }

    } catch (IOException e) {
      throw new RuntimeException("Error reading term docs.", e);
    } finally {
      try {
        if (tDocs != null) tDocs.close();
      } catch (IOException ex) {
        throw new RuntimeException("Error closing term docs.", ex);
      }
    }

    return retVal;
  }


}
