package anl.aida.ae.lucene;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.uima.cas.CAS;

import anl.aida.util.AIDAException;

/**
 * Interface for classes that build Lucene document fields from
 * UIMA annotations.
 *
 */
public interface FieldBuilder {
  
  /**
   * If this field holds keywords, then this gets
   * the keyword type (e.g. "BioAgent"), otherwise
   * this returns null.
   * 
   * @return the keyword type, if applicable, otherwise null.
   */
  String getKeywordType();
  
  /**
   * Gets the name of the field this builder will create.
   * 
   * @return the name of the field this builder will create.
   */
  String getFieldName();

  /**
   * Builds a Field in the specified Document
   * using values from the specified CAS.
   *
   * @param cas the CAS to use to build the field
   * @param doc the doc to add the fields to
   */
  Field build(CAS cas, Document doc, Keywords keywords) throws AIDAException;

}
