package anl.aida.ae.lucene;

import anl.aida.util.Filter;

import org.apache.lucene.index.Term;

import java.util.HashSet;
import java.util.Set;

/**
 * Filters Terms based on their field.
 *
 */
public class FieldFilter implements Filter<Term> {

  private Set<String> fields = new HashSet<String>();

  /**
   * Creates a FieldFilter that will pass a terms if its field is one of the specified
   * fields.
   *
   * @param fields the allowable fields.
   */
  public FieldFilter(String... fields) {
    for (String field : fields) {
      this.fields.add(field);
    }
  }

  /**
   * Evaluates the term against its field.
   *
   * @param item the item to evaluate
   * @return true if the term's field is a member of the allowable fields, otherwise false.
   */
  public boolean evaluate(Term item) {
    return fields.contains(item.field());
  }
}
