/**
 * 
 */
package anl.aida.ae.lucene;

import java.io.IOException;
import java.io.StringReader;

import org.apache.lucene.document.DateTools;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.index.IndexWriter;

import anl.aida.reader.ReaderResult;
import anl.aida.reader.ReaderResultProcessor;
import anl.aida.util.AIDAException;

/**
 * Processes ReaderResults by adding them to a LuceneIndex.
 * 
 */
public class LuceneRRProcessor implements ReaderResultProcessor {
  
  private IndexWriter writer;
  
  
  

  /**
   * Creates a {@link LuceneRRProcessor} that will write
   * the specified index. 
   * 
   * @param writer
   */
  public LuceneRRProcessor(IndexWriter writer) {
    this.writer = writer;
  }


  /**
   * Transforms the reader result to Document and then adds that document to a
   * lucene index.
   * 
   * @param result the result to add
   */
  @Override
  public void process(ReaderResult result) throws AIDAException {
    try {
      ReaderResult.Error error = result.isValid();
      if (error == ReaderResult.Error.NONE) {
	Document doc = new Document();
	// Add the path of the file as a field named "path". Use a field that is
	// indexed (i.e. searchable), but don't tokenize the field into words.
	Field field = new Field(LuceneConstants.TITLE_FIELD, result.getTitle(), Field.Store.YES,
	    Field.Index.ANALYZED);
	field.setBoost(1);
	doc.add(field);

	doc.add(new Field(LuceneConstants.CONTENTS_FIELD, new StringReader(result.getContents())));
	doc.add(new Field(LuceneConstants.PATH_FIELD, result.getLocation(), Field.Store.YES,
	    Field.Index.NOT_ANALYZED));
	doc.add(new Field(LuceneConstants.TIMESTAMP_FIELD, DateTools.timeToString(result.getDate()
	    .getJavaDate().getTime(), DateTools.Resolution.DAY), Field.Store.YES,
	    Field.Index.ANALYZED));
	doc.add(new Field(LuceneConstants.AUTHOR_FIELD, result.getAuthors().getAuthors(),
	    Field.Store.YES, Field.Index.ANALYZED));
	doc.add(new Field(LuceneConstants.SOURCE_FIELD, result.getSource(), Field.Store.YES,
	    Field.Index.NOT_ANALYZED));
	doc.add(new Field(LuceneConstants.LINK_FIELD, result.getLink(), Field.Store.YES,
	    Field.Index.NOT_ANALYZED));
	
	writer.addDocument(doc);
      } else {
	throw new AIDAException("ReaderResult is invalid. Bad " + error);
      }
    } catch (IOException ex) {
      throw new AIDAException("Error writing document to index", ex);
    }
  }


  @Override
  public void done() throws IOException {}


  @Override
  public void initialize() throws AIDAException {}
  
  

}
