package anl.aida.ae.lucene;

import java.util.Properties;
import java.io.InputStream;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

/**
 * Term count db data access object.
 *
 */
public class TermCountDAO {

  private static final String PROPS_FILE = "term_count_db.properties";

  Properties dbProps;

  public TermCountDAO() throws IOException, ClassNotFoundException {
    loadDBProps();
    String driverName = dbProps.getProperty("driver.name");
    Class.forName(driverName);
  }

  private void loadDBProps() throws IOException {
    InputStream dbPropsStream = getClass().getResourceAsStream(PROPS_FILE);
    dbProps = new Properties();
    dbProps.load(dbPropsStream);
  }

  /**
   * Creates a db connection based on the info in the
   * term_count_db.properties file.
   *
   * @return the created connection
   *
   * @throws SQLException if there was an error creating the DB connection.
   */
  public Connection createConnection() throws SQLException {
    String dbUrl = dbProps.getProperty("db.url");
    Properties props = new Properties();
    props.put("user", dbProps.get("user.name"));
    props.put("password", dbProps.get("user.password"));

    return DriverManager.getConnection(dbUrl, props);
  }
}
