package anl.aida.ae.lucene;

import org.apache.lucene.analysis.Token;
import org.apache.lucene.analysis.TokenStream;

import anl.aida.util.AIDAUtils;

import java.util.List;
import java.util.ArrayList;
import java.io.IOException;

/**
 * Builds a TokenStream instance using user supplied Tokens.
 *
 */
public class TokenStreamBuilder {

  private static class StringTokenStream extends TokenStream {

    private int index = 0;
    private List<String> tokens;

    private StringTokenStream(List<String> tokens) {
      this.tokens = new ArrayList<String>(tokens);
    }

    @Override
    public Token next(Token token) throws IOException {
      if (index >= tokens.size()) return null;
      
      String val = tokens.get(index);
      token.reinit(val, 0, val.length());
      index++;

      return token;

    }

    @Override
    public void reset() throws IOException {
      super.reset();
      index = 0;
    }
  }

  private List<String> tokens = new ArrayList<String>();

  /**
   * Adds a String to be returned as an individual Token.
   *
   * @param token the string token.
   */
  public void addToken(String token) {
    tokens.add(AIDAUtils.toInternalFormat(token));
  }

  /**
   * Builds a TokenStream that will return all the Tokens
   * added up to now.
   *
   * @return a TokenStream that will return all the Tokens
   * added up to now.
   */
  public TokenStream buildTokenStream() {
    return new StringTokenStream(tokens);
  }
  
  /**
   * Gets the number of tokens that have been added.
   * 
   * @return the number of tokens that have been added.
   */
  public int tokenCount() {
    return tokens.size();
  }

  /**
   * Clears the list of tokens.
   */
  public void clear() {
    tokens.clear();
  }
}
