package anl.aida.core;

import java.util.Iterator;
import java.util.List;

/**
 * An iterable over an STC subsetted by a list of document ids.
 */
public class DocumentIterable implements Iterable<MIFSDocument> {

  private ScoredTermsCollection stc;
  private List<DocumentID> docs;

  public DocumentIterable(ScoredTermsCollection stc, List<DocumentID> docs) {
    this.stc = stc;
    this.docs = docs;
  }

  /**
   * Returns an iterator over a set of elements of type T.
   *
   * @return an Iterator.
   */
  public Iterator<MIFSDocument> iterator() {
    return new Iter();
  }

  private class Iter implements Iterator<MIFSDocument> {

    private int index = 0;

    public boolean hasNext() {
      return index < docs.size();
    }

    public MIFSDocument next() {
      DocumentID id = docs.get(index);
      index++;
      return stc.getDocument(id.getSTCIndex());
    }

    public void remove() {
      throw new UnsupportedOperationException();
    }
  }
}
