package anl.aida.core;

import anl.aida.util.DateRange;
import anl.aida.util.AIDAException;

import org.apache.lucene.store.IndexInput;
import org.apache.lucene.store.MMapDirectory;

import java.io.IOException;

/**
 * Reads a ScoredTermsCollection from a file written by a
 * STCWriter.
 */
public class STCReader {

  public STCHeader readHeader(String file) throws IOException {
    MMapDirectory mmap = null;
    IndexInput in = null;
    try {
      mmap = new MMapDirectory();
      in = mmap.openInput(file);
      long start = in.readLong();
      long end = in.readLong();
      DateRange range = new DateRange(start, end);
      int docs = in.readInt();
      int terms = in.readInt();

      return new STCHeader(docs, terms, range);
    } finally {
      if (in != null) in.close();
      if (mmap != null) mmap.close();
    }

  }


  /**
   * Reads and returns a StoredTermsCollection written by a STCWriter.
   *
   * @param file the path of the file to read
   * @return the StoredTermsCollection created from the file.
   * @throws IOException   if there is an error reading the file.
   * @throws AIDAException if there is an error reading the file.
   */
  public ScoredTermsCollection readCollection(String file) throws IOException, AIDAException {
    MMapDirectory mmap = null;
    IndexInput in = null;
    STCBuilder builder = null;
    try {
      mmap = new MMapDirectory();
      in = mmap.openInput(file);
      long start = in.readLong();
      long end = in.readLong();
      DateRange range = new DateRange(start, end);

      int docCount = in.readInt();
      int termCount = in.readInt();
      builder = new STCBuilder(range);
      for (int i = 0; i < docCount; i++) {
        String title = in.readString();
        String path = in.readString();
        builder.addDocument(i, title, path);
      }

      for (int i = 0; i < termCount; i++) {
        String termText = in.readString();
        int tDocCount = in.readInt();
        int maxF = in.readInt();
        int minF = in.readInt();
        float maxS = Float.intBitsToFloat(in.readInt());
        float minS = Float.intBitsToFloat(in.readInt());
        builder.addTerm(termText, tDocCount, minF, maxF, minS, maxS);
      }

      long length = in.length();
      while (in.getFilePointer() < length) {
        int termIndex = in.readInt();
        int docIndex = in.readInt();
        int freq = in.readInt();
        float score = Float.intBitsToFloat(in.readInt());
        builder.addScore(docIndex, termIndex, freq, score);
      }

    } finally {
      if (in != null) in.close();
      if (mmap != null) mmap.close();

    }

    return builder.createSTC(false);
  }
}
