/**
 * 
 */
package anl.aida.data;

import java.util.HashMap;
import java.util.Map;

import javax.measure.unit.Unit;

/**
 * DefaultCoordinateAxis implementation.
 * 
 */
public abstract class AbstractCoordAxis implements CoordAxis {
  
  protected AxisData data;
  protected Map<String, String> attributes = new HashMap<String, String>();
  
  /**
   * @param data
   */
  public AbstractCoordAxis(AxisData data) {
    this.data = data;
  }

  /* (non-Javadoc)
   * @see anl.aida.data.CoordAxis#getAxisType()
   */
  @Override
  public AxisType getAxisType() {
    return data.getType();
  }
  
  

  /* (non-Javadoc)
   * @see anl.aida.data.CoordAxis#getAttribute(java.lang.String)
   */
  @Override
  public String getAttribute(String name) {
    return attributes.get(name);
  }
  
  /**
   * Sets the named attributes value
   * 
   * @param name the name of the attribute
   * @param value the attribute value
   */
  public void setAttribute(String name, String value) {
    attributes.put(name, value);
  }

  /* (non-Javadoc)
   * @see anl.aida.data.CoordAxis#getDescription()
   */
  @Override
  public String getDescription() {
    return data.getDescription();
  }

  /* (non-Javadoc)
   * @see anl.aida.data.CoordAxis#getName()
   */
  @Override
  public String getName() {
    return data.getName();
  }

  /* (non-Javadoc)
   * @see anl.aida.data.CoordAxis#getRange()
   */
  @Override
  public Range getRange() {
    return data.getRange();
  }

  /* (non-Javadoc)
   * @see anl.aida.data.CoordAxis#getUnits()
   */
  @Override
  public Unit<?> getUnits() {
    return data.getUnit();
  }
  
  /**
   * Gets whether or not the this axis is compatible with the specified axis.
   * Two axes are compatible if they can be used together in the same formula.
   * 
   * @param axis
   *          the axis to check for compatibility
   * @return true if this axis is compatibilty with the other axis, otherise
   *         false.
   */
  public boolean isCompatible(CoordAxis axis) {
    // identity test
    if (axis == this)
      return true;
    long extent = axis.getRange().getExtent();
    if (axis.getAxisType().equals(this.getAxisType()) && extent == this.getRange().getExtent()) {
      for (int i = 0; i < extent; i++) {
	if (getValue(i) != axis.getValue(i))
	  return false;
      }
      return true;
    }
    return false;
  }
}
