/**
 * 
 */
package anl.aida.data;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Abstract base class for datasets.
 * 
 */
public abstract class AbstractDataset implements Dataset {
  
  protected String alias = "";
  protected Map<String, Variable> varsByName = new HashMap<String, Variable>();
  protected Map<Variable, DataFrame> frames = new HashMap<Variable, DataFrame>();
  
  protected void addVariable(Variable var) {
    varsByName.put(var.getName(), var);
  }
  
  /**
   * Gets the named variable.
   * 
   * @param name the variables name
   * 
   * @return the named variable.
   */
  public Variable getVariable(String name) {
    return varsByName.get(name);
  }
  
  /*
   * (non-Javadoc)
   * 
   * @see anl.aida.data.Dataset#getVariables()
   */
  @Override
  public List<Variable> getVariables() {
    return new ArrayList<Variable>(varsByName.values());
  }


  /* (non-Javadoc)
   * @see anl.aida.data.Dataset#getAlias()
   */
  @Override
  public String getAlias() {
    return alias;
  }

  /* (non-Javadoc)
   * @see anl.aida.data.Dataset#setAlias(java.lang.String)
   */
  @Override
  public void setAlias(String alias) {
    this.alias = alias;
  }

  /*
   * (non-Javadoc)
   * 
   * @see anl.aida.data.Dataset#getDataFrame(anl.aida.data.Variable)
   */
  @Override
  public DataFrame getDataFrame(Variable var) {
    return frames.get(var);
  }
  
  @Override
  public Axes<CoordAxis> getAxes(Variable var) {
    // this really only works when the data frames are small
    // otherwise the axes info should be elsewhere.
    Axes<DataFrameAxis> axes = getDataFrame(var).getAxes();
    List<CoordAxis> list = new ArrayList<CoordAxis>();
    for (CoordAxis axis : axes.getAxes()) {
      list.add(axis);
    }
    Axes<CoordAxis> out = new Axes<CoordAxis>(list);
    return out;
  }
}
