/**
 * 
 */
package anl.aida.data;

import java.util.ArrayList;
import java.util.List;

/**
 * Checks the compatibility of Axes.
 * 
 */
public class CompatibilityChecker {
  
  public static class Result {
    private boolean timeCompatible, categoryCompatible, mapCompatible, categoryAllNull;
    
    public Result() {
      timeCompatible = categoryCompatible = mapCompatible = true;
    }

    public boolean isTimeCompatible() {
      return timeCompatible;
    }

    public boolean isCategoryCompatible() {
      return categoryCompatible;
    }

    public boolean isMapCompatible() {
      return mapCompatible;
    }
    
    public boolean isCategoryAllNull() {
      return categoryAllNull;
    }
    
    public String getMessage() {
      StringBuilder buf = new StringBuilder();
      if (!timeCompatible || !categoryCompatible) {
	if (!timeCompatible) buf.append("Incompatible Time Dimensions");
	if (!categoryCompatible) {
	  if (buf.length() > 0) buf.append("; ");
	  buf.append("Incompatible Group Dimensions");
	}
	buf.append(".");
      } else if (!mapCompatible) {
	//buf.append("Map plot can only display a single variable and the variable must have a spatial dimension.");
      }
      return buf.toString();
    }
  }

  
  private List<Axes<? extends CoordAxis>> axesList = new ArrayList<Axes<? extends CoordAxis>>();

  public void addAxes(Axes<? extends CoordAxis> axes) {
    axesList.add(axes);
  }

  private boolean isCompatible(CoordAxis axis1, CoordAxis axis2) {
    if (axis1 == null && axis2 != null)
      return false;
    if (axis1 != null && axis2 == null)
      return false;
    if (axis1 == null && axis2 == null) return true;

    return axis1.isCompatible(axis2);
  }
  
  /**
   * Gets the number of axes currently added for checking.
   * 
   * @return the number of axes currently added for checking.
   */
  public int axesCount() {
    return axesList.size();
  }

  /**
   * Checks the compatibilty of the added Axes and returns the result.
   * 
   * @return the result of the compatibilty check.
   */
  public Result check() {
    Result result = new Result();
    if (axesList.size() == 0) {
      result.categoryCompatible = result.mapCompatible = result.timeCompatible = false;
    } else if (axesList.size() == 1) {
      result.mapCompatible = axesList.get(0).getAxis(AxisType.NAMED_LOCATION) != null;
      result.categoryAllNull = axesList.get(0).getNonTimeAxis() == null;
    } else if (axesList.size() > 1) {
      
      result.mapCompatible = false;
      CoordAxis timeAxis = axesList.get(0).getTimeAxis();
      CoordAxis groupAxis = axesList.get(0).getNonTimeAxis();
      for (int i = 1; i < axesList.size(); i++) {
	Axes<? extends CoordAxis> axes = axesList.get(i);
	result.timeCompatible = isCompatible(timeAxis, axes.getTimeAxis());
	result.categoryCompatible = isCompatible(groupAxis, axes.getNonTimeAxis());
      }
      
      result.categoryAllNull = groupAxis == null && result.isCategoryCompatible();
    } 
    return result;
  }
}
