package anl.aida.data;

import ucar.ma2.Index;

/**
 * Encapsulates an index into a DataFrame data array.
 * 
 * @version $Revision$ $Date$
 */
public class DataFrameIndex {

  private int xIndex, yIndex, tIndex, kIndex;
  private int[] indices;
  Index index;

  /**
   * Creates a DataFrameIndex for the specified DataFrame.
   * 
   * @param frame
   *          the DataFrame that this is an index for
   */
  public DataFrameIndex(DataFrame frame) {
    Axes<DataFrameAxis> axes = frame.getAxes();
   
    for (DataFrameAxis axis : axes.getAxes()) {
      AxisType axisType = axis.getAxisType();
      int index = axis.getArrayIndex();
      if (axisType == AxisType.TIME) {
	tIndex = axis.getArrayIndex();
      } else if (axisType == AxisType.X_AXIS) {
	xIndex = index;
      } else if (axisType == AxisType.Y_AXIS) {
	yIndex = index;
      } else if (axisType == AxisType.LAYER) {
	kIndex = index;
      }
    }
    
    indices = new int[axes.size()];
    index = frame.getArray().getIndex();
  }

  /**
   * Sets the time step index value.
   * 
   * @param timeStep
   *          the time step.
   */
  public void setTime(int timeStep) {
    indices[tIndex] = timeStep;
    index.set(indices);
  }

  /**
   * Sets the layer index value.
   * 
   * @param layer
   *          the layer index value
   */
  public void setLayer(int layer) {
    indices[kIndex] = layer;
    index.set(indices);
  }
  
  /**
   * Sets the first two index values.
   * 
   * @param index0
   * @param index1
   */
  public void set(int index0, int index1) {
    indices[0] = index0;
    indices[1] = index1;
    index.set(indices);
  }
  
  /**
   * Sets the first index value.
   * 
   * @param index0 the index value
   */
  public void set(int index0) {
    index.set(index0);
  }

  /**
   * Sets all the indices for all the coordinate axes.
   * 
   * @param timeStep
   *          the time ste p index
   * @param layer
   *          the layer index
   * @param x
   *          the x index
   * @param y
   *          the y index
   */
  public void set(int timeStep, int layer, int x, int y) {
    indices[xIndex] = x;
    indices[yIndex] = y;
    indices[tIndex] = timeStep;
    indices[kIndex] = layer;
    try {
      index.set(indices);
    } catch (Exception e) {
      System.out.println("error timeStep layer x y " + timeStep + " " + layer + " " + x + " " + y);
    }
  }

  /**
   * Sets the x and y index value
   * 
   * @param x
   *          the x value
   * @param y
   *          the y value
   */
  public void setXY(int x, int y) {
    indices[xIndex] = x;
    indices[yIndex] = y;
    index.set(indices);
  }
}
