package anl.aida.data;

import java.util.Date;

/**
 * Time axis for a DataFrame.
 * 
 * @version $Revision$ $Date$
 */
public class DataFrameTimeAxis extends DataFrameAxis implements TimeCoordAxis {

  /**
   * Creates a DataFrameAxis for the specified axis with origin and extent equal
   * to that of the axis.
   * 
   * @param axis
   *          the axis
   * @param index
   *          the array index of this axis
   */
  public DataFrameTimeAxis(TimeCoordAxis axis, int index) {
    super(axis, index);
  }

  /**
   * Creates a DataFrameAxis for the specified axis with the specified origin
   * and extent.
   * 
   * @param origin
   *          the origin of the range
   * @param extent
   *          the extent of the range
   * @param axis
   *          the axis
   * @param index
   *          the array index of this axis
   */
  public DataFrameTimeAxis(TimeCoordAxis axis, int origin, int extent, int index) {
    super(axis, origin, extent, index);
  }

  /**
   * Creates a DataFrameAxis from the specified axis and index. This
   * DataFrameAxis will have the same origin and extent as the specified axis.
   * 
   * @param axis
   *          the axis to create this DataFrameAxis from
   * @param index
   *          the array index of this axis
   */
  public DataFrameTimeAxis(DataFrameTimeAxis axis, int index) {
    super(axis, index);
  }

  /**
   * Returns the Date for the specified timestep, or null if the timestep is
   * invalid.
   * 
   * @param timestep
   *          the time step we want the date for.
   * @return the Date for the specified timestep
   */
  public Date getDate(int timestep) {
    return ((TimeCoordAxis) this.axis).getDate(timestep);
  }

  /**
   * Gets the timestep that corresponds to the specified date. If no such
   * timestep is found then return Axes.TIME_STEP_NOT_FOUND.
   * 
   * @param date
   *          the date whose time step we want
   * @return the timestep that corresponds to the specified date. If no such
   *         timestep is found then return Axes.TIME_STEP_NOT_FOUND.
   */
  public int getTimeStep(Date date) {
    return ((TimeCoordAxis) this.axis).getTimeStep(date);
  }
}
