package anl.aida.data;

import java.util.List;

/**
 * Interface for classes that function as sources / sets of data.
 * 
 */
public interface Dataset {
  
  /**
   * Gets the name of the dataset. 
   * 
   * @return the name of the dataset. 
   */
  String getName();
  
  /**
   * Gets the named variable.
   * 
   * @param name the variables name
   * 
   * @return the named variable.
   */
  Variable getVariable(String name);
  
  /**
   * Gets the alias of this dataset.
   * 
   * @return the alias of this dataset
   */
  String getAlias();
  
  /**
   * Sets the alias of this Dataset.
   * 
   * @param alias the new alias
   */
  void setAlias(String alias);
  
  /**
   * Gets the list of variables in this dataset.
   * 
   * @return the list of variables in this dataset.
   */
  List<Variable> getVariables();
  
  /**
   * Gets a DataFrame for the specified variable.
   * 
   * @param var the variable to get the DataFrame for
   * 
   * @return a DataFrame for the specified variable.
   */
  DataFrame getDataFrame(Variable var);
  
  /**
   * Gets the specified variable's dimensional axes.
   * 
   * @param var the variable
   * @return the specified variable's dimensional axes.
   */
  Axes<CoordAxis> getAxes(Variable var);

}
