package anl.aida.data;

import javax.measure.unit.Unit;

/**
 * class that stores metadata on a variable in a dataset
 * 
 * @see DefaultVariable
 * @version $Revision$ $Date$
 * 
 */
public class DefaultVariable implements Variable {

  protected Unit<?> unit;
  protected String name, fullName;
  protected String description;
  protected Dataset dataset;

  public DefaultVariable(String name, String description, Unit<?> unit, Dataset dataset) {
    this.name = name;
    this.description = description;
    this.unit = unit;
    this.dataset = dataset;
    fullName = new StringBuffer(name).append(" (").append(Units.getFormattedName(unit)).append(")")
	.toString();
  }

  /**
   * Gets the dataset in which this variable appears. This may be null if the
   * Variable is a composite composed of multiple datasets.
   * 
   * @return the dataset in which this variable appears.
   */
  public Dataset getDataset() {
    return dataset;
  }

  /**
   * Get the description of a variable
   * 
   * @return the description
   */
  public String getDescription() {
    return description;
  }

  /**
   * Get the name of the variable
   * 
   * @return the name
   */
  public String getName() {
    return name;
  }

  /**
   * 
   * @return the variables name plus its unit represention
   */
  public String toString() {
    return fullName;
  }

  /**
   * Gets this Variable's standard of measurement.
   * 
   * @return this Variable's standard of measurement.
   */
  public Unit<?> getUnit() {
    return unit;
  }
}
