/**
 * 
 */
package anl.aida.data;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import javax.measure.unit.NonSI;
import javax.measure.unit.SI;
import javax.measure.unit.Unit;

/**
 * TimeAxis with a regular interval
 * 
 */
public class RegularTimeAxis extends AbstractCoordAxis implements TimeCoordAxis {

  private int interval;
  private TimeIntervalUnit iUnit;
  private Date startDate;

  private Calendar cal = GregorianCalendar.getInstance();

  public RegularTimeAxis(Date start, int interval, AxisData data) {
    super(data);
    this.startDate = start;
    this.interval = interval;

    Unit<?> unit = data.getUnit();
    if (unit.equals(NonSI.DAY))
      iUnit = TimeIntervalUnit.DAY_OF_YEAR;
    else if (unit.equals(NonSI.HOUR))
      iUnit = TimeIntervalUnit.HOUR;
    else if (unit.equals(SI.SECOND))
      iUnit = TimeIntervalUnit.SECOND;
    else if (unit.equals(NonSI.YEAR)) 
      iUnit = TimeIntervalUnit.YEAR;
    else if (unit.equals(NonSI.MONTH))
      iUnit = TimeIntervalUnit.MONTH;
    else if (unit.equals(SI.SECOND.divide(1000)))
      iUnit = TimeIntervalUnit.MILLISECOND;
    else if (unit.equals(NonSI.MINUTE))
      iUnit = TimeIntervalUnit.MINUTE;
    else if (unit.equals(NonSI.WEEK)) 
      iUnit = TimeIntervalUnit.WEEK;
    else
      throw new IllegalArgumentException("Unknown interval unit");

  }

  @Override
  public Date getDate(int timestep) {
    int delta = interval * timestep;
    cal.setTime(startDate);
    cal.add(iUnit.getCalendarEquivalent(), delta);
    return cal.getTime();

  }

  @Override
  public int getTimeStep(Date date) {
    Range range = data.getRange();
    for (int i = range.origin, n = (int) range.getExtent(); i < n; i++) {
      if (getDate(i).equals(date))
	return i;
    }

    return Axes.TIME_STEP_NOT_FOUND;
  }

  @Override
  public AxisType getAxisType() {
    return AxisType.TIME;
  }

  @Override
  public double getValue(int index) {
    Date date = getDate(index);
    return date.getTime();
  }
  
  /**
   * Gets the value as String at the specified index.
   * 
   * @param index the index of the value to get
   * @return the value as String at the specified index.
   */
  public String getValueAsString(int index) {
    return getDate(index).toString();
  }
}
