package anl.aida.formula;

import java.util.List;

import anl.aida.data.AxisRange;
import anl.aida.data.DataFrame;
import anl.aida.data.DataManager;

/**
 * Encapsulates a pave style formula and its evaluation.
 * 
 * @version $Revision$ $Date$
 */
public interface Formula {
  
  /**
   * Validates the formula.
   * 
   * @param manager
   *          the data manager used to validate the formula
   * @param ranges
   *          the ranges that constrain the validation
   *          
   * @return a ValidationResult containing the results of the validation.
   */
  
  ValidationResult validate(DataManager manager, List<AxisRange> ranges);
  
  /**
   * Evaluates the formula to produce a DataFrame.
   * 
   * @param manager the DataManager used to help evaluate the formula
   * @param ranges
   *          the ranges that constrain the evaluation
   * 
   * @throws IllegalFormulaException
   *           if the formula is unable to be executed.
   */
  DataFrame evaluate(DataManager manager, List<AxisRange> ranges) throws IllegalFormulaException;
  
  /**
   * Gets the String representation of the formula.
   *  
   * @return the String representation of the formula.
   */
  String getFormulaAsString();
  
 
  
  /**
   * Overrides any other time range and uses this one in the formula.
   * 
   * @param range
   *          the time range to evaluate against
   * @throws anl.verdi.formula.IllegalFormulaException
   *           if there is error setting the time range.
   *
  void overrideTimeRange(DateRange range) throws IllegalFormulaException;


  /**
   * Evaluates the formula to produce a DataFrame.
   * 
   * @param manager
   *          the manager used to get read access to the relevant data
   * @param ranges
   *          the ranges that constrain the evaluation
   * @return a DataFrame containing the result of evaluating the formula.
   * @throws IllegalFormulaException
   *           if the formula is unable to be executed.
   *
  DataFrame evaluate(DataManager manager, List<AxisRange> ranges) throws IllegalFormulaException;

  /**
   * Evaluates the formula to produce a DataFrame.
   * 
   * @param manager
   *          the manager used to get read access to the relevant data
   * @param ranges
   *          the ranges that constrain the evaluation
   * @param convertUnits
   *          whether or not to convert the units when evaluating the formula
   * @return a DataFrame containing the result of evaluating the formula.
   * @throws IllegalFormulaException
   *           if the formula is unable to be executed.
   *
  DataFrame evaluate(DataManager manager, List<AxisRange> ranges, boolean convertUnits)
      throws IllegalFormulaException;

  /**
   * Validates the formula. This checks that the units, layers domains and
   * timestep are coherent and within range.
   * 
   * @param manager
   *          the data manager used to validate the formula
   * @param ranges
   *          the ranges that will filter or constrain the evaluation.
   * @return a ValidationResult containing the results of the validation.
   *
  ValidationResult validate(DataManager manager, List<AxisRange> ranges);
  */
}
