package anl.aida.formula;

import javax.measure.unit.Unit;

import anl.aida.data.DataFrame;
import anl.aida.data.DataManager;
import anl.aida.data.Dataset;
import anl.aida.data.Range;
import anl.aida.data.Variable;

/**
 * @version $Revision$ $Date$
 */
public class FormulaVariable {

  public static FormulaVariable createVariable(String name, DataManager manager)
      throws IllegalFormulaException {
    String[] split = manager.splitVarName(name);
    FormulaVariable var = new FormulaVariable(split[1], split[0]);
    var.init(manager);
    return var;
  }

  private String name;
  private String alias;
  private String aliasedName;
  private Dataset dataset;
  private Variable variable;
  private Range timeStepRange;

  private FormulaVariable(String name, String alias) {
    this.alias = alias;
    this.name = name;
    this.aliasedName = name + alias;
  }

  public Variable getVariable() {
    return variable;
  }

  private void init(DataManager manager) throws IllegalFormulaException {
    if (dataset == null) {
      dataset = manager.getDataset(alias);
      if (dataset == null)
	throw new IllegalFormulaException("Alias '" + alias
	    + "' does not identify a loaded dataset.");
      variable = dataset.getVariable(name);

      if (variable == null)
	throw new IllegalFormulaException("Variable '" + name + "' not found in dataset '" + alias
	    + "'");
    }
  }

  public Range getTimeStepRange() {
    return timeStepRange;
  }

  public void setTimeStepRange(Range timeStepRange) {
    this.timeStepRange = timeStepRange;
  }

  /**
   * Evaluates this FormulaVariable and returns the resulting DataFrame.
   * 
   * @return the data frame that results from formula evaluation
   */
  public DataFrame evaluate() {
    return variable.getDataset().getDataFrame(variable);
  }

  /*
   * public Range getTimeStepRange() { return timeStepRange; }
   * 
   * public void setTimeStepRange(Range timeStepRange) { this.timeStepRange =
   * timeStepRange; }
   * 
   * public DataFrame evaluate(List<AxisRange> ranges) {
   * 
   * List<AxisRange> myRanges = new ArrayList<AxisRange>(ranges); if
   * (timeStepRange != null) { removeTimeStepRange(myRanges); myRanges.add(new
   * AxisRange(dataset.getCoordAxes().getTimeAxis(), timeStepRange.getOrigin(),
   * (int) timeStepRange.getExtent())); } return reader.getValues(dataset,
   * myRanges, variable); }
   * 
   * 
   * private void removeTimeStepRange(List<AxisRange> ranges) { for
   * (Iterator<AxisRange> iter = ranges.iterator(); iter.hasNext();) { AxisRange
   * range = iter.next(); if (range.getAxisType() == AxisType.TIME)
   * iter.remove(); } }
   */

  public String getAliasedName() {
    return aliasedName;
  }

  public String getName() {
    return name;
  }

  public Dataset getDataset() {
    return dataset;
  }

  public Unit<?> getUnit() {
    return variable.getUnit();
  }
}
