/**
 * 
 */
package anl.aida.reader;

import java.io.IOException;
import java.util.Date;

/**
 * Reads content from AllAfrica web pages. This assumes
 * that web pages are the printable versions.
 * 
 */
public class AllAfricaHTMLReader implements ContentReader {
  
  private class Processor implements DocumentProcessor {
    
    boolean done = false;
    boolean start = false;

    @Override
    public boolean done() {
      return done;
    }

    @Override
    // there's very little non-content on the AA printable pages
    // so once we hit the title we start collecting and stop
    // when we hit line that starts with Copyright
    public void processLine(String line) {
      line = line.trim();
      if (!start && line.startsWith(title)) {
	start = true;
      } else if (start) {
	if (line.startsWith("Copyright")) {
	  done = true;
	  start = false;
	} else {
	  int index = line.indexOf("<http:");
	  if (index != -1) {
	    content.append(line.substring(0, index));
	    int end = line.indexOf(">");
	    if (end != -1 && end != line.length()) {
	      content.append(line.substring(end + 1, line.length()));
	    }
	  } else {
	    content.append(line);
	  }
	  content.append(" ");
	}
      }
    }

    @Override
    public void reset() {
      done = false;
      start = false;
    }
  }
  
  private StringExtractor extractor = new StringExtractor();
  private Processor processor = new Processor();
  private StringBuilder content;
  private String title;
  

  /* (non-Javadoc)
   * @see anl.aida.reader.ContentReader#read(java.lang.String, java.lang.String, java.util.Date, java.lang.String)
   */
  @Override
  public ReaderResult read(String url, String title, Date date, String author) throws IOException {
    ReaderResult result = new ReaderResult();
    content = new StringBuilder();
    this.title = title.trim();

    result.setTitle(title);
    result.setDate(new ParsedDate(date));
    result.setLocation(url);
    result.setSource(ReaderConstants.ALL_AFRICA_SOURCE);
    result.setLink("");

    ParsedAuthors authors = new ParsedAuthors();
    authors.addName("", author, "");
    result.setAuthors(authors);

    processor.reset();
    extractor.run(url, processor);
    result.setContents(content.toString());

    return result;
  }
}
