package anl.aida.reader;

import java.io.IOException;
import java.util.Date;

/**
 * Scrapes article content from <b>archived</b> Chicago Tribune pages.
 * 
 */
public class ChicagoTribuneHTMLReader implements ContentReader {

  // appends as content lines between
  // <http://archives.chicagotribune.com/2009 and "Save/Share"
  private class Processor implements DocumentProcessor {

    private static final int WAITING = 0;
    private static final int COLLECTING = 1;
    private static final int STOPPED = 2;
    
    private int status = WAITING;

    public void processLine(String line) {
      // System.out.println("line = " + line);
      if (status == COLLECTING && !line.startsWith("<http:")) {
	if (line.startsWith("Save/Share:")) {
	  status = STOPPED;
	} else {
	  content.append(line);
	  content.append(" ");
	}
      }

      if (status == WAITING && line.startsWith("<http://archives.chicagotribune.com/"))
	status = COLLECTING;
    }
    
    @Override
    public boolean done() {
      return status == STOPPED;
    }
    
    public void reset() {
      status = WAITING;
    }
  }

  private StringExtractor extractor = new StringExtractor();
  private DocumentProcessor processor = new Processor();
  private StringBuilder content;
  private ReaderResult result;

  /**
   * Reads the text from the specified URL. The title and date are expected to
   * be provided from another source, e.g. an RSS feed entry or an archive
   * "index."
   * 
   * @param url
   *          the url to read from
   * @param title
   *          the article title
   * @param date
   *          the date of the article
   * @return the result of reading and parsing the text at the url
   * @throws IOException
   *           if there is an error while reading
   */
  public ReaderResult read(String url, String title, Date date, String author) throws IOException {
    result = new ReaderResult();
    content = new StringBuilder();

    result.setTitle(title);
    result.setDate(new ParsedDate(date));
    result.setLocation(url);
    result.setSource(ReaderConstants.CHICAGO_TRIBUNE_SOURCE);
    result.setLink("");

    ParsedAuthors authors = new ParsedAuthors();
    authors.addName("", author, "");
    result.setAuthors(authors);

    processor.reset();
    extractor.run(url, processor);
    result.setContents(content.toString());

    return result;
  }
}
