/**
 * 
 */
package anl.aida.reader;

import java.io.File;
import java.io.IOException;
import java.util.Date;

import org.apache.uima.resource.ConfigurableResource;
import org.apache.uima.resource.ResourceInitializationException;

import anl.aida.util.StandardIndexLineParser;

/**
 * AIDAComponentReader implementation for reading Chicago Tribune 
 * web articles. This uses the {@link ChicagoTribuneHTMLReader} to read
 * <b>archived</b> web entries. This may not work with non-archived
 * material. The list of links should be in a file in the "standard" 
 * index format. Namely,
 * 
 * timestamp:::link:::title:::author
 * 
 * The author or authors is optional.
 */
public class ChicagoTribuneReader extends AbstractAIDAComponentReader {
  
  public static final String INDEX_FILE = "anl.aida.reader.CTIndex";
  
  private ContentReader ctReader = new ChicagoTribuneHTMLReader();
  
  public ChicagoTribuneReader() {
    // uncomment to cache the content
    /*
    try {
      super.processors.add(new ResultCacher("./corpus/ct", "ct_cache_index.txt"));
    } catch (IOException e) {
      UIMAFramework.getLogger(getClass()).log(Level.SEVERE, "Unable to create result cacher", e);
    }
    */  
  }

  /* (non-Javadoc)
   * @see anl.aida.reader.AbstractAIDAComponentReader#getNextResult()
   */
  @Override
  protected ReaderResult getNextResult() throws IOException {
    lineItems = indexIter.nextParsedLine(lineItems);
    location = lineItems[StandardIndexLineParser.URL];
    Date date = new Date(Long.parseLong(lineItems[StandardIndexLineParser.TIMESTAMP]));
    checkDate(date);
    
    return ctReader.read(location,  lineItems[StandardIndexLineParser.TITLE], date, "");
  }
  
  @Override
  protected String getIndexFileKey() {
    return INDEX_FILE;
  }

  @Override
  public void initialize(ConfigurableResource resource, Date startDate) throws ResourceInitializationException {
    super.initialize(resource, startDate);
    String[] line = indexIter.nextParsedLine(new String[4]);
    indexIter.rewind();
    String url = line[StandardIndexLineParser.URL];
    if (!url.startsWith("http://")) {
      String indexFile = (String)resource.getConfigParameterValue(INDEX_FILE);
      File file = new File(indexFile);
      ctReader = new CachedContentReader(ReaderConstants.CHICAGO_TRIBUNE_SOURCE, file.getParent());
    }
  }
}


