package anl.aida.reader;


/**
 * Interface for classes that can parse a line of text.
 * The intention is that LineParsers can be chained together to
 * produce a full parser. For example, one might look for a
 * Date and then when that is found next() would return the 
 * parser that parses the documents content.
 * 
 */
public interface LineParser {
  
  /**
   * Parses the specified line of text. The StringBuilder
   * should accumulate the documents true content in contrast to
   * any metadata that might be parsed such as the date and so forth.
   * 
   * @param line the line of text to parse
   * @param buf the buffer to put the actual content in
   * @return true if 
   * next() should be called, otherwise false. 
   */
  boolean parse(String line, StringBuilder buf);
  
  /**
   * Returns the next LineParser in the chain.
   * 
   * @return the next LineParser in the chain.
   */
  LineParser next();
}
