package anl.aida.reader;

import java.io.IOException;
import java.util.Date;

/**
 * Reads the html from an NTARC article page. This stores as content the lines
 * of text between "Thanks for Visiting" and an http link containing "reblog".
 * When visiting a page without a true cookie accepting browser, the Thanks For
 * Visiting message should always show up.
 * 
 */
public class NTARCHtmlReader implements ContentReader {

  private class NTARProcessor implements DocumentProcessor {

    boolean collect = false;
    boolean done = false;

    public void processLine(String line) {
      if (collect) {
	if (line.startsWith("<http:")) {
	  if (collect && line.contains("reblog")) {
	    collect = false;
	    done = true;
	  }
	} else if (line.startsWith("Share this post") || line.startsWith("Tags:<http")) {
	  collect = false;
	  done = true;
	} else {
	  // System.out.println("line = " + line);
	  content.append(line);
	  content.append(" ");
	}
      }

      if (!collect && line.endsWith("Thanks for visiting!"))
	collect = true;
    }

    @Override
    public boolean done() {
      return done;
    }

    public void reset() {
      collect = false;
      done = false;
    }

  }

  private StringExtractor extractor = new StringExtractor();
  private DocumentProcessor processor = new NTARProcessor();
  private StringBuilder content;
  private ReaderResult result;

  /**
   * Parses the text from the specified URL. The title and date are expected to
   * be provided from another source, e.g. an RSS feed entry or an "index."
   * 
   * @param url
   *          the url to read from
   * @param title
   *          the article title
   * @param date
   *          the date of the article
   * @return the result of reading and parsing the text at the url
   * @throws IOException
   *           if there is an error while reading
   */
  public ReaderResult read(String url, String title, Date date, String author) throws IOException {
    result = new ReaderResult();
    content = new StringBuilder();

    result.setTitle(title);
    result.setDate(new ParsedDate(date));
    result.setLocation(url);
    result.setSource(ReaderConstants.NTARC_SOURCE);
    result.setLink("");

    ParsedAuthors authors = new ParsedAuthors();
    authors.addName("", "", "");
    result.setAuthors(authors);

    processor.reset();
    extractor.run(url, processor);
    result.setContents(content.toString());

    return result;
  }
}
