package anl.aida.reader;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Calendar;
import java.util.GregorianCalendar;

public class NTARCIndexMaker {

  private static class ArchiveMaker implements DocumentProcessor {

    // timestamp:::link:::title:::author

    private PrintWriter writer;
    public long timeStamp;
    public String prefix;

    public ArchiveMaker(String file) throws IOException {
      writer = new PrintWriter(new BufferedWriter(new FileWriter(file)));
    }

    public void processLine(String line) {
      // System.out.println("line = " + line);
      line = line.trim();
      //System.out.println("line = " + line);
      if (line.startsWith("<" + prefix) && !line.contains("/#comments>")) {
	//System.out.println("line = " + line);
	int end = line.indexOf(">");
	String link = line.substring(1, end);
	String title = line.substring(end + 1);

	writer.printf("%d:::%s:::%s:::%n", timeStamp, link, title);
	//System.out.printf("item:%n\t%d%n\t%s%n\t%s%n", timeStamp, link, title);
	

      }
    }

    @Override
    public boolean done() {
      return false;
    }

    public void close() throws IOException {
      writer.close();
    }

    /*
     * (non-Javadoc)
     * 
     * @see anl.aida.reader.DocumentProcessor#reset()
     */
    @Override
    public void reset() {
      // TODO Auto-generated method stub

    }
  }

  public static void main(String[] args) throws IOException {
    StringExtractor extractor = new StringExtractor();
    ArchiveMaker maker = new ArchiveMaker("./ntarc_archive_index.txt");
    int[] daysEnd = { 31, 30, 31 };
    int[] months = { 2, 3, 4 };

    Calendar cal = GregorianCalendar.getInstance();
    cal.set(Calendar.YEAR, 2009);
    cal.set(Calendar.HOUR_OF_DAY, 9);

    for (int index = 0; index < months.length; index++) {
      int month = months[index];

      cal.set(Calendar.MONTH, month);

      for (int day = 1; day <= daysEnd[index]; day++) {
	cal.set(Calendar.DAY_OF_MONTH, day);
	maker.timeStamp = cal.getTime().getTime();

	String sDay = String.valueOf(day);
	if (day < 10) {
	  sDay = "0" + day;
	}

	String sMonth = "0" + (month + 1);

	String url = String.format("http://www.nationalterroralert.com/updates/2009/%s/%s/",
	    sMonth, sDay);

	maker.prefix = url;
	System.out.println("Processing URL: " + url);
	extractor.run(url, maker);
      }
    }
    maker.close();
  }

}
