/**
 * 
 */
package anl.aida.reader;

import java.io.File;
import java.io.IOException;
import java.util.Date;

import org.apache.uima.resource.ConfigurableResource;
import org.apache.uima.resource.ResourceInitializationException;

import anl.aida.util.StandardIndexLineParser;

/**
 * AIDAComponentReader implementation for reading NTARC web entries.
 * The list of links should be in a file in the "standard" index
 * format. Namely,
 * 
 * timestamp:::link:::title:::author
 * 
 * The author or authors is optional.
 */
public class NTARCReader extends AbstractAIDAComponentReader {
  
  public static final String INDEX_FILE = "anl.aida.reader.NTARCIndex";
  
  private ContentReader ntarcReader;
  
  public NTARCReader() {
    // uncomment to cache the content
   /*
    try {
      super.processors.add(new ResultCacher("./corpus/ntarc", "ntarc_cache_index.txt"));
    } catch (IOException e) {
      UIMAFramework.getLogger(getClass()).log(Level.SEVERE, "Unable to create result cacher", e);
    }
    */
  
  }

  /* (non-Javadoc)
   * @see anl.aida.reader.AbstractAIDAComponentReader#getNextResult()
   */
  @Override
  protected ReaderResult getNextResult() throws IOException {
    lineItems = indexIter.nextParsedLine(lineItems);
    location = lineItems[StandardIndexLineParser.URL];
    Date date = new Date(Long.parseLong(lineItems[StandardIndexLineParser.TIMESTAMP]));
    checkDate(date);
    
    //System.out.printf("item:%n\t%s%n\t%s%n\t%s%n", location.trim(), lineItems[StandardIndexLineParser.TIMESTAMP], 
    //	date);
    
    return ntarcReader.read(location,  lineItems[StandardIndexLineParser.TITLE], date, "");
  }

  /* (non-Javadoc)
   * @see anl.aida.reader.AbstractAIDAComponentReader#getIndexFileKey()
   */
  @Override
  protected String getIndexFileKey() {
    return INDEX_FILE;
  }

  @Override
  public void initialize(ConfigurableResource resource, Date startDate) throws ResourceInitializationException {
    super.initialize(resource, startDate);
    String[] line = indexIter.nextParsedLine(new String[4]);
    indexIter.rewind();
    String url = line[StandardIndexLineParser.URL];
    if (url.startsWith("http://")) {
      ntarcReader = new NTARCHtmlReader();
    } else {
      String indexFile = (String)resource.getConfigParameterValue(INDEX_FILE);
      File file = new File(indexFile);
      ntarcReader = new CachedContentReader(ReaderConstants.NTARC_SOURCE, file.getParent());
    }
  }
}
