package anl.aida.reader;

import java.io.BufferedReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import com.google.gson.Gson;

/**
 * Quick and dirty code to create a list of articles from
 * NYT search API results. The list will have one article 
 * entry per line. Each line contains the date, article link,
 * title and author, delimited by ":::".
 * 
 */
public class NYTIndexMaker {
  
  private static final String key = "ac62121689125b05259b6103e6fc3c44:16:59234110";
  
  private Calendar cal = GregorianCalendar.getInstance();
  
  private PrintWriter writer;
  private DateFormat format = new SimpleDateFormat("yyyyMMdd");
  
  public NYTIndexMaker(String file) throws IOException {
    writer = new PrintWriter(new FileWriter(file));
  }
  
  public void run() throws IOException, ParseException {
    Gson gson = new Gson();
    String query = URLEncoder.encode(" ", "UTF-8");

    String[] facets = new String[2];
    facets[0] = URLEncoder.encode("nytd_section_facet:[U.S.]", "UTF-8");
    facets[1] = URLEncoder.encode("nytd_section_facet:[World]", "UTF-8");
    facets[1] = URLEncoder.encode("nytd_section_facet:[Science]", "UTF-8");
    
    String dateRange = "begin_date=20060101&end_date=20070711";
    int offset = 0;
    int totalOffsets = 1;

    for (int i = 0; i < facets.length; i++) {
      String facet = facets[i];
      while (offset <= totalOffsets) {
	String urlString = String
	    .format(
		"http://api.nytimes.com/svc/search/v1/article?query=%s+%s&fields=date,title,url,byline&%s&offset=%d&api-key=%s",
		query, facet, dateRange, offset, key);
	System.out.println("Processing: " + urlString);

	URL url = new URL(urlString);
	HttpURLConnection connection = (HttpURLConnection) url.openConnection();
	BufferedReader reader = new BufferedReader(new InputStreamReader(connection
	    .getInputStream()));

	NYTSearchResult result = gson.fromJson(reader, NYTSearchResult.class);
	for (NYTEntry entry : result.getResults()) {
	  writeLine(entry);
	}

	if (offset == 0) {
	  // parse the total offsets
	  totalOffsets = (int) Math.floor(result.getTotal() / 10f);
	}
	offset++;
      }
      offset = 0;
    }

    writer.flush();
    writer.close();
  }
  
  private void writeLine(NYTEntry entry) throws ParseException {
    Date date = format.parse(entry.getDate());
    cal.setTime(date);
    cal.set(Calendar.HOUR_OF_DAY, 9);
    
    String sep = ":::";
    writer.print(cal.getTime().getTime());
    writer.print(sep);
    writer.print(entry.getUrl());
    writer.print(sep);
    String title = entry.getTitle().replace("’", "'");
    title = title.replace("‘", "'");
    writer.print(title);
    String author = entry.getByline() == null ? "" : entry.getByline();
    if (author.startsWith("By ")) author = author.substring(3);
    writer.print(sep);
    writer.print(author);
    writer.println();
  }
  
  public static void main(String[] args) throws IOException, ParseException {
    NYTIndexMaker maker = new NYTIndexMaker("./nytimes_archive_index.txt");
    maker.run();
  }
}
