package anl.aida.reader;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.text.ParseException;
import java.util.Date;

import anl.aida.util.AIDAException;

/**
 * Quick and dirty class to read nytimes_archive_index.txt
 * and read / parse each article in the index using the
 * NYTimesHTMLReader.
 * 
 */
public class NYTimesArchiveReader {
  
  private static final String DELIMITER = ":::";
  
  /**
   * Parses the lines in the nytimes archive index file for url, title, etc.
   * These are then passed to the NYTimesHTMLReader. The result of that
   * is then passed to the ReaderResultProcessor.
   * 
   * @param proc
   * @throws IOException
   * @throws ParseException
   * @throws AIDAException
   */
  public void run(ReaderResultProcessor proc) throws IOException, ParseException, AIDAException {
    NYTimesHTMLReader nyReader = new NYTimesHTMLReader();

    BufferedReader reader = null;
    try {
      reader = new BufferedReader(new FileReader("./corpus/indices/nytimes_archive_index.txt"));
      String line;
      while ((line = reader.readLine()) != null) {
	line = line.trim();
	
	int index = line.indexOf(DELIMITER);
	long time = Long.parseLong(line.substring(0, index));
	Date date = new Date(time);
	
	index += 3;
	int end = line.indexOf(DELIMITER, index);
	String url = line.substring(index, end);
	
	index = end + 3;
	end = line.indexOf(DELIMITER, index);
	String title = line.substring(index, end);
	
	String author = line.substring(end + 3);
	index = author.indexOf(";");
	if (index > -1) author = author.substring(0, index);
	
	ReaderResult result = nyReader.read(url, title, date, author);
	proc.process(result);
      }
    } finally {
      reader.close();
    }
  }
}

