package anl.aida.reader;

import anl.aida.types.Date;
import anl.aida.types.DocumentMetaData;
import anl.aida.types.Name;

import org.apache.uima.cas.CASException;
import org.apache.uima.jcas.JCas;
import org.apache.uima.jcas.cas.FSArray;

import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.events.EndElement;
import javax.xml.stream.events.StartElement;
import javax.xml.stream.events.XMLEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;

/**
 *         Date: Jan 20, 2009 3:43:32 PM
 */
public class PMXMLReader {

  private static final String TITLE = "ArticleTitle";
  private static final String AUTHOR = "Author";

  private static final String PUBDATE = "PubDate";
  private static final String MEDLINE_DATE = "MedlineDate";
  private static final String YEAR = "Year";
  private static final String MONTH = "Month";
  private static final String DAY = "Day";

  private static final String ABSTRACT = "AbstractText";
  private static final String FNAME = "FirstName";
  private static final String LNAME = "LastName";
  private static final String INITIALS = "Initials";

  /*
   <ArticleTitle>Remote-sensing infrared thermography.</ArticleTitle>
            <Pagination>
                <MedlinePgn>76-83</MedlinePgn>
            </Pagination>
            <Abstract>
                <AbstractText>The outbreak of the severe acute respiratory syndrome (SARS) in 2003 has ignited studies and research (and even the general public interest) in the field of infrared (IR) imaging systems for blindmass human fever screening to control the spread of the pandemic. The ideal device for blind mass fever screening should be speedy, noninvasive, and able to accurately detect peoplewith fever. IR thermography has been used to detect inflammatory abnormalities and has the potential to serve as a tool for mass screening of fever. This article reviews the IR fever-screening systems and suggests the performance and environmental requirements for characterizing thermography for possible fever screening, during the onset of a pandemic, under indoor controlled-environmental conditions.</AbstractText>
            </Abstract>
            <AuthorList>
                <Author>
                    <LastName>Ng</LastName>
                    <FirstName>Eddie</FirstName>
                    <Initials>E</Initials>
                </Author>
                <Author>
                    <LastName>Acharya</LastName>
                    <FirstName>Rajendra</FirstName>
                    <Initials>R</Initials>
                </Author>
            </AuthorList>

            <PubDate>
<Year>2001</Year>
<Month>Apr</Month>
<Day>15</Day>
</PubDate>

<PubDate>
<MedlineDate>2000 Spring</MedlineDate>
</PubDate>
   */

  private static class Author {

    String fname = "";
    String lname = "";
    String initials = "";

    Name build(JCas cas) {
      Name name = new Name(cas);
      name.setFirstName(fname);
      name.setLastName(lname);
      name.setInitials(initials);
      return name;
    }
  }

  private static class DateRep {
    int day = -1, month = -1, year = -1;
    static SimpleDateFormat fmt = new SimpleDateFormat("MMM");
    static Calendar cal = GregorianCalendar.getInstance();

    void month(String month) {
      try {
        cal.setTime(fmt.parse(month));
        this.month = cal.get(Calendar.MONTH) + 1;
      } catch (ParseException e) {
        e.printStackTrace();
      }

    }

    Date build(JCas cas) {
      Date date = new Date(cas);
      date.setDay(day);
      date.setMonth(month);
      date.setYear(year);

      return date;
    }

    // !! THIS IS JUST A HACK TO GET A DATE !!
    // !! THE MEDLINE FORMAT IS MORE COMPLICATED !!
    void parse(String medlineDate) {
      String year = medlineDate.substring(0, medlineDate.indexOf(" "));
      this.year = Integer.parseInt(year);
    }
  }

  private List<Author> authors = new ArrayList<Author>();
  private Author author = null;
  private DateRep date = null;


  public String run(DocumentMetaData docData, File file) throws XMLStreamException, FileNotFoundException,
          CASException {
    authors.clear();
    XMLInputFactory factory = XMLInputFactory.newInstance();
    XMLEventReader reader = factory.createXMLEventReader(new BufferedReader(new FileReader(file)));
    JCas jCas = docData.getCAS().getJCas();
    String absText = "";
    try {
      while (reader.hasNext()) {
        XMLEvent evt = reader.nextEvent();
        if (evt.isStartElement()) {
          StartElement elmt = evt.asStartElement();
          String qName = elmt.getName().getLocalPart();

          if (qName.equals(TITLE)) {
            docData.setTitle(reader.getElementText());
          } else if (qName.equals(AUTHOR)) {
            author = new Author();
          } else if (qName.equals(ABSTRACT)) {
            absText = reader.getElementText();
          } else if (qName.equals(FNAME)) {
            author.fname = reader.getElementText();
          } else if (qName.equals(LNAME)) {
            author.lname = reader.getElementText();
          } else if (qName.equals(INITIALS)) {
            author.initials = reader.getElementText();
          } else if (qName.equals(PUBDATE)) {
            date = new DateRep();
          } else if (qName.equals(YEAR) && date != null) {
            date.year = Integer.parseInt(reader.getElementText());
          } else if (qName.equals(MONTH) && date != null) {
            date.month(reader.getElementText());
          } else if (qName.equals(DAY) && date != null) {
            date.day = Integer.parseInt(reader.getElementText());
          } else if (qName.equals(MEDLINE_DATE) && date != null) {
            date.parse(reader.getElementText());
          }

        } else if (evt.isEndElement()) {
          EndElement elmt = evt.asEndElement();
          String qName = elmt.getName().getLocalPart();
          if (qName.equals(AUTHOR)) {
            authors.add(author);
          } else if (qName.equals(PUBDATE)) {
            docData.setDate(date.build(jCas));
            date = null;
          }
        }
      }
    } finally {
      reader.close();
    }




    FSArray fsArray = new FSArray(jCas, authors.size());
    for (int i = 0, n = authors.size(); i < n; i++) {
      fsArray.set(i, authors.get(i).build(jCas));
    }
    docData.setAuthors(fsArray);

    return absText;
  }
}
