package anl.aida.reader;

import java.io.IOException;

import org.htmlparser.Tag;
import org.htmlparser.beans.StringBean;

public class PreTextExtractor {
  
  @SuppressWarnings("serial")
  public static class PreBean extends StringBean {
    
    private boolean on = false;

    @Override
    public void visitEndTag(Tag tag) {
      super.visitEndTag(tag);
      if (!mIsPre && on) {
	mBuffer.append("\nPRE_OFF\n");
	on = false;
      }
    }

    @Override
    public void visitTag(Tag tag) {
      super.visitTag(tag);
      if (mIsPre && !on) {
	mBuffer.append("\nPRE_ON\n");
	on = true;
      }
    }
    
    public void reset() {
      on = false;
      mIsPre = false;
    }
  }
  
  public static class DocProc implements DocumentProcessor {
    
    private StringBuilder buf = new StringBuilder();
    boolean preOn = false;
    boolean done = false;
    public void processLine(String line) {
	if (line.startsWith("PRE_OFF") && preOn) {
	  preOn = false;
	  done = true;
	}
	
	if (preOn) {
	  line.trim();
	  buf.append(line);
	  buf.append("\n");
	}
	if (line.startsWith("PRE_ON") && !preOn)  preOn = true;
    }

    @Override
    public boolean done() {
	return done;
    }

    @Override
    public void reset() {
      done = false;
      preOn = false;
      buf = new StringBuilder();
    }
  };
  
  private DocProc docProc = new DocProc();
  private StringExtractor extractor;
  private PreBean bean = new PreBean();
  
  public PreTextExtractor() {
    extractor = new StringExtractor(bean);
  }
  
  public String extract(String url) throws IOException {
    docProc.reset();
    bean.reset();
    extractor.run(url, docProc);
    return docProc.buf.toString();
  }
}
