package anl.aida.reader;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;

/**
 * Makes a standard index file from PRobo 
 */
public class ProMedIndexMaker {

  private static class ArchiveMaker implements DocumentProcessor {

    // timestamp:::link:::title:::author

    private PrintWriter writer;
    String prefix = "http://www.promedmail.com/pls/otn/";
    DateFormat format = new SimpleDateFormat("dd-MMM-yy");
    long timestamp;

    public ArchiveMaker(String file) throws IOException {
      writer = new PrintWriter(new BufferedWriter(new FileWriter(file)));
    }

    public void processLine(String line) {
      line = line.trim();
      //System.out.println("line = " + line);
      if (line.contains("-MAR-") || line.contains("-APR-") || line.contains("-MAY-")) {
	// its date so set the timestamp
	try {
	  timestamp = format.parse(line).getTime();
	} catch (ParseException e) {
	  e.printStackTrace();
	}
      } else if (line.startsWith("<file")) {
	int end = line.indexOf(">");
	String link = prefix + line.substring(line.indexOf("f?p"), end);
	String title = line.substring(end + 1);
	//System.out.printf("%d:::%s:::%s%n",timestamp, link.trim(), title.trim());
	writer.printf("%d:::%s:::%s:::%n",timestamp, link.trim(), title.trim());
      }
    }

    @Override
    public boolean done() {
      return false;
    }

    public void close() throws IOException {
      writer.close();
    }

    /*
     * (non-Javadoc)
     * 
     * @see anl.aida.reader.DocumentProcessor#reset()
     */
    @Override
    public void reset() {
      // TODO Auto-generated method stub

    }
  }

  public static void main(String[] args) throws IOException {
    StringExtractor extractor = new StringExtractor();
    ArchiveMaker maker = new ArchiveMaker("./promed_archive_index.txt");
    String url ="file:///Users/nick/Documents/workspace/fusion/Core/scripts/promed_index_source.txt";
    extractor.run(url, maker);
    maker.close();
  }

}
