/**
 * 
 */
package anl.aida.reader;

import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;

import org.apache.uima.collection.CollectionException;
import org.apache.uima.resource.ConfigurableResource;
import org.apache.uima.resource.ResourceInitializationException;

import anl.aida.util.StandardIndexLineParser;

/**
 * AIDAComponentReader implementation for reading archived ProMed files.
 * Archived means that files are saved in some directory as individual files
 * rather than some email format. The names of the files should be
 * ProMedMail_*.txt where * is some number. This will process all the files in
 * the directory.
 * 
 */
public class ProMedReader extends AbstractAIDAComponentReader {

  // private static final String INPUT_DIR = "promed_input_directory";
  public static final String INDEX_FILE = "anl.aida.reader.ProMedIndex";

  private ProMedTxtReader reader = new ProMedTxtReader();
  private ProMedPreProcessor proc;
  private Reader in;
  private PreTextExtractor extractor = new PreTextExtractor();
  private Set<String> badTitleWords = new HashSet<String>();

  private Date date;
  
  public ProMedReader() {
    badTitleWords.add("calendar");
    badTitleWords.add("frequently asked questions");
  }

  /*
   * (non-Javadoc)
   * 
   * @see anl.aida.reader.AIDAComponentReader#close()
   */
  @Override
  public void close() throws IOException {
    if (proc != null)
      proc.close();
  }

  /*
   * (non-Javadoc)
   * 
   * @see anl.aida.reader.AbstractAIDAComponentReader#getNextResult()
   */
  @Override
  protected ReaderResult getNextResult() throws IOException {
    ReaderResult result = reader.run(location, in);
    checkDate(date);
    result.setDate(new ParsedDate(date));
    return result;
  }

  /*
   * (non-Javadoc)
   * 
   * @see anl.aida.reader.AbstractAIDAComponentReader#postNext()
   */
  @Override
  protected void postNext() throws IOException {
    next();
  }

  /*
   * (non-Javadoc)
   * 
   * @see anl.aida.reader.AbstractAIDAComponentReader#getIndexFileKey()
   */
  @Override
  protected String getIndexFileKey() {
    return INDEX_FILE;
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * anl.aida.reader.AIDAComponentReader#initialize(org.apache.uima.resource
   * .ConfigurableResource)
   */
  @Override
  public void initialize(ConfigurableResource resource, Date cacheStartDate) throws ResourceInitializationException {
    super.initialize(resource, cacheStartDate);
    /*
     * File input = new File((String)
     * resource.getConfigParameterValue(INPUT_DIR)); files = input.listFiles(new
     * FilenameFilter() { public boolean accept(File file, String name) { return
     * name.endsWith(".txt") && name.startsWith("ProMedMail"); } });
     */

    try {
      next();
    } catch (IOException ex) {
      throw new ResourceInitializationException(ex);
    }
  }

  private void next() throws IOException {
    if (proc == null) {
      incrementFile();
    }

    if (proc != null) in = proc.next();
    if (in == null && indexIter.hasNext()) {
      proc.close();
      incrementFile();
      if (proc != null) in = proc.next();
    }
  }
  
  private boolean isUsefulContent(String title) {
    title = title.toLowerCase();
    for (String badWords : badTitleWords) {
      if (title.contains(badWords)) return false;
    }
    return true;
    
  }

  private void incrementFile() throws IOException {
    // some mail we want to skip -- like calendar announcements
    boolean find = true;
    while (indexIter.hasNext() && find) {
      lineItems = indexIter.nextParsedLine(lineItems);
      location = lineItems[StandardIndexLineParser.URL];
      date = new Date(Long.parseLong(lineItems[StandardIndexLineParser.TIMESTAMP]));
      // some mail we want to skip -- like calendar announcements
      find = !isUsefulContent(lineItems[StandardIndexLineParser.TITLE]);
    }
    if (!find) {
      //System.out.println("location = " + location);
      proc = new ProMedPreProcessor(new StringReader(extractor.extract(location)));
    }
  }

  public boolean hasNext() throws IOException, CollectionException {
    return in != null;
  }

  @Override
  protected String getDocumentURL() {
    return location;
  }
}
