package anl.aida.reader;

import anl.aida.types.DocumentMetaData;

import org.apache.uima.cas.CAS;
import org.apache.uima.cas.CASException;
import org.apache.uima.collection.CollectionException;
import org.apache.uima.collection.CollectionReader_ImplBase;
import org.apache.uima.jcas.JCas;
import org.apache.uima.resource.ResourceInitializationException;
import org.apache.uima.util.Progress;
import org.apache.uima.util.ProgressImpl;

import java.io.File;
import java.io.IOException;
import java.io.FilenameFilter;

/**
 * CollectionReaders for the pub med abstracts in xml format.
 *
 */
public class PubMedReader extends CollectionReader_ImplBase {

  private static final String INPUT_DIR = "input_directory";

  private File[] files;
  private int index = 0;
  private PMXMLReader reader = new PMXMLReader();


  @Override
  public void initialize() throws ResourceInitializationException {
    File input = new File((String) getConfigParameterValue(INPUT_DIR));
    files = input.listFiles(new FilenameFilter() {
      public boolean accept(File file, String name) {
        return name.endsWith(".xml");
      }
    });
  }

  public void getNext(CAS cas) throws IOException, CollectionException {
    File file = files[index];
    JCas jcas;
    try {
      jcas = cas.getJCas();
    } catch (CASException e) {
      throw new CollectionException(e);
    }

    DocumentMetaData data = new DocumentMetaData(jcas, 0, (int) file.length());
    data.setLocation(file.toURI().toURL().toExternalForm());
    
    try {
      String content = reader.run(data, file);
      jcas.setDocumentText(content);
    } catch (Exception ex) {
      throw new CollectionException(ex);
    }

    data.addToIndexes();
    index++;
  }

  
  public boolean hasNext() throws IOException, CollectionException {
    return index < files.length;
  }

  
  public Progress[] getProgress() {
	  int size = files == null ? 0 : files.length;
    return new Progress[] {new ProgressImpl(index, size, Progress.ENTITIES)};
  }

 
  public void close() throws IOException {}
}
