package anl.aida.reader;

import java.io.IOException;

import anl.aida.util.AIDAException;

/**
 * Processes a {@link ReaderResult} is someone implementor
 * specific way. For example, an implemenation might add the
 * results to a lucene index, or add them to a JCas etc. 
 * 
 */
public interface ReaderResultProcessor {
  
  /**
   * Initializes the processor.
   * 
   * @throws AIDAException
   */
  void initialize() throws AIDAException;
  
  /**
   * Process the specified result.
   * 
   * @param result the result to process
   * @throws AIDAException if there is an error processing the result
   */
  void process(ReaderResult result) throws AIDAException;
  
  /**
   * Notifies the processor that it is done processing results.
   * 
   * @throws AIDAException
   */
  void done() throws IOException;

}
