/**
 * 
 */
package anl.aida.reader;

import static anl.aida.reader.ReaderConstants.INDEX_LINE_DELIMETER;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import anl.aida.util.FileUtilities;
import anl.aida.util.AIDAException;

/**
 * Result processor that will cache the contents of the
 * the result in a file and create an index entry for that.
 * The first line of the cached contents will be the original
 * location. 
 * 
 */
public class ResultCacher implements ReaderResultProcessor {

  private PrintWriter indexOut;
  private ZipOutputStream zipOut;
  private PrintWriter contentOut;
  private String zipName;
  private int entryIndex = 0;

  public ResultCacher(String cacheDir, String indexName) throws IOException {
    File dir = new File(cacheDir);
    if (dir.exists()) {
      FileUtilities.delete(dir);
    }
    dir.mkdirs();
    
    File file = new File(cacheDir, indexName);
    zipName = indexName;
    int index =  zipName.indexOf(".");
    if (index != -1) {
      zipName = zipName.substring(0, index) + ".zip";
    }
    
    zipOut = new ZipOutputStream(new FileOutputStream(new File(cacheDir, zipName)));
    contentOut = new PrintWriter(new BufferedWriter(new OutputStreamWriter(zipOut)));
    indexOut = new PrintWriter(new BufferedWriter(new FileWriter(file)));
  }

  @Override
  public void done() throws IOException {
    indexOut.close();
    contentOut.close();
  }

  @Override
  public void initialize() throws AIDAException {
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * anl.aida.reader.ReaderResultProcessor#process(anl.aida.reader.ReaderResult)
   */
  @Override
  public void process(ReaderResult result) throws AIDAException {
    try {
      zipOut.putNextEntry(new ZipEntry(String.valueOf(entryIndex)));
      String url = "./" + zipName + "!" + entryIndex;
      //PrintWriter out = new PrintWriter(new BufferedWriter(new OutputStreamWriter(zipOut)));
      contentOut.println(result.getLocation());
      contentOut.print(result.getContents());
      contentOut.flush();
      entryIndex++;

      long time = result.getDate().getJavaDate().getTime();
      indexOut.printf("%d%s%s%s%s%s%n", time, INDEX_LINE_DELIMETER, url, INDEX_LINE_DELIMETER,
	  result.getTitle(), INDEX_LINE_DELIMETER, result.getAuthors().getAuthors());
    } catch (IOException ex) {
      throw new AIDAException(ex);
    }
  }
}
