package anl.aida.reader.local;

import java.io.File;
import java.io.IOException;
import java.util.Date;

import org.apache.uima.resource.ConfigurableResource;
import org.apache.uima.resource.ResourceInitializationException;

import anl.aida.reader.AbstractAIDAComponentReader;
import anl.aida.reader.CachedContentReader;
import anl.aida.reader.ContentReader;
import anl.aida.reader.ReaderResult;
import anl.aida.util.StandardIndexLineParser;

public class MuscatineJournalReader extends AbstractAIDAComponentReader{
	  public static final String INDEX_FILE = "anl.aida.reader.local.MuscatineJournalIndex";
	  
	  private ContentReader ltReader = new MuscatineJournalHTMLReader(ReaderConstantsMiner.MUSCATINEJOURNAL_SOURCE, "showComments=",
			  "/articles/");
	  
	  public MuscatineJournalReader() {
	    // uncomment to cache the content
	    /*
	    try {
	      super.processors.add(new ResultCacher("./corpus/ct", "ct_cache_index.txt"));
	    } catch (IOException e) {
	      UIMAFramework.getLogger(getClass()).log(Level.SEVERE, "Unable to create result cacher", e);
	    }
	    */  
	  }

	  /* (non-Javadoc)
	   * @see anl.aida.reader.AbstractAIDAComponentReader#getNextResult()
	   */
	  @Override
	  protected ReaderResult getNextResult() throws IOException {
	    lineItems = indexIter.nextParsedLine(lineItems);
	    location = lineItems[StandardIndexLineParser.URL];
	    Date date = new Date(Long.parseLong(lineItems[StandardIndexLineParser.TIMESTAMP]));
	    checkDate(date);
	    
	    return ltReader.read(location,  lineItems[StandardIndexLineParser.TITLE], date, "");
	  }
	  
	  @Override
	  protected String getIndexFileKey() {
	    return INDEX_FILE;
	  }

	  @Override
	  public void initialize(ConfigurableResource resource, Date startDate) throws ResourceInitializationException {
	    super.initialize(resource, startDate);
	    String[] line = indexIter.nextParsedLine(new String[4]);
	    indexIter.rewind();
	    String url = line[StandardIndexLineParser.URL];
	    if (!url.startsWith("http://")) {
	      String indexFile = (String)resource.getConfigParameterValue(INDEX_FILE);
	      File file = new File(indexFile);
	      ltReader = new CachedContentReader(ReaderConstantsMiner.MUSCATINEJOURNAL_SOURCE, file.getParent());
	    }
	  }
}
