package anl.aida.reader.local;

import java.io.IOException;
import java.util.Date;

import anl.aida.reader.ContentReader;
import anl.aida.reader.DocumentProcessor;
import anl.aida.reader.ParsedAuthors;
import anl.aida.reader.ParsedDate;
import anl.aida.reader.ReaderResult;
import anl.aida.reader.StringExtractor;

public class StandardHTMLReader implements ContentReader{
	 protected String source;
	 protected String parse;
	 protected String stop;

	// appends as content lines between
	public StandardHTMLReader(String source,String stop,String parse){
		this.source=source;
		this.stop=stop;
		this.parse=parse;
	}
	 
	  private class Processor implements DocumentProcessor {

	    private static final int WAITING = 0;
	    private static final int COLLECTING = 1;
	    private static final int STOPPED = 2;
	    
	    private int status = WAITING;

	    public void processLine(String line) {
	      // System.out.println("line = " + line);
//	    	if(line.contains("Heineman signed the bill")){
//	    		System.out.println("stop");
//	    	}
	    	if (line.contains(stop)) { 
	    		  status = STOPPED;
	    	  } 
	    	  else if(!line.startsWith("<http://")&& !line.startsWith("<https://")){
	    		  content.append(line);
	    		  content.append(" ");
	         }
	      

	      if (status == WAITING)
	    	  status = COLLECTING;
	    }
	    
	    @Override
	    public boolean done() {
	      return status == STOPPED;
	    }
	    
	    public void reset() {
	      status = WAITING;
	    }
	  }

	  private StringExtractor extractor = new StringExtractor();
	  private DocumentProcessor processor = new Processor();
	  private StringBuilder content;
	  private ReaderResult result;

	  /**
	   * Reads the text from the specified URL. The title and date are expected to
	   * be provided from another source, e.g. an RSS feed entry or an archive
	   * "index."
	   * 
	   * @param url
	   *          the url to read from
	   * @param title
	   *          the article title
	   * @param date
	   *          the date of the article
	   * @return the result of reading and parsing the text at the url
	   * @throws IOException
	   *           if there is an error while reading
	   */
	  public ReaderResult read(String url, String title, Date date, String author) throws IOException {
	    result = new ReaderResult();
	    content = new StringBuilder();

	    result.setTitle(title);
	    result.setDate(new ParsedDate(date));
	    result.setLocation(url);
	    result.setSource(source);
	    result.setLink("");

	    ParsedAuthors authors = new ParsedAuthors();
	    authors.addName("", author, "");
	    result.setAuthors(authors);

	    processor.reset();
	    extractor.run(url, processor);
	    result.setContents(content.toString());

	    return result;
	  }
}
