package anl.aida.score;

import anl.aida.core.ScoredTermsCollection;

import java.util.HashMap;
import java.util.Map;

/**
 * Abstract base class for scorers that use an STC. This
 * does some caching of the term indices to improve speed.
 *
 */
public class AbstractSTCScorer {

  protected ScoredTermsCollection stc;
  private Map<String, Integer> termIndexCache = new HashMap<String, Integer>();

  public AbstractSTCScorer(ScoredTermsCollection stc) {
    this.stc = stc;
  }

  /**
   * Gets the stc term index for the specified term.
   *
   * @param term the term whose index we want to get
   * @return the stc term index for the specified term.
   */
  protected int getTermIndex(String term) {
    Integer val = termIndexCache.get(term);
    if (val == null) {
      val = stc.getTermIndex(term);
      termIndexCache.put(term, val);
    }

    return val;
  }
}
