package anl.aida.score;

import anl.aida.core.DocumentID;
import anl.aida.core.ScoredTermsCollection;

/**
 * TermScoreFilter implemenation that passes a document if the term
 * occurs in the document.
 *
 */
public class OccurrenceFilter implements TermScoreFilter {

  private TFScorer scorer;

  public OccurrenceFilter(ScoredTermsCollection stc) {
    scorer = new TFScorer(stc);
  }

  /**
   * Evaluates the term and document to see if the term occurs
   * in the document.
   *
   * @param term the term to evaluate against
   * @param id   the id of the document to evaluate
   * @return true if the document passes the filter w/r to the specified term, otherwise false.
   */
  public boolean evaluate(String term, DocumentID id) {
    return scorer.getScore(term, id) > 0; 
  }
}
