package anl.aida.score;

import anl.aida.core.DocumentID;
import anl.aida.sv.SemanticVectorStore;
import anl.aida.util.MIFSRuntimeException;
import anl.aida.util.VectorUtilities;

import java.io.IOException;

/**
 * Scorer that returns the semantic vector similarity score for a term
 * w/r to a document.
 */
public class SVScorer implements Scorer {

  private SemanticVectorStore vectorStore;

  public SVScorer(SemanticVectorStore vectorStore) {
    this.vectorStore = vectorStore;
  }

  /**
   * Gets the term frequency for the specified term w/r to the specified document.
   *
   * @param term the term whose score we want
   * @param id   the id of the document
   * @return the term frequency for the specified term w/r to
   *         the specified document.
   */
  public float getScore(String term, DocumentID id) throws MIFSRuntimeException {
    try {
      float[] termVec = vectorStore.getTermVector(term);
      // term is not found which may be expected if the term doesn't exist in this
      // particular slice of the cache.
      if (termVec == null) return 0;
      
      float[] docVec = vectorStore.getDocumentVector(id.getSTCIndex());
      return VectorUtilities.scalarProduct(termVec, docVec);
    } catch (IOException ex) {
      throw new MIFSRuntimeException("Error getting document vector", ex);
    }
  }
}