package anl.aida.score;

import anl.aida.core.DocumentID;

/**
 * TermScoreFilter that passes documents whose term score
 * is greater than or equal to some threshold.
 *
 */
public class ScorerThresholdFilter implements TermScoreFilter {

  private Scorer scorer;
  private float threshold;

  public ScorerThresholdFilter(Scorer scorer, float threshold) {
    this.scorer = scorer;
    this.threshold = threshold;
  }

  /**
   * Evaluates the term and document.
   *
   * @param term the term to evaluate against
   * @param id   the id of the document to evaluate
   * @return true if the document passes the filter w/r to the specified term, otherwise false.
   */
  public boolean evaluate(String term, DocumentID id) {
    float score =  scorer.getScore(term, id);
    return score >= threshold;
  }
}
