package anl.aida.score;

import anl.aida.core.DocumentID;
import anl.aida.core.ScoredTermsCollection;

/**
 * Scorer that returns the tf/idf score for a term in a
 * document.
 *
 */
public class TFIDFScorer extends AbstractSTCScorer implements Scorer {

  public TFIDFScorer(ScoredTermsCollection stc) {
    super(stc);
  }

  /**
   * Gets the tf/idf score for the specified term w/r to the specified document.
   *
   * @param term the term whose score we want
   * @param id   the id of the document
   * @return the tf/idf score for the specified term w/r to
   *         the specified document.
   */
  public float getScore(String term, DocumentID id) {
    int termIndex = getTermIndex(term);
    if (termIndex == -1) return 0;
    return stc.getScore(id.getSTCIndex(), termIndex);
  }
}
