package anl.aida.sv;

import anl.aida.core.DocumentID;

import org.apache.lucene.store.IndexOutput;
import org.apache.lucene.store.MMapDirectory;
import pitt.search.semanticvectors.ObjectVector;

import java.io.IOException;
import java.util.Enumeration;

/**
 * VectorStoreWriter that stores that assumes the Vector's object is a DocumentID.
 *
 */
public class DocumentVectorStoreWriter {


  /**
   * @param vectorFile    The name of the file to write to
   * @param objectVectors The vector store to be written to disk
   * @throws IOException if there is an error writing the vectors
   */
  public void writeVectors(String vectorFile, DocumentVectors objectVectors) throws IOException {
    IndexOutput outputStream = null;
    try {
      Enumeration<ObjectVector> vecEnum = objectVectors.getAllVectors();
      MMapDirectory dir = new MMapDirectory();
      outputStream = dir.createOutput(vectorFile);

      /* Write header giving number of dimensions for all vectors. */
      outputStream.writeString("-dimensions");
      outputStream.writeInt(ObjectVector.vecLength);

      float[] tmpVector;
      /* Write each vector. */
      while (vecEnum.hasMoreElements()) {
        ObjectVector objectVector = vecEnum.nextElement();
        DocumentID id = (DocumentID) objectVector.getObject();
        outputStream.writeInt(id.getSTCIndex());
        outputStream.writeString(id.getPath());
        tmpVector = objectVector.getVector();
        for (int i = 0; i < ObjectVector.vecLength; ++i) {
          outputStream.writeInt(Float.floatToIntBits(tmpVector[i]));
        }
      }
    } finally {

      if (outputStream != null) outputStream.close();

    }
  }
}
