package anl.aida.sv;

import anl.aida.core.DocumentID;
import anl.aida.core.ScoredTermsCollection;
import anl.aida.util.IntIterator;
import anl.aida.util.VectorUtilities;
import pitt.search.semanticvectors.ObjectVector;
import pitt.search.semanticvectors.VectorStore;
import pitt.search.semanticvectors.VectorStoreRAM;

import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

/**
 *         Date: Feb 4, 2009 4:04:36 PM
 */
public class DocumentVectors implements VectorStore {

  private VectorStoreRAM docVectors;

  public DocumentVectors(ScoredTermsCollection stc, TermVectorsFromSTC termVectors) {
    Map<String, float[]> vectors = new HashMap<String, float[]>();
    docVectors = new VectorStoreRAM();
    for (int i = 0; i < stc.getDocumentCount(); i++) {
      float[] docVector = new float[ObjectVector.vecLength];
      String path = stc.getDocument(i).getPath();
      vectors.put(path, docVector);
    }

    Enumeration<ObjectVector> termEnum = termVectors.getAllVectors();
    while (termEnum.hasMoreElements()) {
      ObjectVector termVector = termEnum.nextElement();
      float[] vector = termVector.getVector();
      String word = termVector.getObject().toString();

      int termId = termVectors.getIDFor(word);

      for (IntIterator iter = stc.getDocumentsFor(termId); iter.hasNext();) {
        int id = iter.next();
        String path = stc.getDocument(id).getPath();
        float[] docVector = vectors.get(path);
        float score = stc.getScore(id, termId);
        for (int j = 0; j < ObjectVector.vecLength; ++j) {
          docVector[j] += score * vector[j];
        }
      }
    }

    for (int i = 0; i < stc.getDocumentCount(); i++) {
      String path = stc.getDocument(i).getPath();
      float[] docVector = vectors.get(path);
      docVector = VectorUtilities.normalizeVector(docVector);
      docVectors.putVector(new DocumentID(i, path), docVector);
    }
    vectors.clear();
  }

  public float[] getVector(Object id) {
    return this.docVectors.getVector(id);
  }

  public Enumeration<ObjectVector> getAllVectors() {
    return this.docVectors.getAllVectors();
  }

  public int getNumVectors() {
    return this.docVectors.getNumVectors();
  }
}
