package anl.aida.sv;

import java.io.IOException;

import pitt.search.semanticvectors.VectorStoreWriter;
import anl.aida.core.MIFSTerm;
import anl.aida.core.ScoredTermsCollection;
import anl.aida.util.Filter;

/**
 * Creates a semantic vectors index, consisting of the terms and doc files.
 *
 */
public class SVIndexBuilder {

  private int seedLength;
  private Filter<MIFSTerm> filter;

  /**
   * Creates an SVIndexBuilder with the specified seedlength and minFreq.
   *
   * @param seedLength the seed length used to build the vectors.
   * @param minFreq    the minimum number of documents a word must appear in order to
   *                   be included in the index.
   */
  public SVIndexBuilder(int seedLength, final int minFreq) {
    this(seedLength, new Filter<MIFSTerm>() {

      @Override
      public boolean evaluate(MIFSTerm item) {
	return item.getDocumentCount() >= minFreq;
      }
    
    });
  }
  
  /**
   * Creates an SVIndexBuilder with the specified seedlength and minFreq.
   *
   * @param seedLength the seed length used to build the vectors.
   * @param minFreq    the minimum number of documents a word must appear in order to
   *                   be included in the index.
   */
  public SVIndexBuilder(int seedLength, Filter<MIFSTerm> filter) {
    this.seedLength = seedLength;
    this.filter = filter;
  }

  public void buildIndex(String termVecFile, String docVecFile, ScoredTermsCollection stc) throws IOException {
    TermVectorsFromSTC termVectors = new TermVectorsFromSTC(seedLength, filter, stc);
    DocumentVectors docVectors = new DocumentVectors(stc, termVectors);

    VectorStoreWriter writer = new VectorStoreWriter();
    writer.WriteVectors(termVecFile, termVectors);
    DocumentVectorStoreWriter docWriter = new DocumentVectorStoreWriter();
    docWriter.writeVectors(docVecFile, docVectors);
  }
}
