package anl.aida.sv;

import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

import pitt.search.semanticvectors.ObjectVector;
import pitt.search.semanticvectors.VectorStore;
import pitt.search.semanticvectors.VectorStoreSparseRAM;
import anl.aida.core.MIFSTerm;
import anl.aida.core.ScoredTermsCollection;
import anl.aida.util.Filter;
import anl.aida.util.IntIterator;
import anl.aida.util.VectorUtilities;

/**
 * Creates Semantic Vector style term and doc vectors from
 * an STC data store.
 *
 */
public class TermVectorsFromSTC implements VectorStore {

  private Hashtable<String, ObjectVector> termVectors;
  private VectorStoreSparseRAM basicDocVectors;
  private Map<String, Integer> textToId = new HashMap<String, Integer>();


  /**
   * @param seedLength
   * @param minFreq    the minimum number of documents a term must appear
   *                   in to be added to the TermVectors.
   * @param stc
   */
  public TermVectorsFromSTC(int seedLength, final int minFreq, ScoredTermsCollection stc) {
    this(seedLength, new Filter<MIFSTerm>() {
      @Override
      public boolean evaluate(MIFSTerm item) {
	return item.getDocumentCount() >= minFreq;
      }
    }, stc);
  }
  
  /**
   * @param seedLength
   * @param minFreq    terms that pass this filter are added to the semantic vector
   * term store
   * @param stc
   */
  public TermVectorsFromSTC(int seedLength, Filter<MIFSTerm> filter, ScoredTermsCollection stc) {

    basicDocVectors = new VectorStoreSparseRAM();
    basicDocVectors.CreateRandomVectors(stc.getDocumentCount(), seedLength);

    termVectors = new Hashtable<String, ObjectVector>();
    for (int i = 0, n = stc.getTermCount(); i < n; i++) {
      MIFSTerm term = stc.getTerm(i);
      if (filter.evaluate(term)) {
        textToId.put(term.getText(), i);
        float[] termVector = new float[ObjectVector.vecLength];
        for (IntIterator iter = stc.getDocumentsFor(i); iter.hasNext();) {
          int j = iter.next();
          String docID = String.valueOf(j);
          float[] docVector = this.basicDocVectors.getVector(docID);
          float score = stc.getScore(j, i);
          
          //if (score > 0) System.out.printf("\t%s%n",stc.getDocument(j).getTitle());

          for (int v = 0; v < ObjectVector.vecLength; v++) {
            termVector[v] += score * docVector[v];
          }
        }

        termVector = VectorUtilities.normalizeVector(termVector);
        termVectors.put(term.getText(), new ObjectVector(term.getText(), termVector));
      }
    }
  }

  public Integer getIDFor(String term) {
    return textToId.get(term);
  }

  public float[] getVector(Object term) {
    return termVectors.get(term).getVector();
  }

  public Enumeration getAllVectors() {
    return termVectors.elements();
  }

  public int getNumVectors() {
    return termVectors.size();
  }
}
