package anl.aida.types;

import java.util.GregorianCalendar;
import java.util.Calendar;

/**
 * Utility methods for working with anl.aida.types.Dates.
 * This assumes Date is 1-based -- that is, January is month 1
 * rather than java.util.Date where January is month 0.
 *
 * @author Nick Collier
 */
public class DateUtils {

  private static final Calendar cal = GregorianCalendar.getInstance();

  /**
   * Creates a Java Date from an anl.aida.types.Date.
   *
   * @param date the Date type
   * @return the created Date
   */
  public java.util.Date toJDate(Date date) {
    cal.set(date.getYear(), date.getMonth() - 1, date.getDay());
    return cal.getTime();
  }

  /**
   * Gets the number of milliseconds since January 1, 1970, 00:00:00 GMT
   * represented by the specified Date object.
   *
   * @param date the Date whose time we want
   * @return the number of milliseconds since January 1, 1970, 00:00:00 GMT
   * represented by the specified Date object.
   */
  public long toTime(Date date) {
    return toJDate(date).getTime();
  }


}
