package anl.aida.util;

import java.util.ArrayList;
import java.util.List;

/**
 * An aggregate Filter that returns true if each of its
 * members Filters returns true.
 *
 */
public class AndFilter<T> implements Filter<T> {

  private List<Filter<T>> filters = new ArrayList<Filter<T>>();

  public AndFilter(Filter<T>... filters) {
    for (Filter<T> filter : filters) {
      this.filters.add(filter);
    }
  }

  /**
   * Evaluates the specified item against all the filters in this filter
   * and returns true if they all pass, otherwise false.
   *
   * @param item the item to evaluate
   * @return true if the item passes all the filters, otherwise false.
   */
  public boolean evaluate(T item) {
    for (Filter<T> filter : filters) {
      if (!filter.evaluate(item)) return false;
    }
    return true;
  }
}
