package anl.aida.util;

import java.util.Date;

/**
 * Encapsulates a date range.
 * 
 */
public class DateRange {

  private Date start, end;
  private int hashCode;

  public DateRange(DateRange range) {
    this(range.start.getTime(), range.end.getTime());
  }

  public DateRange(long start, long end) {
    this.start = new Date(start);
    this.end = new Date(end);

    hashCode = 17;
    hashCode = 31 * hashCode + this.start.hashCode();
    hashCode = 31 * hashCode + this.end.hashCode();
  }

  public DateRange(Date start, Date end) {
    this(start.getTime(), end.getTime());
  }

  public Date getStart() {
    return start;
  }

  public Date getEnd() {
    return end;
  }

  public DateRange overlap(DateRange other) {
    long start = this.start.getTime();
    long end = this.end.getTime();
    long oStart = other.start.getTime();
    long oEnd = other.end.getTime();
    
    if (start >= oStart && end <= oEnd) {
      // this is within other
      return new DateRange(start, end);
    }

    if (oStart >= start && oEnd <= end) {
      // other is within this
      return new DateRange(other.start, other.end);
    }

    if (start >= oStart && start <= oEnd && end > oEnd) {
      // overlaps between this start and other end
      return new DateRange(start, oEnd);
    }

    if (start <= oStart && end >= oStart && end < oEnd) {
      // overlaps between other.start and this.end
      return new DateRange(oStart, end);
    }

    return null;
  }

  public boolean equals(Object obj) {
    if (obj instanceof DateRange) {
      DateRange other = (DateRange) obj;
      return other.start.equals(start) && other.end.equals(end);
    }

    return false;
  }

  public int hashCode() {
    return hashCode;
  }

  public String toString() {
    return "DateRange[" + start + " - " + end + "]";
  }
}
