package anl.aida.util;

import java.util.*;

/**
 * Produces DateRanges over some interval.
 *
 */
public class DateRangeProducer {

  Calendar cal = GregorianCalendar.getInstance();

  private int rangeLength, totalDays;
  private Date start, end;

  /**
   * If totalDays is less than timeSliceInDays then no ranges will be returned.
   *
   * @param start             the starting date
   * @param totalDays         the total number of days over which to produce ranges
   * @param rangeLengthInDays the individual range length
   */
  public DateRangeProducer(Date start, int totalDays, int rangeLengthInDays) {
    this.start = start;
    this.totalDays = totalDays;
    this.rangeLength = rangeLengthInDays;

    cal.setTime(start);
    cal.add(Calendar.DAY_OF_MONTH, totalDays);
    end = cal.getTime();
    cal.setTime(start);
  }

  public Iterable<DateRange> ranges() {
    return new RangeIter();
  }

  public Date getStart() {
    return start;
  }

  public Date getEnd() {
    return end;
  }

  /**
   * Gets the individual range length.
   * @return
   */
  public int getRangeLength() {
    return rangeLength;
  }

  public int getTotalDays() {
    return totalDays;
  }

  private class RangeIter implements Iterator<DateRange>, Iterable<DateRange> {


    private DateRange next;
    private int index = 0;

    public RangeIter() {
      next = computeNext();
    }

    private DateRange computeNext() {
      DateRange range = null;
      cal.setTime(start);
      cal.add(Calendar.DAY_OF_MONTH, index);
      Date rStart = cal.getTime();

      int end = index + rangeLength;
      if (end <= totalDays) {
        cal.add(Calendar.DAY_OF_MONTH, rangeLength);
        Date rEnd = cal.getTime();
        range = new DateRange(rStart, rEnd);
      }

      index++;
      return range;
    }

    public boolean hasNext() {
      return next != null;
    }

    public DateRange next() {
      if (next == null) {
        throw new NoSuchElementException();
      }
      DateRange tmp = next;
      next = computeNext();
      return tmp;
    }

    public void remove() {
      throw new UnsupportedOperationException();
    }

    public Iterator<DateRange> iterator() {
      return this;
    }
  }
}
