package anl.aida.util;

/**
 * A simple pair class that represents a tuple with two elements.<p/>
 * <p/>
 * This class is not implemented to handle null pair values, however it will
 * accept them.
 *
 */
public class Pair<X, Y> {

  private X first;
  private Y second;
  private int hashCode;

  /**
   * Constructs this class with the specified first and second values
   *
   * @param first  the first value
   * @param second the second value
   */
  public Pair(X first, Y second) {
    this.first = first;
    this.second = second;
    hashCode = 17;
    hashCode += 31 * hashCode + first.hashCode();
    hashCode += 31 * hashCode + second.hashCode();

  }

  /**
   * Compares this pair to another object.
   *
   * @return if the other object is a pair with the equal first and second
   *         members
   */
  public boolean equals(Object obj) {
    if (obj == this)
      return true;
    if (obj instanceof Pair) {
      Pair objPair = (Pair) obj;
      return (objPair.first.equals(first) && objPair.second
              .equals(second));
    }

    return false;
  }


  public int hashCode() {
    return hashCode;
  }
  
  public String toString() {
    return "[" + first.toString() + ", " + second.toString() + "]";
  }

  /**
   * Retrieves the first element in the tuple (first, second).
   *
   * @return the first element
   */
  public X getFirst() {
    return first;
  }

  /**
   * Retrieves the second element in the tuple (first, second).
   *
   * @return the second element
   */
  public Y getSecond() {
    return second;
  }
}
