package anl.aida.util;

import java.util.ArrayList;
import java.util.List;

import org.apache.uima.collection.metadata.CasProcessorConfigurationParameterSettings;
import org.apache.uima.collection.metadata.NameValuePair;

/**
 * A Settings class that can be used to override the default settings
 *  in a UIMA processor.
 *  
 */
public class UIMAParameterSettings implements CasProcessorConfigurationParameterSettings {
  

  private static class NameValuePairImpl implements NameValuePair {

    private String name;
    private Object value;

    /**
     * @param name
     * @param value
     */
    public NameValuePairImpl(String name, Object value) {
      this.name = name;
      this.value = value;
    }

    @Override
    public String getName() {
      return name;
    }

    @Override
    public Object getValue() {
      return value;
    }

    @Override
    public void setName(String name) {
      this.name = name;

    }

    @Override
    public void setValue(Object value) {
      this.value = value;

    }
  }
  
  private List<NameValuePair> pairs = new ArrayList<NameValuePair>();
  
  /**
   * Adds the name value pair to this UIMAParameterSettings.
   * 
   * @param name the pair name
   * @param value the pair value
   */
  public void addPair(String name, Object value) {
    pairs.add(new NameValuePairImpl(name, value));
  }

  @Override
  public NameValuePair[] getParameterSettings() {
    NameValuePair[] array = new NameValuePair[pairs.size()];
    pairs.toArray(array);
    return array;
  }

  @Override
  public Object getParameterValue(String name) {
    for (NameValuePair pair : pairs) {
	if (pair.getName().equals(name)) return pair.getValue();
    }
    return null;
  }

  @Override
  public void setParameterValue(String name, Object value) {
    for (NameValuePair pair : pairs) {
	if (pair.getName().equals(name)) {
	  pair.setValue(value);
	  break;
	}
    }
  }

}
