package anl.aida.util;

/**
 *         Date: Feb 13, 2009 11:33:50 AM
 */
public class VectorUtilities {

  /**
   * Returns the scalar product (dot product) of two vectors
   * for normalized vectors this is the same as cosine similarity.
   *
   * @param vec1 First vector.
   * @param vec2 Second vector.
   * @return the product
   */
  public static float scalarProduct(float[] vec1, float[] vec2) {
    float result = 0;
    for (int i = 0; i < vec1.length; ++i) {
      result += vec1[i] * vec2[i];
    }
    return result;
  }

  /**
   * Returns the scalar product (dot product) of two vectors
   * for normalized vectors this is the same as cosine similarity.
   *
   * @param vec1 First vector.
   * @param vec2 Second vector.
   * @return the product
   */
  public static float scalarProduct(float[] vec1, double[] vec2) {
    float result = 0;
    for (int i = 0; i < vec1.length; ++i) {
      result += vec1[i] * vec2[i];
    }
    return result;
  }

  public static float[] doubleToFloat(double[] vec) {
    float[] vals = new float[vec.length];
    for (int i = 0; i < vals.length; i++) {
      vals[i] = (float) vec[i];
    }

    return vals;
  }

  public static double[] floatToDouble(float[] vec) {
    double[] vals = new double[vec.length];
    for (int i = 0; i < vals.length; i++) {
      vals[i] = vec[i];
    }

    return vals;
  }

  /**
   * Returns the normalized version of a vector, i.e. same direction,
   * unit length. This does NOT effect the input vector.
   *
   * @param vec Vector whose normalized version is requested.
   * @return the normalized vector
   */
  public static float[] normalizeVector(float[] vec) {
    float norm = 0;
    float[] tmpVec = new float[vec.length];
    System.arraycopy(vec, 0, tmpVec, 0, vec.length);

    for (int i = 0; i < tmpVec.length; ++i) {
      norm += tmpVec[i] * tmpVec[i];
    }
    norm = (float) Math.sqrt(norm);
    for (int i = 0; i < tmpVec.length; ++i) {
      tmpVec[i] = tmpVec[i] / norm;
    }
    return tmpVec;
  }
}
