package anl.fusion.example;

import gate.*;
import gate.creole.*;
import org.apache.log4j.BasicConfigurator;

import java.io.File;
import java.net.MalformedURLException;

/**
 *         Date: Jan 8, 2009 3:19:25 PM
 */
public class GateExample {

  static {
    BasicConfigurator.configure();
  }


  public GateExample() {
    File gateHome = new File("./gate_app");
    Gate.setGateHome(gateHome);
    Gate.setUserConfigFile(new File(gateHome, "user-gate.xml"));
    Gate.setUserSessionFile(new File(gateHome, ".gate.session"));
    try {
      Gate.init();
      Gate.getCreoleRegister().registerDirectories(new File(gateHome, "/plugins/ANNIE").toURI().toURL()); 

    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  private SerialAnalyserController createPipeline(String[] pResources) throws ResourceInstantiationException,
          MalformedURLException {
    Corpus corpus = Factory.newCorpus("Temp Corpus");
    corpus.add(Factory.newDocument(new File("./corpus/alice/alice.txt").toURI().toURL()));
    SerialAnalyserController pipeline =
            (SerialAnalyserController) Factory.createResource("gate.creole.SerialAnalyserController");
    for (String res : pResources) {
      ProcessingResource resource = (ProcessingResource) Factory.createResource(res);
      pipeline.add(resource);
    }
    pipeline.setCorpus(corpus);

    return pipeline;
  }

  private void printResults(Document doc, String annotationType) {

    FeatureMap map = doc.getFeatures();
    System.out.println("Doc Features:");
    for (Object key : map.keySet()) {
      System.out.println("\tFeature key:" + key);
      System.out.println("\tFeature Value: " + map.get(key));
    }

    System.out.println();
    AnnotationSet anns = annotationType == null ? doc.getAnnotations() : doc.getAnnotations().get(annotationType);
    for (Annotation ann : anns) {
      System.out.print(ann);
    }
  }

  public void tagGazetteerTypes() throws MalformedURLException, ResourceInstantiationException, ExecutionException {
    String[] resources = {"gate.creole.tokeniser.DefaultTokeniser",
            "gate.creole.gazetteer.DefaultGazetteer"};

    SerialAnalyserController pipeline = createPipeline(resources);
    pipeline.execute();

    printResults((Document) pipeline.getCorpus().get(0), ANNIEConstants.LOOKUP_ANNOTATION_TYPE);

  }

  public void tagSentence() throws ResourceInstantiationException, MalformedURLException, ExecutionException {
    String[] resources = {"gate.creole.tokeniser.DefaultTokeniser",
            "gate.creole.gazetteer.DefaultGazetteer",
            "gate.creole.splitter.SentenceSplitter"};

    SerialAnalyserController pipeline = createPipeline(resources);
    pipeline.execute();

    printResults((Document) pipeline.getCorpus().get(0), ANNIEConstants.SENTENCE_ANNOTATION_TYPE);
  }

  public void tagPOS() throws ResourceInstantiationException, MalformedURLException, ExecutionException {
    String[] resources = {"gate.creole.tokeniser.DefaultTokeniser",
            /*"gate.creole.gazetteer.DefaultGazetteer",
            "gate.creole.splitter.SentenceSplitter",
            "gate.creole.POSTagger"*/};

    SerialAnalyserController pipeline = createPipeline(resources);
    // this is a clunky way of getting the POSTagger
    // we want to set the output annotation type to POS, so
    // we can easily print the POS related annotations. Without
    // setting the output annotation type to POS, the type will
    // default to Token which will also contain non-POS info.
    // The category feature will contain the part of speech.
    for (Object obj : pipeline.getPRs()) {
      if (obj instanceof POSTagger) {
        ((POSTagger)obj).setOutputAnnotationType("POS");
      }
    }
    pipeline.execute();
    printResults((Document) pipeline.getCorpus().get(0), "Token");
  }

  public static void main(String[] args) {
    GateExample exp = new GateExample();
    try {
      //exp.tagSentence();
      //exp.tagGazetteerTypes();
      exp.tagPOS();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
}
