package anl.fusion.example;

import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.TermFreqVector;

import java.io.File;
import java.io.IOException;

/**
 *
 */
public class LuceneExample {

  private static final String EXAMPLES_DIR = "./corpus/alice/";
  private static final String INDEX_DIR = "./indices/alice";


  public void index() throws IOException {
    IndexWriter writer = null;
    try {
      writer = new IndexWriter(INDEX_DIR, new StandardAnalyzer(), IndexWriter.MaxFieldLength.UNLIMITED);
      File dir = new File(EXAMPLES_DIR);

      for (File file : dir.listFiles()) {
        System.out.println("Adding document: " + file.getName());
        writer.addDocument(FileDocument.document(file));
      }

      writer.optimize();
    } finally {
      if (writer != null) writer.close();
    }
  }

  public void readIndex() throws IOException {
    IndexReader reader = null;
    try {
      reader = IndexReader.open(INDEX_DIR);
      for (int i = 0; i < reader.numDocs(); i++) {
        System.out.println(reader.document(i).get("path"));
        TermFreqVector vec = reader.getTermFreqVector(i, "contents");
        for (int j = 0; j < vec.size(); j++) {
          String term = vec.getTerms()[j];
          int count = vec.getTermFrequencies()[j];
          System.out.printf("\t%s: %d%n", term, count);
        }
      }
    } finally {
      if (reader != null) reader.close();
    }

  }


  public static void main(String[] args) {
    LuceneExample exp = new LuceneExample();
    try {
      File file = new File(INDEX_DIR);
      if (!file.exists()) file.mkdirs();
      if (file.listFiles() == null || file.listFiles().length == 0) exp.index();
      exp.readIndex();
    } catch (IOException ex) {
      ex.printStackTrace();
    }
  }
}
