package anl.fusion.example;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.*;
import org.apache.lucene.search.DefaultSimilarity;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Similarity;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * 
 */
public class LuceneFedExample {

  private static final String EXAMPLES_DIR = "./corpus/federalist/";
  private static final String INDEX_DIR = "./indices/federalist/";

  private class Mapper extends TermVectorMapper {

    private final String[] functionWords = {"work", "probability", "particularly",
            "matter", "kind", "vigor",
            "language", "innovation", "direction", "apt", "according", "considerable",
            "consequently", "commonly", "though", "always", "whilst", "while", "enough",
            "both", "although", "to", "this", "there", "on", "of", "by", "an", "also", "upon"};

    private Map<String, Integer> counts = new HashMap<String, Integer>();

    private Mapper() {
      super(true, true);
      for (String word : functionWords) {
        counts.put(word, 0);
      }
    }

    @Override
    public void setExpectations(String s, int i, boolean b, boolean b1) {
    }

    @Override
    public void map(String token, int count, TermVectorOffsetInfo[] termVectorOffsetInfos, int[] ints) {
      if (counts.containsKey(token)) {
        counts.put(token, count);
      }
    }

    public String toString() {
      return counts.toString();
    }
  }

  public void index() throws IOException {
    IndexWriter writer = null;
    Analyzer analyzer = new StandardAnalyzer();
    //Analyzer analyzer = new SnowballAnalyzer("English", StandardAnalyzer.STOP_WORDS);
    try {
      writer = new IndexWriter(INDEX_DIR, analyzer, IndexWriter.MaxFieldLength.UNLIMITED);
      File dir = new File(EXAMPLES_DIR);
      FederalistDocCreator creator = new FederalistDocCreator();

      for (Document doc : creator.createDocuments(dir)) {
        System.out.println("Adding document: " + doc.getField("path"));
        writer.addDocument(doc);
      }
      writer.optimize();

    } finally {
      if (writer != null) writer.close();
    }
  }

  public void readIndex() throws IOException {
    IndexReader reader = null;
    try {

      reader = IndexReader.open(INDEX_DIR);
      IndexSearcher searcher = new IndexSearcher(reader);

      int[] docs = new int[32];         // buffered doc numbers
      int[] freqs = new int[32];        // buffered term freqs

      DefaultSimilarity similarity = new DefaultSimilarity();
      // cache the idf for frequencies up to 32
      float[] scoreCache = new float[32];
      for (int i = 0; i < 32; i++) {
        scoreCache[i] = similarity.tf(i);
      }

      TermEnum terms = reader.terms();
      while (terms.next()) {
        Term term = terms.term();
        byte[] norms = reader.norms(term.field());
        float idf = similarity.idf(term, searcher);
        TermDocs termDocs = reader.termDocs(term);
        int max = termDocs.read(docs, freqs);
        while (max != 0) {
          for (int index = 0; index < max; index++) {
            int doc = docs[index];
            int freq = freqs[index];
            float tf = freq < 32 ? scoreCache[freq] : similarity.tf(freq);
            float fieldNorm = norms == null ? 0f : Similarity.decodeNorm(norms[doc]);
            float score = tf * idf * fieldNorm;

            System.out.printf("%s %s %d %f%n", term.text(), term.field(), doc, score);
          }
          max = termDocs.read(docs, freqs);
        }
        termDocs.close();
      }
      terms.close();


    } finally {
      if (reader != null) reader.close();
    }

  }

  public static void main(String[] args) {
    LuceneFedExample exp = new LuceneFedExample();
    try {

      File file = new File(INDEX_DIR);
      if (!file.exists()) file.mkdirs();
      if (file.listFiles() == null || file.listFiles().length == 0) exp.index();
      exp.readIndex();
    } catch (IOException ex) {
      ex.printStackTrace();
    }
  }
}