package anl.fusion.example;

import anl.aida.core.KeywordGenerator;
import anl.aida.core.STCReader;
import anl.aida.core.ScoredTermsCollection;
import anl.aida.sv.SVKeywordGenerator;
import anl.aida.sv.SemanticVectorStore;
import anl.aida.util.EnumIter;
import anl.aida.util.HighestTermScores;
import anl.aida.util.AIDAException;
import anl.aida.util.TermScore;
import pitt.search.semanticvectors.*;

import java.io.IOException;
import java.util.*;

/**
 *         Date: Feb 4, 2009 11:45:02 AM
 */
public class SVExample {

  public void keywords() throws IOException, AIDAException, ZeroVectorException {
    STCReader reader = new STCReader();
    ScoredTermsCollection stc = reader.readCollection("./Core/test_data/cluster_test_stc.bin");
    KeywordGenerator generator = new SVKeywordGenerator(new SemanticVectorStore("./docs.bin", "./terms.bin"));
    for (int i = 0; i < 10; i++) {
      System.out.println(stc.getDocument(i).getPath());
      HighestTermScores results = generator.generate(stc.getDocument(i).getPath(), 4);
      for (TermScore result : results.termScores()) {
        System.out.printf("\t%s:%f%n", result.getTerm(), result.getScore());
      }
    }
  }

  public void docs() throws IOException, ZeroVectorException {
    List<String> docs = new ArrayList<String>();

    VectorStoreReader queryReader = new VectorStoreReader("./docs.bin");
    EnumIter<ObjectVector> iter = new EnumIter<ObjectVector>(queryReader.getAllVectors());
    for (ObjectVector vec : iter) {
      docs.add(vec.getObject().toString());
    }

    VectorStoreReader searchReader = new VectorStoreReader("./terms.bin");
    Map<String, Integer> counts = new HashMap<String, Integer>();

    HighestTermScores highestScores = new HighestTermScores(40);
    for (String doc : docs) {
      VectorSearcher vecSearcher = new VectorSearcher.VectorSearcherCosine(queryReader, searchReader, null,
              new String[]{doc});


      LinkedList results = vecSearcher.getNearestNeighbors(100);
      for (int i = 0, n = results.size(); i < n; i++) {
        SearchResult result = (SearchResult) results.get(i);
        String term = ((ObjectVector) result.getObject()).getObject().toString();
        Integer val = counts.get(term);
        if (val == null) val = 0;
        counts.put(term, new Integer(val.intValue() + 1));
        highestScores.add(term, result.getScore());

      }
    }

    HighestTermScores highestCounts = new HighestTermScores(40);

    System.out.println("\n\n");
    System.out.println("counts.size() = " + counts.size());
    for (Map.Entry<String, Integer> entry : counts.entrySet()) {
      //System.out.printf("%s: %d%n", entry.getKey(), entry.getValue());
      highestCounts.add(entry.getKey(), entry.getValue());
    }

    System.out.println("Hightest Doc Frequency");
    for (TermScore score : highestCounts.termScores()) {
      System.out.println("\t" + score);
    }

    System.out.println("Hightest Scores");
    for (TermScore score : highestScores.termScores()) {
      System.out.println("\t" + score);
    }
  }

  public static void main(String[] args) throws IOException, ZeroVectorException, AIDAException {
    SVExample exp = new SVExample();
    exp.keywords();
  }
}
