package anl.gtou;

import gate.FeatureMap;

import java.io.IOException;
import java.util.Map;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.FeatureStructure;
import org.apache.uima.cas.TypeSystem;

import anl.aida.ae.gate.FeatureBuilder;
import anl.aida.util.AIDAUtils;

public class AgentConverter implements FeatureBuilder {

  private Feature kind;
  private Feature subKind;
  private Feature annotationString;
  private Feature canonicalName;
  private Map<String, String> canonicalNames;

  public AgentConverter(TypeSystem system) throws IOException {
    kind = system.getFeatureByFullName("anl.aida.types.Agent:kind");
    subKind = system.getFeatureByFullName("anl.aida.types.Agent:subKind");
    annotationString = system.getFeatureByFullName("anl.aida.types.Agent:annotationString");
    canonicalName = system.getFeatureByFullName("anl.aida.types.Agent:canonicalName");

    canonicalNames = AIDAUtils.loadMap("./data/CanonicalBioAgent_allLower.txt", "=");
  }

  // FeatureStructure is UIMA : FeatureMap is Gate
  @Override
  public void build(FeatureStructure parent, FeatureMap map) {
    String kindGate = (String) map.get("kind");
    parent.setStringValue(kind, kindGate);

    String subKindGate = (String) map.get("sub_kind");
    parent.setStringValue(subKind, subKindGate);

    String matchedStringGate = (String) map.get("annotationString");
    if (matchedStringGate != null) {
      parent.setStringValue(annotationString, matchedStringGate);
      String canonicalNameFromMap = (String) canonicalNames.get(matchedStringGate.toLowerCase());
      parent.setStringValue(canonicalName, canonicalNameFromMap);
    }
  }
}
