package anl.aida;

import anl.aida.ae.gate.*;
import gate.*;
import gate.creole.ResourceInstantiationException;
import gate.util.GateException;
import gate.util.InvalidOffsetException;
import org.apache.uima.UIMAFramework;
import org.apache.uima.analysis_engine.AnalysisEngine;
import org.apache.uima.analysis_engine.AnalysisEngineProcessException;
import org.apache.uima.cas.*;
import org.apache.uima.cas.text.AnnotationFS;
import org.apache.uima.cas.text.AnnotationIndex;
import org.apache.uima.resource.ResourceInitializationException;
import org.apache.uima.resource.ResourceSpecifier;
import org.apache.uima.util.InvalidXMLException;
import org.apache.uima.util.XMLInputSource;
import org.junit.After;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.xml.sax.SAXException;

import javax.xml.parsers.ParserConfigurationException;
import java.io.File;
import java.io.IOException;
import java.util.*;


/**
 * 
 */
public class GateAnnotatorTests {

  private static String EXAMPLE_TEXT = "There was nothing so VERY remarkable in that; nor did Alice think it so " +
          "VERY much out of the way to hear the Rabbit say to itself, 'Oh dear!" +
          "Oh dear! I shall be late!' (when she thought it over afterwards, it " +
          "occurred to her that she ought to have wondered at this, but at the time " +
          "it all seemed quite natural); but when the Rabbit actually TOOK A WATCH " +
          "OUT OF ITS WAISTCOAT-POCKET, and looked at it, and then hurried on, " +
          "Alice started to her feet, for it flashed across her mind that she had " +
          "never before seen a rabbit with either a waistcoat-pocket, or a watch " +
          "to take out of it, and burning with curiosity, she ran across the field " +
          "after it, and fortunately was just in time to see it pop down a large " +
          "rabbit-hole under the hedge.";

  private CAS cas;

  @BeforeClass
  public static void init() {
    File gateHome = new File("/Applications/GATE-5.0-beta1");
    Gate.setGateHome(gateHome);
    Gate.setPluginsHome(new File(gateHome, "plugins"));
    Gate.setUserConfigFile(new File(gateHome, "gate.xml"));
    try {
      Gate.init();

    } catch (GateException e) {
      e.printStackTrace();
    }
  }

  @Before
  public void setUp() throws IOException, InvalidXMLException, ResourceInitializationException,
          AnalysisEngineProcessException {
    XMLInputSource in = new XMLInputSource("./desc/julie_token_ae.xml");
    ResourceSpecifier spec = UIMAFramework.getXMLParser().parseResourceSpecifier(in);
    AnalysisEngine ae = UIMAFramework.produceAnalysisEngine(spec);
    cas = ae.newCAS();
    cas.setDocumentText(EXAMPLE_TEXT);
    ae.process(cas);
  }

  @After
  public void tearDown() {
    cas.reset();
  }

  @Test
  public void testOutput() throws ResourceInstantiationException, UGMappingException, InvalidOffsetException {
    TypeSystem system = cas.getTypeSystem();
    Type type = system.getType("de.julielab.jules.types.Annotation");
    OutputAnnotationBuilder builder = new OutputAnnotationBuilder("DT", type, false);

    Feature feature = system.getFeatureByFullName("de.julielab.jules.types.Annotation:confidence");
    builder.addFeatureBuilder(new OutputFeatureBuilder("confidence", feature, FeatureType.STRING));

    Feature f = system.getFeatureByFullName("de.julielab.jules.types.Token:posTag");
    System.out.println("f = " + f);

    Document doc = Factory.newDocument(cas.getDocumentText());
    AnnotationSet set = doc.getAnnotations();

    FeatureMap featureMap = Factory.newFeatureMap();
    featureMap.put("confidence", "good");
    set.add(10L, 12L, "DT", featureMap);

    featureMap = Factory.newFeatureMap();
    featureMap.put("confidence", "bad");
    set.add(20L, 40L, "DT", featureMap);

    featureMap = Factory.newFeatureMap();
    featureMap.put("confidence", "middling");
    set.add(50L, 60L, "DT", featureMap);

    Map<Integer, FeatureStructure> map = new HashMap<Integer, FeatureStructure>();
    int casCount = cas.getAnnotationIndex().size();
    builder.run(cas, doc, map);
    assertEquals(casCount + 3, cas.getAnnotationIndex().size());


    AnnotationIndex index = cas.getAnnotationIndex(type);
    boolean a = false, b = false, c = false;
    for (Iterator iter = index.iterator(); iter.hasNext();) {
      AnnotationFS fs = (AnnotationFS) iter.next();
      if (fs.getBegin() == 10 && fs.getEnd() == 12) {
        a = true;
        assertEquals("good", fs.getStringValue(feature));
      } else if (fs.getBegin() == 20 && fs.getEnd() == 40) {
        b = true;
        assertEquals("bad", fs.getStringValue(feature));
      } else if (fs.getBegin() == 50 && fs.getEnd() == 60) {
        c = true;
        assertEquals("middling", fs.getStringValue(feature));
      }
    }
    assertTrue(a);
    assertTrue(b);
    assertTrue(c);
  }

  @Test
  public void outputXMLTest() throws ResourceInitializationException, InvalidOffsetException,
          ResourceInstantiationException, UGMappingException, IOException, ParserConfigurationException, SAXException {
    BuilderFactory fac = new BuilderFactory("./test/anl/mifs/mapping_test.xml");
    TypeSystem system = cas.getTypeSystem();

    Type type = system.getType("de.julielab.jules.types.Annotation");
    Feature feature = system.getFeatureByFullName("de.julielab.jules.types.Annotation:confidence");

    List<InputAnnotationBuilder> inputs = new ArrayList<InputAnnotationBuilder>();
    List<OutputAnnotationBuilder> outputs = new ArrayList<OutputAnnotationBuilder>();
    fac.build(system, inputs, outputs);
    OutputAnnotationBuilder builder = outputs.get(0);

    Document doc = Factory.newDocument(cas.getDocumentText());
    AnnotationSet set = doc.getAnnotations();

    FeatureMap featureMap = Factory.newFeatureMap();
    featureMap.put("confidence", "good");
    set.add(10L, 12L, "DT", featureMap);

    featureMap = Factory.newFeatureMap();
    featureMap.put("confidence", "bad");
    set.add(20L, 40L, "DT", featureMap);

    featureMap = Factory.newFeatureMap();
    featureMap.put("confidence", "middling");
    set.add(50L, 60L, "DT", featureMap);

    Map<Integer, FeatureStructure> map = new HashMap<Integer, FeatureStructure>();
    int casCount = cas.getAnnotationIndex().size();

    builder.run(cas, doc, map);
    assertEquals(casCount + 3, cas.getAnnotationIndex().size());

    AnnotationIndex index = cas.getAnnotationIndex(type);
    boolean a = false, b = false, c = false;
    for (Iterator iter = index.iterator(); iter.hasNext();) {
      AnnotationFS fs = (AnnotationFS) iter.next();
      if (fs.getBegin() == 10 && fs.getEnd() == 12) {
        a = true;
        assertEquals("good", fs.getStringValue(feature));
      } else if (fs.getBegin() == 20 && fs.getEnd() == 40) {
        b = true;
        assertEquals("bad", fs.getStringValue(feature));
      } else if (fs.getBegin() == 50 && fs.getEnd() == 60) {
        c = true;
        assertEquals("middling", fs.getStringValue(feature));
      }
    }
    assertTrue(a);
    assertTrue(b);
    assertTrue(c);

  }

  @Test
  public void builderXMLTest() throws ResourceInitializationException, ResourceInstantiationException,
          UGMappingException, IOException, ParserConfigurationException, SAXException {
    BuilderFactory fac = new BuilderFactory("./test/anl/mifs/mapping_test.xml");
    TypeSystem system = cas.getTypeSystem();
    List<InputAnnotationBuilder> inputs = new ArrayList<InputAnnotationBuilder>();
    List<OutputAnnotationBuilder> outputs = new ArrayList<OutputAnnotationBuilder>();
    fac.build(system, inputs, outputs);
    InputAnnotationBuilder builder = inputs.get(0);

    Type type = system.getType("de.julielab.jules.types.GeniaPOSTag");
    Feature feature = system.getFeatureByFullName("de.julielab.jules.types.GeniaPOSTag:value");

    Document doc = Factory.newDocument(cas.getDocumentText());
    Map<Integer, FeatureStructure> map = new HashMap<Integer, FeatureStructure>();
    builder.run(cas, doc, map);
    assertEquals(cas.getAnnotationIndex(type).size(), map.size());

    AnnotationSet annots = doc.getAnnotations();
    for (Integer val : map.keySet()) {
      Annotation gateA = annots.get(val);
      AnnotationFS uimaA = (AnnotationFS) map.get(val);
      assertEquals("POS", gateA.getType());
      assertEquals((long) uimaA.getBegin(), (long)gateA.getStartNode().getOffset());
      assertEquals((long) uimaA.getEnd(), (long)gateA.getEndNode().getOffset());

      FeatureMap gateMap = gateA.getFeatures();
      String featureVal = uimaA.getStringValue(feature);
      assertEquals(featureVal, gateMap.get("value"));
    }
  }

  @Test
  public void builderTest() throws ResourceInstantiationException, UGMappingException {
    TypeSystem system = cas.getTypeSystem();
    Type type = system.getType("de.julielab.jules.types.GeniaPOSTag");
    InputAnnotationBuilder builder = new InputAnnotationBuilder("POS", type, true);
    Feature feature = system.getFeatureByFullName("de.julielab.jules.types.GeniaPOSTag:value");
    builder.addFeatureBuilder(new InputFeatureBuilder("value", feature, FeatureType.STRING));

    Map<Integer, FeatureStructure> map = new HashMap<Integer, FeatureStructure>();
    Document doc = Factory.newDocument(cas.getDocumentText());
    builder.run(cas, doc, map);
    assertEquals(cas.getAnnotationIndex(type).size(), map.size());

    AnnotationSet annots = doc.getAnnotations();
    for (Integer val : map.keySet()) {
      Annotation gateA = annots.get(val);
      AnnotationFS uimaA = (AnnotationFS) map.get(val);
      assertEquals("POS", gateA.getType());
      assertEquals((long) uimaA.getBegin(), (long)gateA.getStartNode().getOffset());
      assertEquals((long) uimaA.getEnd(), (long)gateA.getEndNode().getOffset());

      FeatureMap gateMap = gateA.getFeatures();
      String featureVal = uimaA.getStringValue(feature);
      assertEquals(featureVal, gateMap.get("value"));
    }
  }

  @Test
  public void builderOutUpdateTest() throws ResourceInstantiationException, UGMappingException {
    TypeSystem system = cas.getTypeSystem();
    Type type = system.getType("de.julielab.jules.types.GeniaPOSTag");
    InputAnnotationBuilder builder = new InputAnnotationBuilder("POS", type, true);
    Feature feature = system.getFeatureByFullName("de.julielab.jules.types.GeniaPOSTag:value");
    builder.addFeatureBuilder(new InputFeatureBuilder("value", feature, FeatureType.STRING));

    Map<Integer, FeatureStructure> map = new HashMap<Integer, FeatureStructure>();
    Document doc = Factory.newDocument(cas.getDocumentText());
    builder.run(cas, doc, map);
    assertEquals(cas.getAnnotationIndex(type).size(), map.size());

    // updates the feature map in the GATE annotation
    // that update should then be carried over into the
    // UIMA annotation
    AnnotationSet annots = doc.getAnnotations();
    for (Integer val : map.keySet()) {
      Annotation gateA = annots.get(val);
      FeatureMap gateMap = gateA.getFeatures();
      String fVal = (String) gateMap.get("value");
      gateMap.put("value", fVal + val);
    }

    OutputAnnotationBuilder oBuilder = new OutputAnnotationBuilder("POS", type, true);
    oBuilder.addFeatureBuilder(new OutputFeatureBuilder("value", feature, FeatureType.STRING));
    oBuilder.run(cas, doc, map);

    for (Integer val : map.keySet()) {
      Annotation gateA = annots.get(val);
      AnnotationFS uimaA = (AnnotationFS) map.get(val);
      assertEquals("POS", gateA.getType());
      assertEquals((long) uimaA.getBegin(), (long)gateA.getStartNode().getOffset());
      assertEquals((long) uimaA.getEnd(), (long)gateA.getEndNode().getOffset());

      FeatureMap gateMap = gateA.getFeatures();
      String featureVal = uimaA.getStringValue(feature);
      assertEquals(featureVal, gateMap.get("value"));
    }
  }

  @Test
  public void builderOutUpdateXMLTest() throws ResourceInitializationException, ResourceInstantiationException,
          UGMappingException, IOException, ParserConfigurationException, SAXException {
    BuilderFactory fac = new BuilderFactory("./test/anl/mifs/mapping_test.xml");
    TypeSystem system = cas.getTypeSystem();
    List<InputAnnotationBuilder> inputs = new ArrayList<InputAnnotationBuilder>();
    List<OutputAnnotationBuilder> outputs = new ArrayList<OutputAnnotationBuilder>();
    fac.build(system, inputs, outputs);
    InputAnnotationBuilder builder = inputs.get(0);

    Type type = system.getType("de.julielab.jules.types.GeniaPOSTag");
    Feature feature = system.getFeatureByFullName("de.julielab.jules.types.GeniaPOSTag:value");

    Document doc = Factory.newDocument(cas.getDocumentText());
    Map<Integer, FeatureStructure> map = new HashMap<Integer, FeatureStructure>();
    builder.run(cas, doc, map);
    assertEquals(cas.getAnnotationIndex(type).size(), map.size());

    // updates the feature map in the GATE annotation
    // that update should then be carried over into the
    // UIMA annotation
    AnnotationSet annots = doc.getAnnotations();
    for (Integer val : map.keySet()) {
      Annotation gateA = annots.get(val);
      FeatureMap gateMap = gateA.getFeatures();
      String fVal = (String) gateMap.get("value");
      gateMap.put("value", fVal + val);
    }

    outputs.get(0).run(cas, doc, map);
    outputs.get(1).run(cas, doc, map);

    for (Integer val : map.keySet()) {
      Annotation gateA = annots.get(val);
      AnnotationFS uimaA = (AnnotationFS) map.get(val);
      assertEquals("POS", gateA.getType());
      assertEquals((long) uimaA.getBegin(), (long)gateA.getStartNode().getOffset());
      assertEquals((long) uimaA.getEnd(), (long)gateA.getEndNode().getOffset());

      FeatureMap gateMap = gateA.getFeatures();
      String featureVal = uimaA.getStringValue(feature);
      assertEquals(gateMap.get("value"), featureVal);
    }
  }
}
