package anl.aida;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.lucene.index.IndexReader;
import org.junit.Test;

import anl.aida.ae.lucene.FieldFilter;
import anl.aida.ae.lucene.STCCreator;
import anl.aida.ae.lucene.WordFilter;
import anl.aida.core.MIFSDocument;
import anl.aida.core.MIFSTerm;
import anl.aida.core.STCReader;
import anl.aida.core.STCWriter;
import anl.aida.core.ScoredDocumentCallback;
import anl.aida.core.ScoredTermCallback;
import anl.aida.core.ScoredTermsCollection;
import anl.aida.sv.SVIndexBuilder;
import anl.aida.sv.SVKeywordGenerator;
import anl.aida.sv.SemanticVectorStore;
import anl.aida.util.DateRange;
import anl.aida.util.HighestTermScores;
import anl.aida.util.AIDAException;
import anl.aida.util.TermScore;

/**
 * 
 */
public class TermsTest {

  // dummy date range
  private DateRange range = new DateRange(new Date(), new Date());

  // Terms in the content section of the PUB_5 index.
  private static final String[] PUB5_TERMS = { "0", "0.001", "0.01", "0.034", "0.25", "0.32",
      "0.39", "0.51", "0.82", "1.6", "16", "16.2", "18", "1993", "20.6", "2003", "2005", "2006",
      "2007", "2008", "3", "3.7", "4", "45", "493", "7", "9", "9.8", "able", "abnormalities",
      "about", "abstract", "abundant", "accomplished", "accumulating", "accurately", "acute",
      "adaptation", "adequate", "after", "aid", "along", "also", "although", "among", "amphibian",
      "another", "anticipated", "applicable", "appraisal", "approach", "article", "asia",
      "associated", "autumn", "autumns", "availability", "background", "bank", "based",
      "batrachochytrium", "bd", "because", "become", "been", "before", "being", "belgium", "best",
      "beyond", "blind", "blindmass", "borne", "both", "breeding", "broad", "c", "calcarata",
      "called", "case", "causal", "cause", "ceratophrys", "chacophrys", "changes",
      "characteristics", "characterizing", "chytrid", "chytridiomycosis", "climate", "coexistence",
      "colder", "collected", "commonly", "conclusion", "conditions", "connection", "considered",
      "consisted", "continuous", "control", "controlled", "coping", "cornuta", "correlations",
      "counseling", "countries", "cranwelli", "culture", "cyclic", "data", "decade", "deciduous",
      "decreased", "demonstrating", "dendrobatidis", "describe", "designed", "detect", "detected",
      "developing", "development", "device", "died", "discovered", "disease", "due", "during", "e",
      "each", "ecological", "effective", "effects", "effort", "emerging", "encountered", "endemic",
      "enhance", "enhancing", "environmental", "epidemic", "epidemica", "even", "evidence",
      "evolution", "examined", "exist", "exotic", "expected", "explaining", "facilitating",
      "failure", "farms", "fecal", "fever", "field", "first", "folkman", "following", "food",
      "forecasting", "formation", "found", "framework", "frogs", "from", "fungus", "further",
      "furthermore", "future", "general", "genetic", "genotype", "genotypes", "global", "goal",
      "goes", "greer", "hantavirus", "has", "have", "health", "healthcare", "healthy", "hepatitis",
      "hev", "high", "higher", "highly", "homeostasis", "hospitals", "hotter", "human",
      "hypothesis", "ideal", "ignited", "illness", "imaging", "immediately", "important",
      "improve", "incidence", "incidences", "include", "increased", "increasing", "indicating",
      "individuals", "indoor", "induced", "infected", "infections", "infectious", "inflammatory",
      "influenza", "infrared", "interest", "interventions", "introduction", "investigation", "ir",
      "itself", "japan", "july", "justice", "kept", "known", "laevis", "last", "leadership",
      "leaf", "lepidobatrachus", "leptodactylus", "lethal", "level", "levels", "likely", "lima",
      "linked", "local", "long", "magnet", "maintained", "maintaining", "mass", "mast", "material",
      "may", "measures", "mechanism", "mental", "might", "models", "moist", "month", "more",
      "moreover", "most", "ne", "nephropathia", "noninvasive", "normal", "numbers", "observed",
      "occidozyga", "occurrence", "one", "only", "onset", "order", "organizational", "ornata",
      "our", "out", "outbreak", "outbreaks", "outspoken", "overall", "owner", "p", "pandemic",
      "patients", "peak", "peaks", "pentadactylus", "people", "peoplewith", "performance",
      "pierotti", "pig", "plethodontohyla", "population", "porcine", "positives", "possibility",
      "possible", "potential", "preceded", "preceding", "precipitation", "predict", "preparation",
      "present", "primed", "private", "procedures", "promoted", "provision", "psychological",
      "psychologically", "public", "putting", "r", "rare", "recently", "recombination",
      "recommendation", "record", "reduce", "reducing", "relate", "related", "relational", "renal",
      "report", "representative", "requirements", "research", "reserves", "reservoir",
      "resilience", "respiratory", "responsible", "resulting", "results", "revealed", "reviews",
      "rising", "risk", "rna", "rodent", "same", "sample", "samples", "sanitary", "sanitation",
      "sars", "screening", "seasonal", "seeds", "sensitive", "serve", "severe", "severity",
      "shanghai", "sharply", "should", "significant", "significantly", "since", "singled",
      "source", "spearman", "species", "specific", "speedy", "spread", "spring", "staff", "staple",
      "state", "statistically", "stress", "stricter", "studies", "suggested", "suggests", "summer",
      "summers", "support", "supported", "supports", "surrounding", "survival", "survived",
      "syndrome", "systems", "temperature", "temperatures", "term", "than", "therefore",
      "thermography", "three", "through", "thus", "time", "times", "tool", "total", "training",
      "treatment", "trees", "tried", "tuberata", "two", "under", "unpredictable", "until", "used",
      "viral", "virtually", "virus", "vole", "voles", "warmer", "warming", "wasted", "we", "well",
      "were", "when", "whereas", "whereby", "while", "winter", "within", "workers", "working",
      "year", "year-1", "year-2", "years", "yet", "zoonosis" };

  class TestDocCallback implements ScoredDocumentCallback {

    List<String> docPath = new ArrayList<String>();

    public void newDocument(int docIndex, MIFSDocument document) {
      String path = document.getPath();
      docPath.add(path.substring(path.lastIndexOf("/"), path.length()));
    }

    public void apply(int termIndex, MIFSTerm term, int freq, float score) {
      // System.out.printf("%s: %f%n", term, score);
    }
  }

  class TestTermCallback implements ScoredTermCallback {

    List<String> terms = new ArrayList<String>();

    public void newTerm(int termIndex, MIFSTerm term) {
      terms.add(term.getText());
    }

    public void apply(int docIndex, MIFSDocument doc, int freq, float score) {

    }
  }

  @Test
  public void testSTCollectionCreator() throws AIDAException, IOException {
    STCCreator creator = new STCCreator();
    IndexReader reader = IndexReader.open("./test_data/pub_med_5");
    ScoredTermsCollection st = creator.createCollection(reader, new FieldFilter("contents"), range);

    TestDocCallback dc = new TestDocCallback();
    st.iterateByDocument(dc);
    assertEquals(5, dc.docPath.size());
    Set<String> expected = new HashSet<String>();
    for (int i = 0; i < 5; i++) {
      expected.add("/pub_med_abstract_" + (i + 1) + ".xml");
    }

    for (int i = 0; i < 5; i++) {
      assertTrue(expected.remove(dc.docPath.get(i)));
    }
    assertEquals(0, expected.size());

    // THIS WILL FAIL IF THE STCCreator processes
    // more than the Contents field term!!!
    TestTermCallback tc = new TestTermCallback();
    st.iterateByTerm(tc);

    assertEquals(PUB5_TERMS.length, tc.terms.size());
    for (int i = 0; i < PUB5_TERMS.length; i++) {
      assertEquals(PUB5_TERMS[i], tc.terms.get(i));
    }

    MIFSTerm maxTerm = null;
    int max = 0;
    for (MIFSTerm term : st.terms()) {
      if (term.getDocumentCount() > max) {
	max = term.getDocumentCount();
	maxTerm = term;
      }
    }

    assertEquals("during", maxTerm.getText());
    assertEquals(4, max);
  }

  @Test
  public void testSVCreation() throws IOException, AIDAException {
    STCCreator creator = new STCCreator();
    IndexReader reader = IndexReader.open("./test_data/test_index");

    // AndFilter<Term> filter = new AndFilter<Term>(new FieldFilter("contents"),
    // new WordFilter(1));
    ScoredTermsCollection st = creator.createCollection(reader, new WordFilter(2), range);
    reader.close();

    SVIndexBuilder indexBuilder = new SVIndexBuilder(20, 10);
    indexBuilder.buildIndex("../terms.bin", "../docs.bin", st);
  }

  @Test
  public void testIO() throws IOException, AIDAException {
    System.out.println("Creating ...");
    STCCreator creator = new STCCreator();
    IndexReader reader = IndexReader.open("./test_data/test_index");
    ScoredTermsCollection st = creator.createCollection(reader, new WordFilter(2), range);
    System.out.printf("Terms X Docs: %d%n", st.getDocumentCount() * st.getTermCount());
    reader.close();
    reader = null;

    Calendar cal = GregorianCalendar.getInstance();
    cal.setTime(new Date());
    cal.add(Calendar.DAY_OF_MONTH, -1);

    System.out.println("Writing ...");
    STCWriter writer = new STCWriter();
    writer.writeCollection("./test_data/cluster_test_stc.bin", st);

    System.out.println("Reading ...");
    STCReader stcReader = new STCReader();
    ScoredTermsCollection copy = stcReader.readCollection("./test_data/cluster_test_stc.bin");

    System.out.println("Comparing ...");
    assertEquals(st.getDocumentCount(), copy.getDocumentCount());
    assertEquals(st.getTermCount(), copy.getTermCount());
    assertEquals(st.getRange(), copy.getRange());

    for (int j = 0; j < st.getTermCount(); j++) {
      MIFSTerm oTerm = st.getTerm(j);
      MIFSTerm cTerm = copy.getTerm(j);
      assertEquals(oTerm.getText(), cTerm.getText());
      assertEquals(oTerm.getDocumentCount(), cTerm.getDocumentCount());
      assertEquals(oTerm.getMinFrequency(), cTerm.getMinFrequency());
      assertEquals(oTerm.getMaxFrequency(), cTerm.getMaxFrequency());
      assertEquals(oTerm.getMinScore(), cTerm.getMinScore(), 0);
      assertEquals(oTerm.getMaxScore(), cTerm.getMaxScore(), 0);

    }

    for (int i = 0; i < st.getDocumentCount(); i++) {
      MIFSDocument oDoc = st.getDocument(i);
      MIFSDocument cDoc = copy.getDocument(i);
      assertEquals(oDoc.getTitle(), cDoc.getTitle());
      assertEquals(oDoc.getPath(), cDoc.getPath());

      for (int j = 0; j < st.getTermCount(); j++) {
	assertEquals(st.getFrequency(i, j), copy.getFrequency(i, j));
	assertEquals(st.getScore(i, j), copy.getScore(i, j), 0);
      }
    }
  }
}
