package anl.aida.ae.lucene;

import org.apache.commons.vfs.FileSystemException;
import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.DateTools;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.index.IndexWriter;

import anl.aida.ae.lucene.LuceneConstants;

import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.events.StartElement;
import javax.xml.stream.events.XMLEvent;
import java.io.*;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

/**
 * Builds a sample lucene index representing a full cache.
 *
 */
public class LIndexBuilder {

  private static final String TITLE = "ArticleTitle";
  private static final String AUTHOR = "Author";

  private static final String PUBDATE = "PubDate";
  private static final String MEDLINE_DATE = "MedlineDate";
  private static final String YEAR = "Year";
  private static final String MONTH = "Month";
  private static final String DAY = "Day";

  private static final String ABSTRACT = "AbstractText";
  private static final String FNAME = "FirstName";
  private static final String LNAME = "LastName";
  private static final String INITIALS = "Initials";

  private static final String INDEX_DIR = "./indices/cache";


  static {
    BasicConfigurator.configure();
    Logger.getRootLogger().setLevel(Level.INFO);
  }

  public LIndexBuilder() throws FileSystemException {

    //FileSystemManager fsManager = VFS.getManager();
    //FileObject abs = fsManager.resolveFile("tgz:///Users/nick/Documents/workspace/fusion/Core/corpus/abstracts.tgz");
    //files = abs.getChildren()[0].getChildren();
  }

  public void build() throws IOException {
    File dir = new File(INDEX_DIR);
    for (File file : dir.listFiles()) {
      file.delete();
    }
    IndexWriter writer = null;
    try {
      writer = new IndexWriter(INDEX_DIR, new StandardAnalyzer(), IndexWriter.MaxFieldLength.UNLIMITED);
      buildDocuments(writer);
      writer.optimize();
    } catch (Exception ex) {
      ex.printStackTrace();
    } finally {
      if (writer != null) writer.close();
    }

  }


  private void buildDocuments(IndexWriter writer) throws IOException, XMLStreamException {
    Calendar cal = GregorianCalendar.getInstance();
    cal.set(2008, 9, 1, 0, 0);
    Date date = cal.getTime();
    int dateIndex = 0;
    File files[] = new File("/tmp/pub_med").listFiles();

    for (int i = 0; i < 15000; i++) {
      File file = files[i];
      Document doc = createDocument(file);
      doc.add(new Field(LuceneConstants.PATH_FIELD, file.toURI().toURL().toExternalForm(), Field.Store.YES,
              Field.Index.NOT_ANALYZED));
      doc.add(new Field(LuceneConstants.TIMESTAMP_FIELD, DateTools.timeToString(date.getTime(),
              DateTools.Resolution.DAY), Field.Store.YES, Field.Index.ANALYZED));

      writer.addDocument(doc);
      dateIndex++;
      if (dateIndex >= 250) {
        // increment the date
        dateIndex = 0;
        cal.add(Calendar.DAY_OF_MONTH, 1);
        date = cal.getTime();
        System.out.printf("Index at: %d, Date: %s%n", i, date.toString());
      }
    }
  }


  private Document createDocument(File file) throws XMLStreamException, FileNotFoundException {
    String title = "";
    String absText = "";

    XMLInputFactory factory = XMLInputFactory.newInstance();
    XMLEventReader reader = factory.createXMLEventReader(new BufferedInputStream(new FileInputStream(file)));

    try {
      while (reader.hasNext()) {
        XMLEvent evt = reader.nextEvent();
        if (evt.isStartElement()) {
          StartElement elmt = evt.asStartElement();
          String qName = elmt.getName().getLocalPart();

          if (qName.equals(TITLE)) {
            title = reader.getElementText();
            /*
          } else if (qName.equals(AUTHOR)) {
            author = new Author();
            */
          } else if (qName.equals(ABSTRACT)) {
            absText = reader.getElementText();

          } /* else if (qName.equals(FNAME)) {
            author.fname = reader.getElementText();
          } else if (qName.equals(LNAME)) {
            author.lname = reader.getElementText();
          } else if (qName.equals(INITIALS)) {
            author.initials = reader.getElementText();
          } else if (qName.equals(PUBDATE)) {
            date = new DateRep();
          } else if (qName.equals(YEAR) && date != null) {
            date.year = Integer.parseInt(reader.getElementText());
          } else if (qName.equals(MONTH) && date != null) {
            date.month(reader.getElementText());
          } else if (qName.equals(DAY) && date != null) {
            date.day = Integer.parseInt(reader.getElementText());
          } else if (qName.equals(MEDLINE_DATE) && date != null) {
            date.parse(reader.getElementText());
          }
          */


          /*
          } else if (evt.isEndElement()) {
            EndElement elmt = evt.asEndElement();
            String qName = elmt.getName().getLocalPart();
            if (qName.equals(AUTHOR)) {
              authors.add(author);
            } else if (qName.equals(PUBDATE)) {
              docData.setDate(date.build(jCas));
              date = null;
            }
          }
          */


        }
      }
    } finally {
      reader.close();
    }

    Document doc = new Document();
    // Add the path of the file as a field named "path".  Use a field that is
    // indexed (i.e. searchable), but don't tokenize the field into words.


    Field field = new Field(LuceneConstants.TITLE_FIELD, title, Field.Store.YES, Field.Index.ANALYZED);
    field.setBoost(1);
    doc.add(field);

    doc.add(new Field(LuceneConstants.CONTENTS_FIELD, new StringReader(absText)));

    return doc;
  }


  public static void main(String[] args) {
    try {
      LIndexBuilder builder = new LIndexBuilder();
      builder.build();
    } catch (FileSystemException e) {
      e.printStackTrace();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }


}
