package anl.aida.reader;

import org.apache.uima.UIMAFramework;
import org.apache.uima.analysis_engine.AnalysisEngineProcessException;
import org.apache.uima.cas.CAS;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationIndex;
import org.apache.uima.collection.CollectionReader;
import org.apache.uima.collection.CollectionException;
import org.apache.uima.resource.CasManager;
import org.apache.uima.resource.ResourceInitializationException;
import org.apache.uima.resource.ResourceSpecifier;
import org.apache.uima.resource.metadata.ProcessingResourceMetaData;
import org.apache.uima.util.InvalidXMLException;
import org.apache.uima.util.XMLInputSource;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import java.io.IOException;

import static junit.framework.Assert.assertEquals;
import anl.aida.types.DocumentMetaData;

/*
 *         Date: Jan 20, 2009 4:47:18 PM
 */
public class PMTests {

  private CAS cas;
  private CollectionReader reader;

  @Before
  public void setUp() throws IOException, InvalidXMLException, ResourceInitializationException,
          AnalysisEngineProcessException {
    XMLInputSource in = new XMLInputSource("./desc/PubMedXMLReader.xml");
    ResourceSpecifier spec = UIMAFramework.getXMLParser().parseResourceSpecifier(in);
    reader = UIMAFramework.produceCollectionReader(spec);

    CasManager casManager = reader.getResourceManager().getCasManager();
    //  Added line:

    casManager.addMetaData((ProcessingResourceMetaData) reader.getMetaData());
    casManager.defineCasPool("pool", 2, null);
    cas = casManager.getCas("pool");
  }

  @After
  public void tearDown() {
    cas.reset();
  }

  @Test
  public void readerTest() throws CollectionException, IOException {
    Type type = cas.getTypeSystem().getType("anl.mifs.types.DocumentMetaData");
    while (reader.hasNext()) {
      reader.getNext(cas);
      AnnotationIndex idx = cas.getAnnotationIndex(type);
      assertEquals(1, idx.size());
      DocumentMetaData data = (DocumentMetaData) idx.iterator().next();
      cas.reset();
    }

  }
}
