package anl.aida.reader;

import static junit.framework.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.util.Calendar;
import java.util.GregorianCalendar;

import org.apache.uima.UIMAFramework;
import org.apache.uima.analysis_engine.AnalysisEngineProcessException;
import org.apache.uima.cas.CAS;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationIndex;
import org.apache.uima.collection.CollectionException;
import org.apache.uima.collection.CollectionReader;
import org.apache.uima.collection.CollectionReaderDescription;
import org.apache.uima.resource.CasManager;
import org.apache.uima.resource.ResourceInitializationException;
import org.apache.uima.resource.metadata.ConfigurationParameterSettings;
import org.apache.uima.resource.metadata.ProcessingResourceMetaData;
import org.apache.uima.util.InvalidXMLException;
import org.apache.uima.util.XMLInputSource;
import org.junit.Test;

import anl.aida.reader.ParsedDate;
import anl.aida.reader.PreTextExtractor;
import anl.aida.reader.ProMedPreProcessor;
import anl.aida.reader.ProMedReader;
import anl.aida.reader.ProMedTxtReader;
import anl.aida.reader.ReaderConstants;
import anl.aida.reader.ReaderResult;
import anl.aida.types.Date;
import anl.aida.types.DocumentMetaData;
import anl.aida.types.Name;

public class ProMedReaderTest {
  
  private static final String title_74132 = "KOI HERPESVIRUS DISEASE - UNITED KINGDOM (02): (ENGLAND)";
  private static final String url_74132 = "http://www.promedmail.org/pls/otn/f?p=2400:1202:1692285671124542::NO::F2400_P1202_CHECK_DISPLAY,F2400_P1202_PUB_MAIL_ID:X,74132";
  // year, month, day
  private static final int[] date_74132 = {2008, 8, 25};
  private static final String author_74132 = "Interactive Investor, Thomson Financial News report [edited]";
  private static final String link_74132 = "http://www.iii.co.uk/news/?type=afxnews&articleid=6912766&subject=companies&action=article";
  private static final String contents_74132 = "A ProMED-mail post <<http://www.promedmail.org>http://www.promedmail.org> ProMED-mail is a program of the International Society for Infectious Diseases <<http://www.isid.org>http://www.isid.org>  Date: Thu 25 Sep 2008 Source: Interactive Investor, Thomson Financial News report [edited] <<http://www.iii.co.uk/news/?type=afxnews&articleid=6912766&subject=companies&action=article>http://www.iii.co.uk/news/?type=afxnews&articleid=6912766&subject=companies&action=article>   The UK's Department for Environment, Food and Rural Affairs (Defra) said movements of fish in and out of a fishery in East Sussex have been restricted, following the confirmation of Koi herpesvirus (KHV) disease.  Defra has issued a designated area order prohibiting the movement of fish to and from Moor Hall Pools, Moor Hall Farm, Ninfield, Nr Battle, East Sussex.  KHV Disease affects common carp and carp varieties such as Koi. It can result in high rates of fish mortalities. There are no implications for human health, the department said in a statement.";
  
  private static final String title_74172 = "YELLOW FEVER - AFRICA (07): GUINEA (N'ZEREKORE)";
  private static final String url_74172 = "http://www.promedmail.org/pls/otn/f?p=2400:1202:1692285671124542::NO::F2400_P1202_CHECK_DISPLAY,F2400_P1202_PUB_MAIL_ID:X,74172";
  // year, month, day
  private static final int[] date_74172 = {2008, 8, 29};
  private static final String author_74172 = "WHO Disease Outbreak News [edited]";
  private static final String link_74172 = "http://www.who.int/csr/don/2008_09_29/en/index.html";
  private static final String contents_74172 = "A ProMED-mail post <<http://www.promedmail.org>http://www.promedmail.org> ProMED-mail is a program of the International Society for Infectious Diseases <<http://www.isid.org>http://www.isid.org>  Date: Mon, 29 Sep 2008 Source: WHO Disease Outbreak News [edited] <<http://www.who.int/csr/don/2008_09_29/en/index.html>http://www.who.int/csr/don/2008_09_29/en/index.html>   The Ministry of Health, Guinea has reported 2 cases of yellow fever on 20 Aug and 12 Sep 2008. The cases have been laboratory confirmed by the Institut Pasteur, Dakar. The 1st case was reported in a 24-year-old male living in the sub-prefecture of Bounouma, Prefecture of N'zerekore: the man reported first having symptoms on 26 Jul 2008, and blood samples were taken on 4 and 5 Aug 2008. The 2nd case was reported in the 3rd week of August 2008 in the Urban Commune of N'zerekore in another 24-year-old male. Neither case had any history of yellow fever vaccination.  A yellow fever outbreak investigation team composed of experts from the MoH and WHO visited 10 sub-prefectures, the Urban Commune of N'zerekore and 2 other sub-prefectures in the Prefecture of Lola, from the 5-14 Sep 2008.  The mission concluded that: (i) according to the yellow fever case definition, 14 suspected cases were found retrospectively, one of these cases died having presented with symptoms of fever and jaundice; (ii) Most of the prefectures were vaccinated in 2005 with a reported coverage of 95%; (iii) The only areas where a low coverage rate was reported (45%) were Bounama sub-prefecture, Urban Commune and the Ivorian Refugee Camp of Kouankan 2.  On the basis of these results, the Ministry of Health decided to carry out a reactive mass vaccination campaign in these 3 areas, in October 2008, targeting 140 342 people. The campaign will be conducted with the technical assistance of WHO and the support of the GAVI alliance (the Global Alliance for Vaccines and Immunization). The MoH has submitted a request for yellow fever vaccine from the global emergency stockpile, managed by the International Coordinating Group on Vaccine Provision for Yellow Fever Control (YF-ICG). The yellow fever emergency stockpile is financed by the GAVI Alliance and managed by the YF-ICG. The YF-ICG is composed of representatives from UNICEF, Medecins sans Frontieres (MSF), the International Federation of the Red Cross (IFRC), and the World Health Organization, which acts as its secretariat.";
  
  private static final String title_75235 = "AVIAN INFLUENZA (123): TAIWAN (SUSPECTED, REQUEST FOR INFORMATION), CHINA (HONG KONG)";
  private static final String url_75235 = "http://www.promedmail.org/pls/otn/f?p=2400:1202:1692285671124542::NO::F2400_P1202_CHECK_DISPLAY,F2400_P1202_PUB_MAIL_ID:X,75235";
  // year, month, day
  private static final int[] date_75235_a = {2008, 11, 18};
  private static final String author_75235_a = "Taiwan News [edited]";
  private static final String link_75235_a = "http://www.etaiwannews.com/etn/news_content.php?id=816681";
  private static final String contents_75235_a = "[1] Taiwan, suspected H5N2, RFI Date: Thu 18 Dec 2008 Source: Taiwan News [edited] <<http://www.etaiwannews.com/etn/news_content.php?id=816681>http://www.etaiwannews.com/etn/news_content.php?id=816681>   Poultry farmers complain of impact of unconfirmed bird flu reports" +
  		" ------------------------------------------------------------------ Japan's decision to halt poultry imports from " +
  		"Taiwan following a news report about a suspected bird flu outbreak on the island has sparked widespread discontent among " +
  		"Taiwan's poultry farmers, who demanded Thursday [18 Dec 2008] that the government swiftly resolve the problem. " +
  		"Poultry farmers, particularly those in southern Taiwan, complained that their livelihood will be impacted and " +
  		"lashed out at what they called irresponsible media reports.  The problem was sparked Wednesday when the Liberty " +
  		"Times reported that a large number of chickens at a poultry farm in Tainan County were confirmed by a government " +
  		"laboratory to have died from the H5N2 bird flu virus last month [November 2008]. The report alleged that the " +
  		"Cabinet-level Council of Agriculture (COA) sought to cover up the matter so as not to mar the launch of direct " +
  		"air, shipping and postal links between Taiwan and mainland China on Monday.  The COA denied the report. " +
  		"Huang Kuo-ching, deputy director of the COA's Bureau of Animal and Plant Health Inspection and Quarantine, " +
  		"said Wednesday [17 Dec 2008] that there is no H5N2 avian flu outbreak in Taiwan, except for an incident in " +
  		"late October at a poultry farm in Luchu township, Kaohsiung County.  As the fatality rate among chickens at " +
  		"the farm was below 3 percent, which is far lower than the average mortality rate in cases of the low-pathogenic " +
  		"H5N2 virus, the COA was hesitant to categorize it as an epidemic, Huang explained, adding that the council did " +
  		"not try to cover up any bird flu outbreak. All the remaining 18 000 chickens at the farm were culled in an effort " +
  		"to prevent a spread, Huang said, adding that the situation was under control and that nothing unusual has been " +
  		"detected so far.  Despite the COA's explanation, Japan announced later Wednesday that it was immediately halting " +
  		"poultry meat imports from Taiwan on a temporary basis to prevent the H5N2 bird flu virus from entering its territory.  T" +
  		"ang Jung-sheng, chairman of the Kaohsiung County chicken-raising association, said Thursday [18 Dec 2008] that " +
  		"such unconfirmed reports will hurt the domestic industry, particularly at the current time of economic recession when " +
  		"consumers are paring back spending and buying less chicken. Representatives of industry associations across the country " +
  		"will call a meeting Thursday in central Taiwan's Changhua County to discuss countermeasures in the face of the latest " +
  		"development, said Tang.  An executive at the Taiwan's largest frozen poultry meat exporting company said the Liberty " +
  		"Times report has dealt a heavy blow to Taiwan's frozen duck meat exports to Japan. One of his company's duck meat " +
  		"containers shipped to Japan has been destroyed there, he said. Taiwan's frozen chicken meat exports are uncompetitive " +
  		"in the international market, but the company's duck meat exports are popular in Japan and are sold in the quality-conscious " +
  		"Japanese market at a similar price level as U.S. beef, he said.  Duck farmers urged the COA to quickly provide a clear " +
  		"explanation to Japan to help farmers reduce possible losses. Chen Chien-chu, a duck raiser in Fongyuan township in " +
  		"Changhua County in central Taiwan said he felt very distressed after learning of the news, since most of his " +
  		"ducks are exported to Japan at a very stable price.  \"Poultry farmers suffer a lot of monetary losses every " +
  		"time there are such news report,\" Chen complained. \"I lost some NTD 2 million (USD 60 000) in just one month " +
  		"last time when there was a similar false report,\" he said.  The value of Taiwan's poultry meat exports to Japan " +
  		"totals around USD 23 million per year, with frozen duck meat as the bulk. In 2007, Japan imported 5128 tons of poultry " +
  		"meat and 148 tons of chicken eggs from Taiwan, Japanese government statistics showed.  The COA is still trying to track " +
  		"the cause of the deaths among the dead chickens, and will publish the findings upon completion of tests, the COA's " +
  		"Huang said. Meanwhile, the impact of the Japan's newest measure is relatively small in the rest of the country where " +
  		"the poultry industry is not prominent.";
  
  private static final int[] date_75235_b = {2008, 11, 19};
  private static final String author_75235_b = "The Standard, Hong Kong [edited]";
  private static final String link_75235_b = "http://www.thestandard.com.hk/news_detail.asp?pp_cat=11&art_id=76068&sid=21959514&con_type=3";
  private static final String contents_75235_b = "[2] Hong Kong update Date: Fri 19 Dec 2008 Source: The Standard, Hong Kong [edited] <<http://www.thestandard.com.hk/news_detail.asp?pp_cat=11&art_id=76068&sid=21959514&con_type=3>http://www.thestandard.com.hk/news_detail.asp?pp_cat=11&art_id=76068&sid=21959514&con_type=3>   Dead chickens get H5N1 mutation all-clear ----------------------------------------- " +
  		"The H5N1 virus found in dead chickens in a Yuen Long farm had not mutated, Undersecretary for Food and Health " +
  		"Gabriel Leung told the Legislative Council's panel on food safety and environmental hygiene yesterday " +
  		"[Thu 18 Dec 2008]. The genetic sequencing of the bird flu virus detected in the farm on 9 Dec 2008 did " +
  		"not contain obvious differences from previous viruses, Leung told the panel. Leung also said more tests " +
  		"were needed to find out how the chickens contracted the virus.  There was also good news for the public: " +
  		"Live chickens can be sold again as early as 30 Dec 2008 if things go smoothly. The early end to the normal " +
  		"21-day no-chicken ban means the birds will once again be on the menu for Lunar New Year family festivities.  " +
  		"Meanwhile, an Agriculture Fisheries and Conservation Department spokeswoman yesterday [18 Dec 2008] explained " +
  		"why the government has been using an H5N2 vaccine manufactured in the Netherlands to protect local chickens " +
  		"from the flu since 2003, even though recent outbreaks showed the virus strain to be subtype H5N1. She said " +
  		"when vaccines were 1st introduced to local chicken farms, tests had shown the H5N2 vaccine to be effective " +
  		"against the viruses found in South China.  \"The H5N2 vaccine is also effective against H5N1 viruses,\" the " +
  		"spokeswoman said, adding that in 2006, the government had let US authorities test the vaccine, which also showed " +
  		"it is effective for use in southern China. \"However, no vaccine is 100 percent effective. The government has set " +
  		"up an investigation group on avian influenza to look into the matter,\" she said.  [Byline: Adele Wong]";

  Calendar cal = GregorianCalendar.getInstance();
  
  private CAS cas;
  private CollectionReader reader;
  
  @Test
  public void sample() throws IOException {
    PreTextExtractor extractor = new PreTextExtractor();
    ProMedTxtReader reader = new ProMedTxtReader();
    String url = "http://www.promedmail.com/pls/otn/f?p=2400:1202:2953026041433572::NO::F2400_P1202_CHECK_DISPLAY,F2400_P1202_PUB_MAIL_ID:X,76594";
    ProMedPreProcessor proc = new ProMedPreProcessor(new StringReader(extractor.extract(url)));
    Reader in = proc.next();
    int i = 0;
    while (in != null) {
      ReaderResult result = reader.run(url, in);
      // fake date as Date is passed in from the index file
      result.setDate(new ParsedDate(new java.util.Date()));
      assertTrue(String.valueOf(i), result.isValid() == ReaderResult.Error.NONE);
      in = proc.next();
      i++;
    }
    
  }
  
  @Test
  public void testTextReader() throws FileNotFoundException, IOException {
    PreTextExtractor extractor = new PreTextExtractor();
    ProMedTxtReader reader = new ProMedTxtReader();
    File file = new File("./test_data/ProMedMail_74132.txt");
    ProMedPreProcessor proc = new ProMedPreProcessor(new StringReader(extractor.extract(url_74132)));
    Reader in = proc.next();
    assertNotNull(in);
    ReaderResult result = reader.run(file.getAbsolutePath(), in);
    assertNotNull(result);
    assertEquals(title_74132, result.getTitle());
    
    /*
    cal.setTime(result.getDate().getJavaDate());
    assertEquals(cal.get(Calendar.YEAR), date_74132[0]);
    assertEquals(cal.get(Calendar.MONTH), date_74132[1]);
    assertEquals(cal.get(Calendar.DAY_OF_MONTH), date_74132[2]);
    */
    
    //assertEquals(author_74132, result.getAuthors().getAuthors());
    //assertEquals(link_74132, result.getLink());
    assertEquals(ReaderConstants.PRO_MED_SOURCE, result.getSource());
    assertEquals(contents_74132.trim(), result.getContents().trim());
    
    assertNull(proc.next());
    
    
    file = new File("./test_data/ProMedMail_74172.txt");
    proc = new ProMedPreProcessor(new StringReader(extractor.extract(url_74172)));
    in = proc.next();
    assertNotNull(in);
    result = reader.run(file.getAbsolutePath(), in);
    assertNotNull(result);
    assertEquals(title_74172, result.getTitle());
    
    /*
    cal.setTime(result.getDate().getJavaDate());
    assertEquals(cal.get(Calendar.YEAR), date_74172[0]);
    assertEquals(cal.get(Calendar.MONTH), date_74172[1]);
    assertEquals(cal.get(Calendar.DAY_OF_MONTH), date_74172[2]);
    */
    
    //assertEquals(author_74172, result.getAuthors().getAuthors());
    //assertEquals(link_74172, result.getLink());
    assertEquals(ReaderConstants.PRO_MED_SOURCE, result.getSource());
    assertEquals(contents_74172.trim(), result.getContents().trim());
    
    assertNull(proc.next());
  }
  
  @Test
  public void testMultiPartFile() throws FileNotFoundException, IOException {
    PreTextExtractor extractor = new PreTextExtractor();
    ProMedTxtReader reader = new ProMedTxtReader();
    File file = new File("./test_data/ProMedMail_75235.txt");
    ProMedPreProcessor proc = new ProMedPreProcessor(new StringReader(extractor.extract(url_75235)));
    Reader in = proc.next();
    assertNotNull(in);
    ReaderResult result = reader.run(file.getAbsolutePath(), in);
    assertNotNull(result);

    assertEquals(title_75235, result.getTitle());

    /*
    cal.setTime(result.getDate().getJavaDate());
    assertEquals(cal.get(Calendar.YEAR), date_75235_a[0]);
    assertEquals(cal.get(Calendar.MONTH), date_75235_a[1]);
    assertEquals(cal.get(Calendar.DAY_OF_MONTH), date_75235_a[2]);
    */

    //assertEquals(author_75235_a, result.getAuthors().getAuthors());
    //assertEquals(link_75235_a, result.getLink());
    assertEquals(ReaderConstants.PRO_MED_SOURCE, result.getSource());
    assertEquals(contents_75235_a.trim(), result.getContents().trim());

    in = proc.next();
    assertNotNull(in);
    result = reader.run(file.getAbsolutePath(), in);
    assertNotNull(result);

    assertEquals(title_75235, result.getTitle());

    /*
    cal.setTime(result.getDate().getJavaDate());
    assertEquals(cal.get(Calendar.YEAR), date_75235_b[0]);
    assertEquals(cal.get(Calendar.MONTH), date_75235_b[1]);
    assertEquals(cal.get(Calendar.DAY_OF_MONTH), date_75235_b[2]);
    */

    //assertEquals(author_75235_b, result.getAuthors().getAuthors());
    //assertEquals(link_75235_b, result.getLink());
    assertEquals(ReaderConstants.PRO_MED_SOURCE, result.getSource());
    assertEquals(contents_75235_b.trim(), result.getContents().trim());

    assertNull(proc.next());
  }

  public void setUpCAS(String indexFileLocation) throws IOException, InvalidXMLException, ResourceInitializationException,
          AnalysisEngineProcessException {
    XMLInputSource in = new XMLInputSource("./desc/MIFSCompositeReader.xml");
    CollectionReaderDescription spec = (CollectionReaderDescription) UIMAFramework.getXMLParser().parseResourceSpecifier(in);
    ConfigurationParameterSettings settings = spec.getCollectionReaderMetaData().getConfigurationParameterSettings();
    settings.setParameterValue(ProMedReader.INDEX_FILE, indexFileLocation);
    // todo switch to use configurator
    //settings.setParameterValue(MIFSCompositeReader.COMPONENTS, "./test_data/component_readers_promed_only.txt");
    reader = UIMAFramework.produceCollectionReader(spec);
    
    CasManager casManager = reader.getResourceManager().getCasManager();
    casManager.addMetaData((ProcessingResourceMetaData) reader.getMetaData());
    casManager.defineCasPool("pool", 2, null);
    cas = casManager.getCas("pool");
  }

  public void tearDownCAS() {
    cas.reset();
  }
  
  @Test
  public void readerSmokeTest() throws CollectionException, IOException, AnalysisEngineProcessException, 
  InvalidXMLException, ResourceInitializationException {
    setUpCAS("./promed_archive_index.txt");
    Type type = cas.getTypeSystem().getType("anl.mifs.types.DocumentMetaData");
    int count = 0;
    while (reader.hasNext()) {
      reader.getNext(cas);
      AnnotationIndex idx = cas.getAnnotationIndex(type);
      assertEquals(1, idx.size());
      DocumentMetaData data = (DocumentMetaData) idx.iterator().next();
      assertTrue(data.getCAS().getDocumentText().length() > 0);
      assertTrue(data.getLocation().length() > 0);
      assertTrue(data.getAuthors(0).getLastName().length() > 0);
      assertTrue(data.getDate().getDay() > 0);
      assertTrue(data.getDate().getMonth() > 0);
      cas.reset();
      count++;
    }
    tearDownCAS();
  }
}
