package anl.aida.reader;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.text.ParseException;
import java.util.Calendar;
import java.util.Date;

import org.apache.uima.UIMAFramework;
import org.apache.uima.analysis_engine.AnalysisEngineProcessException;
import org.apache.uima.cas.CAS;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationIndex;
import org.apache.uima.collection.CollectionException;
import org.apache.uima.collection.CollectionReader;
import org.apache.uima.collection.CollectionReaderDescription;
import org.apache.uima.resource.CasManager;
import org.apache.uima.resource.ResourceInitializationException;
import org.apache.uima.resource.metadata.ConfigurationParameterSettings;
import org.apache.uima.resource.metadata.ProcessingResourceMetaData;
import org.apache.uima.util.InvalidXMLException;
import org.apache.uima.util.XMLInputSource;
import org.junit.Test;

import anl.aida.reader.AllAfricaHTMLReader;
import anl.aida.reader.CTArchiveReader;
import anl.aida.reader.ChicagoTribuneHTMLReader;
import anl.aida.reader.ChicagoTribuneReader;
import anl.aida.reader.NTARCHtmlReader;
import anl.aida.reader.NTARCReader;
import anl.aida.reader.NYTimesArchiveReader;
import anl.aida.reader.NYTimesHTMLReader;
import anl.aida.reader.NYTimesReader;
import anl.aida.reader.ReaderConstants;
import anl.aida.reader.ReaderResult;
import anl.aida.reader.ReaderResultProcessor;
import anl.aida.types.DocumentMetaData;
import anl.aida.util.AIDAException;
import anl.aida.util.StandardIndexLineParser;

/**
 * Tests of the web scrapers.
 * 
 */
public class ScraperTests {
  
  Calendar cal = Calendar.getInstance();
  
  private static String NTARC_CONTENT = "Aspen Bomb Suspect A man delivered bombs wrapped as Christmas presents to " +
  		"two banks Wednesday along with notes threatening mass death if they did not turn over tens of " +
  		"thousands of dollars, Aspen police said Thursday. Authorities were quickly alerted, and the man " +
  		"apparently halted his plan  leaving two bombs in an alley that were likely intended for other " +
  		"banks, police said. The man, identified as James Blanning, 72, who had had previous contacts with " +
  		"law enforcement, was later found dead after apparently shooting himself, police said. His body was " +
  		"found in his car in a rural area east of Aspen. The incidents Wednesday sparked the evacuation of a " +
  		"16-block area of Aspen on New Years Eve, one of the busiest days of the year for businesses in Aspen. " +
  		"Bomb squads ultimately detonated the devices, and no one was injured. Evacuation orders were lifted early " +
  		"Thursday. The following is the text of four identical notes left with gasoline bombs by a man who was found " +
  		"dead and who police said was trying to extort Aspen, Colo., banks on New Years Eve: YOU HAD BETTER BE A VERY C" +
  		"OOL INDIVIDUAL AND NOT START A PANIC OR MANY IN ASPEN WILL PAY A HORRIBLE PRICE IN BLOOD. WE HAVE ALL OF THE COP " +
  		"THREATS TO US UNDER VERY SOPHISTICATED ELECTRONIC SURVEILLANCE. DO NOT F WITH US OR THERE WILL BE MASS DEATH LIKE " +
  		"WE HAVE ALL BEEN PART OF OVER IN THAT FING QUICKSAND TRAP THAT ROVES AND CHANEYS MONKEY BUS PUT US INTO WHERE SO " +
  		"MANY OF OUR SOUL MATES AND BROTHERS DIED VERY HORRIBLE DEATHS. DO NOT MOVE OR COVER THE VERY BIG FIRE CRACKER IN THE " +
  		"PLASTIC CONTAINER. UNIQUE CHEMICALS AND ELECTRONICS. TAKE THIS SHEET AND THE WHITE BOX TO THE ACTING HEAD TELLER. " +
  		"THE OTHER THREE ASPEN BANKS ARE GOING THROUGH THE SAME THING THIS ONE IS AND THIS IS AS MUCH A SUICIDE MISSION AS " +
  		"A BANK ROBBERY, SO, IF IT GETS F-D UP, OUR SOULS HAVE ALREADY BEEN DESTROYED AND WE WILL ALL DIE LIKE THE " +
  		"RAG-HEAD MARTYRS THAT WE WERE KILLING OVER THERE. THEY TRAINED US GOOD FOR THIS SORT OF THING. PUT $60,000 I" +
  		"N USED $100s IN THE WHITE BOX. ANY DYES, TRACKERS, OR OTHER BULLST WILL CAUSE DISASTER TO ALL. IN 20 MINUTES " +
  		"HAVE SOMEONE TAKE THE WHITE BOX OUTSIDE THE FRONT OF THE BANK AND WAIT UNTIL SOMEONE COMES UP AND POINTS DIRECTLY " +
  		"TO THE BOX WITHOUT SAYING ANYTHING, THEN GIVE IT TO HIM. HE HAS BEEN AND IS WELL COVERED. AFTER 2 HOURS THE FIRE " +
  		"CRACKER (IED), IF NOT DISTURBED, WILL AUTOMATICALLY DEACTIVATE AND CAN BE MOVED TO A SAFE PLACE (THE DUMP?) TO BE " +
  		"DISPOSED OF. FOR VERY UNIQUE CHEMICAL REASONS, WE RECOMMEND THAT ALL BE SET ON FIRE. WE LEARNED SOME NASTY CHEMISTRY " +
  		"FROM THE MARTYRS, ETC. PS - FOR ADDED INSURANCE, THERE IS ALSO A FIFTH FIRE CRACKER HIDDEN IN A HIGH END WATERING " +
  		"HOLE THAT WE WILL REMOVE AFTER WE ARE FULLY CLEAR AND KNOW FOR SURE ALL HAS GONE WELL. F THE WHOLE WORLD, ALREADY. " +
  		"Source: Aspen Police Department";
  
  private static String NTARC2_CONTENT = "A flight heading to Charlotte from Lynchburg had to be delayed because of comments made by a passenger."
      + " The 23-year-old man was in the lobby of the airport around 2:30 Sunday afternoon when he was overheard talking on his cell phone. Witnesses say he told whomever he was speaking with "
      + "that the plane wouldnt make it to Charlotte and that it would fall out of the sky."
      + " He broke his phone in half when he was approached by Transportation Security Administration officials. He told them he had he was God, but he was injured "
      + "so he couldnt show them his powers. He then told deputies with the Campbell County Sheriffs Office that he had been treated "
      + "at a mental hospital in West Virginia. The man was taken to Lynchburg General Hospital on an emergency custody order for a mental evaluation. He was not charged with anything."
      + " Explosives trained dogs searched his luggage, his car, and the airplane, but they didnt find anything suspicious.";
  
  private static String TRIB_CONTENT_A = "Police arrested an Aurora man who allegedly damaged the door to an " +
  		"upholstery business and used a piece of door trim to bash the headlights of the store owners " +
  		"car. The store owner and David J. Fitzpatrick previously had a relationship, according to Kane " +
  		"County sheriffs police, and the owner reportedly locked Fitzpatrick out of the Terry Avenue shop " +
  		"in Aurora when he showed up Monday about 12:45 p.m. Fitzpatrick, 44, reportedly kicked and punched " +
  		"the front door, and vandalized the owners car, causing more than $400 in damage. Fitzpatrick, of the " +
  		"600 block of South Lake Street, was charged with two counts of criminal damage to property.";
  
  private static String TRIB_CONTENT_B = "Installing red-light cameras at busy intersections is a growth industry, " +
  		"but one northwest suburban community has pulled the plug on a highly profitable camera amid " +
  		"worries a premier shopping venue was getting a reputation as a ticket destination. Located at " +
  		"Meacham and Woodfield Roads near Woodfield Mall, the camera raked in $1 million in fines for " +
  		"Schaumburg between November and January. The tickets came in a flood, some 10,000 of them, " +
  		"almost all for drivers not stopping before turning right on red. At $100 a ticket, the villages " +
  		"coffers were overflowing. But amid a flurry of complaintssome from scofflaws threatening never " +
  		"to shop again at the mallthe village recently decided to deactivate the camera, saying it wasnt " +
  		"improving safety that much anyway. We want to be as friendly to customers as we are to villagers, " +
  		"and continuing to enforce the right turn on red when it wasnt doing anything to improve public " +
  		"safety didnt make sense, said Village Manager Ken Fritz. Although crashes at the intersection " +
  		"decreased while the cameras were rolling, the reduction was slightfrom 10 before their installation " +
  		"to 7 during the same period after they were put up, officials said. The camera, one of two at the intersection, " +
  		"can still capture drivers traveling straight through the intersection. Schaumburgs deactivation request " +
  		"was the first for Red Speed Illinois, the Lombard company that has camera contracts with 51 " +
  		"Chicago-area municipalities. The fact that they did issue so many violations indicates that " +
  		"they do have a problem, said Red Speed spokeswoman Debra Beerup, adding that the village is " +
  		"considering installing cameras at several more intersections. Schaumburgs decision isnt " +
  		"unprecedented. Two years ago, Bolingbrook turned a camera off after six months and dozens of " +
  		"complaints from residents. From November 2007 to May, 13,000 tickets and warnings were issued. " +
  		"The village collected $639,000 said Village Atty. James Boan. Some of the complaints in Schaumburg " +
  		"came from drivers who thought the punishment didnt fit what they considered a minor infraction, " +
  		"said George Dunham, a trustee and member of the villages Public Safety Committee. Other complaints, " +
  		"a bit nastier, were from drivers who said they will never shop at Woodfield Mall again, Dunham said. " +
  		"Alex Rodriguez, who works near the Schaumburg intersection, said the constant flashing of the camera " +
  		"prompted him to label it the the blue light special. He said turning off the camera was the right thing " +
  		"to do. He hopes other cameras in the Chicago area are also deactivated. But Robin Baran of Palatine " +
  		"disagreed, saying the camera was important for traffic safety. And she sees it as fair way to collect " +
  		"revenue. Tribune reporter Megan Twohey contributed to this report. <cstarks@tribune.com>cstarks@tribune.com";
  
  
  private CAS cas;
  private CollectionReader reader;
 

  @Test
  public void testNTARCHTML() throws IOException {
    
  
    String url = "http://www.nationalterroralert.com/updates/2009/01/01/aspen-bombs-wrapped-like-christmas-gifts-delivered-to-banks-suspect-found-dead/"; //http://www.nationalterroralert.com/updates/2009/03/17/suspect-in-alleged-ny-terror-plot-released-in-lebanon/";
    String title = "Suspect In Alleged NY Terror Plot Released In Lebanon";
    cal.set(2008, 2, 17);
    
    
    NTARCHtmlReader reader = new NTARCHtmlReader();
    ReaderResult result = reader.read(url, title, cal.getTime(), "");
    assertEquals(title, result.getTitle());
    assertEquals(NTARC_CONTENT.trim(), result.getContents().trim());
    assertEquals(ReaderConstants.NTARC_SOURCE, result.getSource());
    assertEquals(url, result.getLocation());
  }
  
  @Test
  public void testNTARCHTML2() throws IOException {
    String url = "http://www.nationalterroralert.com/updates/2009/05/31/plane-delayed-in-lynchburg-after-suspicious-comments/";
    NTARCHtmlReader reader = new NTARCHtmlReader();
    String title = "Plane Delayed In Lynchburg After Suspicious Comments";
    ReaderResult result = reader.read(url, title, cal.getTime(), "");
    assertEquals(NTARC2_CONTENT.trim(), result.getContents().trim());
    assertEquals(ReaderConstants.NTARC_SOURCE, result.getSource());
    assertEquals(url, result.getLocation());
    
  }
  
  @Test
  public void testTribHTML() throws IOException {
    String url = "http://archives.chicagotribune.com/2009/jan/01/local/chi-upholstry-fight-wblotter01jan01";
    String title = "Man charged with damaging Aurora business, owners car";
    
    ChicagoTribuneHTMLReader reader = new ChicagoTribuneHTMLReader();
    ReaderResult result = reader.read(url, title, new Date(), "");
    assertEquals(title, result.getTitle());
    assertEquals(TRIB_CONTENT_A.trim(), result.getContents().trim());
    assertEquals(ReaderConstants.CHICAGO_TRIBUNE_SOURCE, result.getSource());
    assertEquals(url, result.getLocation());
    
    
    url = "http://archives.chicagotribune.com/2009/feb/26/local/chi-red-lights-26-feb26";
    title = "Schaumburg turns off red-light camera near Woodfield";
    
    result = reader.read(url, title, new Date(), "");
    assertEquals(title, result.getTitle());
    assertEquals(TRIB_CONTENT_B.trim(), result.getContents().trim());
    assertEquals(ReaderConstants.CHICAGO_TRIBUNE_SOURCE, result.getSource());
    assertEquals(url, result.getLocation());
  }
  
  /*
  @Test
  public void testBadArticles() throws IOException {
    String url = "http://www.nytimes.com/2009/03/01/education/01campus.html";
    NYTimesHTMLReader reader = new NYTimesHTMLReader();
    ReaderResult result = reader.run(url, "foo", new Date(), "Me");
  }
  */
  
  private static String NYTIMES_CONTENT_A = "Sri Lankan soldiers fought their way into the capital of the rebel " +
  		"Tamil Tigers on Friday for the first time in a decade, military officials said. Government " +
  		"troops entered the rebel stronghold, a northern town called Kilinochchi, from two sides, " +
  		"senior military officials told The Associated Press on condition of anonymity in line with " +
  		"government rules. The latest attack came after months of fierce battles on the outskirts. " +
  		"The level of fighting inside the town remained unclear. The military had predicted Thursday " +
  		"that soldiers would seize the town in two days. The fall of Kilinochchi would be devastating to " +
  		"the separatist Liberation Tigers of Tamil Eelam, who have used the town as their political and " +
  		"military headquarters for the past 10 years and have created structures for an independent state, such as " +
  		"a police force, courts, and tax offices. Recent government offensives have forced the rebels out of much of " +
  		"their territory. The Sri Lankan president, Mahinda Rajapaksa, has promised to crush the rebel group and end " +
  		"the nation's civil war this year. The fighting began in 1983, and a truce brokered by Norway in 2002 was " +
  		"officially called off last year with both sides openly violating the agreement. The rebels could not immediately " +
  		"be reached for comment. But the political leader of the rebels, Balasingham Nadesan, said on Tuesday that they " +
  		"began as a guerrilla group and would be able to keep fighting even if they lost much of the territory they " +
  		"controlled in the north.";
 
  @Test
  public void testNYTimesHTML() throws IOException {
    // 1230904800000:::http://query.nytimes.com/gst/fullpage.html?res=9B06E6D91E3CF931A35752C0A96F9C8B63:::
    // Rebel Capital Under Attack In Sri Lanka:::THE ASSOCIATED PRESS
    String url = "http://query.nytimes.com/gst/fullpage.html?res=9B06E6D91E3CF931A35752C0A96F9C8B63";
    String title = "Rebel Capital Under Attack In Sri Lanka";
    String author = "THE ASSOCIATED PRESS";
    Date date = new Date(1230904800000L);
    
    NYTimesHTMLReader reader = new NYTimesHTMLReader();
   
    ReaderResult result = reader.read(url, title, date, author);
    assertEquals(url, result.getLocation());
    assertEquals(title, result.getTitle());
    assertEquals(author, result.getAuthors().getAuthors());
    assertEquals(date, result.getDate().getJavaDate());
    assertEquals(NYTIMES_CONTENT_A.trim(), result.getContents().trim());
    
    
    // 1230818400000:::http://www.nytimes.com/2009/01/01/world/middleeast/01mideast.html:::
    // ISRAEL REJECTS GAZA CEASE-FIRE, BUT OFFERS AID:::ETHAN BRONNER; 
    // Reporting was contributed by Taghreed El-Khodary from Gaza; Steven Erlanger from 
    
    url = "http://www.nytimes.com/2009/01/01/world/middleeast/01mideast.html";
    title = "ISRAEL REJECTS GAZA CEASE-FIRE, BUT OFFERS AID";
    author = "ETHAN BRONNER";
    date = new Date(1230818400000L);
    
    result = reader.read(url, title, date, author);
    assertEquals(url, result.getLocation());
    assertEquals(title, result.getTitle());
    assertEquals(author, result.getAuthors().getAuthors());
    assertEquals(date, result.getDate().getJavaDate());
    assertTrue(result.getContents().trim().length() > 0);
    //System.out.println("result.getContents() = " + result.getContents());
  }
  
  @Test
  public void testNY() throws IOException {
    NYTimesHTMLReader reader = new NYTimesHTMLReader();
    String url = "http://query.nytimes.com/gst/fullpage.html?res=9505EFDF1639F932A05756C0A96F9C8B63";
    ReaderResult result = reader.read(url, "", new Date(), "");
    
  }
  
  private static String AAContent_1 = "6 April 2009 Liberian President Ellen Johnson Sirleaf arrived Thursday in the " +
  		"United States of America at the start of a three-week annual vacation, which will also include a book launch" +
  		" and an annual routine medical checkup. An Executive Mansion dispatch from Washington, D.C. says the President " +
  		"will spend most part of the week in Los Angeles, California, where she will participate in a number of events " +
  		"including the Distinguished Lecture Series. While in Los Angeles, the Liberian leader will officially launch " +
  		"her book, entitled, \"This Child Will Be Great.\" The President departs California on Friday for Minnesota " +
  		"where she will deliver the keynote address at the Carlson Lecture, hosted by the University of Minnesota. " +
  		"President Johnson Sirleaf is also expected to meet with Liberians residing in that part of the United States " +
  		"while in Minnesota. Part of the President's schedule will also include fund raising events in support of the " +
  		"Liberia Education Trust (LET), a non-governmental organization, which supports Government's education drive, " +
  		"through the provision of scholarships, schools and teachers.";
  
  private static String AAContent_2 = "Katy Gabel 6 April 2009 Nairobi  Kenyan Justice Minister Martha Karua resigned Monday," +
  		" citing frustration with the pace of government reforms. Karua's resignation comes two days after President " +
  		"Mwai Kibaki's appointment of five High Court judges Friday without Karua's approval. Karua was not aware of " +
  		"the swearing-in ceremony until it was reported by the media. According to the Daily Nation, after hearing of " +
  		"the appointments Karua described herself as powerless: \"If my hands are tied and the Judiciary continues to " +
  		"be used as a place where people sacked from parastatals are recycled, the agenda is forestalled and all reforms " +
  		"are annihilated, then I better leave and fights for the rights of ordinary [citizens].\" At a summit in Geneva " +
  		"last week, former United Nations chief Kofi Annan expressed frustration with Kenya's \"grand coalition\" " +
  		"Government. \"Ordinary Kenyans interpret the slow pace of reform as lack of political will on the part of " +
  		"their leaders,\" he said. \"They see it, rightly or wrongly, as proof that the political elite are putting " +
  		"their own partisan interests above the interest of the nation as a whole.\" At a press conference Monday, " +
  		"Karua told reporters \"I will now be able to totally disagree with anything that is anti-reform.\" Her " +
  		"resignation is widely viewed as a move to secure her bid for the presidency in 2012. Karua, a long-time " +
  		"politician from Kenya's Central Province, has already launched a campaign web site at www.marthakarua.net. " +
  		"Kenya's coalition was formed in March 2008 to end violence and instability following the disputed December " +
  		"2007 presidential elections."; 

  
  @Test
  public void testAllAfricaHTMLReader() throws IOException {
    AllAfricaHTMLReader reader = new AllAfricaHTMLReader();
    String author = "Smith";
    Date date = new Date();
    String title = "Liberia: President Sirleaf Kicks-Off U.S. Visit With Weeklong Activities in Los Angeles, California";
    ReaderResult result = reader.read("http://allafrica.com/stories/printable/200904060876.html", title, date, author);
    assertEquals(ReaderResult.Error.NONE, result.isValid());
    assertEquals(title, result.getTitle());
    assertEquals(date, result.getDate().getJavaDate());
    assertEquals(author, result.getAuthors().getAuthors());
    assertEquals(AAContent_1, result.getContents().trim());
    
    title = "Kenya: Justice Minister Resigns";
    result = reader.read("http://allafrica.com/stories/printable/200904061324.html", title, date, author);
    assertEquals(ReaderResult.Error.NONE, result.isValid());
    assertEquals(title, result.getTitle());
    assertEquals(date, result.getDate().getJavaDate());
    assertEquals(author, result.getAuthors().getAuthors());
    assertEquals(AAContent_2, result.getContents().trim());
  }
  
  @Test
  public void testLineParser() {
    String[] lines = {"1235916000000:::http://www.nationalterroralert.com/updates/2009/03/01/company-discovers-marine-one-security-breach/::: Company Discovers Marine One Security Breach:::Nick Collier",
      "1235916280000:::http://www.nationalterroralert.com/updates/2009/03/01/us-says-iran-has-enough-material-for-nuclear-bomb/::: U.S. Says Iran Has Enough Material for Nuclear Bomb:::",
      "1235916320000:::http://www.nationalterroralert.com/updates/2009/03/01/homeland-security-seeks-next-generation-real-id/::: Homeland Security Seeks Next Generation REAL ID:::Bob Smith"};
    
    String[] timestamps = {"1235916000000", "1235916280000", "1235916320000"};
    String[] urls = {"http://www.nationalterroralert.com/updates/2009/03/01/company-discovers-marine-one-security-breach/",
	"http://www.nationalterroralert.com/updates/2009/03/01/us-says-iran-has-enough-material-for-nuclear-bomb/",
	"http://www.nationalterroralert.com/updates/2009/03/01/homeland-security-seeks-next-generation-real-id/"};
    String[] titles = {"Company Discovers Marine One Security Breach", 
	"U.S. Says Iran Has Enough Material for Nuclear Bomb",
	"Homeland Security Seeks Next Generation REAL ID"};
    String[] authors = {"Nick Collier", "", "Bob Smith"};
    
    StandardIndexLineParser parser = new StandardIndexLineParser();
    String[] result = new String[4];
    for (int i = 0; i < lines.length; i++) {
      result = parser.parse(lines[i], result);
      assertEquals(timestamps[i], result[StandardIndexLineParser.TIMESTAMP]);
      assertEquals(urls[i], result[StandardIndexLineParser.URL]);
      assertEquals(titles[i], result[StandardIndexLineParser.TITLE]);
      assertEquals(authors[i], result[StandardIndexLineParser.AUTHOR]);
    }
  }
  
  private static class RRProcessor  implements ReaderResultProcessor {
    
    int count;
    String expectedSource;
    
    public RRProcessor(String expectedSource) {
      this.expectedSource = expectedSource;
    }

    @Override
    public void process(ReaderResult result) throws AIDAException {
      count++;
      assertEquals(expectedSource, result.getSource());
      ReaderResult.Error error = result.isValid();
      if (error != ReaderResult.Error.NONE) {
	System.out.println("result.getError() = " + error);
	System.out.println("result = " + result);
	//fail();
      }
    }

    @Override
    public void done() throws IOException {}

    @Override
    public void initialize() throws AIDAException {}
  }
  
  //@Test
  public void testNYTArchiveReader() throws IOException, ParseException, AIDAException {
    NYTimesArchiveReader reader = new NYTimesArchiveReader();
    RRProcessor proc = new RRProcessor(ReaderConstants.NYTIMES_SOURCE);
    reader.run(proc);
    
    assertEquals(2549, proc.count);
  }
  
  //@Test
  public void testTRIBArchiveReader() throws IOException, ParseException, AIDAException {
    CTArchiveReader reader = new CTArchiveReader();
    RRProcessor proc = new RRProcessor(ReaderConstants.CHICAGO_TRIBUNE_SOURCE);
    reader.run(proc);
    
    // 8658 is the number of entries in the trib_archive_index
    // which is where the individual articles are pulled from
    assertEquals(8658, proc.count);
  }
  
  public ConfigurationParameterSettings setUpCAS(String compsFile, String indexKey, String indexValue) throws IOException, InvalidXMLException,
      ResourceInitializationException, AnalysisEngineProcessException {
    XMLInputSource in = new XMLInputSource("./desc/MIFSCompositeReader.xml");
    CollectionReaderDescription spec = (CollectionReaderDescription) UIMAFramework.getXMLParser()
	.parseResourceSpecifier(in);
    ConfigurationParameterSettings settings = spec.getCollectionReaderMetaData()
	.getConfigurationParameterSettings();
    // todo switch to use configurator
    //settings.setParameterValue(MIFSCompositeReader.COMPONENTS, compsFile);
    if (indexKey != null) settings.setParameterValue(indexKey, indexValue);
    reader = UIMAFramework.produceCollectionReader(spec);

    CasManager casManager = reader.getResourceManager().getCasManager();
    casManager.addMetaData((ProcessingResourceMetaData) reader.getMetaData());
    casManager.defineCasPool("pool", 2, null);
    cas = casManager.getCas("pool");
    
    return settings;
  }

  public void tearDownCAS() {
    cas.reset();
  }
  
  @Test
  public void testNTARCReader() throws IOException, ParseException, AnalysisEngineProcessException,
      InvalidXMLException, ResourceInitializationException, CollectionException {
    try {
    ConfigurationParameterSettings settings = setUpCAS("./test_data/component_readers_ntarc_only.txt",
	null, null);
	//NTARCReader.INDEX_FILE, "./ntarc_archive_index.txt");
    
    // get the index file and see how many lines are in it
    // in order to be sure that we read that many lines
    String indexFile = (String)settings.getParameterValue(NTARCReader.INDEX_FILE);
    BufferedReader bReader = new BufferedReader(new FileReader(indexFile));
    int expectedCount = 0;
    while (bReader.readLine() != null) {
      expectedCount++;
    }
    bReader.close();
    
    NTARCHtmlReader ntarcReader = new NTARCHtmlReader();
    
    Type type = cas.getTypeSystem().getType("anl.mifs.types.DocumentMetaData");
    int count = 0;
    while (reader.hasNext()) {
      reader.getNext(cas);
      AnnotationIndex idx = cas.getAnnotationIndex(type);
      assertEquals(1, idx.size());
      DocumentMetaData data = (DocumentMetaData) idx.iterator().next();
      assertTrue(data.getCAS().getDocumentText().length() > 0);
      assertTrue(data.getLocation().length() > 0);
      assertTrue(data.getDate().getDay() > 0);
      assertTrue(data.getDate().getMonth() > 0);
      
      // don't care about the title and date,
      // but testing the content is the same.
      ReaderResult result = ntarcReader.read(data.getLocation(), "Foo", new Date(), "");
      assertEquals(cas.getDocumentText().trim(), result.getContents().trim());
     
      
      cas.reset();
      count++;
    }
    tearDownCAS();
    assertEquals(expectedCount, count);
    assertTrue(count > 0);
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }
  
  @Test
  public void testNYTReader() throws IOException, ParseException, AnalysisEngineProcessException,
      InvalidXMLException, ResourceInitializationException, CollectionException {
    ConfigurationParameterSettings settings = setUpCAS("./test_data/component_readers_nyt_only.txt", null, null); 
    //NYTimesReader.INDEX_FILE, "./nytimes_archive_index.txt");
    
    // get the index file and see how many lines are in it
    // in order to be sure that we read that many lines
    String indexFile = (String)settings.getParameterValue(NYTimesReader.INDEX_FILE);
    BufferedReader bReader = new BufferedReader(new FileReader(indexFile));
    int expectedCount = 0;
    while (bReader.readLine() != null) {
      expectedCount++;
    }
    bReader.close();
    
    NYTimesHTMLReader nytReader = new NYTimesHTMLReader();
    
    Type type = cas.getTypeSystem().getType("anl.mifs.types.DocumentMetaData");
    int count = 0;
    while (reader.hasNext()) {
      reader.getNext(cas);
      AnnotationIndex idx = cas.getAnnotationIndex(type);
      assertEquals(1, idx.size());
      DocumentMetaData data = (DocumentMetaData) idx.iterator().next();
      assertTrue(data.getCAS().getDocumentText().length() > 0);
      assertTrue(data.getLocation().length() > 0);
      assertTrue(data.getDate().getDay() > 0);
      assertTrue(data.getDate().getMonth() > 0);
      
      // don't care about the title, date or author
      // but testing the content is the same.
      ReaderResult result = nytReader.read(data.getLocation(), "Foo", new Date(), "ntc");
      assertEquals(cas.getDocumentText().trim(), result.getContents().trim());
      
      cas.reset();
      count++;
    }
    tearDownCAS();
    assertEquals(expectedCount, count);
    assertTrue(count > 0);
  }
  
  @Test
  public void testCTReader() throws IOException, ParseException, AnalysisEngineProcessException,
      InvalidXMLException, ResourceInitializationException, CollectionException {
    ConfigurationParameterSettings settings = setUpCAS("./test_data/component_readers_ct_only.txt", 
	ChicagoTribuneReader.INDEX_FILE, "./trib_archive_index.txt");
    
    // get the index file and see how many lines are in it
    // in order to be sure that we read that many lines
    String indexFile = (String)settings.getParameterValue(ChicagoTribuneReader.INDEX_FILE);
    BufferedReader bReader = new BufferedReader(new FileReader(indexFile));
    int expectedCount = 0;
    while (bReader.readLine() != null) {
      expectedCount++;
    }
    bReader.close();
    
    ChicagoTribuneHTMLReader ctReader = new ChicagoTribuneHTMLReader();
    
    Type type = cas.getTypeSystem().getType("anl.mifs.types.DocumentMetaData");
    int count = 0;
    while (reader.hasNext()) {
      try {
      reader.getNext(cas);
      } catch (Exception ex) {
	ex.printStackTrace();
      }
     
      AnnotationIndex idx = cas.getAnnotationIndex(type);
      assertEquals(1, idx.size());
      DocumentMetaData data = (DocumentMetaData) idx.iterator().next();
      assertTrue(data.getCAS().getDocumentText().length() > 0);
      assertTrue(data.getLocation().length() > 0);
      assertTrue(data.getDate().getDay() > 0);
      assertTrue(data.getDate().getMonth() > 0);
      
      /*
      // don't care about the title, date or author
      // but testing the content is the same.
      ReaderResult result = ctReader.read(data.getLocation(), "Foo", new Date(), "");
      assertEquals(cas.getDocumentText().trim(), result.getContents().trim());
      */
      
      cas.reset();
      count++;
      if (count % 100 == 0) {
	System.out.println("Parsed " + count + " items" );
      }
     
    }
    tearDownCAS();
    assertEquals(expectedCount, count);
    assertTrue(count > 0);
  }
}
